#ifndef CONTROLS_H
#define CONTROLS_H

#include <gtk/gtk.h>
#include <libgwydgets/gwydgets.h>

/* TEMPORARY SOLUTION UNTIL GUI DEFINITIONS ARE DISTRIBUTED TO THE TOOLS */

/* Main */

typedef struct {
    GtkWidget *window_main;
    GtkWidget *scrolledwindow_main;
    GtkWidget *vbox_main;
    
    GtkWidget *notebook_tools;
    GtkWidget *hbox_buttons;
    
    GtkWidget *tab_contact, *tab_op, *tab_op_odr, *tab_tangent,
	*tab_hertz, *tab_slopes, *tab_ph2, *tab_apopins, *tab_epwork ;
    /* *tab_creep, *tab_relax, *tab_ppopins */

    GtkWidget *tab_contact_label, *tab_op_label, *tab_op_odr_label, *tab_tangent_label,
	*tab_hertz_label, *tab_slopes_label, *tab_ph2_label, *tab_apopins_label, *tab_epwork_label;
    /*  *tab_creep_label, *tab_relax_label, *tab_ppopins_label */
    
    GtkWidget *button_open, *button_save, *button_help, *button_about, *button_quit;
    GtkWidget *dialog_about;
} MainControls;

/* Contact */

enum FDPlotMode {
    PLOT_FD = 0,
    PLOT_FD_TIME
};

typedef struct {
    GtkWidget *contact_gui;
    //GtkWidget *tool_label;

    /* Controls (left) */
    GtkWidget *vbox_ctrls;

    /* info */
    GtkWidget *frame_info;
    GtkWidget *table_info;
    GtkWidget *hmax, *Fmax;

    /* graph mode */
    GtkWidget *hbox_switch;
    GtkWidget *button_switch_to_fd, *button_switch_to_fd_time;
    enum FDPlotMode fd_plot_mode;

    /* NEW contact, load, hold, unload */
    GtkWidget *frame_contact_hold;
    GtkWidget *table_contact_hold;
    GtkWidget *status_contact_load, *status_load_hold, *status_hold_unload,
	*status_end_unload;
    GtkWidget *button_contact_point, *button_load_hold,	*button_hold_unload,
	*button_end_unload;
    GtkWidget *button_split_auto;

    /* save data */
    GtkWidget *button_save_data;

    /* area */
    GtkWidget *frame_area;
    GtkWidget *table_area;
    GtkWidget *area_function_button;
    GtkWidget *area_function_label;

    /* sample & indenter */
    GtkWidget *frame_sample_indenter;
    GtkWidget *table_sample_indenter;
    GtkObject *nu, *Ei, *nui;
    GtkWidget *button_reset;

    /* Graph (right) */
    GtkWidget *hbox_graph;

    GtkWidget *graph_fd;
    GtkWidget *vbox_zoom_buttons_fd, *vbox_zoom_outer_fd;
    GtkWidget *button_zoom_in_fd, *button_zoom_out_fd, *button_zoom_restore_fd;
   
    GtkWidget *graph_fd_time;
    GtkWidget *vbox_zoom_buttons_fd_time, *vbox_zoom_outer_fd_time;
    GtkWidget *button_zoom_in_fd_time, *button_zoom_out_fd_time, *button_zoom_restore_fd_time;

    GwySelection *selection_fd, *selection_fd_time;

    /* stacks of graph models for zooming */
    GSList *zoomstack_fd, *zoomstack_fd_time;

    /* these should point to structures used by currently displayed graph */
    GwyGraphModel *graph_model_fd, *graph_model_fd_time;
    GwyGraphCurveModel *cmodelload, *cmodelhold, *cmodelunload;
    GwyGraphCurveModel *cmodelFtime, *cmodelhtime;
    GwyGraphCurveModel *cmodel_contact_load, *cmodel_load_hold, *cmodel_hold_unload, *cmodel_end_unload;
} ContactControls;

/* Area */

typedef struct {
    GtkWidget *dialog;
    GtkWidget *hbox;

    GtkWidget *area_table;
    
    /* OP */
    GtkWidget *opp_button;
    GtkWidget **area_polynom;
    
    /* Raw data */
    GtkWidget *rd_button;
    GtkWidget *area_entry;
    GtkWidget *area_file_button;

    /* Graph */
    GtkWidget *area_graph;
    GwyGraphModel *gmodel;
    GwyGraphCurveModel *cmodel;
} AreaControls;

/* OP */

typedef struct {
    GtkWidget *op_gui;

    /* Ctrls (left) */
    GtkWidget *ctrls;

    /* info */
    GtkWidget *frame_info;
    GtkWidget *table_info;
    GtkWidget *hmax, *Fmax;

    /* parameters */
    GtkWidget *frame_parameters;
    GtkWidget *table_parameters;

    GtkWidget *from, *to;
    GtkWidget *from_pct_Fmax, *to_pct_Fmax;
	
    /* fit buttons */
    GtkWidget *hbox_fitbuttons;
    GtkWidget *button_fit_hp, *button_fit_S;

    /* results */
    GtkWidget *frame_results;
    GtkWidget *table_results;

    GtkWidget *hp, *m, *eps, *hc, *S, *Aphc;
    GtkWidget *hpchi2, *hpR; /* created but not attached */
    GtkWidget *Schi2, *SR; /* created but not attached */
    GtkWidget *extrapol;
    GtkWidget *Hit, *Er, *Eit;
    GtkWidget *hprange, *hprange_pct_Fmax;
    GtkWidget *Srange, *Srange_pct_Fmax;
      	
    /* unc, save */
    GtkWidget *button_save, *button_unc;

    /* Graph (right) */
    GtkWidget *hbox_graph;
	
    GtkWidget *graph;
    GtkWidget *vbox_zoom_buttons, *vbox_zoom_outer;
    GtkWidget *button_zoom_in, *button_zoom_out, *button_zoom_restore;

    GwySelection *selection;
	
    GSList *zoomstack;

    /* these should point to structures used by currently displayed graph */
    GwyGraphModel *graph_model;
    GwyGraphCurveModel *cmodelunload, *cmodelhpfit, *cmodelSfit;
} OPControls;

typedef struct {
    GtkObject *uh, *uF;
    GtkObject *unu, *unui, *uEi;

    GtkWidget *uSuh, *uSuF;
    GtkWidget *uhcuh, *uhcuF;
    GtkWidget *uAuh, *uAuF;
    GtkWidget *uHituh, *uHituF;

    GtkWidget *uEruh, *uEruF;
    GtkWidget *uEituh, *uEituF;
    GtkWidget *uEitunu, *uEitunui, *uEituEi;

    GtkWidget *uS,*uhc, *uA, *uHit, *uEr, *uEit;

    GtkWidget *window;

    GtkObject *Nmc;
    GtkWidget *mc_warn;

    GtkWidget *vbox;
    GtkWidget *frame_unc_input, *frame_unc_contact, *frame_mc;
    GtkWidget *table_unc_input, *table_unc_contact, *table_mc;
    GtkWidget *button_save, *button_mc;
} OPUncControls;

typedef struct{
    GtkWidget *window;
    gboolean has_window;
    GtkWidget *vbox;
    GtkWidget *frame_input, *frame_results;
    GtkWidget *table_input, *table_results;
    GtkWidget *button_save;
    GtkWidget **button_hist;
    GtkWidget **hist_window;
//	gboolean *has_hist_window;
} OPMCControls;

/* OP ODR */

typedef struct{
    GtkWidget *opodr_gui;

    /* Controls (left) */
    GtkWidget *ctrls;

    /* info */
    GtkWidget *frame_info;
    GtkWidget *table_info;

    GtkWidget *hmax, *Fmax;
	
    /* parameters */
    GtkWidget *frame_parameters;
    GtkWidget *table_parameters;

    GtkWidget *from, *to;	
    GtkWidget *from_pct_Fmax, *to_pct_Fmax;	

    /* fit button */
    GtkWidget *button_fit;

    /* results */
    GtkWidget *frame_results;
    GtkWidget *table_results;

    GtkWidget *fitinfo;
    GtkWidget *hp, *m, *eps, *hc, *S, *Aphc;
    GtkWidget *extrapol;
    GtkWidget *Hit, *Er, *Eit;
    GtkWidget *range, *range_pct_Fmax;

    /* log, unc, save */
    GtkWidget *button_log, *button_unc, *button_save;

    /* log windows */
    GtkWidget *log_error_window, *log_report_window;

    /* Graph (right) */
    GtkWidget *hbox_graph;
	
    GtkWidget *graph;
    GtkWidget *vbox_zoom_buttons, *vbox_zoom_outer;
    GtkWidget *button_zoom_in, *button_zoom_out, *button_zoom_restore;

    GwySelection *selection;

    GSList *zoomstack;

    /* these should point to structures used by currently displayed graph */
    GwyGraphModel *graph_model;
    GwyGraphCurveModel *cmodelunload, *cmodelSfit;
} OPODRControls;

typedef struct {
    GtkObject *uh, *uF;
    GtkObject *unu, *unui, *uEi;

    GtkWidget *uhcuh, *uhcuF;
    GtkWidget *uSuh, *uSuF;
    GtkWidget *umuh, *umuF;
    GtkWidget *uAuh, *uAuF;
    GtkWidget *uHituh, *uHituF;

    GtkWidget *uEruh, *uEruF;
    GtkWidget *uEituh, *uEituF;
    GtkWidget *uEitunu, *uEitunui, *uEituEi;

    GtkWidget *uS, *um,*uhc, *uA, *uHit, *uEr, *uEit;

    GtkWidget *window;

    GtkObject *Nmc;
    GtkWidget *spin_Nmc;
    GtkWidget *mc_warn;

    GtkWidget *vbox;
    GtkWidget *frame_unc_input, *frame_unc_contact, *frame_mc;
    GtkWidget *table_unc_input, *table_unc_contact, *table_mc;
    GtkWidget *button_save, *button_mc;
} OPODRUncControls;

typedef struct {
    GtkWidget *window;
    gboolean has_window;
    GtkWidget *vbox;
    GtkWidget *frame_input, *frame_results;
    GtkWidget *table_input, *table_results;
    GtkWidget *button_save;
    GtkWidget **button_hist;
    GtkWidget **hist_window;
//	gboolean *has_hist_window;
} OPODRMCControls;

/* Tangent */

typedef struct{
    GtkWidget *tangent_gui;

    /* Controls (left) */
    GtkWidget *ctrls;

    /* info */
    GtkWidget *frame_info;
    GtkWidget *table_info;

    GtkWidget *hmax, *Fmax;

    /* parameters */
    GtkWidget *frame_parameters;
    GtkWidget *table_parameters;

    GtkWidget *from, *to;	
    GtkWidget *from_pct_Fmax, *to_pct_Fmax;	

    /* fit button */
    GtkWidget *button_fit;

    /* results */
    GtkWidget *frame_results;
    GtkWidget *table_results;

    GtkWidget *hc, *S, *Aphc;
    GtkWidget *extrapol;
    GtkWidget *chi2, *R; /* created but not attached */
    GtkWidget *Hit, *Er, *Eit;
    GtkWidget *range, *range_pct_Fmax;

    /* unc, save */
    GtkWidget *button_unc, *button_save;

    /* Graph (right) */
    GtkWidget *hbox_graph;
	
    GtkWidget *graph;
    GtkWidget *vbox_zoom_buttons, *vbox_zoom_outer;
    GtkWidget *button_zoom_in, *button_zoom_out, *button_zoom_restore;

    GwySelection *selection;

    GSList *zoomstack;

    GwyGraphModel *graph_model;
    GwyGraphCurveModel *cmodelunload, *cmodelSfit;
} TangentControls;

typedef struct {
    GtkObject *uh, *uF;
    GtkObject *unu, *unui, *uEi;

    GtkWidget *uhcuh, *uhcuF;
    GtkWidget *uSuh, *uSuF;
    GtkWidget *uAuh, *uAuF;
    GtkWidget *uHituh, *uHituF;

    GtkWidget *uEruh, *uEruF;
    GtkWidget *uEituh, *uEituF;
    GtkWidget *uEitunu, *uEitunui, *uEituEi;

    GtkWidget *uS,*uhc, *uA, *uHit, *uEr, *uEit;

    GtkObject *Nmc;
    GtkWidget *mc_warn;

    GtkWidget *window;
    GtkWidget *vbox;
    GtkWidget *frame_unc_input, *frame_unc_contact, *frame_mc;
    GtkWidget *table_unc_input, *table_unc_contact, *table_mc;
    GtkWidget *button_save, *button_mc;
} TangentUncControls;

typedef struct {
    GtkWidget *window;
    gboolean has_window;
    GtkWidget *vbox;
    GtkWidget *frame_input, *frame_results;
    GtkWidget *table_input, *table_results;
    GtkWidget *button_save;
    GtkWidget **button_hist;
    GtkWidget **hist_window;
//	gboolean *has_hist_window;
} TangentMCControls;

/* Hertz */
typedef struct{
    GtkWidget *hertz_gui;

    /* Controls (left) */
    GtkWidget *ctrls;

    /* info */
    GtkWidget *frame_info;
    GtkWidget *table_info;

    GtkWidget *hmax, *Fmax;

    /* parameters */
    GtkWidget *frame_parameters;
    GtkWidget *table_parameters;

    GtkWidget *from, *to;	

    /*radio buttons to switch between and R and E mode */
    GtkWidget *radio_R, *radio_Er, *radio_Eit;

//	GtkObject *radiusinp, *Eitinp, *Erinp;	/*input according to radio buttons */
    GtkWidget *radiusinp, *Eitinp, *Erinp;	/*input according to radio buttons */

    /* fit button */
    GtkWidget *button_fit;

    /* results */
    GtkWidget *frame_results;
    GtkWidget *table_results;

    GtkWidget *chi2; /* created but not attached */
    GtkWidget *Erres, *Eitres, *radiusres; /*display results */
    GtkWidget *range;	

    /* unc, save */
    GtkWidget *button_unc, *button_save;

    /* Graph (right) */
    GtkWidget *hbox_graph;
	
    GtkWidget *graph;
    GtkWidget *vbox_zoom_buttons, *vbox_zoom_outer;
    GtkWidget *button_zoom_in, *button_zoom_out, *button_zoom_restore;

    GwySelection *selection;

    GSList *zoomstack;
	
    GwyGraphModel *graph_model;
    GwyGraphCurveModel *cmodelload, *cmodelfit;
} HertzControls;

typedef struct {
    GtkObject *uh, *uF;
    GtkObject *unu, *unui, *uEi;
    GtkObject *uradius;
    GtkWidget *uEruh, *uEruF;
    GtkWidget *uEituh, *uEituF;
    GtkWidget *uEitunu, *uEitunui, *uEituEi;
    GtkWidget *uEruradius, *uEituradius;

    GtkWidget *uEr, *uEit;

    GtkObject *Nmc;
    GtkWidget *mc_warn;

    GtkWidget *window;
    GtkWidget *vbox;
    GtkWidget *frame_unc_input, *frame_unc_contact, *frame_mc;
    GtkWidget *table_unc_input, *table_unc_contact, *table_mc;
    GtkWidget *button_save, *button_mc;
} HertzUncControls;

typedef struct {
    GtkWidget *window;
    gboolean has_window;
    GtkWidget *vbox;
    GtkWidget *frame_input, *frame_results;
    GtkWidget *table_input, *table_results;
    GtkWidget *button_save;
    GtkWidget **button_hist;
    GtkWidget **hist_window;
//	gboolean *has_hist_window;
} HertzMCControls;

/* Two slopes */

typedef struct{
    GtkWidget *slopes_gui;

    /* Ctrls (left) */
    GtkWidget *ctrls;

    /* info */
    GtkWidget *frame_info;
    GtkWidget *table_info;
    GtkWidget *hmax, *Fmax;

    /* parameters */
    GtkWidget *frame_parameters;
    GtkWidget *table_parameters;

    GtkWidget *from, *to;
    GtkWidget *from_pct_Fmax, *to_pct_Fmax;
	
    /* fit buttons */
    GtkWidget *hbox_fitbuttons;
    GtkWidget *button_fit_load, *button_fit_unload;

    /* results */
    GtkWidget *frame_results;
    GtkWidget *table_results;

    GtkWidget *fitinfoload, *fitinfounload;
    GtkWidget *hp, *m, *eps, *h0, *n, *Sload, *Sunload;
    GtkWidget *Hit, *Er, *Eit, *Aphc;
    GtkWidget *loadrange, *loadrange_pct_Fmax;
    GtkWidget *unloadrange, *unloadrange_pct_Fmax;
      	
    /* unc, save */
    GtkWidget *button_save, *button_unc;

    /* Log */
    GtkWidget *hbox_logbuttons;
    GtkWidget *button_log_load, *button_log_unload;
    GtkWidget *load_log_error_window, *unload_log_error_window; 
    GtkWidget *load_log_report_window, *unload_log_report_window; 


    /* Graph (right) */
    GtkWidget *hbox_graph;
	
    GtkWidget *graph;
    GtkWidget *vbox_zoom_buttons, *vbox_zoom_outer;
    GtkWidget *button_zoom_in, *button_zoom_out, *button_zoom_restore;

    GwySelection *selection;
	
    GSList *zoomstack;

    /* these should point to structures used by currently displayed graph */
    GwyGraphModel *graph_model;
    GwyGraphCurveModel *cmodelload, *cmodelhold, *cmodelunload, *cmodelloadfit, *cmodelunloadfit;
} SlopesControls;

typedef struct {
    GtkObject *uh, *uF;
    GtkObject *unu, *unui, *uEi;

    GtkWidget *uSluh, *uSluF;
    GtkWidget *uSuuh, *uSuuF;
    GtkWidget *umuh, *umuF;
    GtkWidget *unuh, *unuF;
    GtkWidget *uAuh, *uAuF;
    GtkWidget *uHituh, *uHituF;

    GtkWidget *uEruh, *uEruF;
    GtkWidget *uEituh, *uEituF;
    GtkWidget *uEitunu, *uEitunui, *uEituEi;

    GtkWidget *uSu, *uSl, *un, *um,*uA, *uHit, *uEr, *uEit;

    GtkWidget *window;

    GtkObject *Nmc;
    GtkWidget *spin_Nmc;
    GtkWidget *mc_warn;

    GtkWidget *vbox;
    GtkWidget *frame_unc_input, *frame_unc_contact, *frame_mc;
    GtkWidget *table_unc_input, *table_unc_contact, *table_mc;
    GtkWidget *button_save, *button_mc;
} SlopesUncControls;

typedef struct {
    GtkWidget *window;
    gboolean has_window;
    GtkWidget *vbox;
    GtkWidget *frame_input, *frame_results;
    GtkWidget *table_input, *table_results;
    GtkWidget *button_save;
    GtkWidget **button_hist;
    GtkWidget **hist_window;
//	gboolean *has_hist_window;
} SlopesMCControls;



/* Ph2 */

typedef struct{
    GtkWidget *ph2_gui;

    /* Controls (left) */
    GtkWidget *ctrls;

    /* parameters */
    GtkWidget *frame_parameters;
    GtkWidget *table_parameters;

    GtkObject *nmove;

    /* run, save */
    GtkWidget *button_save, *button_run;

    /* Graphs (right) */
    GtkWidget *vbox_graphs;

    GtkWidget *hbox_graph_Fh;
    GtkWidget *graph_Fh;
    GtkWidget *vbox_zoom_buttons_Fh, *vbox_zoom_outer_Fh;
    GtkWidget *button_zoom_in_Fh, *button_zoom_out_Fh, *button_zoom_restore_Fh;

    GtkWidget *hbox_graph_H;
    GtkWidget *graph_H;
    GtkWidget *vbox_zoom_buttons_H, *vbox_zoom_outer_H;
    GtkWidget *button_zoom_in_H, *button_zoom_out_H, *button_zoom_restore_H;

    GSList *zoomstack_Fh, *zoomstack_H;

    GwyGraphModel *graph_modelFh, *graph_modelPh2;
    GwyGraphCurveModel *cmodelload, *cmodelavg, *cmodelPh2, *cmodeldPdh2; 
} Ph2Controls;

/* Pop-ins A */

typedef struct{
    GtkWidget *apop_gui;

    /* Controls (left) */
    GtkWidget *ctrls;

    /* parameters */
    GtkWidget *frame_parameters;
    GtkWidget *table_parameters;

    GtkObject *nmove, *thresh, *thresh2, *wpop, *hpop;
    GtkWidget *button_reset;

    /* run button */
    GtkWidget *button_run;
	
    /* results */
    GtkWidget *frame_results;
    GtkWidget *table_results;

    GtkWidget *npopin;
    GtkWidget *res;

    /* save button */
    GtkWidget *button_save;

    /* Graphs (right) */
    GtkWidget *vbox_graphs;

    GtkWidget *hbox_graph_h;
    GtkWidget *graph_h;
    GtkWidget *vbox_zoom_buttons_h, *vbox_zoom_outer_h;
    GtkWidget *button_zoom_in_h, *button_zoom_out_h, *button_zoom_restore_h;

    GtkWidget *hbox_graph_der;
    GtkWidget *graph_der;
    GtkWidget *vbox_zoom_buttons_der, *vbox_zoom_outer_der;
    GtkWidget *button_zoom_in_der, *button_zoom_out_der, *button_zoom_restore_der;

    GSList *zoomstack_h, *zoomstack_der;
	
    GwyGraphModel *graph_model, *graph_model_der;
    GwyGraphCurveModel *cmodelload, *cmodelhavg, *cmodelder, *cmodelthresh, *cmodelthresh2;
} ApopinControls;

/* Work */

typedef struct{
    GtkWidget *work_gui;

    /* Controls (left) */
    GtkWidget *ctrls;

    /* parameters */
    GtkWidget *frame_parameters;
    GtkWidget *table_parameters;

    GtkObject *nmove;

    /* run button */
    GtkWidget *button_run;

    /* results */
    GtkWidget *frame_results;
    GtkWidget *table_results;

    GtkWidget *workplastic, *workelastic;
    GtkWidget *eta;

    /* unc, save */
    GtkWidget *button_save,  *button_unc;

    /* Graph (right) */
    GtkWidget *hbox_graph;

    GtkWidget *graph;
    GtkWidget *vbox_zoom_buttons, *vbox_zoom_outer;
    GtkWidget *button_zoom_in, *button_zoom_out, *button_zoom_restore;

    GSList *zoomstack;

    /* these should point to structures used by currently displayed graph */
    GwyGraphModel *graph_model;
    GwyGraphCurveModel *cmodelload, *cmodelloadavg;
    GwyGraphCurveModel *cmodelhold, *cmodelholdavg;
    GwyGraphCurveModel *cmodelunload, *cmodelunloadavg;
} WorkControls;

typedef struct {
    GtkWidget *window;
    GtkWidget *table_contact;
} WorkUncControls;

/* Unused */

typedef struct {
    gint neco_aby_se_to_prelozilo;
} CreepControls;

typedef struct {
    gint neco_aby_se_to_prelozilo;
} RelaxationControls;

typedef struct {
    gint neco_aby_se_to_prelozilo;
} LoadControls;

typedef struct {
    GtkWidget *ppop_gui;
    GtkWidget *ctrls;

    GtkWidget *button_save, *button_run;
} PpopinControls;

/* Controls */

typedef struct {
    MainControls *maincontrols;
    ContactControls *contactcontrols;
    LoadControls *loadcontrols;
    AreaControls *areacontrols;

    OPControls *opcontrols;
    OPUncControls *opunccontrols;
    OPMCControls *opmccontrols;

    OPODRControls *opodrcontrols;
    OPODRUncControls *opodrunccontrols;
    OPODRMCControls *opodrmccontrols;

    TangentControls *tgcontrols;
    TangentUncControls *tgunccontrols;
    TangentMCControls *tgmccontrols;

    HertzControls *hertzcontrols;
    HertzUncControls *hertzunccontrols;
    HertzMCControls *hertzmccontrols;

    SlopesControls *slopescontrols;
    SlopesUncControls *slopesunccontrols;
    SlopesMCControls *slopesmccontrols;

    Ph2Controls *ph2controls;

    ApopinControls *apopcontrols;

    WorkControls *workcontrols;
    WorkUncControls *workunccontrols;

    /*not working at the moment */
    PpopinControls *ppopcontrols;
    RelaxationControls *relaxcontrols;
    CreepControls *creepcontrols;
} Controls;

#endif
