#include "fddata.h"

/* get_hmax_Fmax */
#include "indent-toolbox-common.h"

#include <libprocess/gwyprocess.h>

void
backup_hF_all_data(FDdata *fddata)
{
    if (fddata->hall_orig)
	g_object_unref(fddata->hall_orig);

    if (fddata->Fall_orig)
	g_object_unref(fddata->Fall_orig);

    fddata->hall_orig = gwy_data_line_duplicate(fddata->hall);
    fddata->Fall_orig = gwy_data_line_duplicate(fddata->Fall);
}

void 
init_FDdata(FDdata *fddata)
{
    printf("initializing FD data\n");
    fddata->hall = NULL;
    fddata->Fall = NULL;
    fddata->hload = NULL;
    fddata->Fload = NULL;
    fddata->hhold = NULL;
    fddata->Fhold = NULL;
    fddata->hunload = NULL;
    fddata->Funload = NULL;

    fddata->hall_orig = NULL;
    fddata->Fall_orig = NULL;

    fddata->time = NULL;
    fddata->hall_time = NULL;
    fddata->Fall_time = NULL;

    fddata->filename = NULL;
    fddata->Fc = 0.0;
    fddata->hmax = 0;
    fddata->Fmax = 0;
    fddata->i_contact_load = -1;
    fddata->i_load_hold = -1;
    fddata->i_hold_unload = -1;
    fddata->i_end_unload = -1;

    fddata->status_contact_load = POINT_STATUS_NOT_SET;
    fddata->status_load_hold = POINT_STATUS_NOT_SET;
    fddata->status_hold_unload = POINT_STATUS_NOT_SET;
    fddata->status_end_unload = POINT_STATUS_NOT_SET;
}

void
restore_fd_curves(FDdata *fddata)
{
    g_object_unref(fddata->hall);
    g_object_unref(fddata->Fall);
    g_object_unref(fddata->hload);
    g_object_unref(fddata->Fload);
    g_object_unref(fddata->hhold);
    g_object_unref(fddata->Fhold);
    g_object_unref(fddata->hunload);
    g_object_unref(fddata->Funload);

    fddata->hall = gwy_data_line_duplicate(fddata->hall_orig);
    fddata->Fall = gwy_data_line_duplicate(fddata->Fall_orig);
}

gboolean //TODO rewrite to better version, that includes also hold
split_hF_data(FDdata *fddata)
{
    gint i, i_load_hold, i_hold_unload, i_end_unload;
	GwyDataLine *F;
	gdouble uF;
	gboolean isok = TRUE;

	/*fddata->Fmax = gwy_data_line_get_max(fddata->Fall);
	fddata->hmax = gwy_data_line_get_max(fddata->hall);
	*/

	fddata_get_hmax_Fmax(fddata->hall, fddata->Fall, &(fddata->hmax), &(fddata->Fmax));
	/* neriskujeme tady, ze hmax bude defacto nahodila hodnota z hold oblasti? */

	uF = 0.0;
	if (fddata->i_contact_load > 8) {
	    F = gwy_data_line_part_extract(fddata->Fall_orig, 0, fddata->i_contact_load);
	    uF = gwy_data_line_get_rms(F);
	    g_object_unref(F);
	}
	
	/*
	if (uF < DBL_EPSILON )  // do NOT use Fc before it could be set
	    uF = 0.5 * fddata->Fc;
	*/
	
	if (uF < DBL_EPSILON)
	    uF = 0.001 * fddata->Fmax;

	printf("uF: %g\n",uF);

	i = 0;
	
	while ((i < fddata->hall->res) && (fddata->Fall->data[i] < (fddata->Fmax - 3*uF)))
	    i++;
	i_load_hold = i; /* first point very close to Fmax */

	while ((i < fddata->hall->res) && (fddata->hall->data[i] >= 0.0))
	    i++;
	i--;
	i_end_unload = i; /* last available point with nonnegative depth */

	while ((i > (i_load_hold + 1)) && (fddata->Fall->data[i] < (fddata->Fmax - 3*uF)))
	    i--;
	i++;
	i_hold_unload = i; /* last point very close to Fmax */

	fddata->i_load_hold = fddata->i_contact_load + i_load_hold;
	fddata->i_hold_unload = fddata->i_contact_load + i_hold_unload;
	fddata->i_end_unload = fddata->i_contact_load + i_end_unload;

	printf("prislusne indexy: \n");
	printf(" contact/load %d \n", fddata->i_contact_load);
	printf(" load/hold %d \n", fddata->i_load_hold);
	printf(" hold/unload %d \n", fddata->i_hold_unload);
	printf(" unload/end %d \n", fddata->i_end_unload);

	if (fddata->i_load_hold <= fddata->i_contact_load) {
		printf("Warning: Could not split data between load and hold part.\n Proceed with care.\n");
		isok = FALSE;
	}
	if (fddata->i_hold_unload < fddata->i_load_hold) {
		printf("Warning: Could not split data between hold and unload part.\n Proceed with care.\n");
		isok = FALSE;
	}
	if (fddata->i_end_unload <= fddata->i_hold_unload) {
		printf("Warning: Could not split data between unload and end part.\n Proceed with care.\n");
		isok = FALSE;
	}

	/* set_real: aby byl jednotkovy krok */

	fddata->hload = gwy_data_line_part_extract (fddata->hall, 0, i_load_hold + 1);
	fddata->Fload = gwy_data_line_part_extract (fddata->Fall, 0, i_load_hold + 1);
	gwy_data_line_set_real (fddata->hload, fddata->hload->res);
	gwy_data_line_set_real (fddata->Fload, fddata->Fload->res);
	
	i_hold_unload = MAX(i_hold_unload, i_load_hold);
	fddata->hhold = gwy_data_line_part_extract (fddata->hall, i_load_hold, i_hold_unload - i_load_hold + 1);
	fddata->Fhold = gwy_data_line_part_extract (fddata->Fall, i_load_hold, i_hold_unload - i_load_hold + 1);
	gwy_data_line_set_real (fddata->hhold, fddata->hhold->res);
	gwy_data_line_set_real (fddata->Fhold, fddata->Fhold->res);

	i_end_unload = MAX(i_end_unload, i_hold_unload);
	fddata->hunload = gwy_data_line_part_extract (fddata->hall, i_hold_unload, i_end_unload - i_hold_unload + 1);
	fddata->Funload = gwy_data_line_part_extract (fddata->Fall, i_hold_unload, i_end_unload - i_hold_unload + 1);
	gwy_data_line_set_real (fddata->hunload, fddata->hunload->res);
	gwy_data_line_set_real (fddata->Funload, fddata->Funload->res);


	/* maximum values only from unloading curve to match CSM behavior */
	fddata_get_hmax_Fmax(fddata->hunload, fddata->Funload, &(fddata->hmax), &(fddata->Fmax));

	return isok;

	/* zatim nemazat 
	printf("\n\n\n ------------------Loading data-----------------------------\n");
	for (i=0; i< fddata->hload->res; i++) {
		printf("%g %g \n", fddata->hload->data[i], fddata->Fload->data[i]);	
	}
	printf("\n\n\n ------------------Hold data-----------------------------\n");
	for (i=0; i< fddata->hhold->res; i++) {
		printf("%g %g \n", fddata->hhold->data[i], fddata->Fhold->data[i]);	
	}
	printf("\n\n\n ------------------Unloading data-----------------------------\n");
	for (i=0; i< fddata->hunload->res; i++)	{
		printf("%g %g \n", fddata->hunload->data[i], fddata->Funload->data[i]);	
	}
	printf(" \n \n \n -------------------------------------------\n");
	*/
}

gboolean split_hF_data_spec(FDdata *fddata, gint i_contact_load, gint i_load_hold, gint i_hold_unload, gint i_end_unload)
{
    gdouble hc, Fc;

    if ((0 <= i_contact_load) &&
	(i_contact_load <= i_load_hold) &&
	(i_load_hold <= i_hold_unload) &&
	(i_hold_unload <= i_end_unload) &&
	(i_end_unload < gwy_data_line_get_res (fddata->hall_orig))) {

	clear_hF_split_data(fddata);

	hc = fddata->hall_orig->data[i_contact_load];
	Fc = fddata->Fall_orig->data[i_contact_load];

	gwy_data_line_add (fddata->hall, -hc);
	gwy_data_line_add (fddata->Fall, -Fc);

	fddata->Fc = Fc;

	fddata->hload = gwy_data_line_part_extract (fddata->hall, i_contact_load, i_load_hold - i_contact_load + 1);
	fddata->Fload = gwy_data_line_part_extract (fddata->Fall, i_contact_load, i_load_hold - i_contact_load + 1);
	gwy_data_line_set_real (fddata->hload, fddata->hload->res);
	gwy_data_line_set_real (fddata->Fload, fddata->Fload->res);
	
	fddata->hhold = gwy_data_line_part_extract (fddata->hall, i_load_hold, i_hold_unload - i_load_hold + 1);
	fddata->Fhold = gwy_data_line_part_extract (fddata->Fall, i_load_hold, i_hold_unload - i_load_hold + 1);
	gwy_data_line_set_real (fddata->hhold, fddata->hhold->res);
	gwy_data_line_set_real (fddata->Fhold, fddata->Fhold->res);

	fddata->hunload = gwy_data_line_part_extract (fddata->hall, i_hold_unload, i_end_unload - i_hold_unload + 1);
	fddata->Funload = gwy_data_line_part_extract (fddata->Fall, i_hold_unload, i_end_unload - i_hold_unload + 1);
	gwy_data_line_set_real (fddata->hunload, fddata->hunload->res);
	gwy_data_line_set_real (fddata->Funload, fddata->Funload->res);

	gwy_data_line_resize (fddata->hall, i_contact_load, i_end_unload + 1);
	gwy_data_line_resize (fddata->Fall, i_contact_load, i_end_unload + 1);

	fddata->i_contact_load = i_contact_load;
	fddata->i_load_hold = i_load_hold;
	fddata->i_hold_unload = i_hold_unload;
	fddata->i_end_unload = i_end_unload;

	/* maximum values only from unloading curve to match CSM behavior */
	fddata_get_hmax_Fmax(fddata->hunload, fddata->Funload, &(fddata->hmax), &(fddata->Fmax));

	return TRUE;
    }
    else {
	return FALSE;
    }
}


void
clear_hF_all_orig_data (FDdata *fddata)
{
    if (fddata->hall_orig) {
	g_object_unref(fddata->hall_orig);
	fddata->hall_orig = NULL;
    }
    if (fddata->Fall_orig) {
	g_object_unref(fddata->Fall_orig);
	fddata->Fall_orig = NULL;
    }
}

void
clear_hF_all_data (FDdata *fddata)
{
    if (fddata->hall) {
	g_object_unref(fddata->hall);
	fddata->hall = NULL;
    }
    if (fddata->Fall) {
	g_object_unref(fddata->Fall);
	fddata->Fall = NULL;
    }
}

void
clear_hF_time_data (FDdata *fddata)
{
    if (fddata->time) {
	g_object_unref(fddata->time);
	fddata->time = NULL;
    }
    if (fddata->hall_time) {
	g_object_unref(fddata->hall_time);
	fddata->hall_time = NULL;
    }
    if (fddata->Fall_time) {
	g_object_unref(fddata->Fall_time);
	fddata->Fall_time = NULL;
    }
}

void
clear_hF_split_data(FDdata *fddata)
{
    if (fddata->hload) {
	g_object_unref(fddata->hload);
	fddata->hload = NULL;
    }
    if (fddata->Fload) {
	g_object_unref(fddata->Fload);
	fddata->Fload = NULL;
    }
    if (fddata->hhold) {
	g_object_unref(fddata->hhold);
	fddata->hhold = NULL;
    }
    if (fddata->Fhold) {
	g_object_unref(fddata->Fhold);
	fddata->Fhold = NULL;
    }
    if (fddata->hunload) {
	g_object_unref(fddata->hunload);
	fddata->hunload = NULL;
    }
    if (fddata->Funload) {
	g_object_unref(fddata->Funload);
	fddata->Funload = NULL;
    }
}

void
clear_hF_data(FDdata *fddata)
{
    clear_hF_all_orig_data(fddata);
    clear_hF_all_data(fddata);
    clear_hF_time_data(fddata);
    clear_hF_split_data(fddata);
}


gboolean
create_shifted_FDdata(FDdata *FDold, FDdata *FDnew, gint j)
{

	gint i;
	gdouble dh,dF;
	// if there are no points far enough to the left, we cannot create the shifted data
	if (FDold->i_contact_load+j <0)
		return FALSE;


	FDnew->hall = gwy_data_line_duplicate(FDold->hall_orig);
	FDnew->Fall = gwy_data_line_duplicate(FDold->Fall_orig);

	FDnew->i_contact_load = FDold->i_contact_load+j;
	FDnew->Fc = FDold->Fc;

	// split_hF_data(FDnew);
	dh = FDnew->hall->data[FDnew->i_contact_load];
	dF = FDnew->Fall->data[FDnew->i_contact_load];

	gwy_data_line_add(FDnew->hall, -dh);
	gwy_data_line_add(FDnew->Fall, -dF);

	/* refine_split_hF_data(FDnew); */
	split_hF_data(FDnew);

	printf("\n\n\n ------------------Loading data-----------------------------\n");
	for (i=0; i< FDnew->hload->res; i++)
	{
		printf("%g %g \n", FDnew->hload->data[i], FDnew->Fload->data[i]);	
	}
	printf("\n\n\n ------------------Hold data-----------------------------\n");
	for (i=0; i< FDnew->hhold->res; i++)
	{
		printf("%g %g \n", FDnew->hhold->data[i], FDnew->Fhold->data[i]);	
	}
	printf("\n\n\n ------------------Unloading data-----------------------------\n");
	for (i=0; i< FDnew->hunload->res; i++)
	{
		printf("%g %g \n", FDnew->hunload->data[i], FDnew->Funload->data[i]);	
	}
	printf(" \n \n \n -------------------------------------------\n");

	printf("prislusne indexy: \n");
	printf(" contact/load %d \n", FDnew->i_contact_load);
	printf(" load/hold %d \n", FDnew->i_load_hold);
	printf(" hold/unload %d \n", FDnew->i_hold_unload);
	printf(" unload/end %d \n", FDnew->i_end_unload);

	return TRUE;
}
