#include "file-utils.h"
#include "indent-toolbox-common.h"

/* this should be eliminated so that file-utils does not manipulate with data structures (clear_hF_data) */
#include "fddata.h"

/* this should be eliminated so that file-utils are GUI-unrelated */
#include "gui-utils.h"

#include <stdlib.h>

/* this could be removed if fprintf_fddata is made Gwy-independent */
#include <libprocess/gwyprocess.h>

#ifdef G_OS_WIN32
#include "getline.h"
#endif

static gchar* get_file_format_string(enum DataFileFormat fileformat, gint *nvars);
static void fprint_fddata(FILE *file, GwyDataLine *x, GwyDataLine *y);

gboolean // unifikovat, dotahnout
file_contains_data(gchar *filename, enum DataFileFormat fileformat, FileReadStatus *filestatus)
{
    FILE *infile;
    gchar *line = NULL;
    gchar *formatstring;
    size_t size;
    gint numdata, nvars, n;
    gdouble hlp1, hlp2, hlp3;

    infile = fopen(filename, "r");
    if (!infile) {
	*filestatus = READ_FILE_COULD_NOT_OPEN;
	return FALSE;
    }

    if (fileformat == DATA_FILE_FORMAT_NIGET) {
	getline(&line, &size, infile);
	if (sscanf(line, DATAFORMAT_HEADER_FORMAT, &n, &n, &n, &n) != 4) { /* do not care about the values */
	    *filestatus = READ_FILE_FORMAT_MISMATCH;
	    return FALSE;
	}
    }

    formatstring = get_file_format_string(fileformat, &nvars);

    numdata = 0;
    /* 3 points define an elementary loading-unloading curve */
    while ((getline(&line, &size, infile) != EOF) && (numdata < 3)) {
	n = sscanf(line, formatstring, &hlp1, &hlp2, &hlp3);
	if (n == nvars)
	    numdata++;
    }
    
    fclose(infile);
    g_free(formatstring);

    if (numdata == 3) {
	printf("file %s contains at least 3 valid data items, OK\n", filename);
	*filestatus = READ_FILE_OK;
	return TRUE;
    }
    else {
	printf("file %s contains only %d valid data items, not OK\n", filename, numdata);
	*filestatus = READ_FILE_NOT_ENOUGH_ENTRIES;
	return FALSE;
    }
}

FileReadStatus
read_file(FDdata *fddata)
{
    FILE *infile;
    gchar *line = NULL;
    size_t size;
    gint numdata, nvars;
    gint i, j, n;
    gdouble hlp1, hlp2, hlp3;
    gchar *formatstring;

    printf("opening file %s\n", fddata->filename);

    infile = fopen(fddata->filename, "r");
    if (!infile)
	return READ_FILE_COULD_NOT_OPEN;

    formatstring = get_file_format_string(fddata->fileformat, &nvars);

    numdata = 0;
    while (getline(&line, &size, infile) != EOF) {
	n = sscanf(line, formatstring, &hlp1, &hlp2, &hlp3);
	if (n == nvars)
	    numdata++;
    }
    rewind(infile);
    printf("numdata: %d\n",numdata);

    if (numdata < 3) {   /* this should not happen because we have checked the file using file_contains_data */
	g_free(formatstring);
	return READ_FILE_NOT_ENOUGH_ENTRIES;
    }
    else { /* read the data */

	/* move to some calling function to avoid dependency */
	clear_hF_data(fddata);

	fddata->time=gwy_data_line_new(numdata, numdata, FALSE);
	fddata->hall=gwy_data_line_new(numdata, 1.0, FALSE);
	fddata->Fall=gwy_data_line_new(numdata, 1.0, FALSE);

	if (fddata->fileformat == DATA_FILE_FORMAT_NIGET) {
	    getline(&line, &size, infile);
	    sscanf(line, DATAFORMAT_HEADER_FORMAT,
		   &(fddata->i_contact_load), &(fddata->i_load_hold), &(fddata->i_hold_unload), &(fddata->i_end_unload));
	}

	i = 0;
		
	while (getline(&line, &size, infile) != EOF) {
	    n = sscanf(line, formatstring, &hlp1, &hlp2, &hlp3);
	    if (n == nvars){
		switch (fddata->fileformat) {
		case (DATA_FILE_FORMAT_UNHT_D_NM_F_MN):
		    fddata->time->data[i] = i;
		    fddata->hall->data[i] = hlp2;
		    fddata->Fall->data[i] = hlp3;
		    break;

		case (DATA_FILE_FORMAT_UNHT_D_NM_F_UN):
		    fddata->time->data[i] = i;
		    fddata->hall->data[i] = hlp2;
		    fddata->Fall->data[i] = hlp3/1000;
		    break;

		case (DATA_FILE_FORMAT_HYSITRON):
		    fddata->time->data[i] = i;
		    fddata->hall->data[i] = hlp1;
		    fddata->Fall->data[i] = hlp2 / 1000;
		    break;

		case (DATA_FILE_FORMAT_NIGET):
		case (DATA_FILE_FORMAT_D_NM_F_MN):
		    fddata->time->data[i] = i;
		    fddata->hall->data[i] = hlp1;
		    fddata->Fall->data[i] = hlp2;
		    break;

		case (DATA_FILE_FORMAT_F_MN_D_NM):
		    fddata->time->data[i] = i;
		    fddata->hall->data[i] = hlp2;
		    fddata->Fall->data[i] = hlp1;
		    break;

		case (DATA_FILE_FORMAT_F_UN_D_NM):
		    fddata->time->data[i] = i;
		    fddata->hall->data[i] = hlp2;
		    fddata->Fall->data[i] = hlp1 / 1000;
		    break;
		}
		i++;
	    }
	}
	if (i != numdata){ /* should not get here */
	    g_object_unref(fddata->time);
	    g_object_unref(fddata->hall);
	    g_object_unref(fddata->Fall);
	    fddata->hall = NULL;
	    fddata->Fall = NULL;
	    return READ_FILE_FORMAT_MISMATCH;
	}

	fclose(infile);
	free(line);
	g_free(formatstring);

	if (fddata->fileformat != DATA_FILE_FORMAT_NIGET) {
	    fddata->i_contact_load = 0;
	    fddata->i_end_unload = numdata - 1;
	    fddata->status_contact_load = POINT_STATUS_SET_AUTO;
	    fddata->status_end_unload = POINT_STATUS_SET_AUTO;
	}
	else {
	    fddata->status_contact_load = POINT_STATUS_FROM_FILE;
	    fddata->status_load_hold = POINT_STATUS_FROM_FILE;
	    fddata->status_hold_unload = POINT_STATUS_FROM_FILE;
	    fddata->status_end_unload = POINT_STATUS_FROM_FILE;
	}

	/* check if data is not inf or nan */
	for (j = 0; j < fddata->hall->res; j++) {
	    if (isnan(fddata->hall->data[j]) || isnan(fddata->Fall->data[j])) {
		g_object_unref(fddata->hall);
		g_object_unref(fddata->Fall);
		fddata->hall = NULL;
		fddata->Fall = NULL;
		return READ_FILE_CONTAINS_NAN;
	    }
	    if (isinf(fddata->hall->data[j]) || isinf(fddata->Fall->data[j])) {
		g_object_unref(fddata->hall);
		g_object_unref(fddata->Fall);
		fddata->hall = NULL;
		fddata->Fall = NULL;
		return READ_FILE_CONTAINS_INF;
	    }
	}
	return READ_FILE_OK;
    }
}

void
data_write_file(gchar *filename, Args *args)
{
    FILE *file;
    FDdata *fddata;

    fddata = &(args->fddata);
	
    file = fopen(filename, "w");
    if (!file) {
	show_error("Could not open file.");
	return;
    }

    fprintf(file, "# %s \n", fddata->filename);
    if (fddata->hall && fddata->Fall) {
	fprintf(file, "#fddata: hall, Fall\n");
	fprint_fddata(file, fddata->hall, fddata->Fall);
    }

    if (fddata->hload && fddata->Fload) {
	fprintf(file, "#fddata: hload, Fload\n");
	fprint_fddata(file, fddata->hload, fddata->Fload);
    }

    if (fddata->hhold && fddata->Fhold) {
	fprintf(file, "#fddata: hhold, Fhold\n");
	fprint_fddata(file, fddata->hhold, fddata->Fhold);
    }

    if (fddata->hunload && fddata->Funload) {
	fprintf(file, "#fddata: hunload, Funload\n");
	fprint_fddata(file, fddata->hunload, fddata->Funload);
    }

    fclose(file); 
}

/* static functions */

static gchar*
get_file_format_string(enum DataFileFormat fileformat, gint *nvars)
{
    switch (fileformat) {
    case (DATA_FILE_FORMAT_UNHT_D_NM_F_MN):
    case (DATA_FILE_FORMAT_UNHT_D_NM_F_UN):
	*nvars = 3;
	return g_strdup("%lf %lf %lf");
	break;
    case (DATA_FILE_FORMAT_NIGET):
    case (DATA_FILE_FORMAT_HYSITRON):
    case (DATA_FILE_FORMAT_D_NM_F_MN):
    case (DATA_FILE_FORMAT_F_MN_D_NM):
    case (DATA_FILE_FORMAT_F_UN_D_NM):
	*nvars = 2;
	return g_strdup("%lf %lf");
	break;
    default:
	return NULL;
	break;
    }
}

static void
fprint_fddata(FILE *file, GwyDataLine *x, GwyDataLine *y)
{
    gint i;

    for (i = 0; i < x->res; i++)
	fprintf(file, "%f\t%f\n", x->data[i], y->data[i]);
    fprintf(file, "\n\n");
}
