#include "gui-utils.h"

#include <gtk/gtk.h>

/* adapted from Gwydgets */
GtkWidget*
label_new_left(const gchar *text)
{
    GtkWidget *label;

    label = gtk_label_new(text);
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);

    return label;
}

GtkWidget*
label_new_with_markup(const gchar *text)
{
    GtkWidget *label;

    label = gtk_label_new(NULL);
    gtk_label_set_markup(GTK_LABEL(label), text);

    return label;
}

GtkWidget*
label_new_with_markup_with_mnemonic(const gchar *text)
{
    GtkWidget *label;

    label = gtk_label_new(NULL);
    gtk_label_set_markup_with_mnemonic(GTK_LABEL(label), text);

    return label;
}

GtkWidget*
label_new_with_markup_left(const gchar *text)
{
    GtkWidget *label;

    label = gtk_label_new(NULL);
    gtk_label_set_markup(GTK_LABEL(label), text);
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);

    return label;
}

void
remove_table_content(GtkWidget *table)
{
    GList *children, *iter;

    children = gtk_container_get_children(GTK_CONTAINER(table));
    for (iter = children; iter != NULL; iter = g_list_next(iter)) {
	gtk_widget_destroy(GTK_WIDGET(iter->data));
	iter->data = NULL;
    }
    g_list_free(children);
}

void
entry_clear(GtkWidget *entry)
{
    if (gtk_entry_get_text(GTK_ENTRY(entry)))
	gtk_entry_set_text(GTK_ENTRY(entry), "");
}

void
label_clear(GtkWidget *label)
{
    if (gtk_label_get_text(GTK_LABEL(label)))
	gtk_label_set_text(GTK_LABEL(label), NULL);
}

/* TODO: message windows should have parents */
void show_error(const gchar *msg)
{
    GtkWidget *error;
    
    error = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, msg);
    gtk_dialog_run (GTK_DIALOG (error));
    gtk_widget_destroy (error);
}

/* TODO: message windows should have parents */
void show_warning(const gchar *msg)
{
    GtkWidget *warning;
    
    warning = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, msg);
    gtk_dialog_run (GTK_DIALOG (warning));
    gtk_widget_destroy (warning);
}
