#include "indent-toolbox-common.h"

#define _USE_MATH_DEFINES
#include <math.h>

#include <stdlib.h>
#include <string.h>

#include <gtk/gtk.h>

#include <libprocess/gwyprocess.h>
#include <libgwyddion/gwyddion.h>
#include <app/gwyapp.h>

/* eliminate all */
#include "tool_op.h"
#include "tool_op_unc.h"

#ifndef NOFORTRAN
#include "tool_op_odr.h"
#include "tool_op_odr_unc.h"
#include "tool_twoslopes.h"
#endif

#include "tool_tangent.h"
#include "tool_tangent_unc.h"
#include "tool_epwork.h"
#include "tool_epwork_unc.h"
#include "tool_hertz.h"
#include "tool_hertz_unc.h"
#include "tool_Ph2.h"
#include "tool_ppopins.h"
#include "tool_apopins.h"

#define DSWAP(x, y) GWY_SWAP(gdouble, x, y)

/* can be used from math.h */
/*
#define A 12

double
sp_gamma(double z)
{
    const int a = A;
    static double c_space[A];
    static double *c = NULL;
    int k;
    double accm;

    if (c == NULL ) {
        double k1_factrl = 1.0; // (k - 1)!*(-1)^k with 0!==1
        c = c_space;
        c[0] = sqrt(2.0*M_PI);
        for(k = 1; k < a; k++) {
            c[k] = exp(a-k) * pow(a-k, k-0.5) / k1_factrl;
            k1_factrl *= -k;
        }
    }
    accm = c[0];
    for(k = 1; k < a; k++)
        accm += c[k] / (z + k);
    accm *= exp(-(z+a)) * pow(z+a, z+0.5); // Gamma(z+1) 
    return accm/z;
}
*/

/* opsane z Octavi verze */
gdouble
psi(gdouble z)
{
	gdouble h;

	h = 1e-9;
	
	if (z > 0 )
		return  (lgamma(z + h) - lgamma(z -h ))/(2*h);
	else if (z < 0 )
		return  (lgamma(1-z + h) - lgamma(1-z -h ))/(2*h) + M_PI/tan(M_PI*(1-z));
	else
		return  -1e30;

}

/*
=======================================

UNSORTED

=======================================

*/

void
range_to_indices(gdouble from, gdouble to, GwyDataLine *x, gboolean descending, gint *istart, gint *iend, gint *ndata)
{
	gint i=0;

	*ndata = 0;

	if (descending){
		while (i < x->res && x->data[i] > to)
			i++;
		*istart = i;

		while(i< x->res && x->data[i] > from)
			i++;
		*iend = i;
	}
	else{
		while(i< x->res && x->data[i] < from){
			i++;
		}
		*istart = i;
		while (i < x->res && x->data[i] < to){
//			printf("i %d res %d x %g to %g\n", i, x->res, x->data[i], to);
			i++;
		}
		*iend = i;
	}
	*ndata = *iend - *istart;
}


GwyDataLine*
moving_average (GwyDataLine *x, gint nmove) //TODO prepsat hezceji, rychleji, obecnejsi vahy, doplnit okraje
{
	GwyDataLine *data_line;
	gint i,j, s,n;

	if (nmove == 1){
		data_line=gwy_data_line_duplicate(x);
	}
	else{
		s = (nmove -1 )/2; // nmove must be odd
		n = x->res; 
		data_line=gwy_data_line_new_alike(x,TRUE);
		for (i = s; i< n-s; i++){
			for (j = -s; j <= s ; j++){
				data_line->data[i] += x->data[i+j]/nmove;
			}
		}

		for ( i = 0; i < s; i++){
			/*			for (j = MAX(-s,-i); j <= s ; j++){
						data_line->data[i] += x->data[i+j]/(nmove-s+i);
						}
						*/
			data_line->data[i] = x->data[i];
		}
		for (i = n-s; i < n; i++){
			/*			for (j = -s; j <MIN( s,n-i) ; j++){
						data_line->data[i] += x->data[i+j]/(nmove-s+n-1-i);
						}
						*/
			data_line->data[i] = x->data[i];
		}

	}
	return data_line;
}

GwyDataLine*
moving_average_rs (GwyDataLine *x, gint nmove) //TODO prepsat hezceji, rychleji, obecnejsi vahy, doplnit okraje
{
    GwyDataLine *data_line;
    gint i, j, l, u, w, s, n;
    printf("\n");

    if (nmove == 1)
	data_line = gwy_data_line_duplicate(x);
    else {
	s = (nmove - 1) / 2; // nmove must be odd
	n = x->res;
	data_line = gwy_data_line_new_alike(x,TRUE);

	for (i = 0; i < n; i++) {
	    l = MAX(0,   i-s);
	    u = MIN(n-1, i+s);
	    w = u - l + 1;
	    for (j = l; j <= u; j++)
		data_line->data[i] += x->data[j];
	    data_line->data[i] /= w;
	    printf("%g   \t%g\n",x->data[i],data_line->data[i]);
	}
    }
    
    return data_line;
}

void
remove_all_tool_fit_results_labels(Data *data, gboolean areadependent)
{
    gwy_selection_clear(data->controls->opcontrols->selection);
    op_remove_all_fit_results_hp(&(data->args->opdata));
    op_remove_all_fit_labels_hp(data->controls->opcontrols);
    op_remove_all_fit_curves(data->controls->opcontrols);
    if (data->args->opunc.Ec != NULL || data->args->opunc.Hc != NULL)
	op_unc_close_window(data);

#ifndef NOFORTRAN
    gwy_selection_clear(data->controls->opodrcontrols->selection);
    op_odr_remove_all_fit_results(&(data->args->opodrdata));
    op_odr_remove_all_fit_labels(data->controls->opodrcontrols);
    op_odr_remove_all_fit_curves(data->controls->opodrcontrols);
    if (data->args->opodrunc.Ec != NULL || data->args->opodrunc.Hc != NULL)
	op_odr_unc_close_window(data);

    if (!areadependent) {
	gwy_selection_clear(data->controls->slopescontrols->selection);
	slopes_remove_all_fit_results_load(&(data->args->slopesdata));
	slopes_remove_all_fit_results_unload(&(data->args->slopesdata));
	slopes_remove_all_fit_labels_load(data->controls->slopescontrols);
	slopes_remove_all_fit_labels_unload(data->controls->slopescontrols);
	slopes_remove_all_fit_curves(data->controls->slopescontrols);
    }
#endif

    gwy_selection_clear(data->controls->tgcontrols->selection);
    tangent_remove_all_fit_results(&(data->args->tgdata));
    tangent_remove_all_fit_labels(data->controls->tgcontrols);
    tangent_remove_all_fit_curves(data->controls->tgcontrols);
    if (data->args->tgunc.Ec != NULL || data->args->tgunc.Hc != NULL)
	tangent_unc_close_window(data);

    if (!areadependent) {
	gwy_selection_clear(data->controls->hertzcontrols->selection);
	hertz_remove_all_fit_results(&(data->args->hertzdata));
	hertz_remove_all_fit_labels(data->controls->hertzcontrols);
	hertz_remove_all_fit_curves(data->controls->hertzcontrols);
	if (data->args->hertzunc.Ec != NULL)
	    hertz_unc_close_window(data);

	Ph2_remove_curves(data->controls->ph2controls);

	apopin_remove_results(&(data->args->apopdata));
	apopin_remove_labels(data->controls->apopcontrols);

	work_remove_results(&(data->args->workdata));
	work_remove_labels(data->controls->workcontrols);
	if (data->args->workunc.We != NULL || data->args->workunc.Wp != NULL)
	    work_unc_close_window(data);
    }
}

void
fddata_get_hmax_Fmax(GwyDataLine *h, GwyDataLine *F, gdouble *hmax, gdouble *Fmax)
{
	gint i;

	*Fmax = -G_MAXDOUBLE;
	*hmax = -G_MAXDOUBLE;
	for (i = 0; i< F->res; i++){
		if (*Fmax < F->data[i]){
			*Fmax = F->data[i];
			*hmax = h->data[i];
		}
	}
}

/* tyhle dve from,to mozna spojit do jedne */
void
range_from_changed(GtkWidget *wfrom, GtkWidget *wto, GwySelection *selection, gdouble *datafrom, gdouble *data2from)
{
    gdouble from, to;
    gdouble range[2];
    gchar buffer[24];
    const gchar *sfrom, *sto;

    sfrom = gtk_entry_get_text (GTK_ENTRY(wfrom));
    sto   = gtk_entry_get_text (GTK_ENTRY(wto));

    if (strlen(sfrom)) {
	from = strtod(sfrom, NULL);
	if (strlen(sto)) {
	    to = strtod(sto, NULL);

	    if (from > to)
		from = to;

	    range[0] = from;
	    range[1] = to;
	    gwy_selection_set_object(selection, 0, range);
	}
	*datafrom = from;
	*data2from = from;
	g_snprintf(buffer, sizeof(buffer), DEPTH, *datafrom);
	gtk_entry_set_text(GTK_ENTRY(wfrom), buffer);
    }
}

void
range_to_changed(GtkWidget *wfrom, GtkWidget *wto, GwySelection *selection, gdouble *datato, gdouble *data2to)
{
    gdouble from, to;
    gdouble range[2];
    gchar buffer[24];
    const gchar *sfrom, *sto;

    sfrom = gtk_entry_get_text(GTK_ENTRY(wfrom));
    sto   = gtk_entry_get_text(GTK_ENTRY(wto));

    if (strlen(sto)) {
	to  = strtod(sto, NULL);
	if (strlen(sfrom)) {
	    from = strtod(sfrom, NULL);

	    if (from > to)
		to = from;

	    range[0] = from;
	    range[1] = to;
	    gwy_selection_set_object(selection, 0, range);
	}
	*datato = to;
	*data2to = to;
	g_snprintf(buffer, sizeof(buffer), DEPTH, *datato);
	gtk_entry_set_text(GTK_ENTRY(wto), buffer);
    }
}

/* tyhle dve from,to pct Fmax mozna spojit do jedne, pripadne s predchozimi */
gboolean
range_from_pct_Fmax_changed(GtkWidget *from_pct_Fmax, GtkWidget *to_pct_Fmax, GwySelection *selection,
			    const gdouble Fmax, GwyDataLine *hdataline, GwyDataLine *Fdataline,
			    gdouble *datafrom, gdouble *data2from, gdouble *datato, gdouble *data2to,
			    gboolean *Finput, gboolean *Finput2)
{
    gdouble from_pct, to_pct;
    gdouble range[2];
    gint istart, iend, ndata;
    gdouble from_F, to_F;
    gchar buffer[24];
    const gchar *sfrom, *sto;

    sfrom = gtk_entry_get_text(GTK_ENTRY(from_pct_Fmax));
    sto   = gtk_entry_get_text(GTK_ENTRY(to_pct_Fmax));

    if (strlen(sfrom)) {	
	from_pct = strtod(sfrom, NULL);
	if (strlen(sto)) {
	    to_pct  = strtod(sto, NULL);
	    
	    if (from_pct > to_pct)
		from_pct = to_pct;

	    from_F = from_pct * 0.01 * Fmax;
	    to_F = to_pct * 0.01 * Fmax;

	    range_to_indices(from_F, to_F, Fdataline, TRUE, &istart, &iend, &ndata);

	    range[0] = hdataline->data[iend];
	    range[1] = hdataline->data[istart];

	    gwy_selection_set_object(selection, 0, range);

	    *datato = to_pct;
	    *data2to = to_pct;
	    g_snprintf(buffer, sizeof(buffer), DEPTH, *datato); 
	    gtk_entry_set_text(GTK_ENTRY(to_pct_Fmax), buffer); 
	    *Finput = TRUE;
	    *Finput2 = TRUE;
	}

	*datafrom = from_pct;
	*data2from = from_pct;
	g_snprintf(buffer, sizeof(buffer), DEPTH, *datafrom); 
	gtk_entry_set_text(GTK_ENTRY(from_pct_Fmax), buffer); 
    }
    return FALSE;
}

gboolean
range_to_pct_Fmax_changed(GtkWidget *from_pct_Fmax, GtkWidget *to_pct_Fmax, GwySelection *selection,
			  const gdouble Fmax, GwyDataLine *hdataline, GwyDataLine *Fdataline,
			  gdouble *datafrom, gdouble *data2from, gdouble *datato, gdouble *data2to,
			  gboolean *Finput, gboolean *Finput2)
{
    gdouble from_pct, to_pct;
    gdouble range[2];
    gint istart, iend, ndata;
    gdouble from_F, to_F;
    gchar buffer[24];
    const gchar *sfrom, *sto;

    sfrom = gtk_entry_get_text(GTK_ENTRY(from_pct_Fmax));
    sto   = gtk_entry_get_text(GTK_ENTRY(to_pct_Fmax));

    if (strlen(sto)) {
	to_pct = strtod(sto, NULL);
	if (strlen(sfrom)) {
	    from_pct = strtod(sfrom, NULL);
	    if (from_pct > to_pct)
		to_pct = from_pct;

	    from_F = from_pct * 0.01 * Fmax;
	    to_F = to_pct * 0.01 * Fmax;

	    range_to_indices(from_F, to_F, Fdataline, TRUE, &istart, &iend, &ndata);

	    range[0] = hdataline->data[iend];
	    range[1] = hdataline->data[istart];

	    gwy_selection_set_object(selection, 0, range);

	    *datafrom = from_pct;
	    *data2from = from_pct;
	    g_snprintf(buffer, sizeof(buffer), DEPTH, *datafrom); 
	    gtk_entry_set_text(GTK_ENTRY(from_pct_Fmax), buffer); 
	    *Finput = TRUE;
	    *Finput2 = TRUE;
	}
	*datato = to_pct;
	*data2to = to_pct;
	g_snprintf(buffer, sizeof(buffer), DEPTH, *datato); 
	gtk_entry_set_text(GTK_ENTRY(to_pct_Fmax), buffer); 
    }
    return FALSE;
}

void
range_to_indices_array(gdouble from, gdouble to, const gdouble *x, gint l, gboolean descending, gint *istart, gint *iend, gint *ndata)
{
    gint i;

    if (l > 0) {
	if (!descending) {
	    i = 0;
	    while (i < (l - 1) && x[i] < from)
		i++;
	    *istart = i;
	
	    i = l - 1;
	    while (i > *istart && x[i] > to)
		i--;
	    *iend = i;
	}
	else {
	    i = l - 1;
	    while (i > 0 && x[i] < from)
		i--;
	    *iend = i;

	    i = 0;
	    while (i < *iend && x[i] > to)
		i++;
	    *istart = i;
	}

	if ((*istart == *iend) && ((x[*istart] < from) || (x[*istart] > to))) {
	    *istart = -1;
	    *iend = -1;
	    *ndata = 0;
	}
	else    
	    *ndata = *iend - *istart + 1;
    }
    else { /* empty array */
	*istart = -1;
	*iend = -1;
	*ndata = 0;
    }
}

void
range_to_indices_dataline(gdouble from, gdouble to, GwyDataLine *x, gboolean descending, gint *istart, gint *iend, gint *ndata)
{
    range_to_indices_array(from, to, x->data, gwy_data_line_get_res(x), descending, istart, iend, ndata);
}

void
rs_zoom_in(GwyGraph *graph, GSList **stack, gboolean free_y_rescale)
{
    GwyGraphModel *newgmodel, *oldgmodel;
    GwyGraphCurveModel *newcmodel, *oldcmodel;
    GwySelection *selection;
    GtkAllocation widget_alloc;
    
    gint ci, i, istart, iend, ndata, maxndata, oldndata, pointsize;
    gdouble ymin, ymax, avg, rng;
    const gdouble *oldxdata, *oldydata;
    gdouble *newxdata, *newydata;
    gdouble range[2];
    gboolean descending;

    oldgmodel = gwy_graph_get_model (graph);
    newgmodel = gwy_graph_model_new_alike (oldgmodel);

    selection = gwy_graph_area_get_selection (GWY_GRAPH_AREA (gwy_graph_get_area (GWY_GRAPH (graph))), GWY_GRAPH_STATUS_PLAIN);
    /* here PLAIN stands for the current selection mode */

    if (gwy_selection_get_object (selection, 0, range)) {

	if (range[0] > range[1])
	    DSWAP(range[0], range[1]);
	maxndata = 0;

	for (ci = 0; ci < gwy_graph_model_get_n_curves (oldgmodel); ci++) {
	
	    oldcmodel = gwy_graph_model_get_curve (oldgmodel, ci);
	    newcmodel = gwy_graph_curve_model_new_alike (oldcmodel);
	
	    oldxdata = gwy_graph_curve_model_get_xdata (oldcmodel);
	    oldydata = gwy_graph_curve_model_get_ydata (oldcmodel);
	    oldndata = gwy_graph_curve_model_get_ndata (oldcmodel);
	    
	    descending = FALSE;
	    if (oldndata > 0)
		if (oldxdata[0] > oldxdata[oldndata - 1])
		    descending = TRUE;

	    range_to_indices_array (range[0], range[1], oldxdata, oldndata, descending, &istart, &iend, &ndata);

	    newxdata = (gdouble*)g_malloc(ndata * sizeof(gdouble));
	    newydata = (gdouble*)g_malloc(ndata * sizeof(gdouble));

	    for (i = 0; i < ndata; i++) {
		newxdata[i] = oldxdata[istart + i];
		newydata[i] = oldydata[istart + i];
	    }

	    if (free_y_rescale) {

		ymin = DBL_MAX;
		ymax = -DBL_MAX;
		
		for (i = 0; i < ndata; i++) {
		    if (newydata[i] < ymin)
			ymin = newydata[i];
		    if (newydata[i] > ymax)
			ymax = newydata[i];
		}
		
		avg = (ymin + ymax)/2;
		rng = ymax - ymin;
		for (i = 0; i < ndata; i++)
		    newydata[i] = (newydata[i] - avg) / rng;
	    }

	    gwy_graph_curve_model_set_data(newcmodel, newxdata, newydata, ndata);
	    gwy_graph_model_add_curve (newgmodel, newcmodel);

	    if (ndata > maxndata)
		maxndata = ndata;
	}

	/* set point size, roughly based on the number of points in new curves */
	if (maxndata > 0) {
	    gtk_widget_get_allocation (gwy_graph_get_area (GWY_GRAPH (graph)), &widget_alloc);
	    pointsize = widget_alloc.width / (3*maxndata);
	    pointsize = CLAMP(pointsize, 0, 8);
	    if (pointsize >= 3) {
		for (ci = 0; ci < gwy_graph_model_get_n_curves (newgmodel); ci++) {
		    newcmodel = gwy_graph_model_get_curve (newgmodel, ci);
		    g_object_set(newcmodel, "point-size", pointsize, NULL);
		}
	    }
	}

	gwy_graph_set_model (graph, newgmodel);
	*stack = g_slist_prepend (*stack, newgmodel);
	gwy_selection_clear (selection);
    }
}

void
rs_zoom_out(GwyGraph *graph, GSList **stack)
{
    if (g_slist_length (*stack) >= 2) {
	gwy_graph_set_model (graph, (GwyGraphModel*)g_slist_nth_data (*stack, 1));
	g_object_unref (g_slist_nth_data (*stack, 0));
	*stack = g_slist_remove (*stack, g_slist_nth_data (*stack, 0));
    }
}

void
rs_zoom_restore(GwyGraph *graph, GSList **stack)
{
    guint i, n;

    n = g_slist_length (*stack);
    if (n >= 2) {
	for (i = 0; i < n-1; i++) {
	    g_object_unref (g_slist_nth_data (*stack, 0));
	    *stack = g_slist_remove (*stack, g_slist_nth_data (*stack, 0));
	}
	gwy_graph_set_model (graph, g_slist_nth_data (*stack, 0));
    }
}
