#ifndef INDENT_TOOLBOX_COMMON_H
#define INDENT_TOOLBOX_COMMON_H

#include "datatypes.h"
#include "controls.h"

#include <libgwydgets/gwydgets.h>

#define NDER 3
#define NLOAD 2

#define N_MC 10000 // number of Monte Carlo iterations

#define DATAFORMAT_HEADER_FORMAT "#NIGET %d %d %d %d"
#define DATAFORMAT_HEADER_FORMAT_NEWLINE "#NIGET %d %d %d %d\n"

#ifndef NOFORTRAN
enum {
    CONTACT=0,
    OP,
    OPODR,
    TANGENT,
    HERTZ,
    SLOPES,
    DIFF,
    APOP,
    WORK,
    CREEP,
    RELAX,
    PPOP,
};
#else
enum {
    CONTACT=0,
    OP,
    TANGENT,
    HERTZ,
    DIFF,
    APOP,
    WORK,
    CREEP,
    RELAX,
    PPOP,
};
#endif


typedef struct {
    Args *args;
    Controls *controls;
    struct Functions *functions;
} Data;

#define CURVE_COLOR_END_UNLOAD gwy_graph_get_preset_color(0)
#define CURVE_COLOR_LOAD gwy_graph_get_preset_color(1)
#define CURVE_COLOR_HOLD gwy_graph_get_preset_color(2)
#define CURVE_COLOR_UNLOAD gwy_graph_get_preset_color(3)
#define CURVE_COLOR_FIT gwy_graph_get_preset_color(4)
#define CURVE_COLOR_FIT2 gwy_graph_get_preset_color(5)

/* tool_apopins */
#define CURVE_COLOR_DER gwy_graph_get_preset_color(1)
#define CURVE_COLOR_SMOOTH_LOAD gwy_graph_get_preset_color(2)
#define CURVE_COLOR_DER gwy_graph_get_preset_color(1)
#define CURVE_COLOR_TH gwy_graph_get_preset_color(2)
#define CURVE_COLOR_TH2 gwy_graph_get_preset_color(3)
/* tool_Ph2 */
#define CURVE_COLOR_PH2 gwy_graph_get_preset_color(1)
#define CURVE_COLOR_DIFF_PH2 gwy_graph_get_preset_color(2)

#define DEPTH "%.3f"
#define FORCE "%.3f"
#define HARD "%.2f"
#define MODUL "%.4f"
#define NUMBER "%.3f"
#define AREA  "%.0f"
#define SLOPE "%.5g"
#define EPWORK  "%.2f"

#define CTRLS_BORDER 3
#define CTRLS_SPACING 5
#define CTRLS_TABLE_COL_SPACING 10
#define CTRLS_TABLE_ROW_SPACING 3


//double sp_gamma(double z);
gdouble psi(double z);

void range_to_indices(gdouble from, gdouble to, GwyDataLine *x, gboolean descending, gint *istart, gint *iend, gint *ndata);
/* void graph_selected(GwyGraph* graph, Data *data); */

void remove_all_tool_fit_results_labels(Data *data, gboolean areadependent);

GwyDataLine* moving_average (GwyDataLine *x, gint n);

void fddata_get_hmax_Fmax(GwyDataLine *h, GwyDataLine *F, gdouble *hmax, gdouble *Fmax);

void range_to_indices_array(gdouble from, gdouble to, const gdouble *x, gint l, gboolean descending, gint *istart, gint *iend, gint *ndata);
void range_to_indices_dataline(gdouble from, gdouble to, GwyDataLine *x, gboolean descending, gint *istart, gint *iend, gint *ndata);

void rs_zoom_in(GwyGraph *graph, GSList **stack, gboolean free_y_rescale);
void rs_zoom_out(GwyGraph *graph, GSList **stack);
void rs_zoom_restore(GwyGraph *graph, GSList **stack);

void range_from_changed(GtkWidget *wfrom, GtkWidget *wto, GwySelection *selection, gdouble *datato, gdouble *data2to);
void range_to_changed(GtkWidget *wfrom, GtkWidget *wto, GwySelection *selection, gdouble *datato, gdouble *data2to);
gboolean range_from_pct_Fmax_changed(GtkWidget *from_pct_Fmax, GtkWidget *to_pct_Fmax, GwySelection *selection,
				     const gdouble Fmax, GwyDataLine *hdataline, GwyDataLine *Fdataline,
				     gdouble *datafrom, gdouble *data2from, gdouble *datato, gdouble *data2to,
				     gboolean *Finput, gboolean *Finput2);
gboolean range_to_pct_Fmax_changed(GtkWidget *from_pct_Fmax, GtkWidget *to_pct_Fmax, GwySelection *selection,
				   const gdouble Fmax, GwyDataLine *hdataline, GwyDataLine *Fdataline,
				   gdouble *datafrom, gdouble *data2from, gdouble *datato, gdouble *data2to,
				   gboolean *Finput, gboolean *Finput2);

#endif
