#include <string.h>
#include <math.h>
#include <libgwyddion/gwyddion.h>

#include "indent-toolbox-common.h"
#include "settings.h"

#ifdef G_OS_WIN32
#include "getline.h"
#endif

#ifndef NOFORTRAN
enum {
	FC = 0,
	OP_UNC_H,
	OP_UNC_F,
	OP_UNC_NU,
	OP_UNC_NUI,
	OP_UNC_EI,
	OP_NMC,
	OPODR_UNC_H,
	OPODR_UNC_F,
	OPODR_UNC_NU,
	OPODR_UNC_NUI,
	OPODR_UNC_EI,
	OPODR_NMC,
	TG_UNC_H,
	TG_UNC_F,
	TG_UNC_NU,
	TG_UNC_NUI,
	TG_UNC_EI,
	TG_NMC,
	HZ_UNC_H,
	HZ_UNC_F,
	HZ_UNC_NU,
	HZ_UNC_NUI,
	HZ_UNC_EI,
	HZ_UNC_RADIUS,
	HZ_NMC,
	APOP_THRESH,
	APOP_THRESH2,
	APOP_WPOP,
	APOP_HPOP,
	APOP_NMOVE,
	PH2_NMOVE,
	WORK_NMOVE,
	AREA_POLY, // MUST be the last item
};
#else 
enum {
	FC = 0,
	OP_UNC_H,
	OP_UNC_F,
	OP_UNC_NU,
	OP_UNC_NUI,
	OP_UNC_EI,
	OP_NMC,
	TG_UNC_H,
	TG_UNC_F,
	TG_UNC_NU,
	TG_UNC_NUI,
	TG_UNC_EI,
	TG_NMC,
	HZ_UNC_H,
	HZ_UNC_F,
	HZ_UNC_NU,
	HZ_UNC_NUI,
	HZ_UNC_EI,
	HZ_UNC_RADIUS,
	HZ_NMC,
	APOP_THRESH,
	APOP_THRESH2,
	APOP_WPOP,
	APOP_HPOP,
	APOP_NMOVE,
	PH2_NMOVE,
	WORK_NMOVE,
	AREA_POLY,// MUST be the last item
};
#endif

void 
save_settings(const gchar *filename, Args *args)
{

	gint i;
	FILE *file;

	file = fopen(filename, "w");

	if (!file)
		return;

	fprintf(file, "Settings %s\n", g_get_application_name());

	fprintf(file,"%d  %g\n", FC, args->fddata.Fc);

	/* fprintf(file,"%d  %g\n", OP_NU, args->opdata.nu); */
	/* fprintf(file,"%d  %g\n", OP_NUI, args->opdata.nui); */
	/* fprintf(file,"%d  %g\n", OP_EI, args->opdata.Ei); */

	fprintf(file,"%d  %g\n", OP_UNC_H, args->opunc.uh);
	fprintf(file,"%d  %g\n", OP_UNC_F, args->opunc.uF);
	fprintf(file,"%d  %g\n", OP_UNC_NU, args->opunc.unu);
	fprintf(file,"%d  %g\n", OP_UNC_NUI, args->opunc.unui);
	fprintf(file,"%d  %g\n", OP_UNC_EI, args->opunc.uEi);
	fprintf(file,"%d  %d\n", OP_NMC, args->opunc.Nmc);
#ifndef NOFORTRAN
	/* fprintf(file,"%d  %g\n", OPODR_NU, args->opodrdata.nu); */
	/* fprintf(file,"%d  %g\n", OPODR_NUI, args->opodrdata.nui); */
	/* fprintf(file,"%d  %g\n", OPODR_EI, args->opodrdata.Ei); */
	fprintf(file,"%d  %g\n", OPODR_UNC_H, args->opodrunc.uh);
	fprintf(file,"%d  %g\n", OPODR_UNC_F, args->opodrunc.uF);
	fprintf(file,"%d  %g\n", OPODR_UNC_NU, args->opodrunc.unu);
	fprintf(file,"%d  %g\n", OPODR_UNC_NUI, args->opodrunc.unui);
	fprintf(file,"%d  %g\n", OPODR_UNC_EI, args->opodrunc.uEi);
	fprintf(file,"%d  %d\n", OPODR_NMC, args->opodrunc.Nmc);
	/* fprintf(file,"%d  %g\n", SL_NU, args->slopesdata.nu); */
	/* fprintf(file,"%d  %g\n", SL_NUI, args->slopesdata.nui); */
	/* fprintf(file,"%d  %g\n", SL_EI, args->slopesdata.Ei); */
#endif
	/* fprintf(file,"%d  %g\n", TG_NU, args->tgdata.nu); */
	/* fprintf(file,"%d  %g\n", TG_NUI, args->tgdata.nui); */
	/* fprintf(file,"%d  %g\n", TG_EI, args->tgdata.Ei); */
	fprintf(file,"%d  %g\n", TG_UNC_H, args->tgunc.uh);
	fprintf(file,"%d  %g\n", TG_UNC_F, args->tgunc.uF);
	fprintf(file,"%d  %g\n", TG_UNC_NU, args->tgunc.unu);
	fprintf(file,"%d  %g\n", TG_UNC_NUI, args->tgunc.unui);
	fprintf(file,"%d  %g\n", TG_UNC_EI, args->tgunc.uEi);
	fprintf(file,"%d  %d\n", TG_NMC, args->tgunc.Nmc);

	/* fprintf(file,"%d  %g\n", HZ_NU, args->hertzdata.nu); */
	/* fprintf(file,"%d  %g\n", HZ_NUI, args->hertzdata.nui); */
	/* fprintf(file,"%d  %g\n", HZ_EI, args->hertzdata.Ei); */
	fprintf(file,"%d  %g\n", HZ_UNC_H, args->hertzunc.uh);
	fprintf(file,"%d  %g\n", HZ_UNC_F, args->hertzunc.uF);
	fprintf(file,"%d  %g\n", HZ_UNC_NU, args->hertzunc.unu);
	fprintf(file,"%d  %g\n", HZ_UNC_NUI, args->hertzunc.unui);
	fprintf(file,"%d  %g\n", HZ_UNC_EI, args->hertzunc.uEi);
	fprintf(file,"%d  %g\n", HZ_UNC_RADIUS, args->hertzunc.uradius);
	fprintf(file,"%d  %d\n", HZ_NMC, args->hertzunc.Nmc);

	fprintf(file,"%d  %d\n", PH2_NMOVE, args->ph2data.nmove);

	fprintf(file,"%d  %g\n", APOP_THRESH, args->apopdata.thresh);
	fprintf(file,"%d  %g\n", APOP_THRESH2, args->apopdata.thresh2);
	fprintf(file,"%d  %d\n", APOP_WPOP, args->apopdata.wpop);
	fprintf(file,"%d  %g\n", APOP_HPOP, args->apopdata.hpop);
	fprintf(file,"%d  %d\n", APOP_NMOVE, args->apopdata.nmove);

	fprintf(file,"%d  %d\n", WORK_NMOVE, args->workdata.nmove);

	fprintf(file, "%d %d \n", AREA_POLY, args->area.npoly);
	for (i= 0 ;i< args->area.npoly; i++)
		fprintf(file, "%d %g \n", AREA_POLY+1+i, args->area.polycoef[i]);
	fclose(file); 
}

void 
load_settings(const gchar *filename, Args *args)
{
	FILE *file;
	gchar *line = NULL;
	size_t size;
	gint key;
	gdouble value;
	gint n, npoly;
	gboolean polyok=TRUE;
 

	file = fopen(filename, "r");
	if (!file)
		return;

	getline(&line, &size, file);
	if (strstr(line, g_get_application_name()) == NULL){
		return;
	}

	while(getline (&line, &size, file) != EOF){
		n = sscanf(line, " %d %lf", &key, &value);
		if (n == 2){
			switch(key){
				case(FC):
					args->fddata.Fc = MAX(value,0);
					break;
				/* case(OP_NU): */
				/* 	args-> opdata.nu = CLAMP(value, -1.0, 0.5); */
				/* 	break; */
				/* case(OP_NUI): */
				/* 	args-> opdata.nui = CLAMP(value, -1.0, 0.5); */
				/* 	break; */
				/* case(OP_EI): */
				/* 	args-> opdata.Ei = MAX(value, 0); */
				/* 	break; */
				case(OP_UNC_NU):
					args-> opunc.unu = MAX(value, 0);
					break;
				case(OP_UNC_NUI):
					args-> opunc.unui = MAX(value, 0);
					break;
				case(OP_UNC_EI):
					args-> opunc.uEi = MAX(value, 0);
					break;
				case(OP_UNC_H):
					args-> opunc.uh = MAX(value, 0);
					break;
				case(OP_UNC_F):
					args-> opunc.uF = MAX(value,0);
					break;
				case(OP_NMC):
					args->opunc.Nmc = MAX((gint)round(value), 1);
					break;
#ifndef NOFORTRAN
				/* case(OPODR_NU): */
				/* 	args->opodrdata.nu = CLAMP(value, -1.0, 0.5); */
				/* 	break; */
				/* case(OPODR_NUI): */
				/* 	args->opodrdata.nui = CLAMP(value, -1.0, 0.5); */
				/* 	break; */
				/* case(OPODR_EI): */
				/* 	args-> opodrdata.Ei = MAX(value, 0); */
				/* 	break; */
				case(OPODR_UNC_NU):
					args->opodrunc.unu = MAX(value, 0);
					break;
				case(OPODR_UNC_NUI):
					args->opodrunc.unui = MAX(value, 0);
					break;
				case(OPODR_UNC_EI):
					args->opodrunc.uEi = MAX(value, 0);
					break;
				case(OPODR_UNC_H):
					args-> opodrunc.uh = MAX(value, 0);
					break;
				case(OPODR_UNC_F):
					args-> opodrunc.uF = MAX(value, 0);
					break;
				case(OPODR_NMC):
					args->opodrunc.Nmc = MAX((gint)round(value), 1);
					break;
				/* case(SL_NU): */
				/* 	args->slopesdata.nu = CLAMP(value, -1.0, 0.5); */
				/* 	break; */
				/* case(SL_NUI): */
				/* 	args->slopesdata.nui = CLAMP(value, -1.0, 0.5); */
				/* 	break; */
				/* case(SL_EI): */
				/* 	args-> slopesdata.Ei = MAX(value, 0); */
				/* 	break; */

#endif
				/* case(TG_NU): */
				/* 	args->tgdata.nu = CLAMP(value, -1.0, 0.5); */
				/* 	break; */
				/* case(TG_NUI): */
				/* 	args->tgdata.nui = CLAMP(value, -1.0, 0.5); */
				/* 	break; */
				/* case(TG_EI): */
				/* 	args-> tgdata.Ei = MAX(value, 0); */
				/* 	break; */
				case(TG_UNC_NU):
					args-> tgunc.unu = MAX(value, 0);
					break;
				case(TG_UNC_NUI):
					args-> tgunc.unui = MAX(value, 0);
					break;
				case(TG_UNC_EI):
					args-> tgunc.uEi = MAX(value, 0);
					break;
				case(TG_NMC):
					args->tgunc.Nmc = MAX((gint)round(value), 1);
					break;
				case(TG_UNC_H):
					args-> tgunc.uh = MAX(value, 0);
					break;
				case(TG_UNC_F):
					args-> tgunc.uF = MAX(value, 0);
					break;
				/* case(HZ_NU): */
				/* 	args->hertzdata.nu = CLAMP(value, -1.0, 0.5); */
				/* 	break; */
				/* case(HZ_NUI): */
				/* 	args->hertzdata.nui = CLAMP(value, -1.0, 0.5); */
				/* 	break; */
				/* case(HZ_EI): */
				/* 	args-> hertzdata.Ei = MAX(value, 0); */
				/* 	break; */
				case(HZ_UNC_H):
					args-> hertzunc.uh = MAX(value, 0);
					break;
				case(HZ_UNC_F):
					args-> hertzunc.uF = MAX(value, 0);
					break;
				case(HZ_UNC_NU):
					args-> hertzunc.unu = MAX(value, 0);
					break;
				case(HZ_UNC_NUI):
					args-> hertzunc.unui = MAX(value, 0);
					break;
				case(HZ_UNC_EI):
					args-> hertzunc.uEi = MAX(value, 0);
					break;
				case(HZ_UNC_RADIUS):
					args-> hertzunc.uradius = MAX(value, 0);
					break;
				case(HZ_NMC):
					args-> hertzunc.Nmc = MAX((gint)round(value), 1);
					break;
				case(APOP_THRESH):
					args->apopdata.thresh = value;
					break;
				case(APOP_THRESH2):
					args->apopdata.thresh2 = value;
					break;
				case(APOP_WPOP):
					args->apopdata.wpop = MAX((gint)round(value), 0);
					break;
				case(APOP_HPOP):
					args->apopdata.hpop = value;
					break;
				case(APOP_NMOVE):
					args->apopdata.nmove = MAX((gint)round(value),1);
					break;
				case(PH2_NMOVE):
					args->ph2data.nmove = MAX((gint)round(value),1);
					break;
				case(WORK_NMOVE):
					args->workdata.nmove = MAX((gint)round(value),1);
					break;
				case(AREA_POLY):
					npoly = (gint)round(value);
					polyok = (args->area.npoly >= npoly);
					break;
				default:
					if ( polyok  && (key - AREA_POLY <= args->area.npoly)){
						args->area.polycoef[key-AREA_POLY-1] = value;
						break;
					}
					else
						printf("Bad settings %d \n", key);
					break;
			}
		}
	}
}
