#include <math.h>

#include <gtk/gtk.h>

#include <app/file.h>
#include <libprocess/gwyprocess.h>

#include "indent-toolbox-common.h"
#include "datatypes.h"
#include "controls.h"
#include "gui-utils.h"
#include "tool_apopins.h"


#define NDER 3
#define NLOAD 2
#define DEBUG FALSE

#define THRESH_DEFAULT 2
#define THRESH2_DEFAULT 0.07
#define WPOP_DEFAULT 5
#define HPOP_DEFAULT 2.0


static void update_models_apop_h(ApopinControls *apopcontrols);
static void apop_zoom_in_h(Data *data);
static void apop_zoom_out_h(Data *data);

static void update_models_apop_der(ApopinControls *apopcontrols);
static void apop_zoom_in_der(Data *data);
static void apop_zoom_out_der(Data *data);


void
init_Apopindata(Apopindata *apopdata)
{
	apopdata->nmove=1;
	apopdata->npopin=0;
	apopdata->thresh=THRESH_DEFAULT;
	apopdata->thresh2=THRESH2_DEFAULT;
	apopdata->wpop=WPOP_DEFAULT;
	apopdata->hpop=HPOP_DEFAULT;
	apopdata->Fpopin=NULL;
	apopdata->hpopin=NULL;
	apopdata->dhpopin=NULL;
	apopdata->ileft=NULL;
	apopdata->iright=NULL;
	apopdata->havg=NULL;
	apopdata->dh=NULL;
	apopdata->t=NULL;
	apopdata->dth=NULL;
	apopdata->dth2=NULL;
}

static void
hpop_changed(Data *data)
{
	data->args->apopdata.hpop = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->apopcontrols->hpop));
	apopin_run(data);
}
	static void
wpop_changed(Data *data)
{
	data->args->apopdata.wpop = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->apopcontrols->wpop));
	apopin_run(data);
}
	static void
thresh_changed(Data *data)
{
	data->args->apopdata.thresh = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->apopcontrols->thresh));
	gwy_data_line_resample(data->args->apopdata.dth,data->args->fddata.hload->res,GWY_INTERPOLATION_NONE);
	gwy_data_line_clear(data->args->apopdata.dth);
	gwy_data_line_add(data->args->apopdata.dth, data->args->apopdata.thresh);

	apopin_run(data);
}

	static void
thresh2_changed(Data *data)
{
	data->args->apopdata.thresh2 = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->apopcontrols->thresh2));
	gwy_data_line_resample(data->args->apopdata.dth2,data->args->fddata.hload->res,GWY_INTERPOLATION_NONE);
	gwy_data_line_clear(data->args->apopdata.dth2);
	gwy_data_line_add(data->args->apopdata.dth2, data->args->apopdata.thresh2);
	apopin_run(data);
}
	static void
nmove_changed(Data *data)
{
	data->args->apopdata.nmove = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->apopcontrols->nmove));
	apopin_run(data);
}

void
apopin_remove_results(Apopindata *apopdata)
{
	// remove all results
	if (apopdata->npopin > 0){
		g_free(apopdata->Fpopin);
		g_free(apopdata->hpopin);
		g_free(apopdata->dhpopin);
		g_free(apopdata->ileft);
		g_free(apopdata->iright);
		apopdata->Fpopin = NULL;
		apopdata->hpopin = NULL;
		apopdata->dhpopin = NULL;
		apopdata->ileft = NULL;
		apopdata->iright = NULL;
	}
	apopdata->npopin = 0;
}

void
apopin_remove_labels(ApopinControls *apopcontrols)
{
    /* remove table of popins */
    remove_table_content(apopcontrols->res);

    /* remove number of popins */
    label_clear(apopcontrols->npopin);
}

static void reset_default(Data *data)
{
	data->args->apopdata.nmove=1;
	data->args->apopdata.thresh=THRESH_DEFAULT;
	data->args->apopdata.thresh2=THRESH2_DEFAULT;
	data->args->apopdata.wpop=WPOP_DEFAULT;
	data->args->apopdata.hpop=HPOP_DEFAULT;

	gtk_adjustment_set_value(GTK_ADJUSTMENT(data->controls->apopcontrols->nmove), data->args->apopdata.nmove);
	gtk_adjustment_set_value(GTK_ADJUSTMENT(data->controls->apopcontrols->thresh), data->args->apopdata.thresh);
	gtk_adjustment_set_value(GTK_ADJUSTMENT(data->controls->apopcontrols->thresh2), data->args->apopdata.thresh2);
	gtk_adjustment_set_value(GTK_ADJUSTMENT(data->controls->apopcontrols->wpop), data->args->apopdata.wpop);
	gtk_adjustment_set_value(GTK_ADJUSTMENT(data->controls->apopcontrols->hpop), data->args->apopdata.hpop);

	apopin_remove_results(&(data->args->apopdata));
	apopin_remove_labels(data->controls->apopcontrols);
	apopin_active(data);
}

void apopin_active(Data *data)
{
	gint nc,i;
	gchar str[30];
	Apopindata *apopdata;
	ApopinControls *apopctrls;


	apopdata = &(data->args->apopdata);
	apopctrls = data->controls->apopcontrols;

	// check if pseudo-time data have been already created, if not create them
	//check and possibly create constant thresh, thresh2 datalines
	if (data->args->fddata.hload) {
		if (apopdata->t==NULL) {
			apopdata->t = gwy_data_line_new_alike(data->args->fddata.hload,TRUE);
			for (i = 0; i < apopdata->t->res; i++) {
				apopdata->t->data[i] = i;
			}
		}
		if (apopdata->dth == NULL) {
			apopdata->dth = gwy_data_line_new_alike(data->args->fddata.hload,TRUE);
			gwy_data_line_add(apopdata->dth, apopdata->thresh);
		}
		else {
			 gwy_data_line_resample(apopdata->dth,data->args->fddata.hload->res,GWY_INTERPOLATION_NONE);
			 gwy_data_line_clear(apopdata->dth);
			 gwy_data_line_add(apopdata->dth, apopdata->thresh);
		}
		if (apopdata->dth2 == NULL) {
			apopdata->dth2 = gwy_data_line_new_alike(data->args->fddata.hload,TRUE);
			gwy_data_line_add(apopdata->dth2, apopdata->thresh2);
		}
		else {
			 gwy_data_line_resample(apopdata->dth2,data->args->fddata.hload->res,GWY_INTERPOLATION_NONE);
			 gwy_data_line_clear(apopdata->dth2);
			 gwy_data_line_add(apopdata->dth2, apopdata->thresh2);
		}
	
		gwy_graph_curve_model_set_data(apopctrls->cmodelload,
					       apopdata->t->data, data->args->fddata.hload->data,
					       data->args->fddata.hload->res);
	}

	nc = gwy_graph_model_get_n_curves(apopctrls->graph_model);
	if (nc > 1) {
		gwy_graph_curve_model_set_data(apopctrls->cmodelload,
				apopdata->t->data, apopdata->havg->data,
				apopdata->havg->res);
	}
	// check if popin data should be displayed, if not remove them
	if ( nc != apopdata->npopin+NLOAD){
		for (i = 0; i < nc-NLOAD; i++){
			g_snprintf(str, sizeof(str), "Pop-in #%d", i+1);
			gwy_graph_model_remove_curve_by_description(apopctrls->graph_model, str);
		}
	}

	nc = gwy_graph_model_get_n_curves(apopctrls->graph_model_der);
	if (nc == NDER) {
		gwy_graph_curve_model_set_data(apopctrls->cmodelder,
				apopdata->t->data, apopdata->dh->data,
				apopdata->dh->res);
		gwy_graph_curve_model_set_data(apopctrls->cmodelthresh,
				apopdata->t->data, apopdata->dth->data,
				apopdata->dth->res);
		gwy_graph_curve_model_set_data(apopctrls->cmodelthresh2,
				apopdata->t->data, apopdata->dth2->data,
				apopdata->dth2->res);
	}
	// check if popin data should be displayed, if not remove them
	if ( nc != apopdata->npopin+NDER){
		for (i = 0; i < nc-NDER; i++){
			g_snprintf(str, sizeof(str), "Pop-in #%d", i+1);
			gwy_graph_model_remove_curve_by_description(apopctrls->graph_model_der, str);
		}
	}
}

void 
apopin_run(Data *data)
{
	GwyDataLine *x, *xavg;
	GwyDataLine *dx;
	gint n,nind ;
	gint *indices, **groups;
	gint i,j;
	gdouble xm,thresh,upvalue;

	gint prev;
	gint ngr;
	gint *ngrind;

	gint wpop;
	gdouble hpop;
	gdouble thresh2;
	gint ileft;
	gint iright;

	gchar str[30];
	gint npopin;
	GtkWidget *label;
	gint nc;
	GwyDataLine *hpopin, *tpopin, *dhpopin;

	Apopindata *apopdata;
	ApopinControls *apopctrls;
	GwyGraphCurveModel **cmodelpopin;
	GwyGraphCurveModel **cmodeldpopin;


	//local copies for simpler reading
	apopdata = &(data->args->apopdata);
	apopctrls = data->controls->apopcontrols;
	x = data->args->fddata.hload;
	n = x->res;
	thresh = apopdata->thresh;
	thresh2 = apopdata->thresh2;
	wpop = apopdata->wpop;
	hpop = apopdata->hpop;

	// calculate sliding average of depth data
	xavg = moving_average(x, apopdata->nmove);


	//calculate derivative of smoothed depth
	dx = gwy_data_line_new_alike(x,FALSE);
	for ( i = 0; i< n; i++){
		dx->data[i] = gwy_data_line_get_der(xavg,i);
	}

	xm = gwy_data_line_get_avg(dx);
	//upvalue = xm + thresh; // works but would require plotting upvalue instead of thresh in apopdata->dth
	upvalue = thresh;

	indices = (gint *)g_malloc(n * sizeof(gint));
	nind = 0;

	// find indices where the derivative is larger than a certain value (= average derivative + threshhold) or (=threshhold)
	for ( i = 0; i< n; i++){
		if (dx->data[i] > upvalue){
			indices[nind++] = i;
		}
	}

	if (nind ==0){
		ngr = 0;
		npopin = 0;
	}
	else{
		indices = (gint *)g_realloc(indices, nind*sizeof(gint));
		groups = (gint **) g_malloc(nind*sizeof(gint *));
		prev=indices[0];

		for ( i = 0; i< nind; i++){
			groups[i] = (gint *) g_malloc(nind*sizeof(gint ));
		}

		if (DEBUG){
			printf(" nind %d \n", nind);
			printf("indices \n");
			for (i = 1; i< nind; i++){
				printf("indices[%d] %d \n", i, indices[i]);
			}
		}
		ngrind = (gint *) g_malloc(nind*sizeof(gint ));

		// split indices into groups with consecutive indices, ngr = number of these groups, ngrind = number of indices in each group, groups[igr] = indices belonging to group[igr]
		groups[0][0]=prev;
		j=1;
		ngr=0;
		if (DEBUG) 			printf("--------\n");
		for (i = 1; i< nind; i++){
			if (indices[i] == prev+1){
				groups[ngr][j] = indices[i];
				prev++;
				j++;
				if (DEBUG)	printf("add indices[%d]=%d to group %d \n",i, indices[i], ngr);
			}
			else{
				if (DEBUG)	printf("finish previous group at ngr %d \n", ngr);
				ngrind[ngr]=j;
				ngr++;
				if (DEBUG)      printf("start new group at ngr %d \n", ngr);
				prev = indices[i];
				j=0;
				groups[ngr][j]=indices[i];
				j++;
			}
		}
		ngrind[ngr]=j;
		if (DEBUG)     printf("finish previous group at ngr %d \n", ngr);
		ngr++;
	}

	if (DEBUG){
		printf(" ngr %d \n", ngr);
		for (i=0; i< ngr; i++){
			printf("ngrind[%d] %d \n", i, ngrind[i]);
			for (j=0 ; j< ngrind[i]; j++){
				printf(" groups[%d][%d] %d \n",i, j, groups[i][j]);
			}
		}
	}


	//copy local data to Data structure
	apopdata->havg =  gwy_data_line_duplicate(xavg);
	apopdata->dh =  gwy_data_line_duplicate(dx);

	apop_zoom_restore_h(data);
	apop_zoom_restore_der(data);

	// if no sliding average data displayed, add them. update them in any case.
	nc = gwy_graph_model_get_n_curves(apopctrls->graph_model);
	if (nc == 1)
		gwy_graph_model_add_curve(apopctrls->graph_model, apopctrls->cmodelhavg);
	gwy_graph_curve_model_set_data(apopctrls->cmodelhavg, apopdata->t->data, apopdata->havg->data, apopdata->havg->res);

	// if popin data displayed, remove them
	nc = gwy_graph_model_get_n_curves(apopctrls->graph_model);
	for (i = 0; i < nc-NLOAD; i++){
		g_snprintf(str, sizeof(str), "Pop-in #%d", i+1);
		gwy_graph_model_remove_curve_by_description(apopctrls->graph_model, str);
	}
	nc = gwy_graph_model_get_n_curves(apopctrls->graph_model_der);
	for (i = 0; i < nc-NDER; i++){
		g_snprintf(str, sizeof(str), "Pop-in #%d", i+1);
		gwy_graph_model_remove_curve_by_description(apopctrls->graph_model_der, str);
	}

	// if no derivative data displayed, add curve; update them in anycase.
	if (gwy_graph_model_get_n_curves(apopctrls->graph_model_der) == 0){
		gwy_graph_model_add_curve(apopctrls->graph_model_der, apopctrls->cmodelder);
		gwy_graph_model_add_curve(apopctrls->graph_model_der, apopctrls->cmodelthresh);
		gwy_graph_model_add_curve(apopctrls->graph_model_der, apopctrls->cmodelthresh2);
	}
	gwy_graph_curve_model_set_data(apopctrls->cmodelder, apopdata->t->data, apopdata->dh->data, apopdata->t->res);
	gwy_graph_curve_model_set_data(apopctrls->cmodelthresh, apopdata->t->data, apopdata->dth->data, apopdata->t->res);
	gwy_graph_curve_model_set_data(apopctrls->cmodelthresh2, apopdata->t->data, apopdata->dth2->data, apopdata->t->res);


	// clean up old data if necessary
	if (apopdata->Fpopin)
		g_free(apopdata->Fpopin);
	if (apopdata->hpopin)
		g_free(apopdata->hpopin);
	if (apopdata->dhpopin)
		g_free(apopdata->dhpopin);
	if (apopdata->ileft)
		g_free(apopdata->ileft);
	if (apopdata->iright)
		g_free(apopdata->iright);

	//allocate new data
	apopdata->Fpopin = (gdouble *)g_malloc(ngr*sizeof(gdouble));
	apopdata->hpopin = (gdouble *)g_malloc(ngr*sizeof(gdouble));
	apopdata->dhpopin = (gdouble *)g_malloc(ngr*sizeof(gdouble));
	apopdata->ileft = (gint *)g_malloc(ngr*sizeof(gdouble));
	apopdata->iright = (gint *)g_malloc(ngr*sizeof(gdouble));


	// starting from a group of consecutive indices, add more indices at both ends so that the derivative is everywhere above threshhold2
	// if the width of this group is larger than wpop and the height jump associated with it larger than hpop, then this is a pop-in
	npopin=0;
	for (i = 0; i< ngr; i++){
		ileft = groups[i][0];	
		while (ileft >0 && dx->data[ileft] > thresh2 && dx->data[ileft]>dx->data[ileft-1])
			ileft--;
		iright = groups[i][ngrind[i]-1];
		while (iright < n-1 && dx->data[iright] > thresh2 && dx->data[iright]>dx->data[iright+1])
			iright++;

		if ((iright-ileft > wpop) & (x->data[iright]-x->data[ileft] > hpop))
		{
			//it's a real pop-in, write the data to the Data structure
			apopdata->Fpopin[npopin] = gwy_data_line_get_avg(gwy_data_line_part_extract(data->args->fddata.Fload, ileft, iright-ileft+1));
			apopdata->hpopin[npopin] = data->args->fddata.hload->data[ileft];
			apopdata->dhpopin[npopin] =  data->args->fddata.hload->data[iright]-data->args->fddata.hload->data[ileft];
			apopdata->ileft[npopin] = ileft;
			apopdata->iright[npopin] = iright;
			npopin++;
		}
	}
	//realloc data to match
	apopdata->Fpopin = (gdouble *)g_realloc(apopdata->Fpopin,npopin*sizeof(gdouble));
	apopdata->hpopin = (gdouble *)g_realloc(apopdata->hpopin,npopin*sizeof(gdouble));
	apopdata->dhpopin = (gdouble *)g_realloc(apopdata->dhpopin,npopin*sizeof(gdouble));
	apopdata->ileft = (gint *)g_realloc(apopdata->ileft,npopin*sizeof(gint));
	apopdata->iright = (gint *)g_realloc(apopdata->iright,npopin*sizeof(gint));
	apopdata->npopin = npopin;

	//remove all previous results, that were displayed in the table
	remove_table_content(apopctrls->res);
	gtk_table_resize (GTK_TABLE (apopctrls->res), 1, 4);

	// update npopins
	g_snprintf(str, sizeof(str), "%d", apopdata->npopin);
	gtk_label_set_text(GTK_LABEL(apopctrls->npopin), str);


	//create new results table
	i=0;

	gtk_table_resize (GTK_TABLE (apopctrls->res), 1+npopin, 4);

	g_snprintf(str, sizeof(str), "Load / mN");
	label = gtk_label_new(str);
	gtk_table_attach(GTK_TABLE(apopctrls->res), label, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

	g_snprintf(str, sizeof(str), "Depth / nm");
	label = gtk_label_new(str);
	gtk_table_attach(GTK_TABLE(apopctrls->res), label, 2, 3, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

	g_snprintf(str, sizeof(str), "Depth difference / nm");
	label = gtk_label_new(str);
	gtk_table_attach(GTK_TABLE(apopctrls->res), label, 3, 4, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

	i++;

	for (j=0;j < npopin; j++){
		g_snprintf(str, sizeof(str), "#%d:", j+1);
		label = gtk_label_new(str);
		gtk_table_attach(GTK_TABLE(apopctrls->res), label, 0, 1, i,i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

		g_snprintf(str, sizeof(str), FORCE,apopdata->Fpopin[j] );
		label = gtk_label_new(str);
		gtk_table_attach(GTK_TABLE(apopctrls->res), label, 1, 2, i,i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

		g_snprintf(str, sizeof(str), DEPTH,apopdata->hpopin[j] );
		label = gtk_label_new(str);
		gtk_table_attach(GTK_TABLE(apopctrls->res), label, 2, 3, i,i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

		g_snprintf(str, sizeof(str), DEPTH,apopdata->dhpopin[j] );
		label = gtk_label_new(str);
		gtk_table_attach(GTK_TABLE(apopctrls->res), label, 3, 4, i,i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
		i++;
	}
	gtk_widget_show_all(apopctrls->res);

	//create curve for every pop-in and display
	cmodelpopin = (GwyGraphCurveModel **)g_malloc(npopin*sizeof(GwyGraphCurveModel *));
	for (i = 0; i < npopin; i++){
		cmodelpopin[i] = gwy_graph_curve_model_new();
		gwy_graph_model_add_curve(apopctrls->graph_model, cmodelpopin[i]);
		g_snprintf(str, sizeof(str), "Pop-in #%d", i+1);
		g_object_set(cmodelpopin[i],
				"mode", GWY_GRAPH_CURVE_LINE_POINTS,
				"description", str,
				"point-size", 0,
				"point-type", GWY_GRAPH_POINT_CIRCLE,
				"color", gwy_graph_get_preset_color(i+fmax(NLOAD,NDER)+1),
				"line-style", GDK_LINE_SOLID,
				"line-width", 2,
				NULL);
		tpopin = gwy_data_line_part_extract(apopdata->t,apopdata->ileft[i],apopdata->iright[i]-apopdata->ileft[i]+1);
		hpopin = gwy_data_line_part_extract(data->args->fddata.hload,apopdata->ileft[i],apopdata->iright[i]-apopdata->ileft[i]+1);
		gwy_graph_curve_model_set_data(cmodelpopin[i], tpopin->data, hpopin->data,tpopin->res);

	}
	cmodeldpopin = (GwyGraphCurveModel **)g_malloc(npopin*sizeof(GwyGraphCurveModel *));
	for (i = 0; i < npopin; i++){
		cmodeldpopin[i] = gwy_graph_curve_model_new();
		gwy_graph_model_add_curve(apopctrls->graph_model_der, cmodeldpopin[i]);
		g_snprintf(str, sizeof(str), "Pop-in #%d", i+1);
		g_object_set(cmodeldpopin[i],
				"mode", GWY_GRAPH_CURVE_LINE_POINTS,
				"description", str,
				"point-size", 0,
				"point-type", GWY_GRAPH_POINT_CIRCLE,
				"color", gwy_graph_get_preset_color(i+fmax(NLOAD,NDER)+1),
				"line-style", GDK_LINE_SOLID,
				"line-width", 2,
				NULL);
		tpopin = gwy_data_line_part_extract(apopdata->t,apopdata->ileft[i],apopdata->iright[i]-apopdata->ileft[i]+1);
		dhpopin = gwy_data_line_part_extract(apopdata->dh,apopdata->ileft[i],apopdata->iright[i]-apopdata->ileft[i]+1);
		gwy_graph_curve_model_set_data(cmodeldpopin[i], tpopin->data, dhpopin->data,tpopin->res);

	}

}

void apopin_write_file(gchar *filename, Args *args) 
{
	FILE *file;
	gint i,j;
	Apopindata *apopdata;

	if (args->fddata.hall->res == 0 || args->fddata.Fall->res==0) {
	    show_error("No data.");
	    return;
	}

	file = fopen(filename, "w");
	if (!file) {
	    show_error( "Could not open file.");
	    return;
	}

	apopdata = &(args->apopdata);

	fprintf(file, "# %s \n", args->fddata.filename);
	fprintf(file, "#Popins found by method A \n");
	fprintf(file, "#Input parameters:\n");
	fprintf(file, "#moving average window width  %d \n", apopdata->nmove);
	fprintf(file, "#threshhold of depth derivative for identification of popin  %g \n", apopdata->thresh);
	fprintf(file, "#threshhold of depth derivative for determination of popin width/height  %g \n", apopdata->thresh2);
	fprintf(file, "#minimum popin width %d \n", apopdata->wpop);
	fprintf(file, "#minimum popin height %g \n", apopdata->hpop);
	fprintf(file, "\n\n");

	fprintf(file, "\n#Popins found by method A : %d \n",apopdata->npopin );
	if (apopdata->npopin){
		fprintf(file, "#    load/mN     depth/mN     depth difference/mN    index left-end      index right-end \n");
		for (i = 0; i< apopdata->npopin;i++){
			fprintf(file, "#%d   %g    %g      %g    %d   %d \n", i+1, apopdata->Fpopin[i],apopdata->hpopin[i],apopdata->dhpopin[i], apopdata->ileft[i], apopdata->iright[i]);
		}
		fprintf(file, "\n\n");

		fprintf(file, "#Popin depth data \n");
		for (i = 0; i< apopdata->npopin;i++){
			fprintf(file, "#Popin #%d\n", i+1);
			fprintf(file, "# index i    hload/nm       havg/nm    Fload/nm   dh/di    threshold height   threshold width   \n");
			for (j = apopdata->ileft[i]; j<= apopdata->iright[i]; j++)
				fprintf(file, "%g   %g %g   %g   %g %g %g\n",apopdata->t->data[j], args->fddata.hload->data[j], apopdata->havg->data[j],
						args->fddata.Fload->data[j], apopdata->dh->data[j], apopdata->thresh, apopdata->thresh2 );
			fprintf(file, "\n\n");
		}
		fprintf(file, "\n\n");
		fprintf(file, "\n\n");
		fprintf(file, "# index i    hload/nm       havg/nm    Fload/nm   dh/di    threshold height   threshold width   \n");
		for (j = 0; j< apopdata->havg->res; j++)
			fprintf(file, "%g   %g %g   %g   %g %g %g\n",apopdata->t->data[j], args->fddata.hload->data[j], apopdata->havg->data[j],
				       	args->fddata.Fload->data[j], apopdata->dh->data[j], apopdata->thresh, apopdata->thresh2 );

	}
	fclose(file); 
}

static void apopin_save_data( Args *args)
{
	GtkWidget *dialog;
	gchar *filename;
	
	dialog = gtk_file_chooser_dialog_new("Save File",
			NULL,
			GTK_FILE_CHOOSER_ACTION_SAVE,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_SAVE, GTK_RESPONSE_OK,
			NULL);

	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), gwy_app_get_current_directory());

	if ( gtk_dialog_run (GTK_DIALOG ( dialog)) == GTK_RESPONSE_OK)
	{
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		if (gwy_app_file_confirm_overwrite(dialog))
			apopin_write_file(filename, args);
		g_free(filename);
	}

	gtk_widget_destroy ( dialog);
}

// create gui
GtkWidget *
tool_apopins_create (Data *data)
{
	GtkWidget *spin;
	Args *args;
	ApopinControls *apopcontrols;
	gint i;

	args=data->args;
	apopcontrols=data->controls->apopcontrols;

	// GraphAreas: one for the dependence depth-(pseudo)time (== index), one for the derivative of the smoothed depth
	apopcontrols->graph_model = gwy_graph_model_new();
	g_object_set(apopcontrols->graph_model, "title", "Depth", NULL);
	gwy_graph_model_set_axis_label (apopcontrols->graph_model, GTK_POS_BOTTOM, "i");
	gwy_graph_model_set_axis_label (apopcontrols->graph_model, GTK_POS_LEFT, "h / nm");

	apopcontrols->zoomstack_h = NULL;
	apopcontrols->zoomstack_h = g_slist_prepend (apopcontrols->zoomstack_h, apopcontrols->graph_model);

	apopcontrols->graph_h = gwy_graph_new(apopcontrols->graph_model);
	gtk_widget_set_size_request(GTK_WIDGET(apopcontrols->graph_h), 600, 250);

	gwy_graph_enable_user_input(GWY_GRAPH(apopcontrols->graph_h), FALSE);
	gwy_graph_set_status(GWY_GRAPH(apopcontrols->graph_h), GWY_GRAPH_STATUS_XSEL);

	apopcontrols->cmodelload = gwy_graph_curve_model_new();
	apopcontrols->cmodelhavg = gwy_graph_curve_model_new();

	g_object_set(apopcontrols->cmodelload,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "Depth",
			"point-size", 0,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
		        "color", CURVE_COLOR_LOAD,
			"line-style", GDK_LINE_SOLID,
		        "line-width", 2,
			NULL);
	gwy_graph_model_add_curve(apopcontrols->graph_model, apopcontrols->cmodelload);

	g_object_set(apopcontrols->cmodelhavg,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "Smoothed depth",
			"point-size", 0,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
			"color", CURVE_COLOR_SMOOTH_LOAD,
			"line-style", GDK_LINE_SOLID,
		        "line-width", 2,
			NULL);

	apopcontrols->graph_model_der = gwy_graph_model_new();
	g_object_set(apopcontrols->graph_model_der, "title", "Derivative", NULL);
	gwy_graph_model_set_axis_label (apopcontrols->graph_model_der, GTK_POS_BOTTOM, "i");
	gwy_graph_model_set_axis_label (apopcontrols->graph_model_der, GTK_POS_LEFT, "dh/di / nm");

	apopcontrols->zoomstack_der = NULL;
	apopcontrols->zoomstack_der = g_slist_prepend (apopcontrols->zoomstack_der, apopcontrols->graph_model_der);

	apopcontrols->graph_der = gwy_graph_new(apopcontrols->graph_model_der);
	gtk_widget_set_size_request(GTK_WIDGET(apopcontrols->graph_der), 600, 150);

	gwy_graph_enable_user_input(GWY_GRAPH(apopcontrols->graph_der), FALSE);
	gwy_graph_set_status(GWY_GRAPH(apopcontrols->graph_der), GWY_GRAPH_STATUS_XSEL);


	apopcontrols->cmodelder = gwy_graph_curve_model_new();
	apopcontrols->cmodelthresh = gwy_graph_curve_model_new();
	apopcontrols->cmodelthresh2 = gwy_graph_curve_model_new();

	g_object_set(apopcontrols->cmodelder,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "Derivative",
			"point-size", 3,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
			"color", CURVE_COLOR_DER,
			"line-style", GDK_LINE_SOLID,
			NULL);

	g_object_set(apopcontrols->cmodelthresh,
			"mode", GWY_GRAPH_CURVE_LINE,
			"description", "Threshold",
			"color",CURVE_COLOR_TH, 
			"line-style", GDK_LINE_SOLID,
			NULL);

	g_object_set(apopcontrols->cmodelthresh2,
			"mode", GWY_GRAPH_CURVE_LINE,
			"description", "Threshold minimum",
			"color", CURVE_COLOR_TH2,
			"line-style", GDK_LINE_SOLID,
			NULL);


	apopcontrols->ctrls = gtk_vbox_new (FALSE, CTRLS_SPACING);
	
	/* Parameters section */

	apopcontrols->frame_parameters = gtk_frame_new (NULL);
	gtk_frame_set_label_widget (GTK_FRAME (apopcontrols->frame_parameters), GTK_WIDGET (gwy_label_new_header("Parameters")));
	apopcontrols->table_parameters = gtk_table_new (5, 2, FALSE);
	gtk_container_set_border_width (GTK_CONTAINER (apopcontrols->table_parameters), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (apopcontrols->table_parameters), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (apopcontrols->table_parameters), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add ( GTK_CONTAINER (apopcontrols->frame_parameters), apopcontrols->table_parameters);
	
	i = 0;

	gtk_table_attach(GTK_TABLE(apopcontrols->table_parameters), label_new_with_markup_left("Moving average window width"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	apopcontrols->nmove = gtk_adjustment_new(args->apopdata.nmove, 1, 101, 2, 10,0); 
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(apopcontrols->nmove), 1, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(apopcontrols->table_parameters), spin, 1, 2, i, i+1,  GTK_FILL, 0, 0, 0); 
	g_signal_connect_swapped(apopcontrols->nmove, "value-changed", G_CALLBACK(nmove_changed), data); 
	i++;

	gtk_table_attach(GTK_TABLE(apopcontrols->table_parameters), label_new_with_markup_left("Derivative threshhold (identification)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	apopcontrols->thresh = gtk_adjustment_new(args->apopdata.thresh, 0, 10000, 0.001, 0.1,0); 
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(apopcontrols->thresh), 0.01, 3);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(apopcontrols->table_parameters), spin, 1, 2, i, i+1,  GTK_FILL, 0, 0, 0); 
	g_signal_connect_swapped(apopcontrols->thresh, "value-changed", G_CALLBACK(thresh_changed), data); 
	i++;

	gtk_table_attach(GTK_TABLE(apopcontrols->table_parameters), label_new_with_markup_left("Derivative threshhold (width)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	apopcontrols->thresh2 = gtk_adjustment_new(args->apopdata.thresh2, 0, 10000, 0.001, 0.1,0); 
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(apopcontrols->thresh2), 0.01, 3);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(apopcontrols->table_parameters), spin, 1, 2, i, i+1,  GTK_FILL, 0, 0, 0); 
	g_signal_connect_swapped(apopcontrols->thresh2, "value-changed", G_CALLBACK(thresh2_changed), data); 
	i++;

	gtk_table_attach(GTK_TABLE(apopcontrols->table_parameters), label_new_with_markup_left("Minimum pop-in width"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	apopcontrols->wpop = gtk_adjustment_new(args->apopdata.wpop, 0, 100, 1, 10,0); 
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(apopcontrols->wpop), 1, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(apopcontrols->table_parameters), spin, 1, 2, i, i+1,  GTK_FILL, 0, 0, 0); 
	g_signal_connect_swapped(apopcontrols->wpop, "value-changed", G_CALLBACK(wpop_changed), data); 
	i++;

	gtk_table_attach(GTK_TABLE(apopcontrols->table_parameters), label_new_with_markup_left("Minimum pop-in height"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	apopcontrols->hpop = gtk_adjustment_new(args->apopdata.hpop, 0, 100, 0.001, 0.1,0); 
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(apopcontrols->hpop), 0.01, 3);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(apopcontrols->table_parameters), spin, 1, 2, i, i+1,  GTK_FILL, 0, 0, 0); 
	g_signal_connect_swapped(apopcontrols->hpop, "value-changed", G_CALLBACK(hpop_changed), data); 
	i++;

	/* Reset button */
	apopcontrols->button_reset =  gtk_button_new_with_label("Reset to defaults");
	gtk_table_attach(GTK_TABLE(apopcontrols->table_parameters), apopcontrols->button_reset, 0, 2, i, i+1, GTK_FILL, 0,0,0);
	g_signal_connect_swapped(apopcontrols->button_reset, "clicked", G_CALLBACK(reset_default), data);
	i++;
	


	/* Run button */
	apopcontrols->button_run = gtk_button_new_with_mnemonic(gwy_sgettext("verb|_Run"));
	gtk_button_set_image (GTK_BUTTON (apopcontrols->button_run), gtk_image_new_from_stock (GTK_STOCK_MEDIA_PLAY, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (apopcontrols->button_run, "clicked", G_CALLBACK (apopin_run), data);

	/* Results section */
		
	apopcontrols->frame_results = gtk_frame_new (NULL);
	gtk_frame_set_label_widget (GTK_FRAME (apopcontrols->frame_results), GTK_WIDGET (gwy_label_new_header("Results")));
	apopcontrols->table_results = gtk_table_new (2, 2, FALSE);
	gtk_container_set_border_width (GTK_CONTAINER (apopcontrols->table_results), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (apopcontrols->table_results), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (apopcontrols->table_results), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add ( GTK_CONTAINER (apopcontrols->frame_results), apopcontrols->table_results);

	gtk_table_attach (GTK_TABLE (apopcontrols->table_results), label_new_with_markup_left("Found pop-ins:"), 0, 1, 0, 1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

	apopcontrols->npopin = gtk_label_new(NULL);
	gtk_misc_set_alignment (GTK_MISC (apopcontrols->npopin), 0.0, 0.5); 
	gtk_table_attach (GTK_TABLE (apopcontrols->table_results), data->controls->apopcontrols->npopin, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	apopcontrols->res = gtk_table_new (1, 4, FALSE);
	gtk_table_attach (GTK_TABLE (apopcontrols->table_results), apopcontrols->res, 0, 2, 1, 2, GTK_EXPAND | GTK_FILL, 0, 0, 0);

	/* Save button */
	apopcontrols->button_save = gtk_button_new_from_stock (GTK_STOCK_SAVE);
	g_signal_connect_swapped (apopcontrols->button_save, "clicked", G_CALLBACK (apopin_save_data), args);

	/* Zooming */

	apopcontrols->vbox_zoom_buttons_h = gtk_vbox_new (TRUE, 0);
	apopcontrols->vbox_zoom_outer_h = gtk_vbox_new (FALSE, 0);

	apopcontrols->button_zoom_in_h = gtk_button_new ();
	gtk_button_set_image (GTK_BUTTON (apopcontrols->button_zoom_in_h), gtk_image_new_from_stock (GTK_STOCK_ZOOM_IN, GTK_ICON_SIZE_BUTTON));
	gtk_box_pack_start (GTK_BOX (apopcontrols->vbox_zoom_buttons_h), apopcontrols->button_zoom_in_h, TRUE, TRUE, 0);
	g_signal_connect_swapped(apopcontrols->button_zoom_in_h, "clicked", G_CALLBACK (apop_zoom_in_h), data);
	
	apopcontrols->button_zoom_out_h = gtk_button_new ();
	gtk_button_set_image (GTK_BUTTON (apopcontrols->button_zoom_out_h), gtk_image_new_from_stock (GTK_STOCK_ZOOM_OUT, GTK_ICON_SIZE_BUTTON));
	gtk_box_pack_start (GTK_BOX (apopcontrols->vbox_zoom_buttons_h), apopcontrols->button_zoom_out_h, TRUE, TRUE, 0);
	g_signal_connect_swapped(apopcontrols->button_zoom_out_h, "clicked", G_CALLBACK (apop_zoom_out_h), data);

	apopcontrols->button_zoom_restore_h = gtk_button_new ();
	gtk_button_set_image (GTK_BUTTON (apopcontrols->button_zoom_restore_h), gtk_image_new_from_stock (GTK_STOCK_ZOOM_FIT, GTK_ICON_SIZE_BUTTON));
	gtk_box_pack_start (GTK_BOX (apopcontrols->vbox_zoom_buttons_h), apopcontrols->button_zoom_restore_h, TRUE, TRUE, 0);
	g_signal_connect_swapped(apopcontrols->button_zoom_restore_h, "clicked", G_CALLBACK (apop_zoom_restore_h), data);

	apopcontrols->vbox_zoom_buttons_der = gtk_vbox_new (TRUE, 0);
	apopcontrols->vbox_zoom_outer_der = gtk_vbox_new (FALSE, 0);

	apopcontrols->button_zoom_in_der = gtk_button_new ();
	gtk_button_set_image (GTK_BUTTON (apopcontrols->button_zoom_in_der), gtk_image_new_from_stock (GTK_STOCK_ZOOM_IN, GTK_ICON_SIZE_BUTTON));
	gtk_box_pack_start (GTK_BOX (apopcontrols->vbox_zoom_buttons_der), apopcontrols->button_zoom_in_der, TRUE, TRUE, 0);
	g_signal_connect_swapped(apopcontrols->button_zoom_in_der, "clicked", G_CALLBACK (apop_zoom_in_der), data);
	
	apopcontrols->button_zoom_out_der = gtk_button_new ();
	gtk_button_set_image (GTK_BUTTON (apopcontrols->button_zoom_out_der), gtk_image_new_from_stock (GTK_STOCK_ZOOM_OUT, GTK_ICON_SIZE_BUTTON));
	gtk_box_pack_start (GTK_BOX (apopcontrols->vbox_zoom_buttons_der), apopcontrols->button_zoom_out_der, TRUE, TRUE, 0);
	g_signal_connect_swapped(apopcontrols->button_zoom_out_der, "clicked", G_CALLBACK (apop_zoom_out_der), data);

	apopcontrols->button_zoom_restore_der = gtk_button_new ();
	gtk_button_set_image (GTK_BUTTON (apopcontrols->button_zoom_restore_der), gtk_image_new_from_stock (GTK_STOCK_ZOOM_FIT, GTK_ICON_SIZE_BUTTON));
	gtk_box_pack_start (GTK_BOX (apopcontrols->vbox_zoom_buttons_der), apopcontrols->button_zoom_restore_der, TRUE, TRUE, 0);
	g_signal_connect_swapped(apopcontrols->button_zoom_restore_der, "clicked", G_CALLBACK (apop_zoom_restore_der), data);

	
	//final assembly
	apopcontrols->vbox_graphs = gtk_vbox_new (FALSE, CTRLS_SPACING);
	apopcontrols->hbox_graph_h = gtk_hbox_new (FALSE, CTRLS_SPACING);
	apopcontrols->hbox_graph_der = gtk_hbox_new (FALSE, CTRLS_SPACING);
	
	gtk_box_pack_start (GTK_BOX (apopcontrols->hbox_graph_h), apopcontrols->graph_h, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (apopcontrols->hbox_graph_h), apopcontrols->vbox_zoom_outer_h, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (apopcontrols->hbox_graph_der), apopcontrols->graph_der, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (apopcontrols->hbox_graph_der), apopcontrols->vbox_zoom_outer_der, FALSE, FALSE, 0);

	gtk_box_pack_start (GTK_BOX (apopcontrols->vbox_zoom_outer_h), apopcontrols->vbox_zoom_buttons_h, TRUE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (apopcontrols->vbox_zoom_outer_der), apopcontrols->vbox_zoom_buttons_der, TRUE, FALSE, 0);

	gtk_box_pack_start (GTK_BOX (apopcontrols->ctrls), apopcontrols->frame_parameters, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (apopcontrols->ctrls), apopcontrols->button_run, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (apopcontrols->ctrls), apopcontrols->frame_results, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (apopcontrols->ctrls), apopcontrols->button_save, FALSE, FALSE, 0);

	gtk_box_pack_start (GTK_BOX (apopcontrols->vbox_graphs), apopcontrols->hbox_graph_h, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (apopcontrols->vbox_graphs), apopcontrols->hbox_graph_der, TRUE, TRUE, 0);
	
	apopcontrols->apop_gui = gtk_hbox_new (FALSE, CTRLS_SPACING);
	
	gtk_box_pack_start (GTK_BOX (apopcontrols->apop_gui), apopcontrols->ctrls, FALSE, FALSE, 0); /* expand, fill, padding */
	gtk_box_pack_start (GTK_BOX (apopcontrols->apop_gui), apopcontrols->vbox_graphs, TRUE, TRUE, 0);

	gtk_widget_show(apopcontrols->apop_gui);
    
	return apopcontrols->apop_gui;
}

static void
update_models_apop_h(ApopinControls *apopcontrols)
{
    GwyGraphCurveModel *cmodel;
    
    apopcontrols->graph_model = gwy_graph_get_model (GWY_GRAPH(apopcontrols->graph_h));
    cmodel = gwy_graph_model_get_curve_by_description (apopcontrols->graph_model, "Depth");
    if (cmodel)
	apopcontrols->cmodelload = cmodel;
    cmodel = gwy_graph_model_get_curve_by_description (apopcontrols->graph_model, "Smoothed depth");
    if (cmodel)
	apopcontrols->cmodelhavg = cmodel;
}

static void
apop_zoom_in_h(Data *data)
{
    ApopinControls *apopcontrols;

    apopcontrols = data->controls->apopcontrols;
    
    rs_zoom_in (GWY_GRAPH (apopcontrols->graph_h), &(apopcontrols->zoomstack_h), FALSE); /* do not rescale y */
    update_models_apop_h(apopcontrols);
}

static void
apop_zoom_out_h(Data *data)
{
    ApopinControls *apopcontrols;
    
    apopcontrols = data->controls->apopcontrols;
    
    rs_zoom_out (GWY_GRAPH (apopcontrols->graph_h), &(apopcontrols->zoomstack_h));
    update_models_apop_h(apopcontrols);

}

void
apop_zoom_restore_h(Data *data)
{
    ApopinControls *apopcontrols;

    apopcontrols = data->controls->apopcontrols;
    
    rs_zoom_restore (GWY_GRAPH (apopcontrols->graph_h), &(apopcontrols->zoomstack_h));
    update_models_apop_h(apopcontrols);
}

static void
update_models_apop_der(ApopinControls *apopcontrols)
{
    GwyGraphCurveModel *cmodel;
    
    apopcontrols->graph_model_der = gwy_graph_get_model (GWY_GRAPH(apopcontrols->graph_der));
    cmodel = gwy_graph_model_get_curve_by_description (apopcontrols->graph_model_der, "Derivative");
    if (cmodel)
	apopcontrols->cmodelder = cmodel;
    cmodel = gwy_graph_model_get_curve_by_description (apopcontrols->graph_model_der, "Threshold");
    if (cmodel)
	apopcontrols->cmodelthresh = cmodel;
    cmodel = gwy_graph_model_get_curve_by_description (apopcontrols->graph_model_der, "Threshold minimum");
    if (cmodel)
	apopcontrols->cmodelthresh2 = cmodel;
}

static void
apop_zoom_in_der(Data *data)
{
    ApopinControls *apopcontrols;

    apopcontrols = data->controls->apopcontrols;
    
    rs_zoom_in (GWY_GRAPH (apopcontrols->graph_der), &(apopcontrols->zoomstack_der), FALSE); /* do not rescale y */
    update_models_apop_der(apopcontrols);
}

static void
apop_zoom_out_der(Data *data)
{
    ApopinControls *apopcontrols;
    
    apopcontrols = data->controls->apopcontrols;
    
    rs_zoom_out (GWY_GRAPH (apopcontrols->graph_der), &(apopcontrols->zoomstack_der));
    update_models_apop_der(apopcontrols);

}

void
apop_zoom_restore_der(Data *data)
{
    ApopinControls *apopcontrols;

    apopcontrols = data->controls->apopcontrols;
    
    rs_zoom_restore (GWY_GRAPH (apopcontrols->graph_der), &(apopcontrols->zoomstack_der));
    update_models_apop_der(apopcontrols);
}
