#include <gtk/gtk.h>
#include <math.h>
#include <app/file.h>
#include "indent-toolbox-common.h"

#include "datatypes.h"
#include "controls.h"

#include "gui-utils.h"
#include "tool_epwork_unc.h"
#include "tool_epwork.h"

#define CONTACTMAX 2

static void
work_run_shift_contact(Workdata *wdata, FDdata *fddata,gdouble *We, gdouble *Wp,gint j)
{
	Workdata w;
	FDdata fdc;
	gdouble dh,dF;
	gint iend;

	We[j+CONTACTMAX] = -1;
	Wp[j+CONTACTMAX] = -1;

//	printf(" #-----------------------------j = %d ------------------------------------\n\n\n", j);
	// create shifted FDdata
	
	if (fddata->i_contact_load+j < 0)
		return;

	fdc.i_contact_load = fddata->i_contact_load+j;
	fdc.hall_orig = gwy_data_line_duplicate(fddata->hall_orig);
	fdc.Fall_orig = gwy_data_line_duplicate(fddata->Fall_orig);

//	printf(" old contact %d new contact %d  j %d \n", fddata->i_contact_load, fdc.i_contact_load, j);
	
	fdc.hload = gwy_data_line_part_extract(fdc.hall_orig, fdc.i_contact_load, fddata->hload->res-j);
	fdc.Fload = gwy_data_line_part_extract(fdc.Fall_orig, fdc.i_contact_load, fddata->Fload->res-j);
	gwy_data_line_add(fdc.hload, -fdc.hall_orig->data[fdc.i_contact_load]);
	gwy_data_line_add(fdc.Fload, -fdc.Fall_orig->data[fdc.i_contact_load]);

	/*printf(" loading \n");
	printf("old contact \n");
	for (i = 0; i < fddata->hload->res; i++)
		printf("%d %g \n", i, fddata->hload->data[i]);
	printf("new contact \n");
	for (i = 0; i < fdc.hload->res; i++)
		printf("%d %g %g \n", i, fdc.hload->data[i], fddata->hload->data[i+j]-fdc.Fall_orig->data[fdc.i_contact_load]);
		*/

	fdc.hunload = gwy_data_line_duplicate(fddata->hunload);
	fdc.Funload = gwy_data_line_duplicate(fddata->Funload);
	dh = fdc.hall_orig->data[fdc.i_contact_load]-fddata->hall_orig->data[fddata->i_contact_load];
	dF = fdc.Fall_orig->data[fdc.i_contact_load]-fddata->Fall_orig->data[fddata->i_contact_load];
	gwy_data_line_add(fdc.hunload, -dh);
	gwy_data_line_add(fdc.Funload, -dF);
//	printf("dh %g dF %g \n", dh, dF);
	iend = 0;
	while ( iend <fdc.hunload->res && fdc.hunload->data[iend]>0 && fdc.Funload->data[iend]>0)
		iend++;

//	printf("iend %d res %d iend old %d \n", iend, fdc.hunload->res, fddata->i_end_unload);
	fdc.hunload = gwy_data_line_part_extract(fdc.hunload, 0, iend);
	fdc.Funload = gwy_data_line_part_extract(fdc.Funload, 0, iend);

	/*
	printf(" unloading \n");
	printf("old contact \n");
	for (i = 0; i < fddata->hunload->res; i++)
		printf("%d %g %g \n", i, fddata->hunload->data[i], fddata->Funload->data[i]);
	printf("new contact \n");
	for (i = 0; i < fdc.hunload->res; i++)
		printf("%d %g %g %g %g %g\n", i, fdc.hunload->data[i],fddata->hunload->data[i], fddata->hunload->data[i]-dh, fdc.Funload->data[i], fddata->Funload->data[i]);
		*/


	fdc.hhold = gwy_data_line_duplicate(fddata->hhold);
	fdc.Fhold = gwy_data_line_duplicate(fddata->Fhold);
	gwy_data_line_add(fdc.hhold, -dh);
	gwy_data_line_add(fdc.Fhold, -dF);
	
	/*
	printf(" holding \n");
	printf("old contact \n");
	for (i = 0; i < fddata->hhold->res; i++)
		printf("%d %g \n", i, fddata->hhold->data[i]);
	printf("new contact \n");
	for (i = 0; i < fdc.hhold->res; i++)
		printf("%d %g %g \n", i, fdc.hhold->data[i], fddata->hhold->data[i]);
		*/

	/*
	if (!create_shifted_FDdata(&fddata-> &fdc, j))
	{
		printf("could not create shifted FDdata\n");
		return;
	}
	*/

	//shifted Workdata, only nmove
	w.nmove = wdata->nmove;

	work_calc(&w, &fdc);

	We[j+CONTACTMAX] = w.workelastic;
	Wp[j+CONTACTMAX] = w.workplastic;

}


void work_uncertainty_write_file(gchar *filename, Args *args) 
{
	FILE *file;
	WorkUncdata *wunc;
	gint j;

	wunc=&(args->workunc);


	if (args->fddata.hall->res == 0 || args->fddata.Fall->res==0) {
	    show_error("No data.");
	    return;
	}

	file = fopen(filename, "w");
	if (!file) {
	    show_error("Could not open file.");
	    return;
	}

	fprintf(file, "#Changes of the elastic/plastic work corresponding to changes in the contact point \n");
	fprintf(file, "# shift of contact point with respect to the globally chosen contact point (negative - to the left, positive - to the right)   ");
	fprintf(file, "#W_elastic/pJ     W_plastic/pJ \n");
	for (j= -CONTACTMAX; j< CONTACTMAX+1; j++){
		fprintf(file, "# %d   %g   %g \n", j, wunc->We[j+CONTACTMAX], wunc->Wp[j+CONTACTMAX]);
	}

	fclose(file); 
}

static void work_uncertainty_save_data( Args *args)
{
	GtkWidget *dialog;
	gchar *filename;

	dialog = gtk_file_chooser_dialog_new("Save File",
			NULL,
			GTK_FILE_CHOOSER_ACTION_SAVE,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_SAVE, GTK_RESPONSE_OK,
			NULL);

	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), gwy_app_get_current_directory());

	if ( gtk_dialog_run (GTK_DIALOG ( dialog)) == GTK_RESPONSE_OK)
	{
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		if (gwy_app_file_confirm_overwrite(dialog))
			work_uncertainty_write_file(filename, args);
		g_free(filename);
	}
	gtk_widget_destroy ( dialog);
}


void 
work_unc_close_window(Data *data)
{
	g_free(data->args->workunc.We);
	g_free(data->args->workunc.Wp);
	data->args->workunc.We = NULL;
	data->args->workunc.Wp = NULL;
	gtk_widget_destroy(data->controls->workunccontrols->window);
}

void
work_uncertainty (Data *data)
{
	GtkWidget  *vbox, *label;
	GtkWidget *button_save;
	gint i,j;
	WorkUncControls *ctrls;
	WorkUncdata *unc;
	gchar str[50];

	unc = &(data->args->workunc);
        ctrls =	data->controls->workunccontrols;

	ctrls->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->window), "Uncertainty in elastic/plastic work");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->window), 400, 300); 
	g_signal_connect_swapped (ctrls->window, "delete_event", G_CALLBACK (work_unc_close_window), data);

	ctrls->table_contact = gtk_table_new(20, 5, FALSE);
	i=0;
	gtk_table_attach(GTK_TABLE(ctrls->table_contact), gwy_label_new_header("Uncertainties due to choice of contact point"), 0, 4, i,i+1, GTK_FILL, 0, 0, 0);
	i++;
	gtk_table_attach(GTK_TABLE(ctrls->table_contact), gwy_label_new_header("shift contact point"), 0, 1, i,i+1, GTK_FILL, 0, 0, 0);
	gtk_table_attach(GTK_TABLE(ctrls->table_contact), gwy_label_new_header("W<sub>el</sub>/GPa"), 1, 2, i,i+1, GTK_FILL, 0, 0, 0);
	gtk_table_attach(GTK_TABLE(ctrls->table_contact), gwy_label_new_header("W<sub>pl</sub>/GPa"), 2, 3, i,i+1, GTK_FILL, 0, 0, 0);
//	gtk_table_attach(GTK_TABLE(ctrls->table_contact), gwy_label_new_header("weight"), 2, 3, i,i+1, GTK_FILL, 0, 0, 0);
	i++;


	unc->We = (gdouble *)g_malloc((2*CONTACTMAX+1)*sizeof(gdouble));
	unc->Wp = (gdouble *)g_malloc((2*CONTACTMAX+1)*sizeof(gdouble));

	for (j = -CONTACTMAX; j<=CONTACTMAX ;j++){
		label = gtk_label_new(NULL);
		g_snprintf(str, sizeof(str),"%d\n", j); 
		gtk_label_set_text(GTK_LABEL(label), str);
		gtk_table_attach(GTK_TABLE(ctrls->table_contact), label, 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

		work_run_shift_contact(&(data->args->workdata), &(data->args->fddata), unc->We, unc->Wp,j);

		label = gtk_label_new(NULL);
		if (unc->We[j+CONTACTMAX]>0 )
			g_snprintf(str, sizeof(str),EPWORK, unc->We[j+CONTACTMAX]); 
		else
			g_snprintf(str, sizeof(str), "not defined");
		gtk_label_set_text(GTK_LABEL(label), str);
		gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5); 
		gtk_table_attach(GTK_TABLE(ctrls->table_contact), label, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

		label = gtk_label_new(NULL);
		if (unc->Wp[j+CONTACTMAX]>0 )
		g_snprintf(str, sizeof(str),EPWORK, unc->Wp[j+CONTACTMAX]); 
		else
			g_snprintf(str, sizeof(str), "not defined");
		gtk_label_set_text(GTK_LABEL(label), str);
		gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5); 
		gtk_table_attach(GTK_TABLE(ctrls->table_contact), label, 2, 3, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
		i++;
	}


	button_save = gtk_button_new_from_stock (GTK_STOCK_SAVE);
	g_signal_connect_swapped (button_save, "clicked", G_CALLBACK (work_uncertainty_save_data), data->args);

	vbox = gtk_vbox_new(TRUE, 4);
	gtk_box_pack_start(GTK_BOX(vbox), ctrls->table_contact, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), button_save,  TRUE, FALSE,0);
	gtk_container_add(GTK_CONTAINER(ctrls->window), vbox);
	gtk_widget_show_all(ctrls->window);
}
