#include <math.h>

#include <gtk/gtk.h>

#include <app/file.h>

#include "indent-toolbox-common.h"
#include "datatypes.h"
#include "controls.h"
#include "gui-utils.h"
#include "tool_hertz_unc.h"
#include "tool_hertz_mc.h"
#include "tool_hertz.h"


#define CONTACTMAX 2
#define MC_WHEN_WARN 1e6
#define DEBUG TRUE


static void hertz_fit_shift_contact(Hertzdata *hzdata, FDdata *fddata, Instdata *instdata, gdouble *Ec, gint j);


void 
hertz_propagate_uncertainties (Data *data)
{
	gchar str[30];
	HertzUncControls *ctrls;
	HertzUncdata *unc;
	Hertzdata *hz;
	Instdata *inst;
	gint i,j;
	GtkWidget *label;
	gdouble R,Er,Eit, nu,nui,Ei;

	unc = &(data->args->hertzunc);
	hz = &(data->args->hertzdata);
	inst = &(data->args->instdata);
        ctrls =	data->controls->hertzunccontrols;

	if ( !hz->has_fit )
		return;

	if ( unc->Ec ==NULL)
		return;
	
	//make copies to simplify code
	Eit = hz->Eit;
	Er = hz->Er;
	Ei = inst->Ei;
	nui = inst->nui;
	nu = inst->nu;
	R = hz->radius;

	unc->uEruh = hz->reg.unc_slope_x/hz->reg.slope*Er*unc->uh;//Er/a uah = Er/a ua_x*uh
	unc->uEruF = hz->reg.unc_slope_y/hz->reg.slope*Er*unc->uF; //Er/a uaF = Er/a ua_y*uF
	unc->uEruradius = 0.5*Er/R*unc->uradius; // -1/2 *Er/R u(R)

	//uEituh = dEit/ dh uh = dEit / dEr * dEr/dh uh = dEit/dEr * uEruh
	//dEit/dEr = Eit^2/Er^2/ (1-nu^2)

	unc->uEituh = Eit*Eit/Er/Er/(1-nu*nu)*unc->uEruh; 
	unc->uEituF = Eit*Eit/Er/Er/(1-nu*nu)*unc->uEruF; 
	unc->uEituradius = Eit*Eit/Er/Er/(1-nu*nu)*unc->uEruradius; 

	unc->uEitunu =  fabs(2*nu/(1-nu*nu)*Eit*unc->unu); // -2nu/(1-nu^2) Eit u(nu)
	unc->uEitunui = fabs(2*nui/(1-nu*nu)* Eit*Eit /Ei *unc->unui)*1e9; // -2nui/(1-nu^2) Eit^2/Ei u(nui)
	unc->uEituEi =  fabs((1-nui*nui)/(1-nu*nu) *Eit*Eit/ Ei*Ei *unc->uEi)*1e-18; //-(1-nui^2)/(1-nu)^2 Eit^2/Ei^2 u(Ei)


	/* show */
	g_snprintf(str, sizeof(str), MODUL, unc->uEruh);
	gtk_label_set_text(GTK_LABEL(ctrls->uEruh), str);
	g_snprintf(str, sizeof(str), MODUL, unc->uEruF);
	gtk_label_set_text(GTK_LABEL(ctrls->uEruF), str);
	g_snprintf(str, sizeof(str), MODUL, unc->uEruradius);
	gtk_label_set_text(GTK_LABEL(ctrls->uEruradius), str);

	g_snprintf(str, sizeof(str), MODUL, unc->uEituh);
	gtk_label_set_text(GTK_LABEL(ctrls->uEituh), str);
	g_snprintf(str, sizeof(str), MODUL, unc->uEituF);
	gtk_label_set_text(GTK_LABEL(ctrls->uEituF), str);
	g_snprintf(str, sizeof(str), MODUL, unc->uEituradius);
	gtk_label_set_text(GTK_LABEL(ctrls->uEituradius), str);
	g_snprintf(str, sizeof(str), MODUL, unc->uEitunu);
	gtk_label_set_text(GTK_LABEL(ctrls->uEitunu), str);
	g_snprintf(str, sizeof(str), MODUL, unc->uEitunui);
	gtk_label_set_text(GTK_LABEL(ctrls->uEitunui), str);
	g_snprintf(str, sizeof(str), "%.4g", unc->uEituEi);
	gtk_label_set_text(GTK_LABEL(ctrls->uEituEi), str);

	// total uncertainties
	unc->uEr = sqrt(unc->uEruh*unc->uEruh + unc->uEruF*unc->uEruF + unc->uEruradius*unc->uEruradius);
	g_snprintf(str, sizeof(str), MODUL, unc->uEr);
	gtk_label_set_text(GTK_LABEL(ctrls->uEr), str);

	unc->uEit = sqrt(unc->uEituh*unc->uEituh + unc->uEituF*unc->uEituF + unc->uEituradius*unc->uEituradius 
			+unc->uEitunu*unc->uEitunu +unc->uEitunui*unc->uEitunui +unc->uEituEi*unc->uEituEi);
	g_snprintf(str, sizeof(str), MODUL,unc->uEit );
	gtk_label_set_text(GTK_LABEL(ctrls->uEit), str);


	//destroy the labels of the old table_unc_contact
	remove_table_content(ctrls->table_unc_contact);

	//create new table
	i=0;
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact), gwy_label_new_header("Uncertainties due to choice of contact point"), 0, 4, i,i+1, GTK_FILL, 0, 0, 0);
	i++;
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact), gwy_label_new_header("shift contact point"), 0, 1, i,i+1, GTK_FILL, 0, 0, 0);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact), gwy_label_new_header("E<sub>r</sub>/GPa"), 1, 2, i,i+1, GTK_FILL, 0, 0, 0);
//	gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact), gwy_label_new_header("weight"), 2, 3, i,i+1, GTK_FILL, 0, 0, 0);
	i++;

	for (j = -CONTACTMAX; j<=CONTACTMAX ;j++){
		label = gtk_label_new(NULL);
		g_snprintf(str, sizeof(str),"%d\n", j); 
		gtk_label_set_text(GTK_LABEL(label), str);
		gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact), label, 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

		hertz_fit_shift_contact(&(data->args->hertzdata), &(data->args->fddata), &(data->args->instdata), unc->Ec, j);
		label = gtk_label_new(NULL);
		if (unc->Ec[j+CONTACTMAX] > 0)
			g_snprintf(str, sizeof(str),MODUL, unc->Ec[j+CONTACTMAX]); 
		else
			g_snprintf(str, sizeof(str), "not defined");
		gtk_label_set_text(GTK_LABEL(label), str);
		gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5); 
		gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact), label, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

		label = gtk_label_new(NULL);
		g_snprintf(str, sizeof(str),"1/%d", 2*CONTACTMAX+1); 
		gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5); 
		gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact), label, 2, 3, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
		i++;
	}
	gtk_widget_show_all(ctrls->table_unc_contact);

}

static void mc_iterations_changed(Data *data){
	gchar str[500];

	data->args->hertzunc.Nmc = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->hertzunccontrols->Nmc));
	if (data->args->hertzunc.Nmc > MC_WHEN_WARN){
		g_snprintf(str, sizeof(str), "Warning: Monte Carlo calculation for %d iterations may take very long!", data->args->hertzunc.Nmc);
		gtk_label_set_text(GTK_LABEL(data->controls->hertzunccontrols->mc_warn), str);
	}
	else{
		gtk_label_set_text(GTK_LABEL(data->controls->hertzunccontrols->mc_warn), NULL);
	}

}

static void 
unc_depth_changed (Data *data)
{
	data->args->hertzunc.uh = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->hertzunccontrols->uh));
	hertz_propagate_uncertainties (data);
}

static void 
unc_load_changed (Data *data)
{
	data->args->hertzunc.uF = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->hertzunccontrols->uF));
	hertz_propagate_uncertainties (data);
}

static void 
unc_radius_changed (Data *data)
{
	data->args->hertzunc.uradius = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->hertzunccontrols->uradius))*1e-9;
	hertz_propagate_uncertainties (data);
}

static void 
unc_nu_changed (Data *data)
{
	data->args->hertzunc.unu = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->hertzunccontrols->unu));
	hertz_propagate_uncertainties (data);
}

static void 
unc_nui_changed (Data *data)
{
	data->args->hertzunc.unui = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->hertzunccontrols->unui));
	hertz_propagate_uncertainties (data);
}

static void 
unc_Ei_changed (Data *data)
{
	data->args->hertzunc.uEi = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->hertzunccontrols->uEi))*1e9;
	hertz_propagate_uncertainties (data);
}



void hertz_uncertainty_write_file(gchar *filename, Args *args) 
{
	FILE *file;
	Hertzdata *hzdata;
	HertzUncdata *hzunc;
	Instdata *instdata;
	gint j;

	hzdata = &(args->hertzdata);
	hzunc = &(args->hertzunc);
	instdata = &(args->instdata);

	if (args->fddata.hall->res == 0 || args->fddata.Fall->res==0) {
	    show_error("No data.");
	    return;
	}

	file = fopen(filename, "w");
	if (!file) {
	    show_error("Could not open file.");
	    return;
	}

	fprintf(file, "# %s \n", args->fddata.filename);
	fprintf(file, "#Parameters of Hertz analysis \n");
	fprintf(file, "# u(depth): %g nm\n", hzunc->uh);
	fprintf(file, "# u(load): %g nm\n", hzunc->uF);
	fprintf(file, "#tip radius: %g nm       u(tip radius): %g nm\n", hzdata->radius*1e9, hzunc->uradius*1e9);
	fprintf(file, "#nu: %g                 u(nu):  %g  \n", instdata->nu, hzunc->unu);
	fprintf(file, "#nu_indenter: %g       u(nu_indenter):  %g   \n", instdata->nui, hzunc->unui);
	fprintf(file, "#E_indenter: %g        u(E_indenter):   %g GPa\n", instdata->Ei * 1e-9, hzunc->uEi*1e-9); //TODO units
	fprintf(file, "#fitted range:  [%g, %g ] nm \n", hzdata->range[0], hzdata->range[1]);
	fprintf(file, "#\n");

	fprintf(file, "#Results of Hertz analysis \n");
	fprintf(file, "#Er: %g GPa\n", hzdata->Er);
	fprintf(file, "#Eit: %g GPa\n", hzdata->Eit);
	fprintf(file, "#\n");

	fprintf(file, "#Results of the Gaussian propagation of uncertainties \n");
	fprintf(file, "#u(Er) =  %g GPa \n", hzunc->uEr);
	fprintf(file, "#u(Eit) =  %g GPa \n", hzunc->uEit);
	fprintf(file, "#\n");


	fprintf(file, "#Contributions to uncertainty of Er\n");
	fprintf(file, "#u(Er;uh) = %g GPa \n", hzunc->uEruh);
	fprintf(file, "#u(Er;uF) = %g GPa \n", hzunc->uEruF);
	fprintf(file, "#u(Er;uradius) = %g GPa \n", hzunc->uEruradius);
	fprintf(file, "#\n");
	fprintf(file, "#Contributions to uncertainty of Eit\n");
	fprintf(file, "#u(Eit;uh) = %g GPa \n", hzunc->uEituh);
	fprintf(file, "#u(Eit;uF) = %g GPa \n", hzunc->uEituF);
	fprintf(file, "#u(Eit;uradius) = %g GPa \n", hzunc->uEituradius);
	fprintf(file, "#u(Eit;unu) = %g GPa \n", hzunc->uEitunu);
	fprintf(file, "#u(Eit;unui) = %g GPa \n", hzunc->uEitunui);
	fprintf(file, "#u(Eit;uEi) = %g GPa \n", hzunc->uEituEi);


	fprintf(file, "#Changes of the Er corresponding to changes in the contact point \n");
	fprintf(file, "# shift of contact point with respect to the globally chosen contact point (negative - to the left, positive - to the right)   ");
	fprintf(file, "Er/GPa \n");
	for (j= -CONTACTMAX; j< CONTACTMAX+1; j++){
		if ( hzunc->Ec[j+CONTACTMAX] < 0 )
			fprintf(file, "# %d   not defined \n", j);
		else
		fprintf(file, "# %d   %g  \n", j, hzunc->Ec[j+CONTACTMAX]);
	}

	fclose(file); 
}



static void hertz_uncertainty_save_data( Args *args)
{
	GtkWidget *dialog;
	gchar *filename;

	dialog = gtk_file_chooser_dialog_new("Save File",
			NULL,
			GTK_FILE_CHOOSER_ACTION_SAVE,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_SAVE, GTK_RESPONSE_OK,
			NULL);

	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), gwy_app_get_current_directory());

	if ( gtk_dialog_run (GTK_DIALOG ( dialog)) == GTK_RESPONSE_OK)
	{
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		if (gwy_app_file_confirm_overwrite(dialog))
			hertz_uncertainty_write_file(filename, args);
		g_free(filename);
	}
	gtk_widget_destroy ( dialog);
}

static void
hertz_fit_shift_contact(Hertzdata *hzdata, FDdata *fddata, Instdata *instdata, gdouble *Ec, gint j)
{
	Hertzdata hz;
	FDdata fdc;
	gdouble dh;
	gint istart,iend, ndata;
	GwyDataLine *x, *y;

	Ec[j+CONTACTMAX] = -1;

	// create shifted FDdata
	
	if (DEBUG)
		printf("\n\nCreate shifted data for j %d \n", j);

	if (fddata->i_contact_load+j < 0)
		return;

	fdc.i_contact_load = fddata->i_contact_load+j;

	fdc.hall_orig = gwy_data_line_duplicate(fddata->hall_orig);
	fdc.Fall_orig = gwy_data_line_duplicate(fddata->Fall_orig);

	
	fdc.hload = gwy_data_line_part_extract(fdc.hall_orig, fdc.i_contact_load, fddata->hload->res-j);
	fdc.Fload = gwy_data_line_part_extract(fdc.Fall_orig, fdc.i_contact_load, fddata->Fload->res-j);
	gwy_data_line_add(fdc.hload, -fdc.hall_orig->data[fdc.i_contact_load]);
	gwy_data_line_add(fdc.Fload, -fdc.Fall_orig->data[fdc.i_contact_load]);

	//difference in shift between different contactpoints
	dh = fdc.hall_orig->data[fdc.i_contact_load]-fddata->hall_orig->data[fddata->i_contact_load];
	if (DEBUG)
		printf("shift dh %g  \n", dh);
	

	// create working copy of Hertzdat
	// shift integration range to the right only (we assume that we always start at h=0)
	hz = *hzdata;
	hz.range[0] += -dh;
	hz.range[1] += -dh;
	hz.from = hz.range[0];
	hz.to = hz.range[1];

	//check range
	if (hz.range[0] ==  hz.range[1]){
		return;
	}

	//find appropriate data, which should be fitted
	range_to_indices(hz.range[0], hz.range[1], fdc.hload, FALSE, &istart, &iend, &ndata);
	// if there are not enough data in the selected range, not fitting can be performed
	ndata = MIN(ndata, fdc.hload->res - istart);

	if (DEBUG)
		printf("interval has %d points, from %d to %d, hrange [%g, %g ] nm, Frange [%g, %g] mN \n", ndata, istart, iend,
			       fdc.hload->data[istart], fdc.hload->data[iend], fdc.Fload->data[istart], fdc.Fload->data[iend]);

	if (ndata < 2){
		return;
	}

	//get data to be fitted
	x = gwy_data_line_part_extract(fdc.hload, istart, ndata);
	y = gwy_data_line_part_extract(fdc.Fload, istart, ndata);

	hertz_fit(x, y, &hz, instdata);

	// clean up
	g_object_unref(x);
	g_object_unref(y);
	g_object_unref(fdc.hall_orig);
	g_object_unref(fdc.Fall_orig);
	g_object_unref(fdc.hload);
	g_object_unref(fdc.Fload);

	Ec[j+CONTACTMAX] = hz.Er;

	if (DEBUG)
		printf("\n\n");
}

void 
hertz_unc_close_window(Data *data)
{
	g_free(data->args->hertzunc.Ec);
	data->args->hertzunc.Ec = NULL;
	hertz_mc_close_window(data);
	gtk_widget_destroy(data->controls->hertzunccontrols->window);
}

void
hertz_uncertainty (Data *data)
{
	GtkWidget  *label, *spin;
	gint i,j;
	HertzUncControls *ctrls;
	HertzUncdata *unc;
	gchar str[50];


	if (data->args->hertzunc.Ec != NULL){
		gtk_window_present(GTK_WINDOW(data->controls->hertzunccontrols->window));
		return;
	}

	unc = &(data->args->hertzunc);
        ctrls =	data->controls->hertzunccontrols;

	ctrls->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->window), "Uncertainty in Hertz analysis");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->window), 400, 300); 
	g_signal_connect_swapped (ctrls->window, "delete_event", G_CALLBACK (hertz_unc_close_window), data);
	gtk_window_set_transient_for (GTK_WINDOW(ctrls->window), GTK_WINDOW(data->controls->maincontrols->window_main));

	/* Uncertainties due to uncertainties in input values */
	ctrls->frame_unc_input = gtk_frame_new(NULL);
	gtk_frame_set_label_widget (GTK_FRAME (ctrls->frame_unc_input),
				    GTK_WIDGET (gwy_label_new_header("Uncertainties due to uncertainties in input values")));


	ctrls->table_unc_input = gtk_table_new(6, 8, FALSE);

	gtk_container_set_border_width (GTK_CONTAINER (ctrls->table_unc_input), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (ctrls->table_unc_input), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (ctrls->table_unc_input), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add ( GTK_CONTAINER (ctrls->frame_unc_input), ctrls->table_unc_input);

	i=0;

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(E<sub>r</sub>)/GPa"), 3, 4, i, i+1, GTK_FILL, 0, 0, 0); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(E<sub>it</sub>)/GPa"), 4, 5, i, i+1, GTK_FILL, 0, 0, 0); 

	i++;

	/* u(h) */

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(h)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 

	ctrls->uh = gtk_adjustment_new(unc->uh, 0, 1e6, 0.01, 1,0); 
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->uh), 0.001, 3);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	g_signal_connect_swapped(ctrls->uh, "value-changed", G_CALLBACK(unc_depth_changed), data); 

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("nm"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 

	ctrls->uEruh = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEruh), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEruh, 3, 4, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

	ctrls->uEituh = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEituh), 0.0, 0.5); 

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEituh, 4, 5, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

	i++;

	/* u(F) */

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(F)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 

	ctrls->uF = gtk_adjustment_new(unc->uF, 0, 1e6, 0.0001, 0.01,0.); 
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->uF), 0.0001, 4);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	g_signal_connect_swapped(ctrls->uF, "value-changed", G_CALLBACK(unc_load_changed), data); 

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("mN"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);  

	ctrls->uEruF = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEruF), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEruF, 3, 4, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

	ctrls->uEituF = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEituF), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEituF, 4, 5, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

	i++;

	/* u(radius) */

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(radius)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 

	ctrls->uradius = gtk_adjustment_new(unc->uradius*1e9, 0, 1e6, 1, 10,0); 
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->uradius), 0.1, 1);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	g_signal_connect_swapped(ctrls->uradius, "value-changed", G_CALLBACK(unc_radius_changed), data); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("nm"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	
	ctrls->uEruradius = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEruradius), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEruradius, 3, 4, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

	ctrls->uEituradius = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEituradius), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEituradius, 4, 5, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

	i++;

	/* u(nu) */

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(ν)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 

	ctrls->unu = gtk_adjustment_new(unc->unu, 0., 0.5, 0.01, 0.1, 0.); 
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->unu), 0.01, 2); 
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	g_signal_connect_swapped(ctrls->unu, "value-changed", G_CALLBACK(unc_nu_changed), data); 

	ctrls->uEitunu = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEitunu), 0.0, 0.5); 

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEitunu, 4, 5, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

	i++;

	/* u(nu_i) */

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(ν<sub>i</sub>)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 

	ctrls->unui = gtk_adjustment_new(unc->unui, 0, 0.5, 0.01, 0.1, 0.); 
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->unui), 0.01, 2); 
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	g_signal_connect_swapped(ctrls->unui, "value-changed", G_CALLBACK(unc_nui_changed), data); 

	ctrls->uEitunui = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEitunui), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEitunui, 4, 5, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

	i++;

	/* u(E_i) */

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(E<sub>i</sub>)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 

	ctrls->uEi = gtk_adjustment_new(unc->uEi*1e-9, 0, 1e5, 1, 10, 0.); 
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->uEi), 1, 0); 
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	g_signal_connect_swapped(ctrls->uEi, "value-changed", G_CALLBACK(unc_Ei_changed), data); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("GPa"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 

	ctrls->uEituEi = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEituEi), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEituEi, 4, 5, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

	i++;

	/* total */
	
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("Total"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->uEr = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEr), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEr, 3, 4, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	ctrls->uEit = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEit), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEit, 4, 5, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);


	
	/*
	label = gtk_label_new(NULL);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), label, 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	i++;
	label = gtk_label_new(NULL);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), label, 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	i++;
	*/


	/* Uncertainties due to contact point */

	ctrls->frame_unc_contact = gtk_frame_new(NULL);
	gtk_frame_set_label_widget (GTK_FRAME (ctrls->frame_unc_contact),
				    GTK_WIDGET (gwy_label_new_header("Uncertainties due to choice of contact point")));

	ctrls->table_unc_contact = gtk_table_new(6, 3, FALSE);
	gtk_container_set_border_width (GTK_CONTAINER (ctrls->table_unc_contact), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (ctrls->table_unc_contact), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (ctrls->table_unc_contact), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add (GTK_CONTAINER (ctrls->frame_unc_contact), ctrls->table_unc_contact);

	/* table headers */

	i=0;

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact),
		       	gwy_label_new_header("shift contact point"), 0, 1, i,i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact), 
			gwy_label_new_header("E/GPa"), 1, 2, i,i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
//	gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact), gwy_label_new_header("weight"), 2, 3, i,i+1, GTK_FILL, 0, 0, 0);
	i++;


	// Ec is Er, it is not updated at changes in radius
	unc->Ec = (gdouble *)g_malloc((2*CONTACTMAX+1)*sizeof(gdouble));



	for (j = -CONTACTMAX; j<=CONTACTMAX ;j++){

	    /* vertical alignments: j: 1.0 (bottom), others: 0.0 (top); looks quite nice, still probably incorrect */
	    
		label = gtk_label_new(NULL);
		g_snprintf(str, sizeof(str),"%d\n", j); 
		gtk_label_set_text(GTK_LABEL(label), str);
		gtk_misc_set_alignment (GTK_MISC(label), 0.0, 1.0);
		gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact), label, 0, 1, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

		hertz_fit_shift_contact(&(data->args->hertzdata), &(data->args->fddata), &(data->args->instdata), unc->Ec, j);

		//fill in Er
		label = gtk_label_new(NULL);
		if (unc->Ec[j+CONTACTMAX] > 0)
			g_snprintf(str, sizeof(str),MODUL, unc->Ec[j+CONTACTMAX]); 
		else
			g_snprintf(str, sizeof(str), "not defined");
		gtk_label_set_text(GTK_LABEL(label), str);
		gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0); 
		gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

		/*
		label = gtk_label_new(NULL);
		g_snprintf(str, sizeof(str),"1/%d", 2*CONTACTMAX+1); 
		gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5); 
		gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact), label, 2, 3, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
		*/
		i++;

	}

	hertz_propagate_uncertainties (data);

	ctrls->button_save = gtk_button_new_from_stock (GTK_STOCK_SAVE);
	g_signal_connect_swapped (ctrls->button_save, "clicked", G_CALLBACK (hertz_uncertainty_save_data), data->args);

	/* Monte Carlo calculation */
	
	ctrls->frame_mc = gtk_frame_new(NULL);
	gtk_frame_set_label_widget (GTK_FRAME (ctrls->frame_mc),
				    GTK_WIDGET (gwy_label_new_header("Monte Carlo calculation of uncertainties")));

	ctrls->table_mc = gtk_table_new(6, 7, FALSE);

	gtk_container_set_border_width (GTK_CONTAINER (ctrls->table_mc), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (ctrls->table_mc), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (ctrls->table_mc), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add ( GTK_CONTAINER (ctrls->frame_mc), ctrls->table_mc);

	i = 0;
	gtk_table_attach(GTK_TABLE(ctrls->table_mc), gwy_label_new_header("Number of iterations"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->Nmc = gtk_adjustment_new(unc->Nmc, 1, 1e9, 10, 100,0); 
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->Nmc), 1, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(ctrls->table_mc), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	g_signal_connect_swapped(ctrls->Nmc, "value-changed", G_CALLBACK(mc_iterations_changed), data);
	
	i++;

	ctrls->mc_warn = gtk_label_new(NULL);
	gtk_table_attach(GTK_TABLE(ctrls->table_mc), ctrls->mc_warn, 0, 3, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	if (unc->Nmc > MC_WHEN_WARN){
		g_snprintf(str, sizeof(str), "Warning: Monte Carlo calculation for %d iterations may take very long!", unc->Nmc);
		gtk_label_set_text(GTK_LABEL(ctrls->mc_warn), str);
	}

	//no Monte Carlo window without the uncertainty dialog
	data->controls->hertzmccontrols->has_window = FALSE;
	ctrls->button_mc = gtk_button_new_with_mnemonic ("_Monte Carlo");
	gtk_button_set_image (GTK_BUTTON (ctrls->button_mc), gtk_image_new_from_stock (GWY_STOCK_POLYNOM, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (ctrls->button_mc, "clicked", G_CALLBACK (hertz_uncertainty_montecarlo), data);

	ctrls->vbox = gtk_vbox_new(FALSE, CTRLS_SPACING);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->frame_unc_input, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->frame_unc_contact, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->button_save,  FALSE, FALSE,0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->frame_mc, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->button_mc, FALSE, FALSE, 0);

	gtk_container_add(GTK_CONTAINER(ctrls->window), ctrls->vbox);
	gtk_widget_show_all(ctrls->window);
}
