#ifndef NOFORTRAN

#include <math.h>

#include <gtk/gtk.h>

#include <app/file.h>

#include "indent-toolbox-common.h"
#include "datatypes.h"
#include "controls.h"
#include "fit-utils.h"
#include "gui-utils.h"
#include "tool_op_odr_unc.h"
#include "tool_op_odr_mc.h"
#include "tool_op_odr.h"


#define CONTACTMAX 2
#define MC_WHEN_WARN 5e2
#define DEBUG TRUE


static void op_odr_fit_shift_contact(OPODRdata *opodrdata, FDdata *fddata, Instdata *instdata, Area *area, gdouble *Ec, gdouble *Hc, gint j);

void 
op_odr_propagate_uncertainties (Data *data)
{
	gchar str[30];
	OPODRUncControls *ctrls;
	OPODRUncdata *unc;
	OPODRdata *opodr;
	FDdata *fd;
	Instdata *inst;
	gdouble uh, uF;
	gdouble umh, umF, uhph, uhpF;
	gdouble uSh, uSF;
	gdouble z, depsdm;
	gdouble uepsh, uepsF;
	gdouble um, uhp, ueps, uS;
	gdouble beta[3], std_beta[3], covariance[3];
	gint istart, iend,ndata, info;
	GwyDataLine *x, *y;
	gdouble ux,uy;
	gdouble dAdh;
	gdouble m,S,eps, hc, hp, hmax,Fmax, Aphc,Hit,Er,Eit, nu,nui,Ei;
	gdouble covmhph, covmhpF, covmhp;
	gdouble covShch, covShcF, covShc;
	gchar fnm[500];

	unc = &(data->args->opodrunc);
	opodr = &(data->args->opodrdata);
	fd = &(data->args->fddata);
	inst = &(data->args->instdata);
	ctrls =	data->controls->opodrunccontrols;

	if ( !opodr->has_fit )
		return;

	if ( unc->Ec ==NULL || unc->Hc == NULL)
		return;

	//make copies to simplify code
	uh = unc->uh;
	uF = unc->uF;
	m = opodr->m;
	eps = opodr->eps;
	S = opodr->S;
	hp = opodr->hp;
	hc = opodr->hc;
	hmax = fd->hmax;
	Fmax = fd->Fmax;
	Aphc = opodr->Aphc;
	Hit = opodr->Hit;
	Eit = opodr->Eit;
	Er = opodr->Er;
	Ei = inst->Ei;
	nui = inst->nui;
	nu = inst->nu;

	if (DEBUG)
		printf(" \n\nPropagate uncertainties\n");


	/*   u(m)    */

	/* prepare odr fit */

	beta[0] = opodr->alpha;
	beta[1] = opodr->hp;
	beta[2] = opodr->m;

	//find appropriate data, which should be fitted, use same regime as for the main fit
	if (opodr->Finput)
		range_to_indices(opodr->range_pct_Fmax[0]*0.01*fd->Fmax, opodr->range_pct_Fmax[1]*0.01*fd->Fmax, fd->Funload, TRUE, &istart, &iend, &ndata);
	else
		range_to_indices(opodr->range[0], opodr->range[1], fd->hunload, TRUE, &istart, &iend, &ndata);


	x = gwy_data_line_part_extract(fd->hunload, istart, ndata);
	y = gwy_data_line_part_extract(fd->Funload, istart, ndata);

	/* fit with only uh */
	ux = uh;
	uy = 0;
	beta[0] = opodr->alpha;
	beta[1] = opodr->hp;
	beta[2] = opodr->m;
	info = odr_run_fit(x, y, beta, fnm, ux, uy, std_beta, covariance, FALSE); /* do not rescale */
	umh = std_beta[2];
	uhph = std_beta[1];
	covmhph = covariance[0]; 

	if (DEBUG)
		printf(" fit with only uh:\n  alpha %g m %g hp %g \n  u(m) %g u(hp) %g corr(m,hp) %g \n",
			       beta[0], beta[1], beta[2], umh, uhph, covmhph/umh/uhph);	       

	/* fit with only uF */
	ux = 0;
	uy = uF;
	info = odr_run_fit(x, y, beta, fnm, ux, uy, std_beta, covariance, FALSE); /* do not rescale */
	umF = std_beta[2];
	uhpF = std_beta[1];
	covmhpF = covariance[0]; 

	if (DEBUG)
		printf(" fit with only uF:\n  alpha %g m %g hp %g \n  u(m) %g u(hp) %g corr(m,hp) %g \n",
			       beta[0], beta[1], beta[2], umF, uhpF, covmhpF/umF/uhpF);	       

	unc->umuh = umh;
	unc->umuF = umF;

	/*   u(eps)    */
	z = m/(m-1)/2;
	depsdm = eps/m - m/(m -1) * (eps/m -1) -m/2/(m-1)/(m-1)*(eps/m -1)* (psi(z)-psi(z-1./2));
	uepsh = depsdm *umh;
	uepsF = depsdm *umF;
	if (DEBUG){
		printf("depsdm %g  z %g  \n", depsdm,z);
		printf(" u(eps;h) %g u(eps;F) %g  urel(eps;h) %g urel(eps;F) %g \n", uepsh, uepsF, uepsh/eps, uepsF/eps);
	}

	/*    u(S)   */
	uSh = sqrt(umh*umh/m/m +
			uh *uh/(hmax-hp)/(hmax-hp) +
			uhph*uhph/(hmax-hp)/(hmax-hp) +
			2*covmhph/m/(hmax-hp)
		  ) * S; 

	uSF = sqrt(umF*umF/m/m +    
		       	uF*uF/Fmax/Fmax  +    
			uhpF*uhpF/(hmax-hp)/(hmax-hp)+
			2*covmhpF/m/(hmax-hp)
		  ) * S; 
	unc->uSuh = uSh;
	unc->uSuF = uSF;
	/* u(S;h) = S *sqrt [ (umh/m)^2 + (uh/(hmax-hp))^2 + (uhph/(hmax-hp))^2  + 2 cov(m,hp;h)/(m*(hmax-hp))]
	 * u(S;F) = S *sqrt [ (umF/m)^2 + (uFmax/Fmax)^2   + (uhpF/(hmax-hp))^2  + 2 cov(m,hp;F)/(m*(hmax-hp))]
	 */

	if (DEBUG){
		printf(" u(S;h) %g u(S;F) %g  urel(S;h) %g urel(S;F) %g \n", uSh, uSF, uSh/S, uSF/S);
	}

	/*   u(hc)   */
	unc->uhcuh = sqrt((hmax-hc)*(hmax-hc)* depsdm*depsdm *umh*umh + 
			(1-eps)*(1-eps)*uh*uh + 
			eps*eps*uhph*uhph+
			2*(hmax-hc)*eps*depsdm*covmhph);
	unc->uhcuF = sqrt((hmax-hc)*(hmax-hc)* depsdm*depsdm *umF*umF + 
			eps*eps*uhpF*uhpF+
			2*(hmax-hc)*eps*depsdm*covmhpF);
	/* u(hc;h)^2 = (hmax-hc)^2 *(deps/dm)^2 * umh^2 + (1-eps)^2 uh^2 + eps^2 uhph^2 +2( hmax-hc)eps (deps/dm) cov(m,hp;h)
	 * u(hc;F)^2 = (hmax-hc)^2 *(deps/dm)^2 * umF^2 +                + eps^2 uhpF^2 +2( hmax-hc)eps (deps/dm) cov(m,hp;F)
	*/

	if (DEBUG)
		printf(" u(hc;h) %g u(hc;F) %g  urel(hc;h) %g urel(hc;F) %g \n", unc->uhcuh, unc->uhcuF, unc->uhcuh/hc, unc->uhcuF/hc);

	/*    u(A)   */
	dAdh =  eval_area_derivative(&(data->args->area), hc) ;
	unc->uAuh = dAdh * unc->uhcuh;
	unc->uAuF = dAdh * unc->uhcuF;
	if (DEBUG)
		printf(" u(A;h) %g u(A;F) %g  urel(A;h) %g urel(A;F) %g \n", unc->uAuh, unc->uAuF, unc->uAuh/Aphc, unc->uAuF/Aphc);

	/*     u(H_IT)   */
	unc->uHituh = Hit/Aphc*dAdh*unc->uhcuh;
	unc->uHituF = Hit *sqrt(uF*uF/Fmax/Fmax + dAdh*dAdh/Aphc/Aphc*unc->uhcuh*unc->uhcuh);
	if (DEBUG)
		printf(" u(Hit;h) %g u(Hit;F) %g  urel(Hit;h) %g urel(Hit;F) %g \n", unc->uHituh, unc->uHituF, unc->uHituh/Hit, unc->uHituF/Hit);


	/*     u(E_r)   */

	covShch = - S *(1-eps)/(hmax-hp)*uh*uh  - S*(hmax - hp)/m *depsdm* umh*umh +
		S*eps/(hmax-hp)*uhph*uhph + S*(eps/m-depsdm)*covmhph;
	covShcF =  - S*(hmax - hp)/m *depsdm* umF*umF +
		S*eps/(hmax-hp)*uhpF*uhpF + S*(eps/m-depsdm)*covmhpF;
	/*cov(S,hc;h) = -S(1-eps)/(hmax-hp)*uh^2 -S(hmax-hp)/m *(deps/dm) *u(m;h)^2 + S*eps/(hmax-hp)*u(hp;h)^2 +S (eps/m-(deps/dm))cov(m,hp;h)
	 *cov(S,hc;F) =                          -S(hmax-hp)/m *(deps/dm) *u(m;F)^2 + S*eps/(hmax-hp)*u(hp;F)^2 +S (eps/m-(deps/dm))cov(m,hp;F)
	 */

	unc->uEruh = Er*sqrt( uSh*uSh/S/S + unc->uAuh*unc->uAuh/Aphc/Aphc/4 - 1/S/Aphc*dAdh*covShch); 
	unc->uEruF = Er*sqrt( uSF*uSF/S/S + unc->uAuF*unc->uAuF/Aphc/Aphc/4 - 1/S/Aphc*dAdh*covShcF); 
	/* u(Er;h) = sqrt[ (Er/S)^2 u(S;h)^2 + (Er/2A)^2 u(A;h)^2  - Er^2/(S Aphc)* (dA/dh) cov(S,hc;h)]
	 * u(Er;F) = sqrt[ (Er/S)^2 u(S;F)^2 + (Er/2A)^2 u(A;F)^2  - Er^2/(S Aphc)* (dA/dh) cov(S,hc;F)]
	 */
	if (DEBUG)
		printf(" u(Er;h) %g u(Er;F) %g  urel(Er;h) %g urel(Er;F) %g \n", unc->uEruh, unc->uEruF, unc->uEruh/Er, unc->uEruF/Er);


	/*     u(E_IT)   */
	unc->uEituh = Eit*Eit/Er/Er/(1-nu*nu)*unc->uEruh; // Eit^2/Er^2 /(1-nu^2) u(Er;h)
	unc->uEituF = Eit*Eit/Er/Er/(1-nu*nu)*unc->uEruF; // Eit^2/Er^2 /(1-nu^2) u(Er;F)


	unc->uEitunu =  fabs(2*nu/(1-nu*nu)*Eit*unc->unu); // -2nu/(1-nu^2) Eit u(nu)
	unc->uEitunui = fabs(2*nui/(1-nu*nu)* Eit*Eit /Ei *unc->unui)*1e9; // -2nui/(1-nu^2) Eit^2/Ei u(nui)
	unc->uEituEi =  fabs((1-nui*nui)/(1-nu*nu) *Eit*Eit/ Ei*Ei *unc->uEi)*1e-18; //-(1-nui^2)/(1-nu)^2 Eit^2/Ei^2 u(Ei)

	if (DEBUG){
		printf("u(Eit;h) %g  u(Eit;F) %g  urel(Eit;h) %g  urel(Eit;F) %g\n", unc->uEituh, unc->uEituF, unc->uEituh/Eit, unc->uEituF/Eit);
		printf("u(Eit;nu) %g  u(Eit;nui) %g  u(Eit;Ei) %g  urel(Eit;nu) %g  urel(Eit;nui) %g  urel(Eit;Ei) %g\n", unc->uEitunu, unc->uEitunui, unc->uEituEi, unc->uEitunu/Eit, unc->uEitunui/Eit, unc->uEituEi/Eit);
	}



	/* show */
	g_snprintf(str, sizeof(str), NUMBER, unc->umuh);
	gtk_label_set_text(GTK_LABEL(ctrls->umuh), str);
	g_snprintf(str, sizeof(str), NUMBER, unc->umuF);
	gtk_label_set_text(GTK_LABEL(ctrls->umuF), str);

	g_snprintf(str, sizeof(str), SLOPE, unc->uSuh);
	gtk_label_set_text(GTK_LABEL(ctrls->uSuh), str);
	g_snprintf(str, sizeof(str), SLOPE, unc->uSuF);
	gtk_label_set_text(GTK_LABEL(ctrls->uSuF), str);

	g_snprintf(str, sizeof(str), DEPTH, unc->uhcuh);
	gtk_label_set_text(GTK_LABEL(ctrls->uhcuh), str);
	g_snprintf(str, sizeof(str), DEPTH, unc->uhcuF);
	gtk_label_set_text(GTK_LABEL(ctrls->uhcuF), str);

	g_snprintf(str, sizeof(str), AREA, unc->uAuh);
	gtk_label_set_text(GTK_LABEL(ctrls->uAuh), str);
	g_snprintf(str, sizeof(str), AREA, unc->uAuF);
	gtk_label_set_text(GTK_LABEL(ctrls->uAuF), str);

	g_snprintf(str, sizeof(str), HARD, unc->uHituh);
	gtk_label_set_text(GTK_LABEL(ctrls->uHituh), str);
	g_snprintf(str, sizeof(str), HARD, unc->uHituF);
	gtk_label_set_text(GTK_LABEL(ctrls->uHituF), str);

	g_snprintf(str, sizeof(str), MODUL, unc->uEruh);
	gtk_label_set_text(GTK_LABEL(ctrls->uEruh), str);
	g_snprintf(str, sizeof(str), MODUL, unc->uEruF);
	gtk_label_set_text(GTK_LABEL(ctrls->uEruF), str);
	g_snprintf(str, sizeof(str), MODUL, unc->uEituh);
	gtk_label_set_text(GTK_LABEL(ctrls->uEituh), str);
	g_snprintf(str, sizeof(str), MODUL, unc->uEituF);
	gtk_label_set_text(GTK_LABEL(ctrls->uEituF), str);


	g_snprintf(str, sizeof(str), MODUL, unc->uEitunu);
	gtk_label_set_text(GTK_LABEL(ctrls->uEitunu), str);
	g_snprintf(str, sizeof(str), MODUL, unc->uEitunui);
	gtk_label_set_text(GTK_LABEL(ctrls->uEitunui), str);
	g_snprintf(str, sizeof(str), "%.4g", unc->uEituEi);
	gtk_label_set_text(GTK_LABEL(ctrls->uEituEi), str);


	// total uncertainties
	// should take um as given by odr_wrap, otherwise doublecounting
	/* fit with both uh and uF */
	ux = uh;
	uy = uF;
	info = odr_run_fit(x, y, beta, fnm, ux, uy, std_beta, covariance, FALSE); /* do not rescale */
	um = std_beta[2];
	uhp = std_beta[1];
	covmhp = covariance[0]; 

	unc->um = um;
	if (DEBUG)
		printf(" fit with both uh and uF:\n  alpha %g m %g hp %g \n  u(m) %g u(hp) %g corr(m,hp) %g \n",
			       beta[0], beta[1], beta[2], um, uhp, covmhp/um/uhp);	       

	/*   u(eps)    */
	z = m/(m-1)/2;
	depsdm = eps/m - m/(m -1) * (eps/m -1) -m/2/(m-1)/(m-1)*(eps/m -1)* (psi(z)-psi(z-1./2));
	ueps = depsdm *um;
	if (DEBUG){
		printf("depsdm %g  z %g  \n", depsdm,z);
		printf(" u(eps) %g    urel(eps) %g  \n", ueps, ueps/eps);
	}


	/*    u(S)   */
	uS = sqrt(um*um/m/m  + uh *uh/(hmax-hp)/(hmax-hp) +  uF*uF/Fmax/Fmax +
			uhp*uhp/(hmax-hp)/(hmax-hp) + 2*covmhp/m/(hmax-hp)) * S; 
	unc->uS = uS;
	/* u(S) = S *sqrt [ (umh/m)^2 +  (uhmax/(hmax-hp))^2 + (uFmax/Fmax)^2 +  (uhp/(hmax-hp))^2 + 2 cov(m,hp)/(m*(hmax-hp)) ]
	 */

	if (DEBUG){
		printf(" u(S) %g    urel(S) %g \n", uS , uS/S);
	}

	/*   u(hc)   */
	unc->uhc = sqrt((hmax-hc)*(hmax-hc)* depsdm*depsdm *um*um + 
			(1-eps)*(1-eps)*uh*uh + 
			eps*eps*uhp*uhp+
			2*(hmax-hc)*eps*depsdm*covmhp);
	/* u(hc;h)^2 = (hmax-hc)^2 *(deps/dm)^2 * um^2 + (1-eps)^2 uh^2 + eps^2 uhp^2 +2( hmax-hc)eps (deps/dm) cov(m,hp)
	 */

	if (DEBUG)
		printf(" u(hc) %g    urel(hc) %g \n", unc->uhc, unc->uhc/hc);


	/*    u(A)   */
	dAdh =  eval_area_derivative(&(data->args->area), hc) ;
	unc->uA = dAdh * unc->uhc;
	unc->uA = dAdh * unc->uhc;
	if (DEBUG)
		printf(" u(A) %g    urel(A) %g  \n", unc->uA, unc->uA/Aphc);

	/*     u(H_IT)   */
	unc->uHit = Hit*sqrt(uF*uF/Fmax/Fmax + unc->uA*unc->uA/opodr->Aphc/opodr->Aphc); 
	if (DEBUG)
		printf(" u(Hit;h) %g u(Hit;F) %g  urel(Hit;h) %g urel(Hit;F) %g \n", unc->uHituh, unc->uHituF, unc->uHituh/Hit, unc->uHituF/Hit);

	/*     u(E_r)   */

	covShc = - S *(1-eps)/(hmax-hp)*uh*uh  - S*(hmax - hp)/m *depsdm* um*um +
		S*eps/(hmax-hp)*uhp*uhp + S*(eps/m-depsdm)*covmhp;

	/*cov(S,hc;h) = -S(1-eps)/(hmax-hp)*uh^2 -S(hmax-hp)/m *(deps/dm) *u(m)^2 + S*eps/(hmax-hp)*u(hp)^2 +S (eps/m-(deps/dm))cov(m,hp)
	*/
	
	unc->uEr = Er*sqrt( uS*uS/S/S +unc->uA*unc->uA/Aphc/Aphc/4- 1/S/Aphc*dAdh*covShc); 
	/* u(Er) = sqrt[ (Er/S)^2 u(S)^2 + (Er/2A)^2 u(A)^2  - Er^2/(S Aphc)* (dA/dh) cov(S,hc)]
	 */
	if (DEBUG)
		printf(" u(Er) %g   urel(Er) %g  \n", unc->uEr, unc->uEr/Er);


	/*     u(E_IT)   */
	//contribution from h,F
	unc->uEit = Eit*Eit/Er/Er/(1-nu*nu)*unc->uEr; // Eit^2/Er^2 /(1-nu^2) uEr

	//contribution from nu, nui, Ei
	unc->uEitunu =  fabs(2*nu/(1-nu*nu)*Eit*unc->unu); // -2nu/(1-nu^2) Eit u(nu)
	unc->uEitunui = fabs(2*nui/(1-nu*nu)* Eit*Eit /Ei *unc->unui)*1e9; // -2nui/(1-nu^2) Eit^2/Ei u(nui)
	unc->uEituEi =  fabs((1-nui*nui)/(1-nu*nu) *Eit*Eit/Ei*Ei *unc->uEi)*1e-18; //-(1-nui^2)/(1-nu)^2 Eit^2/Ei^2 u(Ei)

	if (DEBUG){
		printf("u(Eit;h,F) %g    urel(Eit;hF) %g  \n", unc->uEit, unc->uEit/Eit);
		printf("u(Eit;nu) %g  u(Eit;nui) %g  u(Eit;Ei) %g  urel(Eit;nu) %g  urel(Eit;nui) %g  urel(Eit;Ei) %g\n", unc->uEitunu, unc->uEitunui, unc->uEituEi, unc->uEitunu/Eit, unc->uEitunui/Eit, unc->uEituEi/Eit);
	}

	unc->uEit = sqrt( unc->uEit*unc->uEit  +unc->uEitunu*unc->uEitunu +unc->uEitunui*unc->uEitunui +unc->uEituEi*unc->uEituEi);
	if (DEBUG)
		printf("u(Eit;hFnuetc) %g    urel(Eit; hF_nu_etc) %g \n", unc->uEit, unc->uEit/Eit);

	if (DEBUG)
		printf(" \nEnd propagation of uncertainties\n\n");


	g_snprintf(str, sizeof(str), NUMBER,  unc->um);
	gtk_label_set_text(GTK_LABEL(ctrls->um), str);

	g_snprintf(str, sizeof(str), SLOPE,  unc->uS);
	gtk_label_set_text(GTK_LABEL(ctrls->uS), str);

	g_snprintf(str, sizeof(str), DEPTH,  unc->uhc);
	gtk_label_set_text(GTK_LABEL(ctrls->uhc), str);

	g_snprintf(str, sizeof(str), AREA, unc->uA);
	gtk_label_set_text(GTK_LABEL(ctrls->uA), str);

	g_snprintf(str, sizeof(str), HARD,  unc->uHit);
	gtk_label_set_text(GTK_LABEL(ctrls->uHit), str);

	g_snprintf(str, sizeof(str), MODUL, unc->uEr);
	gtk_label_set_text(GTK_LABEL(ctrls->uEr), str);

	g_snprintf(str, sizeof(str), MODUL, unc->uEit);
	gtk_label_set_text(GTK_LABEL(ctrls->uEit), str);


	g_object_unref(x);
	g_object_unref(y);

	//table_contact doesn't change with any of the adjustables nu, nui or Ei, uh, uF, unu, uniu, uEi
	//no need to update
}

static gboolean 
mc_iterations_changed(Data *data)
{
	gchar str[500];

	data->args->opodrunc.Nmc = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->opodrunccontrols->Nmc));
	if (data->args->opodrunc.Nmc > MC_WHEN_WARN){
		g_snprintf(str, sizeof(str), "Warning: Monte Carlo calculation for %d iterations may take very long!", data->args->opodrunc.Nmc);
		gtk_label_set_text(GTK_LABEL(data->controls->opodrunccontrols->mc_warn), str);
	}
	else{
		gtk_label_set_text(GTK_LABEL(data->controls->opodrunccontrols->mc_warn), NULL);
	}

	return FALSE;
}

static void 
unc_depth_changed (Data *data)
{
	data->args->opodrunc.uh = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->opodrunccontrols->uh));
	op_odr_propagate_uncertainties (data);
}

static void 
unc_load_changed (Data *data)
{
	data->args->opodrunc.uF = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->opodrunccontrols->uF));
	op_odr_propagate_uncertainties (data);
}

static void 
unc_nu_changed (Data *data)
{
	data->args->opodrunc.unu = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->opodrunccontrols->unu));
	op_odr_propagate_uncertainties (data);
}

static void 
unc_nui_changed (Data *data)
{
	data->args->opodrunc.unui = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->opodrunccontrols->unui));
	op_odr_propagate_uncertainties (data);
}

static void 
unc_Ei_changed (Data *data)
{
	data->args->opodrunc.uEi = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->opodrunccontrols->uEi))*1e9;
	op_odr_propagate_uncertainties (data);
}



void op_odr_uncertainty_write_file(gchar *filename, Args *args) 
{
	FILE *file;
	OPODRdata *opodrdata;
	OPODRUncdata *opodrunc;
	Instdata *instdata;
	gint j;

	opodrdata = &(args->opodrdata);
	opodrunc = &(args->opodrunc);
	instdata = &(args->instdata);

	if (args->fddata.hall->res == 0 || args->fddata.Fall->res==0) {
	    show_error("No data.");
	    return;
	}

	file = fopen(filename, "w");
	if (!file) {
	    show_error("Could not open file.");
	    return;
	}

	fprintf(file, "# %s \n", args->fddata.filename);
	fprintf(file, "#Parameters of Oliver Pharr analysis (ODR fit) \n");
	fprintf(file, "# u(depth): %g nm\n", opodrunc->uh);
	fprintf(file, "# u(load): %g nm\n", opodrunc->uF);
	fprintf(file, "#nu: %g                 u(nu):  %g  \n", instdata->nu, opodrunc->unu);
	fprintf(file, "#nu_indenter: %g       u(nu_indenter):  %g   \n", instdata->nui, opodrunc->unui);
	fprintf(file, "#E_indenter: %g        u(E_indenter):   %g GPa\n", instdata->Ei * 1e-9, opodrunc->uEi*1e-9); //TODO units
	fprintf(file, "#Fitted range: ");
	if (args->opodrdata.Finput)
		fprintf(file, "	[%g , %g ] %% Fmax \n", opodrdata->range_pct_Fmax[0], opodrdata->range_pct_Fmax[1]);
	else
		fprintf(file, "[%g , %g ] nm \n",opodrdata->range[0], opodrdata->range[1]);	
	fprintf(file, "# corresponds : ");
	if (args->opodrdata.Finput)
		fprintf(file, "[%g , %g ] nm \n",opodrdata->range[0], opodrdata->range[1]);	
	else
		fprintf(file, "	[%g , %g ] %% Fmax \n", opodrdata->range_pct_Fmax[0], opodrdata->range_pct_Fmax[1]);
	fprintf(file, "# %d datapoints used.\n", opodrdata->nfitdata);
	fprintf(file, "#\n");

	fprintf(file, "#Results of Oliver Pharr analysis (ODR fit) \n");
	fprintf(file, "#hc: %g nm\n", opodrdata->hc);
	fprintf(file, "#Ap(hc): %g nm^2\n", opodrdata->Aphc);
	fprintf(file, "#Er: %g GPa\n", opodrdata->Er);
	fprintf(file, "#Hit: %g GPa\n", opodrdata->Hit);
	fprintf(file, "#Eit: %g GPa\n", opodrdata->Eit);
	fprintf(file, "#\n");
	fprintf(file, "#\n");

	fprintf(file, "#Results of the Gaussian propagation of uncertainties \n");
	fprintf(file, "#u(m) =  %g  \n", opodrunc->um);
	fprintf(file, "#u(S) =  %g mN/nm \n", opodrunc->uS);
	fprintf(file, "#u(hc) =  %g nm \n", opodrunc->uhc);
	fprintf(file, "#u(Ap(hc)) =  %g nm^2 \n", opodrunc->uA);
	fprintf(file, "#u(Hit) =  %g MPa \n", opodrunc->uHit);
	fprintf(file, "#u(Er) =  %g GPa \n", opodrunc->uEr);
	fprintf(file, "#u(Eit) =  %g GPa \n", opodrunc->uEit);
	fprintf(file, "#\n");


	fprintf(file, "#Contributions to uncertainty of m\n");
	fprintf(file, "#u(m;uh) = %g \n", opodrunc->umuh);
	fprintf(file, "#u(m;uF) = %g \n", opodrunc->umuF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of S\n");
	fprintf(file, "#u(S;uh) = %g mN/nm \n", opodrunc->uSuh);
	fprintf(file, "#u(S;uF) = %g mN/nm \n", opodrunc->uSuF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of hc\n");
	fprintf(file, "#u(hc;uh) = %g nm \n", opodrunc->uhcuh);
	fprintf(file, "#u(hc;uF) = %g nm \n", opodrunc->uhcuF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Ap(hc)\n");
	fprintf(file, "#u(Ap(hc);uh) = %g nm^2 \n", opodrunc->uAuh);
	fprintf(file, "#u(Ap(hc);uF) = %g nm^2 \n", opodrunc->uAuF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Hit\n");
	fprintf(file, "#u(Hit;uh) = %g MPa \n", opodrunc->uHituh);
	fprintf(file, "#u(Hit;uF) = %g MPa \n", opodrunc->uHituF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Er\n");
	fprintf(file, "#u(Er;uh) = %g GPa \n", opodrunc->uEruh);
	fprintf(file, "#u(Er;uF) = %g GPa \n", opodrunc->uEruF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Eit\n");
	fprintf(file, "#u(Eit;uh) = %g GPa \n", opodrunc->uEituh);
	fprintf(file, "#u(Eit;uF) = %g GPa \n", opodrunc->uEituF);
	fprintf(file, "#u(Eit;unu) = %g GPa \n", opodrunc->uEitunu);
	fprintf(file, "#u(Eit;unui) = %g GPa \n", opodrunc->uEitunui);
	fprintf(file, "#u(Eit;uEi) = %g GPa \n", opodrunc->uEituEi);
	fprintf(file, "#\n");
	fprintf(file, "#\n");
	fprintf(file, "#\n");


	fprintf(file, "#Changes of the Er corresponding to changes in the contact point \n");
	fprintf(file, "# shift of contact point with respect to the globally chosen contact point (negative - to the left, positive - to the right)   \n");
	fprintf(file, "#Er/GPa \n");
	for (j= -CONTACTMAX; j< CONTACTMAX+1; j++){
		if ( opodrunc->Ec[j+CONTACTMAX] < 0 )
			fprintf(file, "# %d   not defined \n", j);
		else
			fprintf(file, "# %d   %g  \n", j, opodrunc->Ec[j+CONTACTMAX]);
	}
	fprintf(file, "#\n");
	fprintf(file, "#Changes of the H corresponding to changes in the contact point \n");
	fprintf(file, "# shift of contact point with respect to the globally chosen contact point (negative - to the left, positive - to the right)   \n");
	fprintf(file, "#H/MPa \n");
	for (j= -CONTACTMAX; j< CONTACTMAX+1; j++){
		if ( opodrunc->Hc[j+CONTACTMAX] < 0 )
			fprintf(file, "# %d   not defined \n", j);
		else
			fprintf(file, "# %d   %g  \n", j, opodrunc->Hc[j+CONTACTMAX]);
	}

	fclose(file); 
}





static void op_odr_uncertainty_save_data( Args *args)
{
	GtkWidget *dialog;
	gchar *filename;

	dialog = gtk_file_chooser_dialog_new("Save File",
			NULL,
			GTK_FILE_CHOOSER_ACTION_SAVE,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_SAVE, GTK_RESPONSE_OK,
			NULL);

	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), gwy_app_get_current_directory());

	if ( gtk_dialog_run (GTK_DIALOG ( dialog)) == GTK_RESPONSE_OK)
	{
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		if (gwy_app_file_confirm_overwrite(dialog))
			op_odr_uncertainty_write_file(filename, args);
		g_free(filename);
	}
	gtk_widget_destroy ( dialog);
}

static void
op_odr_fit_shift_contact(OPODRdata *opodrdata, FDdata *fddata, Instdata *instdata, Area *area,gdouble *Ec,gdouble *Hc,gint j)
{
	OPODRdata opodr;
	FDdata fdc;
	gdouble dh,dF;
	gint istart,iend, ndata;
	GwyDataLine *x, *y;

	Ec[j+CONTACTMAX] = -1;
	Hc[j+CONTACTMAX] = -1;

	// create shifted FDdata

	if (fddata->i_contact_load+j < 0)
		return;
	fdc.i_contact_load = fddata->i_contact_load+j;

	fdc.hall_orig = gwy_data_line_duplicate(fddata->hall_orig);
	fdc.Fall_orig = gwy_data_line_duplicate(fddata->Fall_orig);

	fdc.hunload = gwy_data_line_duplicate(fddata->hunload);
	fdc.Funload = gwy_data_line_duplicate(fddata->Funload);

	/*
	   gwy_data_line_add(fdc.hunload, -fdc.hall_orig->data[fdc.i_contact_load]);
	   gwy_data_line_add(fdc.Funload, -fdc.Fall_orig->data[fdc.i_contact_load]);

	   fdc.hmax = fddata.hmax- fdc.hall_orig->data[fdc.i_contact_load];
	   fdc.Fmax = fddata.Fmax- fdc.Fall_orig->data[fdc.i_contact_load];
	   */

	//difference in shift between different contactpoints
	dh = fdc.hall_orig->data[fdc.i_contact_load]-fddata->hall_orig->data[fddata->i_contact_load];
	dF = fdc.Fall_orig->data[fdc.i_contact_load]-fddata->Fall_orig->data[fddata->i_contact_load];

	gwy_data_line_add(fdc.hunload, -dh);
	gwy_data_line_add(fdc.Funload, -dF);


	fdc.hmax = fddata->hmax -dh;
	fdc.Fmax = fddata->Fmax -dF;


	// create working copy of OPODRdata 
	// shift fitting range
	/* opodr.nu = opodrdata->nu; */
	/* opodr.nui = opodrdata->nui; */
	/* opodr.Ei = opodrdata->Ei; */
	opodr.eps = opodrdata->eps;
	opodr.range[0]= opodrdata->range[0]-dh;
	opodr.range[1]= opodrdata->range[1]-dh;
	opodr.from = opodr.range[0];
	opodr.to = opodr.range[1];
	opodr.from_pct_Fmax = opodrdata->from_pct_Fmax;
	opodr.to_pct_Fmax = opodrdata->to_pct_Fmax;
	opodr.has_fit = FALSE;
	opodr.xfit = NULL;
	opodr.yfit = NULL;


	//check range
	if (opodr.from ==  opodr.to){
		return;
	}

	//find appropriate data, which should be fitted, use same regime as for the main fit
	if (opodrdata->Finput)
		range_to_indices(opodr.from_pct_Fmax*0.01*fdc.Fmax, opodr.to_pct_Fmax*0.01*fdc.Fmax, fdc.Funload, TRUE, &istart, &iend, &ndata);
	else
		range_to_indices(opodr.from, opodr.to, fdc.hunload, TRUE, &istart, &iend, &ndata);
	// if there are not enough data in the selected range, not fitting can be performed
	ndata = MIN(ndata, fdc.hunload->res - istart);

	if (ndata < 2){
		return;
	}

	//get data to be fitted
	x = gwy_data_line_part_extract(fdc.hunload, istart, ndata);
	y = gwy_data_line_part_extract(fdc.Funload, istart, ndata);


	// perform fit and get results
	op_odr_fit(x,y,&opodr, &fdc, instdata, area, NULL);

	// clean up
	g_object_unref(x);
	g_object_unref(y);
	g_object_unref(fdc.hall_orig);
	g_object_unref(fdc.Fall_orig);
	g_object_unref(fdc.hunload);
	g_object_unref(fdc.Funload);

	Ec[j+CONTACTMAX] = opodr.Er;
	Hc[j+CONTACTMAX] = opodr.Hit;

}

void 
op_odr_unc_close_window(Data *data)
{
	g_free(data->args->opodrunc.Ec);
	g_free(data->args->opodrunc.Hc);
	data->args->opodrunc.Ec = NULL;
	data->args->opodrunc.Hc = NULL;
	op_odr_mc_close_window(data);
	gtk_widget_destroy(data->controls->opodrunccontrols->window);
}

void
op_odr_uncertainty (Data *data)
{
	GtkWidget *label, *spin;
	gint i, j;
	OPODRUncControls *ctrls;
	OPODRUncdata *unc;
	gchar str[500];

	if ((data->args->opodrunc.Ec != NULL) & (data->args->opodrunc.Hc != NULL )) {
		gtk_window_present (GTK_WINDOW (data->controls->opodrunccontrols->window));
		return;
	}

	unc = &(data->args->opodrunc);
        ctrls =	data->controls->opodrunccontrols;


	ctrls->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->window), "Uncertainty in Oliver Pharr analysis (ODR fit)");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->window), 400, 300); 
	g_signal_connect_swapped (ctrls->window, "delete_event", G_CALLBACK (op_odr_unc_close_window), data);
	gtk_window_set_transient_for (GTK_WINDOW(ctrls->window), GTK_WINDOW(data->controls->maincontrols->window_main));

	/* Uncertainties due to uncertainties in input values */
	
	ctrls->frame_unc_input = gtk_frame_new(NULL);
//	gtk_frame_set_label_widget (GTK_FRAME (ctrls->frame_unc_input),
//				    GTK_WIDGET (gwy_label_new_header("Uncertainties due to uncertainties in input values")));
	gtk_frame_set_label_widget (GTK_FRAME (ctrls->frame_unc_input),
				    GTK_WIDGET (gwy_label_new_header("Uncertainties in input values")));

	ctrls->table_unc_input = gtk_table_new(6, 11, FALSE);

	gtk_container_set_border_width (GTK_CONTAINER (ctrls->table_unc_input), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (ctrls->table_unc_input), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (ctrls->table_unc_input), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add ( GTK_CONTAINER (ctrls->frame_unc_input), ctrls->table_unc_input);

	i = 0;
	
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input),
			 gwy_label_new_header("u(m)"), 3, 4, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input),
			 gwy_label_new_header("u(S)/(mN/nm)"), 4, 5, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input),
			 gwy_label_new_header("u(h<sub>c</sub>)/nm"), 5, 6, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input),
			 gwy_label_new_header("u(A<sub>p</sub>)/nm<sup>2</sup>"), 6, 7, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input),
			 gwy_label_new_header("u(H<sub>it</sub>)/MPa"), 7, 8, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input),
			 gwy_label_new_header("u(E<sub>r</sub>)/GPa"), 8, 9, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input),
			 gwy_label_new_header("u(E<sub>it</sub>)/GPa"), 9, 10, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

	i++;

	/* u(h) */

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(h)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->uh = gtk_adjustment_new(unc->uh, 0, 1e6, 0.01, 1,0); 
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->uh), 0.001, 3);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	g_signal_connect_swapped(ctrls->uh, "value-changed", G_CALLBACK(unc_depth_changed), data);
	
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), label_new_with_markup_left("nm"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);

	ctrls->umuh = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->umuh), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->umuh, 3, 4, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->uSuh = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uSuh), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uSuh, 4, 5, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->uhcuh = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uhcuh), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uhcuh, 5, 6, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->uAuh = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uAuh), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uAuh, 6, 7, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->uHituh = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uHituh), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uHituh, 7, 8, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->uEruh = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEruh), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEruh, 8, 9, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->uEituh = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEituh), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEituh, 9, 10, i, i+1, GTK_FILL, 0, 0, 0);
	
	i++;

	/* u(F) */

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(F)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->uF = gtk_adjustment_new(unc->uF, 0, 1e6, 0.0001, 0.01,0.);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->uF), 0.0001, 4);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	g_signal_connect_swapped(ctrls->uF, "value-changed", G_CALLBACK(unc_load_changed), data);
	
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), label_new_with_markup_left("mN"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);

	ctrls->umuF = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->umuF), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->umuF, 3, 4, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

	ctrls->uSuF = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uSuF), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uSuF, 4, 5, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

	ctrls->uhcuF = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uhcuF), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uhcuF, 5, 6, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	ctrls->uAuF = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uAuF), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uAuF, 6, 7, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	ctrls->uHituF = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uHituF), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uHituF, 7, 8, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	ctrls->uEruF = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEruF), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEruF, 8, 9, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	ctrls->uEituF = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEituF), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEituF, 9, 10, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	i++;

	/* u(nu) */

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(ν)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->unu = gtk_adjustment_new(unc->unu, 0, 0.5, 0.01, 0.1, 0.);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->unu), 0.01, 2);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	g_signal_connect_swapped(ctrls->unu, "value-changed", G_CALLBACK(unc_nu_changed), data);
	
	ctrls->uEitunu = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEitunu), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEitunu, 7, 8, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

	i++;

	/* u(nu_i) */

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(ν<sub>i</sub>)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->unui = gtk_adjustment_new(unc->unui, 0, 0.5, 0.01, 0.1, 0.);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->unui), 0.01, 2);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	g_signal_connect_swapped(ctrls->unui, "value-changed", G_CALLBACK(unc_nui_changed), data);
	
	ctrls->uEitunui = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEitunui), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEitunui, 7, 8, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	i++;

	/* u(E_i) */

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(E<sub>i</sub>)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->uEi = gtk_adjustment_new(unc->uEi*1e-9, 0, 1e5, 1, 10, 0.);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->uEi), 1, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	g_signal_connect_swapped(ctrls->uEi, "value-changed", G_CALLBACK(unc_Ei_changed), data);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), label_new_with_markup_left("GPa"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->uEituEi = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEituEi), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEituEi, 7, 8, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

	i++;

	/* total */
	
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("Total"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->um = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->um), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->um, 3, 4, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	ctrls->uS = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uS), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uS, 4, 5, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

	ctrls->uhc = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uhc), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uhc, 5, 6, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	ctrls->uA = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uA), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uA, 6, 7, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	ctrls->uHit = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uHit), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uHit, 7, 8, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	ctrls->uEr = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEr), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEr, 8, 9, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	ctrls->uEit = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEit), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEit, 9, 10, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);


	/* Uncertainties due to contact point */

	ctrls->frame_unc_contact = gtk_frame_new(NULL);
	gtk_frame_set_label_widget (GTK_FRAME (ctrls->frame_unc_contact),
				    GTK_WIDGET (gwy_label_new_header("Uncertainties due to choice of contact point")));
	
	ctrls->table_unc_contact = gtk_table_new(6, 3, FALSE);
	gtk_container_set_border_width (GTK_CONTAINER (ctrls->table_unc_contact), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (ctrls->table_unc_contact), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (ctrls->table_unc_contact), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add (GTK_CONTAINER (ctrls->frame_unc_contact), ctrls->table_unc_contact);

	/* table headers */

	i=0;
	
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact),
			 gwy_label_new_header("Contact point shift"), 0, 1, i, i+1,  GTK_EXPAND | GTK_FILL, 0, 0, 0);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact),
			 gwy_label_new_header("E<sub>r</sub>/GPa"), 1, 2, i, i+1,  GTK_EXPAND | GTK_FILL, 0, 0, 0);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact),
			 gwy_label_new_header("H<sub>it</sub>/GPa"), 2, 3, i, i+1,  GTK_EXPAND | GTK_FILL, 0, 0, 0);
	// gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact),
	// gwy_label_new_header("weight"), 2, 3, i,i+1,  GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);

	i++;

	// Ec is Er, Hc is Hit
	unc->Ec = (gdouble *)g_malloc((2*CONTACTMAX+1)*sizeof(gdouble));
	unc->Hc = (gdouble *)g_malloc((2*CONTACTMAX+1)*sizeof(gdouble));

	/* values */

	for (j = -CONTACTMAX; j <= CONTACTMAX; j++)	{

	    /* vertical alignments: j: 1.0 (bottom), others: 0.0 (top); looks quite nice, still probably incorrect */
	    
		label = gtk_label_new(NULL);
		g_snprintf(str, sizeof(str),"%d\n", j); 
		gtk_label_set_text (GTK_LABEL(label), str);
		gtk_misc_set_alignment (GTK_MISC(label), 0.0, 1.0);
		gtk_table_attach (GTK_TABLE (ctrls->table_unc_contact), label, 0, 1, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

		op_odr_fit_shift_contact(&(data->args->opodrdata), &(data->args->fddata), &(data->args->instdata), &(data->args->area), unc->Ec, unc->Hc, j);

		//fill in Er
		label = gtk_label_new(NULL);
		if (unc->Ec[j+CONTACTMAX] > 0)
			g_snprintf(str, sizeof(str), MODUL, unc->Ec[j+CONTACTMAX]); 
		else
			g_snprintf(str, sizeof(str), "not defined");
		gtk_label_set_text (GTK_LABEL(label), str);
		gtk_misc_set_alignment (GTK_MISC(label), 0.0, 0.0); 
		gtk_table_attach (GTK_TABLE(ctrls->table_unc_contact), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

		/*
		label = gtk_label_new(NULL);
		g_snprintf(str, sizeof(str),"1/%d", 2*CONTACTMAX+1); 
		gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0); 
		gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact), label, 2, 3, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 
		*/

		//fill in Hit
		label = gtk_label_new(NULL);
		if (unc->Hc[j+CONTACTMAX] > 0)
			g_snprintf(str, sizeof(str), HARD, unc->Hc[j+CONTACTMAX]); 
		else
			g_snprintf(str, sizeof(str), "not defined");
		gtk_label_set_text (GTK_LABEL(label), str);
		gtk_misc_set_alignment (GTK_MISC(label), 0.0, 0.0); 
		gtk_table_attach (GTK_TABLE(ctrls->table_unc_contact), label, 2, 3, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

		i++;
	}

	op_odr_propagate_uncertainties (data);

	ctrls->button_save = gtk_button_new_from_stock (GTK_STOCK_SAVE);
	g_signal_connect_swapped (ctrls->button_save, "clicked", G_CALLBACK (op_odr_uncertainty_save_data), data->args);

	/* Monte Carlo calculation */
	
	ctrls->frame_mc = gtk_frame_new(NULL);
	gtk_frame_set_label_widget (GTK_FRAME (ctrls->frame_mc),
				    GTK_WIDGET (gwy_label_new_header("Monte Carlo calculation of uncertainties")));

	ctrls->table_mc = gtk_table_new(6, 7, FALSE);

	gtk_container_set_border_width (GTK_CONTAINER (ctrls->table_mc), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (ctrls->table_mc), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (ctrls->table_mc), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add ( GTK_CONTAINER (ctrls->frame_mc), ctrls->table_mc);

	i = 0;
	gtk_table_attach(GTK_TABLE(ctrls->table_mc), gwy_label_new_header("Number of iterations"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->Nmc = gtk_adjustment_new(unc->Nmc, 1, 1e9, 10, 100,0); 
	
	ctrls->spin_Nmc = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->Nmc), 1, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(ctrls->spin_Nmc), TRUE);
	gtk_table_attach(GTK_TABLE(ctrls->table_mc), ctrls->spin_Nmc, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	g_signal_connect_swapped(ctrls->Nmc, "value-changed", G_CALLBACK(mc_iterations_changed), data);
	i++;
	

	ctrls->mc_warn = gtk_label_new(NULL);
	gtk_table_attach(GTK_TABLE(ctrls->table_mc), ctrls->mc_warn, 0, 3, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	if (unc->Nmc > MC_WHEN_WARN){
		g_snprintf(str, sizeof(str), "Warning: Monte Carlo calculation for %d iterations may take very long!", unc->Nmc);
		gtk_label_set_text(GTK_LABEL(ctrls->mc_warn), str);
	}

	// no Monte Carlo window without the uncertainty dialog
	data->controls->opodrmccontrols->has_window = FALSE;
	ctrls->button_mc = gtk_button_new_with_mnemonic ("_Monte Carlo");
	gtk_button_set_image (GTK_BUTTON (ctrls->button_mc), gtk_image_new_from_stock (GWY_STOCK_POLYNOM, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (ctrls->button_mc, "clicked", G_CALLBACK (op_odr_uncertainty_montecarlo), data);

	ctrls->vbox = gtk_vbox_new(FALSE, CTRLS_SPACING);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->frame_unc_input, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->frame_unc_contact, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->button_save, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->frame_mc, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->button_mc, FALSE, FALSE, 0);
	
	gtk_container_add(GTK_CONTAINER(ctrls->window), ctrls->vbox);
	gtk_widget_show_all(ctrls->window);
}

#endif
