#include <math.h>

#include <gtk/gtk.h>

#include <app/file.h>

#include "indent-toolbox-common.h"
#include "datatypes.h"
#include "controls.h"
#include "gui-utils.h"
#include "mc-utils.h"
#include "tool_tangent_mc.h"
#include "tool_tangent_unc.h"
#include "tool_tangent.h"


#define NPARAM 6  // hc, Aphc, Hit, Er, Eit,S
#define MAXBAD 5 //maximum percentage of nonsense results before warning is triggered
#define DEBUG FALSE
#define DEBUG2 FALSE //save individual data (h,F) which were fitted, TAKES TIME!!!!


void
init_TangentMCdata (TangentMCdata *tgmc, gint Nmc)
{
	gint i;

	//number of iterations
	tgmc->Nmc = Nmc;

	//how much had to be skipped 
	tgmc->skipmc = 0;

	//dynamically allocate
	tgmc->tgmc = (Tangentdata *)g_malloc(tgmc->Nmc*sizeof(Tangentdata));
	tgmc->mcavg = (gdouble *)g_malloc(NPARAM*sizeof(gdouble));
	tgmc->mcstd = (gdouble *)g_malloc(NPARAM*sizeof(gdouble));
	tgmc->mcdd = (gdouble **) g_malloc(NPARAM*sizeof(gdouble *));
	tgmc->mcxhist = (gdouble **) g_malloc(NPARAM*sizeof(gdouble *));
	tgmc->mcyhist = (gdouble **) g_malloc(NPARAM*sizeof(gdouble *));

	tgmc->mcnstat = (gint)floor(3.49*cbrt(tgmc->Nmc)+0.5);
	for (i = 0; i< NPARAM; i++){
		tgmc->mcdd[i] = (gdouble *) g_malloc(tgmc->Nmc*sizeof(gdouble));
		tgmc->mcxhist[i] = (gdouble *) g_malloc(tgmc->mcnstat*sizeof(gdouble));
		tgmc->mcyhist[i] = (gdouble *) g_malloc(tgmc->mcnstat*sizeof(gdouble));
	}
}

void
destroy_TangentMCdata (TangentMCdata *tgmc)
{
	gint i;


	//free everything
	g_free(tgmc->tgmc);
	g_free(tgmc->mcavg);
	g_free(tgmc->mcstd);

	for (i = 0; i< NPARAM; i++){
		g_free(tgmc->mcdd[i]);
		g_free(tgmc->mcxhist[i]);
		g_free(tgmc->mcyhist[i]);
	}

	g_free(tgmc->mcdd);
	g_free(tgmc->mcxhist);
	g_free(tgmc->mcyhist);

	tgmc->mcnstat = 0;
	tgmc->skipmc = 0;
}

static	gboolean 
nparam_to_string(gint i, gchar *str, gint nstr)
{

	switch (i){
		case 0:
			g_snprintf(str,nstr,"h_c/nm");
			break;
		case 1:
			g_snprintf(str,nstr,"A_p(h_c)/nm^2");
			break;
		case 2:
			g_snprintf(str,nstr,"H_IT/MPa");
			break;
		case 3:
			g_snprintf(str,nstr,"E_r/GPa");
			break;
		case 4:
			g_snprintf(str,nstr,"E_IT/GPa");
			break;
		case 5:
			g_snprintf(str,nstr,"S/(mN/nm)");
			break;
		default:
			printf("Unhandled number of results.\n");
			return FALSE;
	}
	return TRUE;
}

void tangent_mc_write_file(gchar *filename, Args *args) 
{
	FILE *file;
	Tangentdata *tg;
	gint i,j;
	gchar str[200];
	gboolean is;

	if (args->fddata.hall->res == 0 || args->fddata.Fall->res==0) {
	    show_error("No data.");
	    return;
	}

	file = fopen(filename, "w");
	if (!file) {
	    show_error("Could not open file.");
	    return;
	}

	fprintf(file, "# %s \n", args->fddata.filename);
	fprintf(file, "#Monte Carlo simulation of tangent analysis\n");
	fprintf(file, "#Input uncertainties \n");
	fprintf(file, "#u(h) = %g nm \n", args->tgunc.uh);
	fprintf(file, "#u(F) = %g mN \n", args->tgunc.uF);
	fprintf(file, "#Number of iterations  = %d  \n", args->tgmc.Nmc);
	fprintf(file, "\n");

	fprintf(file, "#Parameters of Tangent analysis \n");
	fprintf(file, "#nu: %g                 \n", args->instdata.nu);
	fprintf(file, "#nu_indenter: %g        \n", args->instdata.nui);
	fprintf(file, "#E_indenter: %g      GPa\n", args->instdata.Ei * 1e-9); //TODO units
	fprintf(file, "#Fitted range: ");
	if (args->tgdata.Finput)
		fprintf(file, "	[%g , %g ] %% Fmax \n", args->tgdata.range_pct_Fmax[0], args->tgdata.range_pct_Fmax[1]);
	else
		fprintf(file, "[%g , %g ] nm \n",args->tgdata.range[0], args->tgdata.range[1]);	
	fprintf(file, "# corresponds : ");
	if (args->tgdata.Finput)
		fprintf(file, "[%g , %g ] nm \n",args->tgdata.range[0], args->tgdata.range[1]);	
	else
		fprintf(file, "	[%g , %g ] %% Fmax \n", args->tgdata.range_pct_Fmax[0], args->tgdata.range_pct_Fmax[1]);
	fprintf(file, "# %d datapoints used.\n", args->tgdata.nfitdata);

	fprintf(file, "\n\n");


	fprintf(file, "#Results \n");
	for (i = 0; i < NPARAM; i++){
		is = nparam_to_string(i, str, sizeof(str));
		fprintf(file, "#%s = (%g ± %g ) \n", str, args->tgmc.mcavg[i], args->tgmc.mcstd[i]);
	}

	fprintf(file, "\n\n\n");


	// Histograms

	if (args->tgmc.mcnstat >0){
		fprintf(file,"#Histogram\n");
		for (i=0; i < NPARAM; i++){
			fprintf(file,"#Histogram\n");
			is = nparam_to_string(i, str, sizeof(str));
			if (is){
				fprintf(file,"#%s    count/a.u.  \n", str);
				for (j = 0;j < args->tgmc.mcnstat; j++)
					fprintf(file, "%g %g \n", args->tgmc.mcxhist[i][j], args->tgmc.mcyhist[i][j]);
				fprintf(file, "\n \n");
			}
		}
	}


	fprintf(file, "\n\n\n");

	//Full data
	fprintf(file, "#Full data \n");
	fprintf(file, "#");
	for ( j = 0; j< NPARAM; j++){
		is = nparam_to_string(j, str, sizeof(str));
		fprintf(file, "%s ", str);
	}
	fprintf(file,"\n");

	tg= args->tgmc.tgmc;
	for (i = 0; i< args->tgmc.Nmc; i++){
		fprintf(file, DEPTH, tg->hc);
		fprintf(file,"  ");
		fprintf(file, AREA, tg->Aphc);
		fprintf(file,"  ");
		fprintf(file, HARD, tg->Hit);
		fprintf(file,"  ");
		fprintf(file, MODUL, tg->Er);
		fprintf(file,"  ");
		fprintf(file, MODUL, tg->Eit);
		fprintf(file,"  ");
		fprintf(file, SLOPE, tg->S);
		fprintf(file,"  \n");
		tg++;
	}


	fclose(file); 
}

static void tangent_mc_hist_S( Data *data)
{
	TangentMCControls *ctrls;
	GtkWidget *graph;
	GwyGraphModel *graph_model;
	GwyGraphCurveModel *cmodel;
	gint j=5;

	ctrls = data->controls->tgmccontrols;

	printf("show S histogram \n");
	if (GTK_IS_WIDGET(ctrls->hist_window[j]))
	{
		gtk_window_present(GTK_WINDOW ( ctrls->hist_window[j]));
		return;
	}

	ctrls->hist_window[j] = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->hist_window[j]), "Probability distribution of S");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->hist_window[j]), 300, 200); 
	g_signal_connect (ctrls->hist_window[j], "delete_event", G_CALLBACK (gtk_widget_destroy),NULL); // CHECK

	graph_model = gwy_graph_model_new();
	g_object_set(graph_model, "title", "PDF S", NULL);
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_BOTTOM, "S / (mN/nm)");
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_LEFT, "count / a.u.");	

	graph = gwy_graph_new(graph_model);
	gtk_widget_set_size_request(GTK_WIDGET(graph), 200, 150);

	gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
	gwy_graph_set_status(GWY_GRAPH(graph), GWY_GRAPH_STATUS_XSEL);



	cmodel = gwy_graph_curve_model_new();

	g_object_set(cmodel,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "PDF(S)",
			"point-size", 3,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
			"color", CURVE_COLOR_UNLOAD,
			"line-style", GDK_LINE_SOLID,
			NULL);
	gwy_graph_model_add_curve(graph_model, cmodel);

	gwy_graph_curve_model_set_data(cmodel,data->args->tgmc.mcxhist[j], data->args->tgmc.mcyhist[j], data->args->tgmc.mcnstat);

	gtk_container_add(GTK_CONTAINER(ctrls->hist_window[j]), graph);
	gtk_widget_show_all(ctrls->hist_window[j]);

}


static void tangent_mc_hist_hc( Data *data)
{
	TangentMCControls *ctrls;
	GtkWidget *graph;
	GwyGraphModel *graph_model;
	GwyGraphCurveModel *cmodel;
	gint j=0;

	ctrls = data->controls->tgmccontrols;

	printf("show hc histogram \n");
	if (GTK_IS_WIDGET(ctrls->hist_window[j]))
	{
		gtk_window_present(GTK_WINDOW ( ctrls->hist_window[j]));
		return;
	}

	ctrls->hist_window[j] = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->hist_window[j]), "Probability distribution of h_c");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->hist_window[j]), 300, 200); 
	g_signal_connect (ctrls->hist_window[j], "delete_event", G_CALLBACK (gtk_widget_destroy),NULL); // CHECK

	graph_model = gwy_graph_model_new();
	g_object_set(graph_model, "title", "PDF h_c", NULL);
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_BOTTOM, "h_c / nm");
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_LEFT, "count / a.u.");	

	graph = gwy_graph_new(graph_model);
	gtk_widget_set_size_request(GTK_WIDGET(graph), 200, 150);

	gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
	gwy_graph_set_status(GWY_GRAPH(graph), GWY_GRAPH_STATUS_XSEL);



	cmodel = gwy_graph_curve_model_new();

	g_object_set(cmodel,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "PDF(h_c)",
			"point-size", 3,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
			"color", CURVE_COLOR_UNLOAD,
			"line-style", GDK_LINE_SOLID,
			NULL);
	gwy_graph_model_add_curve(graph_model, cmodel);

	gwy_graph_curve_model_set_data(cmodel,data->args->tgmc.mcxhist[j], data->args->tgmc.mcyhist[j], data->args->tgmc.mcnstat);

	gtk_container_add(GTK_CONTAINER(ctrls->hist_window[j]), graph);
	gtk_widget_show_all(ctrls->hist_window[j]);

}

static void tangent_mc_hist_Aphc( Data *data)
{
	TangentMCControls *ctrls;
	GtkWidget *graph;
	GwyGraphModel *graph_model;
	GwyGraphCurveModel *cmodel;
	gint j=1;

	ctrls = data->controls->tgmccontrols;

	printf("show Aphc histogram \n");
	if (GTK_IS_WIDGET(ctrls->hist_window[j]))
	{
		gtk_window_present(GTK_WINDOW ( ctrls->hist_window[j]));
		return;
	}

	ctrls->hist_window[j] = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->hist_window[j]), "Probability distribution of A_p(h_c)");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->hist_window[j]), 300, 200); 
	g_signal_connect (ctrls->hist_window[j], "delete_event", G_CALLBACK (gtk_widget_destroy),NULL); // CHECK

	graph_model = gwy_graph_model_new();
	g_object_set(graph_model, "title", "PDF h_c", NULL);
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_BOTTOM, "A_p(h_c) / nm^2");
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_LEFT, "count / a.u.");	

	graph = gwy_graph_new(graph_model);
	gtk_widget_set_size_request(GTK_WIDGET(graph), 200, 150);

	gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
	gwy_graph_set_status(GWY_GRAPH(graph), GWY_GRAPH_STATUS_XSEL);

	cmodel = gwy_graph_curve_model_new();

	g_object_set(cmodel,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "PDF(A_p(h_c))",
			"point-size", 3,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
			"color", CURVE_COLOR_UNLOAD,
			"line-style", GDK_LINE_SOLID,
			NULL);
	gwy_graph_model_add_curve(graph_model, cmodel);

	gwy_graph_curve_model_set_data(cmodel,data->args->tgmc.mcxhist[j], data->args->tgmc.mcyhist[j], data->args->tgmc.mcnstat);

	gtk_container_add(GTK_CONTAINER(ctrls->hist_window[j]), graph);
	gtk_widget_show_all(ctrls->hist_window[j]);
}

static void tangent_mc_hist_Hit( Data *data)
{
	TangentMCControls *ctrls;
	GtkWidget *graph;
	GwyGraphModel *graph_model;
	GwyGraphCurveModel *cmodel;
	gint j=2;

	ctrls = data->controls->tgmccontrols;

	printf("show Hit histogram \n");
	if (GTK_IS_WIDGET(ctrls->hist_window[j]))
	{
		gtk_window_present(GTK_WINDOW ( ctrls->hist_window[j]));
		return;
	}

	ctrls->hist_window[j] = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->hist_window[j]), "Probability distribution of H_IT");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->hist_window[j]), 300, 200); 
	g_signal_connect (ctrls->hist_window[j], "delete_event", G_CALLBACK (gtk_widget_destroy),NULL); // CHECK

	graph_model = gwy_graph_model_new();
	g_object_set(graph_model, "title", "PDF H_IT", NULL);
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_BOTTOM, "H_IT / MPa");
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_LEFT, "count / a.u.");	

	graph = gwy_graph_new(graph_model);
	gtk_widget_set_size_request(GTK_WIDGET(graph), 200, 150);

	gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
	gwy_graph_set_status(GWY_GRAPH(graph), GWY_GRAPH_STATUS_XSEL);



	cmodel = gwy_graph_curve_model_new();

	g_object_set(cmodel,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "PDF(H_IT)",
			"point-size", 3,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
			"color", CURVE_COLOR_UNLOAD,
			"line-style", GDK_LINE_SOLID,
			NULL);
	gwy_graph_model_add_curve(graph_model, cmodel);

	gwy_graph_curve_model_set_data(cmodel,data->args->tgmc.mcxhist[j], data->args->tgmc.mcyhist[j], data->args->tgmc.mcnstat);

	gtk_container_add(GTK_CONTAINER(ctrls->hist_window[j]), graph);
	gtk_widget_show_all(ctrls->hist_window[j]);
}

static void tangent_mc_hist_Er( Data *data)
{
	TangentMCControls *ctrls;
	GtkWidget *graph;
	GwyGraphModel *graph_model;
	GwyGraphCurveModel *cmodel;
	gint j=3;

	ctrls = data->controls->tgmccontrols;

	printf("show Er histogram \n");
	if (GTK_IS_WIDGET(ctrls->hist_window[j]))
	{
		gtk_window_present(GTK_WINDOW ( ctrls->hist_window[j]));
		return;
	}

	ctrls->hist_window[j] = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->hist_window[j]), "Probability distribution of E_r");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->hist_window[j]), 300, 200); 
	g_signal_connect (ctrls->hist_window[j], "delete_event", G_CALLBACK (gtk_widget_destroy),NULL); // CHECK

	graph_model = gwy_graph_model_new();
	g_object_set(graph_model, "title", "PDF E_r", NULL);
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_BOTTOM, "E_r / GPa");
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_LEFT, "count / a.u.");	

	graph = gwy_graph_new(graph_model);
	gtk_widget_set_size_request(GTK_WIDGET(graph), 200, 150);

	gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
	gwy_graph_set_status(GWY_GRAPH(graph), GWY_GRAPH_STATUS_XSEL);



	cmodel = gwy_graph_curve_model_new();

	g_object_set(cmodel,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "PDF(E_r)",
			"point-size", 3,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
			"color", CURVE_COLOR_UNLOAD,
			"line-style", GDK_LINE_SOLID,
			NULL);
	gwy_graph_model_add_curve(graph_model, cmodel);

	gwy_graph_curve_model_set_data(cmodel,data->args->tgmc.mcxhist[j], data->args->tgmc.mcyhist[j], data->args->tgmc.mcnstat);

	gtk_container_add(GTK_CONTAINER(ctrls->hist_window[j]), graph);
	gtk_widget_show_all(ctrls->hist_window[j]);
}

static void tangent_mc_hist_Eit( Data *data)
{
	TangentMCControls *ctrls;
	GtkWidget *graph;
	GwyGraphModel *graph_model;
	GwyGraphCurveModel *cmodel;
	gint j=4;

	ctrls = data->controls->tgmccontrols;

	printf("show hc histogram \n");
	if (GTK_IS_WIDGET(ctrls->hist_window[j]))
	{
		gtk_window_present(GTK_WINDOW ( ctrls->hist_window[j]));
		return;
	}

	ctrls->hist_window[j] = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->hist_window[j]), "Probability distribution of E_IT");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->hist_window[j]), 300, 200); 
	g_signal_connect (ctrls->hist_window[j], "delete_event", G_CALLBACK (gtk_widget_destroy),NULL); // CHECK

	graph_model = gwy_graph_model_new();
	g_object_set(graph_model, "title", "PDF E_IT", NULL);
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_BOTTOM, "E_IT / GPa");
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_LEFT, "count / a.u.");	

	graph = gwy_graph_new(graph_model);
	gtk_widget_set_size_request(GTK_WIDGET(graph), 200, 150);

	gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
	gwy_graph_set_status(GWY_GRAPH(graph), GWY_GRAPH_STATUS_XSEL);



	cmodel = gwy_graph_curve_model_new();

	g_object_set(cmodel,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "PDF(E_IT)",
			"point-size", 3,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
			"color", CURVE_COLOR_UNLOAD,
			"line-style", GDK_LINE_SOLID,
			NULL);
	gwy_graph_model_add_curve(graph_model, cmodel);

	gwy_graph_curve_model_set_data(cmodel,data->args->tgmc.mcxhist[j], data->args->tgmc.mcyhist[j], data->args->tgmc.mcnstat);

	gtk_container_add(GTK_CONTAINER(ctrls->hist_window[j]), graph);
	gtk_widget_show_all(ctrls->hist_window[j]);

}

static void tangent_mc_save_data( Args *args)
{
	GtkWidget *dialog;
	gchar *filename;

	dialog = gtk_file_chooser_dialog_new("Save File",
			NULL,
			GTK_FILE_CHOOSER_ACTION_SAVE,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_SAVE, GTK_RESPONSE_OK,
			NULL);

	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), gwy_app_get_current_directory());

	if ( gtk_dialog_run (GTK_DIALOG ( dialog)) == GTK_RESPONSE_OK)
	{
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		if (gwy_app_file_confirm_overwrite(dialog))
			tangent_mc_write_file(filename, args);
		g_free(filename);
	}
	gtk_widget_destroy ( dialog);
}

void 
tangent_mc_close_window(Data *data)
{
	gint j;

	if (data->controls->tgmccontrols->has_window){
		gtk_widget_destroy(data->controls->tgmccontrols->window);

		destroy_TangentMCdata(&(data->args->tgmc));
		data->controls->tgmccontrols->has_window = FALSE;
		for (j=0; j< NPARAM; j++){
			if (GTK_IS_WIDGET(data->controls->tgmccontrols->hist_window[j])){
				gtk_widget_destroy(data->controls->tgmccontrols->hist_window[j]);
			}
		}
		g_free(data->controls->tgmccontrols->hist_window);
		g_free(data->controls->tgmccontrols->button_hist);
	}
}

static void tangent_uncertainty_montecarlo_run_calc( Data *data)
{
	gint istart, iend, ndata;
	gint i, Nmc, j;
	Tangentdata *tg;
	TangentMCdata *mc;
	GwyDataLine *h, *F;
	GwyDataLine *hnew, *Fnew;
	GwyDataLine *hnoise, *Fnoise;
	gdouble hmax_old, Fmax_old;
	GRand *rng;
	gint nn;
	gdouble **dd;
	gdouble *xhist, *yhist;
	gboolean hist;
	FILE *aux;
	FILE *aux2;
	gchar auxstr[1000];

	//not necessary to check range, couldn't get here in the first place

	//find appropriate data, which should be fitted, use same regime as for the main fit
	if (data->args->tgdata.Finput)
		range_to_indices(data->args->tgdata.from_pct_Fmax*0.01*data->args->fddata.Fmax, data->args->tgdata.to_pct_Fmax*0.01*data->args->fddata.Fmax,
			       	data->args->fddata.Funload, TRUE, &istart, &iend, &ndata);
	else
		range_to_indices(data->args->tgdata.from, data->args->tgdata.to, data->args->fddata.hunload, TRUE, &istart, &iend, &ndata);

	// if there are not enough data in the selected range, not fitting can be performed
	ndata = MIN(ndata, data->args->fddata.hunload->res - istart);

	if (ndata < 2){
		return;
	}

	//back up hmax, Fmax
	hmax_old = data->args->fddata.hmax;
	Fmax_old = data->args->fddata.Fmax;

	//get data to be fitted
	h = gwy_data_line_part_extract(data->args->fddata.hunload, istart, ndata);
	F = gwy_data_line_part_extract(data->args->fddata.Funload, istart, ndata);



	Nmc = data->args->tgmc.Nmc;
	tg = data->args->tgmc.tgmc;
	//prepare data
	for (i=0; i< Nmc ;i++){
		init_Tangentdata(tg);
		tg++;
	}

	tg= data->args->tgmc.tgmc;
	//copy input parameters
	for (i=0; i< Nmc ;i++){
		/* tg->nu = data->args->tgdata.nu; */
		/* tg->nui = data->args->tgdata.nui; */
		/* tg->Ei = data->args->tgdata.Ei; */
		//		tg->from = data->args->tgdata.from;
		//		tg->to = data->args->tgdata.to;
		tg++;
	}

	/*
	   for (i=0; i< Nmc ;i++){
	   printf ("nu %g \n", data->args->tgmc.tgmc[i].nu);
	   }
	   */

	// initialize random generator and datafield
	rng= g_rand_new();

	//prepare datafield for noise
	hnoise=gwy_data_line_new_alike(h,FALSE);
	Fnoise=gwy_data_line_new_alike(h,FALSE);

	//prepare datafield for varied fields
	hnew=gwy_data_line_new_alike(h,FALSE);
	Fnew=gwy_data_line_new_alike(F,FALSE);

	tg = data->args->tgmc.tgmc;
	if (DEBUG) 
		aux = fopen("debug_tangent_mc.dat", "w");

	for (i=0; i< Nmc; i++){

		//create noise for h-F data
		generate_uncorrelated_noise(hnoise, data->args->tgunc.uh, rng);
		generate_uncorrelated_noise(Fnoise, data->args->tgunc.uF, rng);
		if (DEBUG){ 
			fprintf(aux,"#MC %d \n", i);
			fprintf(aux,"#noise\n");
			for (j = 0 ; j< hnoise->res; j++)
				fprintf(aux, "%d %g %g \n", j, hnoise->data[j], Fnoise->data[j]);
			fprintf(aux,"\n\n");
		}
			

		//add noise to original h-F data
		sum_data_lines(hnew,h, hnoise);
		sum_data_lines(Fnew,F, Fnoise);

		if (DEBUG){ 
			fprintf(aux,"#MC %d \n", i);
			fprintf(aux,"#new data\n");
			for (j = 0 ; j< hnew->res; j++)
				fprintf(aux, "%d %g %g \n", j, hnew->data[j], Fnew->data[j]);
			fprintf(aux,"\n\n");
		}
		if (DEBUG2){
			g_snprintf(auxstr,sizeof(auxstr), "debug%d.txt", i);
			aux2 = fopen(auxstr, "w");
			for (j = 0 ; j< hnew->res; j++)
				fprintf(aux2, "%g %g \n", hnew->data[j], Fnew->data[j]);
			fclose(aux2);
		}
	
		//add random noise to hmax, Fmax
		data->args->fddata.hmax = hmax_old + gaussian_random_number(rng)*data->args->tgunc.uh;
		data->args->fddata.Fmax = Fmax_old + gaussian_random_number(rng)*data->args->tgunc.uF;

		//run fit
		tangent_fit(hnew,Fnew,tg, &(data->args->fddata), &(data->args->instdata), &(data->args->area), data->args->tgunc.uh, data->args->tgunc.uF);
		tg++;
	}
	if (DEBUG)
		fclose(aux);

	//reset hmax, Fmax to original value
	data->args->fddata.hmax = hmax_old;
	data->args->fddata.Fmax = Fmax_old;

	printf("run Monte Carlo \n");	
	g_object_unref(h);
	g_object_unref(F);
	g_object_unref(hnew);
	g_object_unref(Fnew);
	g_object_unref(hnoise);
	g_object_unref(Fnoise);
	g_rand_free(rng);


	for (i=0; i< Nmc; i++){
		//		printf(" %d Er %g Eit %g Hit %g \n", i, data->args->tgmc.tgmc[i].Er, data->args->tgmc.tgmc[i].Eit, data->args->tgmc.tgmc[i].Hit);
	}

	//calculate averages and stdevs
	//hp, Aphc, Hit, Er, Eit, S

	mc = &(data->args->tgmc);
	for (i = 0; i< NPARAM; i++){
		mc->mcavg[i] =0;
		mc->mcstd[i] =0;
	}

	tg = data->args->tgmc.tgmc;
	nn = 0;
	for (i = 0; i< Nmc; i++){
		//check sanity
		if (tg->hc >0){
			mc->mcavg[0]+=tg->hc;
			mc->mcavg[1]+=tg->Aphc;
			mc->mcavg[2]+=tg->Hit;
			mc->mcavg[3]+=tg->Er;
			mc->mcavg[4]+=tg->Eit;
			mc->mcavg[5]+=tg->S;
			mc->mcstd[0]+=tg->hc*tg->hc;
			mc->mcstd[1]+=tg->Aphc*tg->Aphc;
			mc->mcstd[2]+=tg->Hit*tg->Hit;
			mc->mcstd[3]+=tg->Er*tg->Er;
			mc->mcstd[4]+=tg->Eit*tg->Eit;
			mc->mcstd[5]+=tg->S*tg->S;
			nn++;
		}
		tg++;
	}

	//use only reasonable data for average and standard deviation
	for (i = 0; i< NPARAM; i++){
		mc->mcavg[i] /= nn;
		mc->mcstd[i] /= nn;
		mc->mcstd[i]-= mc->mcavg[i]*mc->mcavg[i];
		mc->mcstd[i] = (mc->mcstd[i] < 0) ? 0 : mc->mcstd[i];
		mc->mcstd[i] = sqrt(mc->mcstd[i]);
	}

	// how many datasets were skipped
	data->args->tgmc.skipmc = Nmc-nn;

	/*
	   printf("mean  stdev \n");
	   for (i = 0; i< NPARAM; i++){
	   printf(" %g   %g \n", avg[i], std[i]);
	   }
	   */

	// Histograms
	//create data structure

	dd = data->args->tgmc.mcdd;
	tg = data->args->tgmc.tgmc;
	for (i = 0; i< Nmc; i++){
		dd[0][i] = tg->hc;
		dd[1][i] = tg->Aphc;
		dd[2][i] = tg->Hit;
		dd[3][i] = tg->Er;
		dd[4][i] = tg->Eit;
		dd[5][i] = tg->S;
		tg++;
	}
	//prepare histogram structure


	for (i=0; i < NPARAM; i++){
		xhist = mc->mcxhist[i];
		yhist = mc->mcyhist[i];
		hist = create_histogram(dd[i], Nmc, xhist, yhist, mc->mcnstat );
		if (!hist){
			printf("Warning: No histogram could be created.\n");
			mc->mcnstat = -1;
			break;
		}
	}

}

void tangent_uncertainty_montecarlo( Data *data)
{

	gint i, j;
	GtkWidget *label;
	gchar str[200];
	GdkColor color;

	TangentMCControls *ctrls;
	TangentMCdata *tgmc;

	gdk_color_parse ("red", &color);


	//if there was a previous window, close it and any histograms, free data
	tangent_mc_close_window(data);

	tgmc = &(data->args->tgmc);

	// initialize for number of iterations which was chosen in the Uncertainties dialog
	init_TangentMCdata(tgmc, data->args->tgunc.Nmc); 

	// run calculation
	tangent_uncertainty_montecarlo_run_calc(data);


	//create window and display results
	ctrls = data ->controls->tgmccontrols;

	ctrls->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->window), "Monte Carlo calculation of uncertainties in tangent analysis");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->window), 400, 300); 
	g_signal_connect_swapped (ctrls->window, "delete_event", G_CALLBACK (tangent_mc_close_window), data);
	gtk_window_set_transient_for (GTK_WINDOW(ctrls->window), GTK_WINDOW(data->controls->maincontrols->window_main));

	ctrls->frame_input = gtk_frame_new(NULL);
	gtk_frame_set_label_widget (GTK_FRAME (ctrls->frame_input),
			GTK_WIDGET (gwy_label_new_header("Input for Monte Carlo simulation")));

	ctrls->table_input = gtk_table_new(3, 4, FALSE);

	gtk_container_set_border_width (GTK_CONTAINER (ctrls->table_input), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (ctrls->table_input), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (ctrls->table_input), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add ( GTK_CONTAINER (ctrls->frame_input), ctrls->table_input);

	i = 0;

	/*          u(h)             */

	gtk_table_attach(GTK_TABLE(ctrls->table_input), gwy_label_new_header("u(h)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);

	label = gtk_label_new(NULL);
	g_snprintf(str,sizeof(str), "%g", data->args->tgunc.uh);
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_input), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	gtk_table_attach(GTK_TABLE(ctrls->table_input), label_new_with_markup_left("nm"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);

	gtk_table_attach(GTK_TABLE(ctrls->table_input), label_new_with_markup_left("normal distribution"), 3, 4, i, i+1, GTK_FILL, 0, 0, 0);

	i++;

	/*          u(F)             */

	gtk_table_attach(GTK_TABLE(ctrls->table_input), gwy_label_new_header("u(F)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);

	label = gtk_label_new(NULL);
	g_snprintf(str,sizeof(str), "%g", data->args->tgunc.uF);
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_input), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	gtk_table_attach(GTK_TABLE(ctrls->table_input), label_new_with_markup_left("mN"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);

	gtk_table_attach(GTK_TABLE(ctrls->table_input), label_new_with_markup_left("normal distribution"), 3, 4, i, i+1, GTK_FILL, 0, 0, 0);

	i++;

	/*          Number of iteration */

	gtk_table_attach(GTK_TABLE(ctrls->table_input), gwy_label_new_header("Number of iterations "), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);

	label = gtk_label_new(NULL);
	g_snprintf(str,sizeof(str), "%d", tgmc->Nmc);
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_input), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	// Results
	ctrls->frame_results = gtk_frame_new(NULL);
	gtk_frame_set_label_widget (GTK_FRAME (ctrls->frame_results),
			GTK_WIDGET (gwy_label_new_header("Results of Monte Carlo simulation")));

	ctrls->table_results = gtk_table_new(NPARAM+1, 4, FALSE);

	gtk_container_set_border_width (GTK_CONTAINER (ctrls->table_results), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (ctrls->table_results), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (ctrls->table_results), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add ( GTK_CONTAINER (ctrls->frame_results), ctrls->table_results);

	i = 0;

	gtk_table_attach(GTK_TABLE(ctrls->table_results), gwy_label_new_header("mean"), 1, 2, i, i+1, GTK_FILL, 0, 0, 0);
	gtk_table_attach(GTK_TABLE(ctrls->table_results), gwy_label_new_header("stdev"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);

	i++;

	ctrls->button_hist = (GtkWidget **)g_malloc(NPARAM*sizeof(GtkWidget *));
	ctrls->hist_window = (GtkWidget **)g_malloc(NPARAM*sizeof(GtkWidget *));

	for (j = 0; j< NPARAM; j++)
		ctrls->hist_window[j] = NULL;

	/*                       h_c           */

	gtk_table_attach(GTK_TABLE(ctrls->table_results),
			gwy_label_new_header("h<sub>c</sub>/nm"), 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units

	label = gtk_label_new(NULL);
	g_snprintf(str, sizeof(str), DEPTH, data->args->tgmc.mcavg[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	label = gtk_label_new(NULL);
	g_snprintf(str, sizeof(str), DEPTH, data->args->tgmc.mcstd[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 2, 3, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	ctrls->button_hist[i-1] = gtk_button_new();
	gtk_button_set_image (GTK_BUTTON (ctrls->button_hist[i-1]), gtk_image_new_from_stock (GWY_STOCK_SPECTRUM, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (ctrls->button_hist[i-1], "clicked", G_CALLBACK (tangent_mc_hist_hc), data);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), ctrls->button_hist[i-1], 3, 4, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 
	i++;

	/*                       A_p(h_c)           */

	gtk_table_attach(GTK_TABLE(ctrls->table_results),
			gwy_label_new_header("A<sub>p</sub>/nm<sup>2</sup>"), 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units

	label = gtk_label_new(NULL);
	g_snprintf(str, sizeof(str), AREA, data->args->tgmc.mcavg[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	label = gtk_label_new(NULL);
	g_snprintf(str, sizeof(str), AREA, data->args->tgmc.mcstd[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 2, 3, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	ctrls->button_hist[i-1] = gtk_button_new();
	gtk_button_set_image (GTK_BUTTON (ctrls->button_hist[i-1]), gtk_image_new_from_stock (GWY_STOCK_SPECTRUM, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (ctrls->button_hist[i-1], "clicked", G_CALLBACK (tangent_mc_hist_Aphc), data);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), ctrls->button_hist[i-1], 3, 4, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 
	i++;

	/*                       H_IT           */

	gtk_table_attach(GTK_TABLE(ctrls->table_results),
			gwy_label_new_header("H<sub>it</sub>/MPa"), 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units

	label = gtk_label_new(NULL);
	g_snprintf(str, sizeof(str), HARD, data->args->tgmc.mcavg[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	label = gtk_label_new(NULL);
	g_snprintf(str, sizeof(str), HARD, data->args->tgmc.mcstd[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 2, 3, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	ctrls->button_hist[i-1] = gtk_button_new();
	gtk_button_set_image (GTK_BUTTON (ctrls->button_hist[i-1]), gtk_image_new_from_stock (GWY_STOCK_SPECTRUM, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (ctrls->button_hist[i-1], "clicked", G_CALLBACK (tangent_mc_hist_Hit), data);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), ctrls->button_hist[i-1], 3, 4, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 
	i++;

	/*                       E_r           */

	gtk_table_attach(GTK_TABLE(ctrls->table_results),
			gwy_label_new_header("E<sub>r</sub>/GPa"), 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units

	label = gtk_label_new(NULL);
	g_snprintf(str, sizeof(str), MODUL, data->args->tgmc.mcavg[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	label = gtk_label_new(NULL);
	g_snprintf(str, sizeof(str), MODUL, data->args->tgmc.mcstd[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 2, 3, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	ctrls->button_hist[i-1] = gtk_button_new();
	gtk_button_set_image (GTK_BUTTON (ctrls->button_hist[i-1]), gtk_image_new_from_stock (GWY_STOCK_SPECTRUM, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (ctrls->button_hist[i-1], "clicked", G_CALLBACK (tangent_mc_hist_Er), data);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), ctrls->button_hist[i-1], 3, 4, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 
	i++;

	/*                       E_IT          */

	gtk_table_attach(GTK_TABLE(ctrls->table_results),
			gwy_label_new_header("E<sub>it</sub>/GPa"), 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units

	label = gtk_label_new(NULL);

	g_snprintf(str, sizeof(str), MODUL, data->args->tgmc.mcavg[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	label = gtk_label_new(NULL);
	g_snprintf(str, sizeof(str), MODUL, data->args->tgmc.mcstd[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 2, 3, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	ctrls->button_hist[i-1] = gtk_button_new();
	gtk_button_set_image (GTK_BUTTON (ctrls->button_hist[i-1]), gtk_image_new_from_stock (GWY_STOCK_SPECTRUM, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (ctrls->button_hist[i-1], "clicked", G_CALLBACK (tangent_mc_hist_Eit), data);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), ctrls->button_hist[i-1], 3, 4, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 
	i++;

	/*                       S          */

	gtk_table_attach(GTK_TABLE(ctrls->table_results),
			gwy_label_new_header("S/(mN/nm)"), 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units

	label = gtk_label_new(NULL);

	g_snprintf(str, sizeof(str), SLOPE, data->args->tgmc.mcavg[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	label = gtk_label_new(NULL);
	g_snprintf(str, sizeof(str), SLOPE, data->args->tgmc.mcstd[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 2, 3, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	ctrls->button_hist[i-1] = gtk_button_new();
	gtk_button_set_image (GTK_BUTTON (ctrls->button_hist[i-1]), gtk_image_new_from_stock (GWY_STOCK_SPECTRUM, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (ctrls->button_hist[i-1], "clicked", G_CALLBACK (tangent_mc_hist_S), data);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), ctrls->button_hist[i-1], 3, 4, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 
	i++;


	if (100.*data->args->tgmc.skipmc/tgmc->Nmc > MAXBAD){
		label = gtk_label_new(NULL);
		g_snprintf(str, sizeof(str), "Warning: Approx %.1f %% of the results had to be removed!", 100.*data->args->tgmc.skipmc/tgmc->Nmc);
		gtk_label_set_text(GTK_LABEL(label),str);
		gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &color);
		gtk_table_attach(GTK_TABLE(ctrls->table_results), label, 0,1, i, i+1, GTK_FILL, GTK_FILL,0,0);
		i++;
	}


	// save data
	ctrls->button_save = gtk_button_new_from_stock (GTK_STOCK_SAVE);
	g_signal_connect_swapped (ctrls->button_save, "clicked", G_CALLBACK (tangent_mc_save_data), data->args);

	ctrls->vbox = gtk_vbox_new(FALSE, CTRLS_SPACING);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->frame_input, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->frame_results, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->button_save, FALSE, FALSE, 0);

	gtk_container_add(GTK_CONTAINER(ctrls->window), ctrls->vbox);
	gtk_widget_show_all(ctrls->window);
	ctrls->has_window = TRUE;

}
