#include <math.h>

#include <gtk/gtk.h>

#include <app/file.h>

#include "indent-toolbox-common.h"
#include "datatypes.h"
#include "controls.h"
#include "fit-utils.h"
#include "gui-utils.h"
#include "tool_tangent_unc.h"
#include "tool_tangent_mc.h"
#include "tool_tangent.h"

#define CONTACTMAX 2
#define MC_WHEN_WARN 1e6
#define DEBUG  TRUE


static void tangent_fit_shift_contact(Tangentdata *tgdata, FDdata *fddata, Instdata *inst, Area *area, gdouble *Sc,gdouble *Ec,gdouble *Hc,gint j);


void 
tangent_propagate_uncertainties (Data *data)
{
	gchar str[30];
	TangentUncControls *ctrls;
	TangentUncdata *unc;
	Tangentdata *tg;
	FDdata *fd;
	Instdata *inst;
	gdouble uh, uF;
	gdouble dAdh;
	gdouble S,eps, hc,  hmax,Fmax, Aphc,Hit,Er,Eit, nu,nui,Ei;
	gint istart, iend, ndata;
	GwyDataLine *x, *y;

	unc = &(data->args->tgunc);
	tg = &(data->args->tgdata);
	fd = &(data->args->fddata);
	inst = &(data->args->instdata);
	ctrls =	data->controls->tgunccontrols;

	if ( !tg->has_fit )
		return;

	if ( unc->Sc == NULL || unc->Ec ==NULL || unc->Hc == NULL)
		return;

	//make copies to simplify code
	uh = unc->uh;
	uF = unc->uF;
	eps = tg->eps;
	S = tg->S;
	hc = tg->hc;
	hmax = fd->hmax;
	Fmax = fd->Fmax;
	Aphc = tg->Aphc;
	Hit = tg->Hit;
	Eit = tg->Eit;
	Er = tg->Er;
	Ei = inst->Ei;
	nui = inst->nui;
	nu = inst->nu;

	
	if (DEBUG)
		printf(" \n\nPropagate uncertainties\n");

	/* constant uncertainty for all data points */
	if (tg->Finput)
		range_to_indices(tg->range_pct_Fmax[0]*0.01*fd->Fmax, tg->range_pct_Fmax[1]*0.01*fd->Fmax, fd->Funload, TRUE, &istart, &iend, &ndata);
	else
		range_to_indices(tg->range[0], tg->range[1], fd->hunload, TRUE, &istart, &iend, &ndata);

	x = gwy_data_line_part_extract(fd->hunload, istart, ndata);
	y = gwy_data_line_part_extract(fd->Funload, istart, ndata);

	if (FIT_TLS)
		total_least_squares_line_fit_const(x, y, &(tg->reg), uh, uF );
	else
		ordinary_least_squares_line_fit_unc_const(x, y, &(tg->reg), uh, uF );
	g_object_unref(x);
	g_object_unref(y);


	unc->uSuh = tg->reg.unc_slope_x;
	unc->uSuF = tg->reg.unc_slope_y;
	if (DEBUG)
		printf("u(S;h) %g u(S;F) %g  urel(S;h) %g urel(S;F) %g  \n", unc->uSuh, unc->uSuF, unc->uSuh/S, unc->uSuF/S);  

	/*    u(hc)   */
	unc->uhcuh = sqrt(uh*uh+ (hmax-hc)*(hmax-hc)/S/S*unc->uSuh*unc->uSuh); 
	unc->uhcuF = sqrt(eps*eps/S/S*uF*uF + (hmax-hc)*(hmax-hc)/S/S *unc->uSuF*unc->uSuF); 
	/* u(hc;h) = sqrt[uh^2+ ((hmax-hc)/S^2)^2 uSh^2]
	 * u(hc;F) = sqrt[(eps/S) uF^2 + ((hmax-hc)/S^2)^2 uSF^2 )
	 */
	if (DEBUG)
		printf("u(hc;h) %g u(hc;F) %g  urel(hc;h) %g urel(hc;F) %g  \n", unc->uhcuh, unc->uhcuF, unc->uhcuh/hc, unc->uhcuF/hc);  

	/*    u(A)   */
	dAdh =  eval_area_derivative(&(data->args->area), hc) ;
	unc->uAuh = dAdh * unc->uhcuh;
	unc->uAuF = dAdh * unc->uhcuF;
	/* u(Ap;h) = dA/dh * u(hc;h)
	 * u(Ap;F) = dA/dh * u(hc;F)
	 */
	if (DEBUG)
		printf("u(A;h) %g u(A;F) %g urel(A;h) %g urel(A;F) %g \n", unc->uAuh, unc->uAuF, unc->uAuh/Aphc, unc->uAuF/Aphc);

	/*     u(H_IT)   */
	unc->uHituh = Hit/Aphc*dAdh*unc->uhcuh;
	unc->uHituF = Hit *sqrt(uF*uF/Fmax/Fmax + dAdh*dAdh/Aphc/Aphc*unc->uhcuF*unc->uhcuF);
	/* u(Hit;h) = Fmax/Aphc^2 * dA/dh *u(hc;h)
	 * u(Hit;F) = sqrt[ (1/Aphc)^2 u(F)^2 + (Fmax/Aphc^2 * dA/dh)^2 *u(hc;F)^2] = Hit sqrt[ u(F)^2/Fmax^2 + (dA/dh  / Aphc)^2 u(hc;F)^2 ]
	 */
	if (DEBUG)
		printf("u(H;h) %g  u(H;F) %g  urel(H;h) %g  urel(H;F) %g\n", unc->uHituh, unc->uHituF, unc->uHituh/Hit, unc->uHituF/Hit);

	/*     u(E_r)   */
	unc->uEruh = Er*sqrt( unc->uSuh*unc->uSuh/S/S + unc->uAuh*unc->uAuh/(Aphc*Aphc*4) - dAdh/Aphc * eps *Fmax/S *unc->uSuh*unc->uSuh/S/S); 
	unc->uEruF = Er*sqrt( unc->uSuF*unc->uSuF/S/S + unc->uAuF*unc->uAuF/(Aphc*Aphc*4) - dAdh/Aphc * eps *Fmax/S *unc->uSuF*unc->uSuF/S/S); 
	/* u(Er;h) = sqrt( (Er/S)^2 u(S;h)^2 + (Er/2A)^2 u(A;h)^2 - Er^2/(S Aphc) *dAdh*eps*Fmax/S^2 *uSh^2 )
	 * u(Er;F) = sqrt( (Er/S)^2 u(S;F)^2 + (Er/2A)^2 u(A;F)^2 - Er^2/(S Aphc) *dAdh*eps*Fmax/S^2 *uSh^2 )
	 */
	if (DEBUG){
		printf("u(Er;h)^2 terms %g %g %g \n",
				unc->uSuh*unc->uSuh/S/S , unc->uAuh*unc->uAuh/(Aphc*Aphc*4),- dAdh/Aphc * eps *Fmax/S *unc->uSuh*unc->uSuh/S/S);
		printf("u(Er;F)^2 terms %g %g %g \n",
				unc->uSuF*unc->uSuF/S/S , unc->uAuF*unc->uAuF/(Aphc*Aphc*4),- dAdh/Aphc * eps *Fmax/S *unc->uSuF*unc->uSuF/S/S);
		printf("u(Er;h) %g  u(Er;F) %g  urel(Er;h) %g  urel(Er;F) %g\n", unc->uEruh, unc->uEruF, unc->uEruh/Er, unc->uEruF/Er);
	}

	/*     u(E_IT)   */
	unc->uEituh = Eit*Eit/Er/Er/(1-nu*nu)*unc->uEruh; 
	unc->uEituF = Eit*Eit/Er/Er/(1-nu*nu)*unc->uEruF;
	unc->uEitunu =  fabs(2*nu/(1-nu*nu)*Eit*unc->unu); 
	unc->uEitunui = fabs(2*nui/(1-nu*nu)* Eit*Eit /Ei *unc->unui)*1e9; 
	unc->uEituEi =  fabs((1-nui*nui)/(1-nu*nu) *Eit*Eit/ Ei*Ei *unc->uEi)*1e-18; 
	/* u(Eit;h) =  Eit^2/Er^2 /(1-nu^2) u(Er;h)
	 * u(Eit;F) =  Eit^2/Er^2 /(1-nu^2) u(Er;F)
	 * u(Eit;nu) = -2nu/(1-nu^2) Eit u(nu)
	 * u(Eit;nui) =  -2nui/(1-nu^2) Eit^2/Ei u(nui)
	 * u(Eit;Ei) = -(1-nui^2)/(1-nu)^2 Eit^2/Ei^2 u(Ei)
	 */

	if (DEBUG){
		printf("u(Eit;h) %g  u(Eit;F) %g  urel(Eit;h) %g  urel(Eit;F) %g\n", unc->uEituh, unc->uEituF, unc->uEituh/Eit, unc->uEituF/Eit);
		printf("u(Eit;nu) %g  u(Eit;nui) %g  u(Eit;Ei) %g  urel(Eit;nu) %g  urel(Eit;nui) %g  urel(Eit;Ei) %g\n", unc->uEitunu, unc->uEitunui, unc->uEituEi, unc->uEitunu/Eit, unc->uEitunui/Eit, unc->uEituEi/Eit);
	}

	if (DEBUG)
		printf(" \nEnd propagation of uncertainties\n\n");

	/* show */
	g_snprintf(str, sizeof(str), SLOPE, unc->uSuh);
	gtk_label_set_text(GTK_LABEL(ctrls->uSuh), str);
	g_snprintf(str, sizeof(str), SLOPE, unc->uSuF);
	gtk_label_set_text(GTK_LABEL(ctrls->uSuF), str);

	g_snprintf(str, sizeof(str), DEPTH, unc->uhcuh);
	gtk_label_set_text(GTK_LABEL(ctrls->uhcuh), str);
	g_snprintf(str, sizeof(str), DEPTH, unc->uhcuF);
	gtk_label_set_text(GTK_LABEL(ctrls->uhcuF), str);

	g_snprintf(str, sizeof(str), AREA, unc->uAuh);
	gtk_label_set_text(GTK_LABEL(ctrls->uAuh), str);
	g_snprintf(str, sizeof(str), AREA, unc->uAuF);
	gtk_label_set_text(GTK_LABEL(ctrls->uAuF), str);

	g_snprintf(str, sizeof(str), HARD, unc->uHituh);
	gtk_label_set_text(GTK_LABEL(ctrls->uHituh), str);
	g_snprintf(str, sizeof(str), HARD, unc->uHituF);
	gtk_label_set_text(GTK_LABEL(ctrls->uHituF), str);

	g_snprintf(str, sizeof(str), MODUL, unc->uEruh);
	gtk_label_set_text(GTK_LABEL(ctrls->uEruh), str);
	g_snprintf(str, sizeof(str), MODUL, unc->uEruF);
	gtk_label_set_text(GTK_LABEL(ctrls->uEruF), str);
	g_snprintf(str, sizeof(str), MODUL, unc->uEituh);
	gtk_label_set_text(GTK_LABEL(ctrls->uEituh), str);
	g_snprintf(str, sizeof(str), MODUL, unc->uEituF);
	gtk_label_set_text(GTK_LABEL(ctrls->uEituF), str);


	g_snprintf(str, sizeof(str), MODUL, unc->uEitunu);
	gtk_label_set_text(GTK_LABEL(ctrls->uEitunu), str);
	g_snprintf(str, sizeof(str), MODUL, unc->uEitunui);
	gtk_label_set_text(GTK_LABEL(ctrls->uEitunui), str);
	g_snprintf(str, sizeof(str), "%.4g", unc->uEituEi);
	gtk_label_set_text(GTK_LABEL(ctrls->uEituEi), str);

	// total uncertainties
	unc->uS =  sqrt(unc->uSuh*unc->uSuh + unc->uSuF*unc->uSuF);
	g_snprintf(str, sizeof(str), SLOPE, unc->uS );
	gtk_label_set_text(GTK_LABEL(ctrls->uS), str);

	unc->uhc =  sqrt(unc->uhcuh*unc->uhcuh + unc->uhcuF*unc->uhcuF);
	g_snprintf(str, sizeof(str), DEPTH, unc->uhc );
	gtk_label_set_text(GTK_LABEL(ctrls->uhc), str);

	unc->uA =  sqrt(unc->uAuh*unc->uAuh + unc->uAuF*unc->uAuF);
	g_snprintf(str, sizeof(str), AREA,unc->uA);
	gtk_label_set_text(GTK_LABEL(ctrls->uA), str);

	unc->uHit = sqrt(unc->uHituh*unc->uHituh + unc->uHituF*unc->uHituF);
	g_snprintf(str, sizeof(str), HARD, unc->uHit );
	gtk_label_set_text(GTK_LABEL(ctrls->uHit), str);

	unc->uEr = sqrt(unc->uEruh*unc->uEruh + unc->uEruF*unc->uEruF);
	g_snprintf(str, sizeof(str), MODUL, unc->uEr);
	gtk_label_set_text(GTK_LABEL(ctrls->uEr), str);

	unc->uEit =sqrt(unc->uEituh*unc->uEituh + unc->uEituF*unc->uEituF
			+unc->uEitunu*unc->uEitunu +unc->uEitunui*unc->uEitunui +unc->uEituEi*unc->uEituEi);
	g_snprintf(str, sizeof(str), MODUL, unc->uEit );
	gtk_label_set_text(GTK_LABEL(ctrls->uEit), str);


	/*table_contact doesn't change with any of the adjustables nu, nui or Ei, uh, uF, unu, uniu, uEi
	no need to update */
	/*gdk_window_raise(gtk_widget_get_window(GTK_WIDGET (ctrls->window)));  TODO */ 
}

static void mc_iterations_changed(Data *data){
	gchar str[500];

	data->args->tgunc.Nmc = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->tgunccontrols->Nmc));
	if (data->args->tgunc.Nmc > MC_WHEN_WARN){
		g_snprintf(str, sizeof(str), "Warning: Monte Carlo calculation for %d iterations may take very long!", data->args->tgunc.Nmc);
		gtk_label_set_text(GTK_LABEL(data->controls->tgunccontrols->mc_warn), str);
	}
	else{
		gtk_label_set_text(GTK_LABEL(data->controls->tgunccontrols->mc_warn), NULL);
	}

}

static void 
unc_depth_changed (Data *data)
{
	data->args->tgunc.uh = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->tgunccontrols->uh));
	tangent_propagate_uncertainties (data);
}

static void 
unc_load_changed (Data *data)
{
	data->args->tgunc.uF = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->tgunccontrols->uF));
	tangent_propagate_uncertainties (data);
}

static void 
unc_nu_changed (Data *data)
{
	data->args->tgunc.unu = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->tgunccontrols->unu));
	tangent_propagate_uncertainties (data);
}

static void 
unc_nui_changed (Data *data)
{
	data->args->tgunc.unui = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->tgunccontrols->unui));
	tangent_propagate_uncertainties (data);
}

static void 
unc_Ei_changed (Data *data)
{
	data->args->tgunc.uEi = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->tgunccontrols->uEi))*1e9;
	tangent_propagate_uncertainties (data);
}


void tangent_uncertainty_write_file(gchar *filename, Args *args) 
{
	FILE *file;
	Tangentdata *tgdata;
	TangentUncdata *tgunc;
	Instdata *instdata;
	gint j;

	tgdata = &(args->tgdata);
	tgunc = &(args->tgunc);
	instdata = &(args->instdata);

	if (args->fddata.hall->res == 0 || args->fddata.Fall->res==0) {
	    show_error("No data.");
	    return;
	}

	file = fopen(filename, "w");
	if (!file) {
	    show_error("Could not open file.");
	    return;
	}

	fprintf(file, "# %s \n", args->fddata.filename);
	fprintf(file, "#Parameters of Tangent analysis \n");
	fprintf(file, "# u(depth): %g nm\n", tgunc->uh);
	fprintf(file, "# u(load): %g nm\n", tgunc->uF);
	fprintf(file, "#nu: %g                 u(nu):  %g  \n", instdata->nu, tgunc->unu);
	fprintf(file, "#nu_indenter: %g       u(nu_indenter):  %g   \n", instdata->nui, tgunc->unui);
	fprintf(file, "#E_indenter: %g        u(E_indenter):   %g GPa\n", instdata->Ei * 1e-9, tgunc->uEi*1e-9); //TODO units
	fprintf(file, "#Fitted range: ");
	if (args->tgdata.Finput)
		fprintf(file, "	[%g , %g ] %% Fmax \n", tgdata->range_pct_Fmax[0], tgdata->range_pct_Fmax[1]);
	else
		fprintf(file, "[%g , %g ] nm \n",tgdata->range[0], tgdata->range[1]);	
	fprintf(file, "# corresponds : ");
	if (args->tgdata.Finput)
		fprintf(file, "[%g , %g ] nm \n",tgdata->range[0], tgdata->range[1]);	
	else
		fprintf(file, "	[%g , %g ] %% Fmax \n", tgdata->range_pct_Fmax[0], tgdata->range_pct_Fmax[1]);
	fprintf(file, "# %d datapoints used.\n", tgdata->nfitdata);


	fprintf(file, "#\n");
	fprintf(file, "#Results of tangent analysis \n");
	fprintf(file, "#S = %g mN/nm\n", tgdata->S);
	fprintf(file, "#hc = %g nm\n", tgdata->hc);
	fprintf(file, "#Ap(hc) = %g nm^2\n", tgdata->Aphc);
	fprintf(file, "#Er = %g GPa\n", tgdata->Er);
	fprintf(file, "#Hit = %g GPa\n", tgdata->Hit);
	fprintf(file, "#Eit = %g GPa\n", tgdata->Eit);
	fprintf(file, "#\n");
	fprintf(file, "#\n");

	fprintf(file, "#Results of the Gaussian propagation of uncertainties \n");
	fprintf(file, "#u(S) =  %g mN/nm \n", tgunc->uS);
	fprintf(file, "#u(hc) =  %g nm \n", tgunc->uhc);
	fprintf(file, "#u(Ap(hc)) =  %g nm^2 \n", tgunc->uA);
	fprintf(file, "#u(Hit) =  %g MPa \n", tgunc->uHit);
	fprintf(file, "#u(Er) =  %g GPa \n", tgunc->uEr);
	fprintf(file, "#u(Eit) =  %g GPa \n", tgunc->uEit);
	fprintf(file, "#\n");


	fprintf(file, "#Contributions to uncertainty of S\n");
	fprintf(file, "#u(S;uh) = %g mN/nm \n", tgunc->uSuh);
	fprintf(file, "#u(S;uF) = %g mN/nm \n", tgunc->uSuF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of hc\n");
	fprintf(file, "#u(hc;uh) = %g nm \n", tgunc->uhcuh);
	fprintf(file, "#u(hc;uF) = %g nm \n", tgunc->uhcuF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Ap(hc)\n");
	fprintf(file, "#u(Ap(hc);uh) = %g nm^2 \n", tgunc->uAuh);
	fprintf(file, "#u(Ap(hc);uF) = %g nm^2 \n", tgunc->uAuF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Hit\n");
	fprintf(file, "#u(Hit;uh) = %g MPa \n", tgunc->uHituh);
	fprintf(file, "#u(Hit;uF) = %g MPa \n", tgunc->uHituF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Er\n");
	fprintf(file, "#u(Er;uh) = %g GPa \n", tgunc->uEruh);
	fprintf(file, "#u(Er;uF) = %g GPa \n", tgunc->uEruF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Eit\n");
	fprintf(file, "#u(Eit;uh) = %g GPa \n", tgunc->uEituh);
	fprintf(file, "#u(Eit;uF) = %g GPa \n", tgunc->uEituF);
	fprintf(file, "#u(Eit;unu) = %g GPa \n", tgunc->uEitunu);
	fprintf(file, "#u(Eit;unui) = %g GPa \n", tgunc->uEitunui);
	fprintf(file, "#u(Eit;uEi) = %g GPa \n", tgunc->uEituEi);
	fprintf(file, "#\n");
	fprintf(file, "#\n");
	fprintf(file, "#\n");


	fprintf(file, "#Changes of the S corresponding to changes in the contact point \n");
	fprintf(file, "# shift of contact point with respect to the globally chosen contact point (negative - to the left, positive - to the right)   \n");
	fprintf(file, "#S/(mN/nm) \n");
	for (j= -CONTACTMAX; j< CONTACTMAX+1; j++){
		if ( tgunc->Sc[j+CONTACTMAX] < 0 )
			fprintf(file, "#d    not defined \n");
		else
			fprintf(file, "# %d   %g  \n", j, tgunc->Sc[j+CONTACTMAX]);
	}
	fprintf(file, "#\n");
	fprintf(file, "#Changes of the Er corresponding to changes in the contact point \n");
	fprintf(file, "# shift of contact point with respect to the globally chosen contact point (negative - to the left, positive - to the right)   \n");
	fprintf(file, "#Er/GPa \n");
	for (j= -CONTACTMAX; j< CONTACTMAX+1; j++){
		if ( tgunc->Ec[j+CONTACTMAX] < 0 )
			fprintf(file, "#d    not defined \n");
		else
			fprintf(file, "# %d   %g  \n", j, tgunc->Ec[j+CONTACTMAX]);
	}
	fprintf(file, "#\n");
	fprintf(file, "#Changes of the H corresponding to changes in the contact point \n");
	fprintf(file, "# shift of contact point with respect to the globally chosen contact point (negative - to the left, positive - to the right)   \n");
	fprintf(file, "#H/MPa \n");
	for (j= -CONTACTMAX; j< CONTACTMAX+1; j++){
		if ( tgunc->Hc[j+CONTACTMAX] < 0 )
			fprintf(file, "# %d   not defined \n", j);
		else
		fprintf(file, "# %d   %g  \n", j, tgunc->Hc[j+CONTACTMAX]);
	}

	fclose(file); 
}





static void tangent_uncertainty_save_data( Args *args)
{
	GtkWidget *dialog;
	gchar *filename;

	dialog = gtk_file_chooser_dialog_new("Save File",
			NULL,
			GTK_FILE_CHOOSER_ACTION_SAVE,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_SAVE, GTK_RESPONSE_OK,
			NULL);

	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), gwy_app_get_current_directory());

	if ( gtk_dialog_run (GTK_DIALOG ( dialog)) == GTK_RESPONSE_OK)
	{
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		if (gwy_app_file_confirm_overwrite(dialog))
			tangent_uncertainty_write_file(filename, args);
		g_free(filename);
	}
	gtk_widget_destroy ( dialog);
}

static void
tangent_fit_shift_contact(Tangentdata *tgdata, FDdata *fddata, Instdata *inst, Area *area, gdouble *Sc,gdouble *Ec,gdouble *Hc,gint j)
{
	Tangentdata tg;
	FDdata fdc;
	gdouble dh,dF;
	gint istart,iend, ndata;
	GwyDataLine *x, *y;

	Sc[j+CONTACTMAX] = -1;
	Ec[j+CONTACTMAX] = -1;
	Hc[j+CONTACTMAX] = -1;

	// create shifted FDdata

	if (DEBUG)
		printf("\n\nCreate shifted data for j %d \n", j);

	if (fddata->i_contact_load+j < 0)
		return;
	
	fdc.i_contact_load = fddata->i_contact_load+j;

	fdc.hall_orig = gwy_data_line_duplicate(fddata->hall_orig);
	fdc.Fall_orig = gwy_data_line_duplicate(fddata->Fall_orig);

	fdc.hunload = gwy_data_line_duplicate(fddata->hunload);
	fdc.Funload = gwy_data_line_duplicate(fddata->Funload);

	/*
	   gwy_data_line_add(fdc.hunload, -fdc.hall_orig->data[fdc.i_contact_load]);
	   gwy_data_line_add(fdc.Funload, -fdc.Fall_orig->data[fdc.i_contact_load]);

	   fdc.hmax = fddata.hmax- fdc.hall_orig->data[fdc.i_contact_load];
	   fdc.Fmax = fddata.Fmax- fdc.Fall_orig->data[fdc.i_contact_load];
	   */

	//difference in shift between different contactpoints
	dh = fdc.hall_orig->data[fdc.i_contact_load]-fddata->hall_orig->data[fddata->i_contact_load];
	dF = fdc.Fall_orig->data[fdc.i_contact_load]-fddata->Fall_orig->data[fddata->i_contact_load];
	if (DEBUG)
		printf("shift dh %g dF %g \n", dh ,dF);

	gwy_data_line_add(fdc.hunload, -dh);
	gwy_data_line_add(fdc.Funload, -dF);


	fdc.hmax = fddata->hmax -dh;
	fdc.Fmax = fddata->Fmax -dF;


	// create working copy of Tangentdata 
	// shift fitting range
	tg = *tgdata;
	tg.range[0] += -dh;
	tg.range[1] += -dh;
	tg.from = tg.range[0];
	tg.to = tg.range[1];

	//check range
	if (tg.range[0] ==  tg.range[1]){
		return;
	}

	//find appropriate data, which should be fitted, use same regime as for the main fit
	if (tgdata->Finput)
		range_to_indices(tg.range_pct_Fmax[0]*0.01*fdc.Fmax, tg.range_pct_Fmax[1]*0.01*fdc.Fmax, fdc.Funload, TRUE, &istart, &iend, &ndata);
	else
		range_to_indices(tg.range[0], tg.range[1], fdc.hunload, TRUE, &istart, &iend, &ndata);

	// if there are not enough data in the selected range, not fitting can be performed
	ndata = MIN(ndata, fdc.hunload->res - istart);

	if (DEBUG)
		printf("interval has %d points, from %d to %d, hrange [%g, %g ] nm, Frange [%g, %g] mN \n", ndata, istart, iend,
			       fdc.hunload->data[iend], fdc.hunload->data[istart], fdc.Funload->data[iend], fdc.Funload->data[istart]);

	if (ndata < 2){
		return;
	}

	//get data to be fitted
	x = gwy_data_line_part_extract(fdc.hunload, istart, ndata);
	y = gwy_data_line_part_extract(fdc.Funload, istart, ndata);


	// perform fit and get results
	tangent_fit(x,y,&tg, &fdc, inst, area, 0, 0);

	// clean up
	g_object_unref(x);
	g_object_unref(y);
	g_object_unref(fdc.hall_orig);
	g_object_unref(fdc.Fall_orig);
	g_object_unref(fdc.hunload);
	g_object_unref(fdc.Funload);

	Sc[j+CONTACTMAX] = tg.S;
	Ec[j+CONTACTMAX] = tg.Er;
	Hc[j+CONTACTMAX] = tg.Hit;

	if (DEBUG)
		printf("\n\n");
}

void 
tangent_unc_close_window(Data *data)
{
	g_free(data->args->tgunc.Sc);
	g_free(data->args->tgunc.Ec);
	g_free(data->args->tgunc.Hc);
	data->args->tgunc.Sc = NULL;
	data->args->tgunc.Ec = NULL;
	data->args->tgunc.Hc = NULL;
	tangent_mc_close_window(data);
	gtk_widget_destroy(data->controls->tgunccontrols->window);
}

void
tangent_uncertainty (Data *data)
{
	GtkWidget *label, *spin;
	gint i, j, k;
	TangentUncControls *ctrls;
	TangentUncdata *unc;
	gchar str[50];

	if ((data->args->tgunc.Sc != NULL) & (data->args->tgunc.Ec != NULL) & (data->args->tgunc.Hc != NULL )) {
		gtk_window_present (GTK_WINDOW (data->controls->tgunccontrols->window));
		return;
	}

	unc = &(data->args->tgunc);
        ctrls =	data->controls->tgunccontrols;


	ctrls->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->window), "Uncertainty in tangent analysis");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->window), 400, 300); 
	g_signal_connect_swapped (ctrls->window, "delete_event", G_CALLBACK (tangent_unc_close_window), data);
	gtk_window_set_transient_for (GTK_WINDOW(ctrls->window), GTK_WINDOW(data->controls->maincontrols->window_main));
	

	/* Uncertainties due to uncertainties in input values */
	
	ctrls->frame_unc_input = gtk_frame_new(NULL);
	gtk_frame_set_label_widget (GTK_FRAME (ctrls->frame_unc_input),
				    GTK_WIDGET (gwy_label_new_header("Uncertainties due to uncertainties in input values")));

	ctrls->table_unc_input = gtk_table_new(6, 10, FALSE);

	gtk_container_set_border_width (GTK_CONTAINER (ctrls->table_unc_input), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (ctrls->table_unc_input), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (ctrls->table_unc_input), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add ( GTK_CONTAINER (ctrls->frame_unc_input), ctrls->table_unc_input);

	i = 0;
	j = 3;
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input),
			 gwy_label_new_header("u(S)/mN.nm<sup>-1</sup>"), j, j+1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units
	j++;
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input),
			 gwy_label_new_header("u(h<sub>c</sub>)/nm"), j, j+1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units
	j++;
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input),
			 gwy_label_new_header("u(A<sub>p</sub>)/nm<sup>2</sup>"), j, j+1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units
	j++;
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input),
			 gwy_label_new_header("u(H<sub>it</sub>)/MPa"), j, j+1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units
	j++;
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input),
			 gwy_label_new_header("u(E<sub>r</sub>)/GPa"), j, j+1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	j++;
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input),
			 gwy_label_new_header("u(E<sub>it</sub>)/GPa"), j, j+1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	j++;

	i++;

	/* u(h) */

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(h)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->uh = gtk_adjustment_new(unc->uh, 0, 1e6, 0.01, 1,0); 
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->uh), 0.001, 3);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	g_signal_connect_swapped(ctrls->uh, "value-changed", G_CALLBACK(unc_depth_changed), data);
	
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), label_new_with_markup_left("nm"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);

	j = 3;
	ctrls->uSuh = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uSuh), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uSuh, j, j+1, i, i+1, GTK_FILL, 0, 0, 0);
	j++;
	
	ctrls->uhcuh = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uhcuh), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uhcuh, j, j+1, i, i+1, GTK_FILL, 0, 0, 0);
	j++;
	
	ctrls->uAuh = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uAuh), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uAuh, j, j+1, i, i+1, GTK_FILL, 0, 0, 0);
	j++;
	
	ctrls->uHituh = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uHituh), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uHituh, j, j+1, i, i+1, GTK_FILL, 0, 0, 0);
	j++;
	
	ctrls->uEruh = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEruh), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEruh, j, j+1, i, i+1, GTK_FILL, 0, 0, 0);
	j++;
	
	ctrls->uEituh = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEituh), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEituh, j, j+1, i, i+1, GTK_FILL, 0, 0, 0);
	j++;
	
	i++;

	/* u(F) */

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(F)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->uF = gtk_adjustment_new(unc->uF, 0, 1e6, 0.0001, 0.01,0.);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->uF), 0.0001, 4);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	g_signal_connect_swapped(ctrls->uF, "value-changed", G_CALLBACK(unc_load_changed), data);
	
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), label_new_with_markup_left("mN"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);

	j = 3;
	ctrls->uSuF = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uSuF), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uSuF, j, j+1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	j++;

	ctrls->uhcuF = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uhcuF), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uhcuF, j, j+1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	j++;

	ctrls->uAuF = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uAuF), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uAuF, j, j+1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	j++;
	
	ctrls->uHituF = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uHituF), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uHituF, j, j+1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	j++;
	
	ctrls->uEruF = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEruF), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEruF, j, j+1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	j++;
	
	ctrls->uEituF = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEituF), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEituF, j, j+1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	j++;
	
	i++;

	/* u(nu) */

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(ν)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->unu = gtk_adjustment_new(unc->unu, 0, 0.5, 0.01, 0.1, 0.);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->unu), 0.01, 2);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	g_signal_connect_swapped(ctrls->unu, "value-changed", G_CALLBACK(unc_nu_changed), data);
	
	ctrls->uEitunu = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEitunu), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEitunu, 8, 9, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

	i++;

	/* u(nu_i) */

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(ν<sub>i</sub>)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->unui = gtk_adjustment_new(unc->unui, 0, 0.5, 0.01, 0.1, 0.);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->unui), 0.01, 2);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	g_signal_connect_swapped(ctrls->unui, "value-changed", G_CALLBACK(unc_nui_changed), data);
	
	ctrls->uEitunui = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEitunui), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEitunui, 8, 9, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	i++;

	/* u(E_i) */

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(E<sub>i</sub>)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->uEi = gtk_adjustment_new(unc->uEi*1e-9, 0, 1e5, 1, 10, 0.);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->uEi), 1, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	g_signal_connect_swapped(ctrls->uEi, "value-changed", G_CALLBACK(unc_Ei_changed), data);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), label_new_with_markup_left("GPa"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->uEituEi = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEituEi), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEituEi, 8, 9, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	i++;

	/* total */
	
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("Total"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);
	
	j = 3;
	ctrls->uS = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uS), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uS, j, j+1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	j++;
	
	ctrls->uhc = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uhc), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uhc, j, j+1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	j++;
	
	ctrls->uA = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uA), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uA, j, j+1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	j++;
	
	ctrls->uHit = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uHit), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uHit, j, j+1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	j++;
	
	ctrls->uEr = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEr), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEr, j, j+1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	j++;
	
	ctrls->uEit = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEit), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEit, j, j+1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	j++;


	/* Uncertainties due to contact point */

	ctrls->frame_unc_contact = gtk_frame_new(NULL);
	gtk_frame_set_label_widget (GTK_FRAME (ctrls->frame_unc_contact),
				    GTK_WIDGET (gwy_label_new_header("Uncertainties due to choice of contact point")));
	
	ctrls->table_unc_contact = gtk_table_new(6, 4, FALSE);
	gtk_container_set_border_width (GTK_CONTAINER (ctrls->table_unc_contact), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (ctrls->table_unc_contact), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (ctrls->table_unc_contact), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add (GTK_CONTAINER (ctrls->frame_unc_contact), ctrls->table_unc_contact);

	/* table headers */

	i=0;
	j=0;
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact),
			 gwy_label_new_header("Contact point shift"), j, j+1, i, i+1,  GTK_EXPAND | GTK_FILL, 0, 0, 0);
	j++;
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact),
			 gwy_label_new_header("S/mN.nm<sup>-1</sup>"), j, j+1, i, i+1,  GTK_EXPAND | GTK_FILL, 0, 0, 0);
	j++;
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact),
			 gwy_label_new_header("E<sub>r</sub>/GPa"), j, j+1, i, i+1,  GTK_EXPAND | GTK_FILL, 0, 0, 0);
	j++;
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact),
			 gwy_label_new_header("H<sub>it</sub>/GPa"), j, j+1, i, i+1,  GTK_EXPAND | GTK_FILL, 0, 0, 0);
	j++;
	// gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact),
	// gwy_label_new_header("weight"), 2, 3, i,i+1,  GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);

	i++;

	// Ec is Er, Hc is Hit, Sc is S
	unc->Ec = (gdouble *)g_malloc((2*CONTACTMAX+1)*sizeof(gdouble));
	unc->Hc = (gdouble *)g_malloc((2*CONTACTMAX+1)*sizeof(gdouble));
	unc->Sc = (gdouble *)g_malloc((2*CONTACTMAX+1)*sizeof(gdouble));

	/* values */

	for (j = -CONTACTMAX; j <= CONTACTMAX; j++)	{

	    /* vertical alignments: j: 1.0 (bottom), others: 0.0 (top); looks quite nice, still probably incorrect */
	    
		k = 0;
		label = gtk_label_new(NULL);
		g_snprintf(str, sizeof(str),"%d\n", j); 
		gtk_label_set_text (GTK_LABEL(label), str);
		gtk_misc_set_alignment (GTK_MISC(label), 0.0, 1.0);
		gtk_table_attach (GTK_TABLE (ctrls->table_unc_contact), label, k, k+1, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 
		k++;

		tangent_fit_shift_contact(&(data->args->tgdata), &(data->args->fddata), &(data->args->instdata), &(data->args->area), unc->Sc, unc->Ec, unc->Hc, j);

		//fill in S
		label = gtk_label_new(NULL);
		if (unc->Sc[j+CONTACTMAX] > 0)
			g_snprintf(str, sizeof(str), SLOPE, unc->Sc[j+CONTACTMAX]); 
		else
			g_snprintf(str, sizeof(str), "not defined");
		gtk_label_set_text (GTK_LABEL(label), str);
		gtk_misc_set_alignment (GTK_MISC(label), 0.0, 0.0); 
		gtk_table_attach (GTK_TABLE(ctrls->table_unc_contact), label, k, k+1, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 
		k++;


		//fill in Er
		label = gtk_label_new(NULL);
		if (unc->Ec[j+CONTACTMAX] > 0)
			g_snprintf(str, sizeof(str), MODUL, unc->Ec[j+CONTACTMAX]); 
		else
			g_snprintf(str, sizeof(str), "not defined");
		gtk_label_set_text (GTK_LABEL(label), str);
		gtk_misc_set_alignment (GTK_MISC(label), 0.0, 0.0); 
		gtk_table_attach (GTK_TABLE(ctrls->table_unc_contact), label, k, k+1, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 
		k++;

		/*
		label = gtk_label_new(NULL);
		g_snprintf(str, sizeof(str),"1/%d", 2*CONTACTMAX+1); 
		gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0); 
		gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact), label, 2, 3, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 
		*/

		//fill in Hit
		label = gtk_label_new(NULL);
		if (unc->Hc[j+CONTACTMAX] > 0)
			g_snprintf(str, sizeof(str), HARD, unc->Hc[j+CONTACTMAX]); 
		else
			g_snprintf(str, sizeof(str), "not defined");
		gtk_label_set_text (GTK_LABEL(label), str);
		gtk_misc_set_alignment (GTK_MISC(label), 0.0, 0.0); 
		gtk_table_attach (GTK_TABLE(ctrls->table_unc_contact), label, k, k+1, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

		i++;

	}

	tangent_propagate_uncertainties (data);

	ctrls->button_save = gtk_button_new_from_stock (GTK_STOCK_SAVE);
	g_signal_connect_swapped (ctrls->button_save, "clicked", G_CALLBACK (tangent_uncertainty_save_data), data->args);

	/* Monte Carlo calculation */
	
	ctrls->frame_mc = gtk_frame_new(NULL);
	gtk_frame_set_label_widget (GTK_FRAME (ctrls->frame_mc),
				    GTK_WIDGET (gwy_label_new_header("Monte Carlo calculation of uncertainties")));

	ctrls->table_mc = gtk_table_new(6, 7, FALSE);

	gtk_container_set_border_width (GTK_CONTAINER (ctrls->table_mc), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (ctrls->table_mc), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (ctrls->table_mc), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add ( GTK_CONTAINER (ctrls->frame_mc), ctrls->table_mc);

	i = 0;
	gtk_table_attach(GTK_TABLE(ctrls->table_mc), gwy_label_new_header("Number of iterations"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->Nmc = gtk_adjustment_new(unc->Nmc, 1, 1e9, 10, 100,0); 
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->Nmc), 1, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(ctrls->table_mc), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	g_signal_connect_swapped(ctrls->Nmc, "value-changed", G_CALLBACK(mc_iterations_changed), data);
	
	i++;

	ctrls->mc_warn = gtk_label_new(NULL);
	gtk_table_attach(GTK_TABLE(ctrls->table_mc), ctrls->mc_warn, 0, 3, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	if (unc->Nmc > MC_WHEN_WARN){
		g_snprintf(str, sizeof(str), "Warning: Monte Carlo calculation for %d iterations may take very long!", unc->Nmc);
		gtk_label_set_text(GTK_LABEL(ctrls->mc_warn), str);
	}

	// no Monte Carlo window without the uncertainty dialog
	data->controls->tgmccontrols->has_window = FALSE;
	ctrls->button_mc = gtk_button_new_with_mnemonic ("_Monte Carlo");
	gtk_button_set_image (GTK_BUTTON (ctrls->button_mc), gtk_image_new_from_stock (GWY_STOCK_POLYNOM, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (ctrls->button_mc, "clicked", G_CALLBACK (tangent_uncertainty_montecarlo), data);

	ctrls->vbox = gtk_vbox_new(FALSE, CTRLS_SPACING);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->frame_unc_input, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->frame_unc_contact, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->button_save, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->frame_mc, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->button_mc, FALSE, FALSE, 0);
	
	gtk_container_add(GTK_CONTAINER(ctrls->window), ctrls->vbox);
	gtk_widget_show_all(ctrls->window);
}
