#ifndef NOFORTRAN

#define _USE_MATH_DEFINES
#include <math.h>

#include <stdlib.h>
#include <string.h>

#include <gtk/gtk.h>

#include <app/file.h>

#include "indent-toolbox-common.h"
#include "datatypes.h"
#include "controls.h"
#include "fit-utils.h"
#include "gui-utils.h"
#include "tool_twoslopes.h"
#include "tool_twoslopes_unc.h"


#define BETA 1.0


static const gchar *logfnmload = "fit.log.load";
static const gchar *logfnmunload = "fit.log.unload";

static void update_models_slopes(SlopesControls *slcontrols);
static void slopes_zoom_in(Data *data);
static void slopes_zoom_out(Data *data);

static void slopes_show_log_load(Data *data);
static void slopes_show_log_unload(Data *data);

void
init_Slopesdata(Slopesdata *sldata)
{
	sldata->gamma = 0;
	sldata->h0=0;
	sldata->n=0;

	sldata->alpha=0;
	sldata->hp=0;
	sldata->m=0;

	sldata->eps=0;
	sldata->Sload=0;
	sldata->Sunload=0;

	sldata->Aphc=0;

	sldata->has_fit_load=FALSE;
	sldata->loadto=0;
	sldata->loadfrom=0;
	sldata->loadto_pct_Fmax=0;
	sldata->loadfrom_pct_Fmax=0;
	sldata->loadrange[0]=0;
	sldata->loadrange[1]=0;
	sldata->loadrange_pct_Fmax[0]=0;
	sldata->loadrange_pct_Fmax[1]=0;

	sldata->has_fit_unload=FALSE;
	sldata->unloadto=0;
	sldata->unloadfrom=0;
	sldata->unloadto_pct_Fmax=0;
	sldata->unloadfrom_pct_Fmax=0;
	sldata->unloadrange[0]=0;
	sldata->unloadrange[1]=0;
	sldata->unloadrange_pct_Fmax[0]=0;
	sldata->unloadrange_pct_Fmax[1]=0;

	sldata->xloadfit=NULL;
	sldata->yloadfit=NULL;
	sldata->xunloadfit=NULL;
	sldata->yunloadfit=NULL;

	sldata->Er=0;
	sldata->Eit=0;
	sldata->Hit=0;

	sldata->Finputload = FALSE;
	sldata->nfitloaddata = 0;
	sldata->Finputunload = FALSE;
	sldata->nfitunloaddata = 0;


	// TODO build logfilename
	sldata->logfnmload =  g_malloc(strlen(logfnmload)*sizeof(gchar));
	strcpy(sldata->logfnmload, logfnmload);

	sldata->infologload = 0;

	sldata->logfnmunload =  g_malloc(strlen(logfnmunload)*sizeof(gchar));
	strcpy(sldata->logfnmunload, logfnmunload);

	sldata->infologunload = 0;
}

void
init_SlopesUncdata (SlopesUncdata *slopesunc)
{
	slopesunc->uh = 1;
	slopesunc->uF = 0.001;
	slopesunc->unu = 0.05;
	slopesunc->unui = 0.04;
	slopesunc->uEi = 20e9;
	slopesunc->Nmc = N_MC;

	slopesunc->ubeta[0] = 0;
	slopesunc->ubeta[1] = 0;
	slopesunc->ubeta[2] = 0;

	slopesunc->umuh = 0;
	slopesunc->umuF = 0;
	slopesunc->unuh = 0;
	slopesunc->unuF = 0;

	slopesunc->uSuuh = 0;
	slopesunc->uSuuF = 0;
	slopesunc->uSluh = 0;
	slopesunc->uSluF = 0;


	slopesunc->uAuh = 0;
	slopesunc->uAuF = 0;
	slopesunc->uA = 0;

	slopesunc->uHituh = 0;
	slopesunc->uHituF = 0;
	slopesunc->uHit = 0;

	slopesunc->uEruh = 0;
	slopesunc->uEruF = 0;
	slopesunc->uEr = 0;

	slopesunc->uEituh = 0;
	slopesunc->uEituF = 0;
	slopesunc->uEitunu = 0;
	slopesunc->uEitunui = 0;
	slopesunc->uEituEi = 0;
	slopesunc->uEit = 0;


	slopesunc->Ec=NULL;
	slopesunc->Hc=NULL;
	slopesunc->Ac=NULL;

}

void
slopes_remove_all_fit_results_unload(Slopesdata *sldata)
{
	//reset all results associated with the powerlaw fit of the unloading curve to zero
	sldata->alpha=0;
	sldata->hp=0;
	sldata->m=0;

	sldata->eps=0;
	sldata->Sunload=0;

	sldata->Aphc=0;
	sldata->Er=0;
	sldata->Eit=0;
	sldata->Hit=0;

	sldata->unloadto=0;
	sldata->unloadfrom=0;
	sldata->unloadrange[0]=0;
	sldata->unloadrange[1]=0;
	sldata->unloadrange_pct_Fmax[0]=0;
	sldata->unloadrange_pct_Fmax[1]=0;

	sldata->Finputunload = FALSE;
	sldata->has_fit_unload = FALSE;
	sldata->nfitunloaddata = 0;

	sldata->infologunload = 0;
}

void
slopes_remove_all_fit_results_load(Slopesdata *sldata)
{
	//reset all results associated with the powerlaw fit of the loading curve to zero
	sldata->gamma=0;
	sldata->h0=0;
	sldata->n=0;

	sldata->Sload=0;

	sldata->Aphc=0;
	sldata->Er=0;
	sldata->Eit=0;
	sldata->Hit=0;

	sldata->loadto=0;
	sldata->loadfrom=0;
	sldata->loadrange[0]=0;
	sldata->loadrange[1]=0;
	sldata->loadrange_pct_Fmax[0]=0;
	sldata->loadrange_pct_Fmax[1]=0;

	sldata->Finputload = FALSE;
	sldata->has_fit_load = FALSE;
	sldata->nfitloaddata = 0;

	sldata->infologload = 0;
}

void
slopes_remove_all_fit_labels_unload(SlopesControls *slcontrols)
{
    /* remove all labels that result from the powerlaw fit of the unloading curve */
    
    entry_clear(slcontrols->from);
    entry_clear(slcontrols->to);
    entry_clear(slcontrols->from_pct_Fmax);
    entry_clear(slcontrols->to_pct_Fmax);

    label_clear(slcontrols->fitinfounload);
    label_clear(slcontrols->hp);
    label_clear(slcontrols->Sunload);
    label_clear(slcontrols->m);
    label_clear(slcontrols->eps);
    label_clear(slcontrols->Aphc);

    label_clear(slcontrols->Hit);
    label_clear(slcontrols->Er);
    label_clear(slcontrols->Eit);
    label_clear(slcontrols->unloadrange);
    label_clear(slcontrols->unloadrange_pct_Fmax);
}

void
slopes_remove_all_fit_labels_load(SlopesControls *slcontrols)
{
    /* remove all labels that result from the powerlaw fit of the loading curve */
    entry_clear(slcontrols->from);
    entry_clear(slcontrols->to);
    entry_clear(slcontrols->from_pct_Fmax);
    entry_clear(slcontrols->to_pct_Fmax);

    label_clear(slcontrols->fitinfoload);
    label_clear(slcontrols->h0);
    label_clear(slcontrols->Sload);
    label_clear(slcontrols->n);
    label_clear(slcontrols->Aphc);

    label_clear(slcontrols->Hit);
    label_clear(slcontrols->Er);
    label_clear(slcontrols->Eit);
    label_clear(slcontrols->loadrange);
    label_clear(slcontrols->loadrange_pct_Fmax);
}

void
slopes_remove_all_fit_curves(SlopesControls *slcontrols)
{
    gwy_graph_model_remove_curve_by_description(slcontrols->graph_model, "Loading fit");
    gwy_graph_model_remove_curve_by_description(slcontrols->graph_model, "Unloading fit");
}

static void recalculate_Eit(Data *data)
{
    gdouble Er;
    gdouble nu, nui, Ei, Eit;
    gchar str[300];

    nu = data->args->instdata.nu;
    nui = data->args->instdata.nui;
    Ei = data->args->instdata.Ei;
    Er = data->args->slopesdata.Er * 1e9;

    // recalc for changed  nu, nui or Ei
    if (Er == 0)
        return;
    
    Eit = (1 - nu*nu) / (1/Er - (1 - nui*nui)/Ei);
    data->args->slopesdata.Eit = Eit * 1e-9;
    g_snprintf(str, sizeof(str), MODUL, data->args->slopesdata.Eit);
    gtk_label_set_text(GTK_LABEL(data->controls->slopescontrols->Eit), str);

}

void slopes_active(Data *data)
{
	gchar str[300];
	GwyGraphCurveModel *cmodel;

	// update loading curve and Fmax and hmax labels
	if (data->args->fddata.hunload && data->args->fddata.hload && data->args->fddata.hhold) {

		gwy_graph_curve_model_set_data(data->controls->slopescontrols->cmodelload,
				data->args->fddata.hload->data, data->args->fddata.Fload->data,
				data->args->fddata.hload->res);

		gwy_graph_curve_model_set_data(data->controls->slopescontrols->cmodelhold,
				data->args->fddata.hhold->data, data->args->fddata.Fhold->data,
				data->args->fddata.hhold->res);

		gwy_graph_curve_model_set_data(data->controls->slopescontrols->cmodelunload,
				data->args->fddata.hunload->data, data->args->fddata.Funload->data,
				data->args->fddata.hunload->res);

		gwy_graph_curve_model_enforce_order (data->controls->slopescontrols->cmodelload);
		gwy_graph_curve_model_enforce_order (data->controls->slopescontrols->cmodelhold);
		gwy_graph_curve_model_enforce_order (data->controls->slopescontrols->cmodelunload);

		g_snprintf(str,sizeof(str), DEPTH, data->args->fddata.hmax);
		gtk_label_set_text(GTK_LABEL(data->controls->slopescontrols->hmax), str);
		g_snprintf(str,sizeof(str), FORCE, data->args->fddata.Fmax);
		gtk_label_set_text(GTK_LABEL(data->controls->slopescontrols->Fmax), str);


		cmodel = gwy_graph_model_get_curve_by_description (data->controls->slopescontrols->graph_model, "Loading fit");
		if (cmodel){
			gwy_graph_curve_model_set_data(data->controls->slopescontrols->cmodelloadfit, 
					data->args->slopesdata.xloadfit->data, data->args->slopesdata.yloadfit->data,  data->args->slopesdata.xloadfit->res);
			gwy_graph_curve_model_enforce_order (data->controls->slopescontrols->cmodelloadfit);
		}
		cmodel = gwy_graph_model_get_curve_by_description (data->controls->slopescontrols->graph_model, "Unloading fit");
		if (cmodel){
			gwy_graph_curve_model_set_data(data->controls->slopescontrols->cmodelunloadfit, 
					data->args->slopesdata.xunloadfit->data, data->args->slopesdata.yunloadfit->data,  data->args->slopesdata.xunloadfit->res);
			gwy_graph_curve_model_enforce_order (data->controls->slopescontrols->cmodelunloadfit);
		}
	}
}

static void
slopes_range_from_changed(Data *data)
{
    SlopesControls *fcontrols;
    Slopesdata *fdata;

    fcontrols = data->controls->slopescontrols;
    fdata = &(data->args->slopesdata);
	
    range_from_changed(fcontrols->from, fcontrols->to, fcontrols->selection, &(fdata->loadfrom), &(fdata->unloadfrom));
}

static void
slopes_range_to_changed(Data *data)
{
    SlopesControls *fcontrols;
    Slopesdata *fdata;

    fcontrols = data->controls->slopescontrols;
    fdata = &(data->args->slopesdata);
	
    range_to_changed(fcontrols->from, fcontrols->to, fcontrols->selection, &(fdata->loadto), &(fdata->unloadto));
}

static gboolean
slopes_range_from_pct_Fmax_changed(Data *data)
{
    SlopesControls *rcontrols;
    Slopesdata *rdata;

    rcontrols = data->controls->slopescontrols;
    rdata = &(data->args->slopesdata);

    return range_from_pct_Fmax_changed(rcontrols->from_pct_Fmax, rcontrols->to_pct_Fmax, rcontrols->selection, data->args->fddata.Fmax, data->args->fddata.hunload, data->args->fddata.Funload,
				       &(rdata->loadfrom_pct_Fmax), &(rdata->unloadfrom_pct_Fmax), &(rdata->loadto_pct_Fmax), &(rdata->unloadto_pct_Fmax), &(rdata->Finputload), &(rdata->Finputunload));
}

static gboolean
slopes_range_to_pct_Fmax_changed(Data *data)
{
    SlopesControls *rcontrols;
    Slopesdata *rdata;

    rcontrols = data->controls->slopescontrols;
    rdata = &(data->args->slopesdata);

    return range_to_pct_Fmax_changed(rcontrols->from_pct_Fmax, rcontrols->to_pct_Fmax, rcontrols->selection, data->args->fddata.Fmax, data->args->fddata.hunload, data->args->fddata.Funload,
				     &(rdata->loadfrom_pct_Fmax), &(rdata->unloadfrom_pct_Fmax), &(rdata->loadto_pct_Fmax), &(rdata->unloadto_pct_Fmax), &(rdata->Finputload), &(rdata->Finputunload));
}

static void
slopes_graph_selected( GwySelection* selection, gint i,Data *data )
{
    gchar buffer[24];
    gdouble range[2];
    gint istart, iend, ndata;
    gdouble Fmax;


    g_return_if_fail(i <= 0);

    //take data from selection and put them in the "from" and "to" values for both hp and S (at this moment, we don't know yet which fit will be chosen), create labels only once
    //SlopesControls has entries from and to, which are filled here
    // and labels hp,unloadrange, where the values of the ranges that were actually used in the corresponding fits  are shown
    //Slopesdata has loadfrom,loadto, unloadfrom,unloadto which are the data from the entries, i.e. from the selection
    //Slopesdata; also has loadrange, unloadrange where the ranges that were actually used in the fit 
    
    gwy_selection_get_object(selection, 0, range);

    data->args->slopesdata.unloadfrom = MIN(range[0], range[1]);
    data->args->slopesdata.unloadto = MAX(range[0], range[1]);

    data->args->slopesdata.loadfrom = MIN(range[0], range[1]);
    data->args->slopesdata.loadto = MAX(range[0], range[1]);

    g_snprintf(buffer, sizeof(buffer), DEPTH, data->args->slopesdata.unloadfrom);
    gtk_entry_set_text(GTK_ENTRY(data->controls->slopescontrols->from), buffer);
    g_snprintf(buffer, sizeof(buffer), DEPTH, data->args->slopesdata.unloadto);
    gtk_entry_set_text(GTK_ENTRY(data->controls->slopescontrols->to), buffer);

    Fmax = data->args->fddata.Fmax;
    range_to_indices(data->args->slopesdata.unloadfrom,data->args->slopesdata.unloadto , data->args->fddata.hunload, TRUE, &istart, &iend, &ndata);
    data->args->slopesdata.loadfrom_pct_Fmax = data->args->fddata.Funload->data[iend]/Fmax*100;
    data->args->slopesdata.loadto_pct_Fmax = data->args->fddata.Funload->data[istart]/Fmax*100;
    data->args->slopesdata.unloadfrom_pct_Fmax = data->args->fddata.Funload->data[iend]/Fmax*100;
    data->args->slopesdata.unloadto_pct_Fmax = data->args->fddata.Funload->data[istart]/Fmax*100;


    g_snprintf(buffer, sizeof(buffer), DEPTH, data->args->slopesdata.unloadfrom_pct_Fmax); 
    gtk_entry_set_text(GTK_ENTRY(data->controls->slopescontrols->from_pct_Fmax), buffer); 
    g_snprintf(buffer, sizeof(buffer), DEPTH, data->args->slopesdata.unloadto_pct_Fmax); 
    gtk_entry_set_text(GTK_ENTRY(data->controls->slopescontrols->to_pct_Fmax), buffer); 

    data->args->slopesdata.Finputload = FALSE;
    data->args->slopesdata.Finputunload = FALSE;
}

void slopes_write_file(gchar *filename, Args *args) 
{
	FILE *file;
	gint i;
	Slopesdata *sldata;
	Instdata *instdata;

	sldata = &(args->slopesdata);
	instdata = &(args->instdata);

	if (args->fddata.hall->res == 0 || args->fddata.Fall->res==0) {
	    show_error("No data.");
	    return;
	}

	file = fopen(filename, "w");
	if (!file) {
	    show_error("Could not open file.");
	    return;
	}

	fprintf(file, "# %s \n", args->fddata.filename);
	//Input parameters
	fprintf(file, "#Parameters of Slopes  analysis \n");
	fprintf(file, "#nu: %g \n", instdata->nu);
	fprintf(file, "#nui: %g \n", instdata->nui);
	fprintf(file, "#Ei: %g GPa\n", instdata->Ei * 1e-9); //TODO units 

	fprintf(file, "\n\n");
	fprintf(file, "#Results of Slopes  analysis \n");
	fprintf(file, "#hmax: %g nm\n", args->fddata.hmax);
	fprintf(file, "#Fmax: %g mN\n", args->fddata.Fmax);
	fprintf(file, "#Fit of loading curve     gamma (h-h0)^n\n");
	fprintf(file, "#gamma: %g mN.nm^{-%g}\n", sldata->gamma, sldata->n);
	fprintf(file, "#h0: %g nm\n", sldata->h0);
	fprintf(file, "#n: %g nm\n", sldata->n);
	fprintf(file, "#Fit of unloading curve     alpha (h-hp)^m\n");
	fprintf(file, "#alpha: %g mN.nm^{-%g}\n", sldata->alpha, sldata->m);
	fprintf(file, "#hp: %g nm\n", sldata->hp);
	fprintf(file, "#m: %g nm\n", sldata->m);
	fprintf(file, "#epsilon: %g \n", sldata->eps);
	fprintf(file, "#Sload: %g mN/nm\n", sldata->Sload);
	fprintf(file, "#Sunload: %g mN/nm\n", sldata->Sunload);
	fprintf(file, "#Aphc: %g nm^2\n", sldata->Aphc);
	fprintf(file, "#Hit: %g MPa\n", sldata->Hit);
	fprintf(file, "#Er: %g GPa\n", sldata->Er);
	fprintf(file, "#Eit: %g GPa\n", sldata->Eit);


	if (args->slopesdata.has_fit_load) {
		fprintf(file, "\n \n");
		fprintf(file, "#Fitted loadrange: ");
		if (args->slopesdata.Finputload)
			fprintf(file, "	[%g , %g ] %% Fmax \n", sldata->loadrange_pct_Fmax[0], sldata->loadrange_pct_Fmax[1]);
		else
			fprintf(file, "[%g , %g ] nm \n",sldata->loadrange[0], sldata->loadrange[1]);	
		fprintf(file, "# corresponds : ");
		if (args->slopesdata.Finputload)
			fprintf(file, "[%g , %g ] nm \n",sldata->loadrange[0], sldata->loadrange[1]);	
		else
			fprintf(file, "	[%g , %g ] %% Fmax \n", sldata->loadrange_pct_Fmax[0], sldata->loadrange_pct_Fmax[1]);
		fprintf(file, "# %d datapoints used.\n", sldata->nfitloaddata);

		fprintf(file,"\n# Loading Fit \n");
		fprintf(file, "# x/nm y/mN \n");
		for (i = 0; i < args->slopesdata.xloadfit->res; i++)
			fprintf(file, "%g %g \n", args->slopesdata.xloadfit->data[i], args->slopesdata.yloadfit->data[i]);
	}
	if (args->slopesdata.has_fit_unload) {
		fprintf(file, "\n \n");
		fprintf(file, "#Fitted unload range: ");
		if (args->slopesdata.Finputunload)
			fprintf(file, "	[%g , %g ] %% Fmax \n", sldata->unloadrange_pct_Fmax[0], sldata->unloadrange_pct_Fmax[1]);
		else
			fprintf(file, "[%g , %g ] nm \n",sldata->unloadrange[0], sldata->unloadrange[1]);	
		fprintf(file, "# corresponds : ");
		if (args->slopesdata.Finputunload)
			fprintf(file, "[%g , %g ] nm \n",sldata->unloadrange[0], sldata->unloadrange[1]);	
		else
			fprintf(file, "	[%g , %g ] %% Fmax \n", sldata->unloadrange_pct_Fmax[0], sldata->unloadrange_pct_Fmax[1]);
		fprintf(file, "# %d datapoints used.\n", sldata->nfitunloaddata);

		fprintf(file,"\n# Unloading Fit \n");
		fprintf(file, "# x/nm y/mN \n");
		for (i = 0; i < args->slopesdata.xunloadfit->res; i++)
			fprintf(file, "%g %g \n", args->slopesdata.xunloadfit->data[i], args->slopesdata.yunloadfit->data[i]);
	}
	fclose(file); 

}

void slopes_save_data(Args *args)
{
	GtkWidget *dialog;
	gchar *filename;

	dialog = gtk_file_chooser_dialog_new("Save File",
			NULL,
			GTK_FILE_CHOOSER_ACTION_SAVE,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_SAVE, GTK_RESPONSE_OK,
			NULL);

	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), gwy_app_get_current_directory());

	if ( gtk_dialog_run (GTK_DIALOG ( dialog)) == GTK_RESPONSE_OK)
	{
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		if (gwy_app_file_confirm_overwrite(dialog))
			slopes_write_file(filename, args);
		g_free(filename);
	}

	gtk_widget_destroy ( dialog);

}

void
slopes_combine_fit_results(Slopesdata *sldata, FDdata *fddata, Instdata *instdata)
{
	gdouble C0=24.5, Fmax ;
	gdouble K;
	// TODO 24.5

	Fmax = fddata->Fmax;

	K= (2*sldata->Sunload - sldata->Sload*sldata->eps*BETA)/sldata->Sunload/sldata->Sload;
	//printf(" K %g K(beta=1) %g  \n", K, (2*sldata->Sunload - sldata->Sload*sldata->eps*BETA)/sldata->Sunload/sldata->Sload);

	//TODO Units
	sldata->Aphc = C0*Fmax*Fmax*K*K;
	sldata->Hit = 1/C0/Fmax/K/K*1e9;

	sldata->Er = sqrt(M_PI/C0)/2/BETA/Fmax/K*sldata->Sunload*1e6;

	sldata->Eit = (1 - instdata->nu * instdata->nu) / (1 / sldata->Er - (1 - instdata->nui * instdata->nui) / (instdata->Ei * 1e-9));

	/*printf("BETA %g \n", BETA);
	printf("Hit %g \n", sldata->Hit);
	printf("Er %g \n", sldata->Er);
	printf("Eit %g \n", sldata->Eit);
	*/
}

gint 
slopes_fit_load(GwyDataLine *x, GwyDataLine *y, Slopesdata *sldata, FDdata *fddata, Instdata *instdata, gdouble estimate[3])
{
        gdouble beta[3], std_beta[3], covariance[3];
	gint info;
	gdouble ux, uy;
	//gint istart,iend,ndata = 0;
//	gdouble r1 = 0.05,r2 = 0.1, rstep = 0.05;


	//fit the data with the function beta[0]*(h-beta[1])^beta[2]
	if (estimate){
		beta[0]=estimate[0];
		beta[1]=estimate[1];
		beta[2]=estimate[2];
	}
	else{
		beta[2] = 2.0;
		beta[0] = fddata->Fmax/pow(fddata->hmax,beta[2]);
		beta[1] = 1;
		/*
		while ( ndata ==0 && r2 < 1.0 ){
			range_to_indices(r1*fddata->Fmax, r2*fddata->Fmax, fddata->Fload,FALSE, &istart, &iend, &ndata); 
			r1+=rstep;
			r2+=rstep;
		}
		if (ndata > 0)
			beta[1] = fddata->hunload->data[iend];
		else
			beta[1] = fddata->hmax*0.05;
			*/
	}
	// no uncertainties at this stage
	ux = 0;
	uy = 0;


	info = odr_run_fit(x, y, beta, sldata->logfnmload, ux, uy, std_beta, covariance, FALSE); /* do not rescale */
//	printf("load after beta %g %g %g  \n\n", beta[0], beta[1], beta[2]);

	// calculate results 
	sldata->gamma = beta[0];
	sldata->h0 = beta[1];
	sldata->n = beta[2];

	sldata->Sload = sldata->n * fddata->Fmax / (fddata->hmax - sldata->h0);

	if (sldata->has_fit_unload){
	    slopes_combine_fit_results(sldata, fddata, instdata);
	}

	//clean up
	sldata->loadrange[0] = sldata->loadfrom;
	sldata->loadrange[1] = sldata->loadto;
	sldata->has_fit_load = TRUE;
	sldata->loadrange_pct_Fmax[0] = sldata->loadfrom_pct_Fmax;
	sldata->loadrange_pct_Fmax[1] = sldata->loadto_pct_Fmax;
	
	return info;
}


gint 
slopes_fit_unload(GwyDataLine *x, GwyDataLine *y, Slopesdata *sldata, FDdata *fddata, Instdata *instdata, gdouble estimate[3])
{
        gdouble beta[3], std_beta[3], covariance[3];
	gint info;
	gdouble ux, uy;
	gint istart,iend,ndata = 0;
	gdouble r1 = 0.05,r2 = 0.1, rstep = 0.05;


	//fit the data with the function beta[0]*(h-beta[1])^beta[2]
	if (estimate){
		beta[0]=estimate[0];
		beta[1]=estimate[1];
		beta[2]=estimate[2];
	}
	else{
		beta[2] = 1.5;
		beta[0] = fddata->Fmax/pow(fddata->hmax,beta[2]);
		while ( ndata ==0 && r2 < 1.0 ){
			range_to_indices(r1*fddata->Fmax, r2*fddata->Fmax, fddata->Funload,TRUE, &istart, &iend, &ndata); 
			r1+=rstep;
			r2+=rstep;
		}
		if (ndata > 0)
			beta[1] = fddata->hunload->data[iend];
		else
			beta[1] = fddata->hmax*0.05;
	}
	// no uncertainties at this stage
	ux = 0;
	uy = 0;

	info = odr_run_fit(x, y, beta, sldata->logfnmunload, ux, uy, std_beta, covariance, FALSE); /* do not rescale */
//	printf("unload after beta %g %g %g  \n\n", beta[0], beta[1], beta[2]);

	// calculate results 
	sldata->alpha = beta[0];
	sldata->hp = beta[1];
	sldata->m = beta[2];
	if (sldata->m <=1)
		sldata->eps = 0.75;
	else
		sldata->eps = sldata->m * (1 - 2*(sldata->m-1) * tgamma(sldata->m / 2 / (sldata->m - 1)) / tgamma(1.0 / 2.0 / (sldata->m - 1)) / sqrt(M_PI));

	sldata->Sunload = sldata->m * fddata->Fmax / (fddata->hmax - sldata->hp);

	if (sldata->has_fit_load)
	    slopes_combine_fit_results(sldata, fddata, instdata);

	//clean up
	sldata->unloadrange[0] = sldata->unloadfrom;
	sldata->unloadrange[1] = sldata->unloadto;
	sldata->has_fit_unload = TRUE;
	sldata->unloadrange_pct_Fmax[0] = sldata->unloadfrom_pct_Fmax;
	sldata->unloadrange_pct_Fmax[1] = sldata->unloadto_pct_Fmax;
	
	return info;
}

void 
slopes_fit_and_update_load(Data *data)
{
	gchar str[300];
	gint i;
	gint istart, iend, ndata;
	gboolean fit=TRUE;
	GwyDataLine *x, *y;
	gint fitinfo;

	Slopesdata *sldata;
	SlopesControls *slcontrols;
	FDdata *fddata;
	Instdata *instdata;
	GdkColor color;
	GwyGraphCurveModel *cmodel;


	gdk_color_parse ("red", &color);



	//TEMPORARY update GtkEntries for ranges
	/*
	   if (data->args->slopesdata.Finputload){
	   range_from_pct_Fmax_changed(data);
	   range_to_pct_Fmax_changed(data);
	   }
	   else{
	   range_from_changed(data);
	   range_to_changed(data);
	   }
	   */

	//create pointers for easier reading
	sldata = &(data->args->slopesdata);
	fddata = &(data->args->fddata);
	instdata = &(data->args->instdata);
	slcontrols = data->controls->slopescontrols;

	//if there was a previous fit, invalidate previous fitted curve
	if (sldata->has_fit_load){
		g_object_unref(sldata->xloadfit);
		g_object_unref(sldata->yloadfit);
	}

	printf("from %g to %g \n", sldata->loadfrom, sldata->loadto);

	//check range
	if (sldata->loadfrom == sldata->loadto){
		fit = FALSE;
	}

	//find appropriate data, which should be fitted
	//default regime is governed by depth, can be switched to load regime by Finput (if the pct_Fmax ranges were used)
	if (sldata->Finputload)
	{
		range_to_indices(sldata->loadfrom_pct_Fmax*0.01*fddata->Fmax, sldata->loadto_pct_Fmax*0.01*fddata->Fmax,
				fddata->Fload, FALSE, &istart, &iend, &ndata);
	}
	else
		range_to_indices(sldata->loadfrom, sldata->loadto, fddata->hload, FALSE, &istart, &iend, &ndata);


	// if there are not enough data in the selected range, no fitting can be performed
	ndata = MIN(ndata, fddata->hunload->res - istart);
	if (ndata < 2){
		fit = FALSE;
	}

	sldata->nfitloaddata = ndata;

	if (fit){
		//get data to be fitted
		x = gwy_data_line_part_extract(fddata->hload, istart, ndata);
		y = gwy_data_line_part_extract(fddata->Fload, istart, ndata);

		//fit the data and evaluate
		fitinfo = slopes_fit_load(x,y, sldata, fddata, instdata, NULL); 

		if (fitinfo >= 1e6){
			sldata->infologload = fitinfo/1000000;
			fitinfo = fitinfo % 1000000;
		}
				
		if (fitinfo >=5){
			g_snprintf(str, sizeof(str), "%s","Warning Load fit: Questionable results \n or fatal errors in fitting!");
			gtk_label_set_text(GTK_LABEL(slcontrols->fitinfoload), str);
			gtk_widget_modify_fg (slcontrols->fitinfoload, GTK_STATE_NORMAL, &color);
			gtk_label_set_line_wrap(GTK_LABEL(slcontrols->fitinfoload), TRUE);
		}
		else if (fitinfo == 4 ){
			g_snprintf(str, sizeof(str), "%s","Warning Load fit: Iteration limit reached!");
			gtk_label_set_text(GTK_LABEL(slcontrols->fitinfoload), str);
			gtk_widget_modify_fg (slcontrols->fitinfoload, GTK_STATE_NORMAL, &color);
			gtk_label_set_line_wrap(GTK_LABEL(slcontrols->fitinfoload), TRUE);
		}
		else {
			gtk_label_set_text(GTK_LABEL(slcontrols->fitinfoload), NULL);
		}

		//update range labels
		g_snprintf(str, sizeof(str), DEPTH, sldata->loadfrom);
		gtk_entry_set_text(GTK_ENTRY(slcontrols->from), str); 
		g_snprintf(str, sizeof(str), DEPTH, sldata->loadto);
		gtk_entry_set_text(GTK_ENTRY(slcontrols->to), str); 

		g_snprintf(str, sizeof(str), DEPTH, sldata->loadfrom_pct_Fmax);
		gtk_entry_set_text(GTK_ENTRY(slcontrols->from_pct_Fmax), str);
		g_snprintf(str, sizeof(str), DEPTH, sldata->loadto_pct_Fmax);
		gtk_entry_set_text(GTK_ENTRY(slcontrols->to_pct_Fmax), str);

		//update labels
		g_snprintf(str, sizeof(str), DEPTH, sldata->h0);
		gtk_label_set_text(GTK_LABEL(slcontrols->h0), str); 
		g_snprintf(str, sizeof(str), NUMBER, sldata->n);
		gtk_label_set_text(GTK_LABEL(slcontrols->n), str); 
		g_snprintf(str, sizeof(str), SLOPE, sldata->Sload);
		gtk_label_set_text(GTK_LABEL(slcontrols->Sload), str); 
		g_snprintf(str, sizeof(str), AREA, sldata->Aphc);
		gtk_label_set_text(GTK_LABEL(slcontrols->Aphc), str); 

		g_snprintf(str, sizeof(str), "[%.3f, %.3f]", sldata->loadfrom, sldata->loadto);
		gtk_label_set_text(GTK_LABEL(slcontrols->loadrange), str);
		g_snprintf(str, sizeof(str), "[%.3f, %.3f]", sldata->loadfrom_pct_Fmax, sldata->loadto_pct_Fmax);
		gtk_label_set_text(GTK_LABEL(slcontrols->loadrange_pct_Fmax), str);

		g_snprintf(str, sizeof(str), HARD, sldata->Hit);
		gtk_label_set_text(GTK_LABEL(slcontrols->Hit), str);
		g_snprintf(str, sizeof(str), MODUL, sldata->Eit);
		gtk_label_set_text(GTK_LABEL(slcontrols->Eit), str);
		g_snprintf(str,sizeof(str), MODUL, sldata->Er);
		gtk_label_set_text(GTK_LABEL(slcontrols->Er), str); 

		//calculate curve of fit
		sldata->xloadfit = gwy_data_line_duplicate(x);
		sldata->yloadfit = gwy_data_line_new_alike(y,FALSE);

		for (i=0; i< sldata->xloadfit->res; i++){
			sldata->yloadfit->data[i] =  sldata->gamma*pow(sldata->xloadfit->data[i]-sldata->h0,sldata->n);
		}

		// clean up
		g_object_unref(x);
		g_object_unref(y);

		//update fitted curve, if necessary create it
		cmodel = gwy_graph_model_get_curve_by_description (slcontrols->graph_model, "Loading fit");
		if (cmodel){
			gwy_graph_curve_model_set_data(slcontrols->cmodelloadfit, sldata->xloadfit->data, sldata->yloadfit->data, sldata->xloadfit->res);
			gwy_graph_curve_model_enforce_order (slcontrols->cmodelloadfit);
		}
		else {
			gwy_graph_curve_model_set_data(slcontrols->cmodelloadfit, sldata->xloadfit->data, sldata->yloadfit->data, sldata->xloadfit->res);
			gwy_graph_curve_model_enforce_order (slcontrols->cmodelloadfit);
			gwy_graph_model_add_curve(slcontrols->graph_model, slcontrols->cmodelloadfit);
			g_object_set(slcontrols->cmodelloadfit,
					"mode", GWY_GRAPH_CURVE_LINE,
					"description", "Loading fit",
					"color", CURVE_COLOR_FIT,
					"line-style", GDK_LINE_SOLID,
					"line-width", 3,
					NULL);
		}
	}
	else{
		slopes_remove_all_fit_labels_load(slcontrols);			
		slopes_remove_all_fit_results_load(sldata);			
		sldata->has_fit_load = FALSE;
		gwy_graph_model_remove_curve_by_description(slcontrols->graph_model, "Loading fit");
	}

	// log of fitting procedure is available
	if (sldata->has_fit_load)
		gtk_widget_set_sensitive(slcontrols->button_log_load, TRUE);
	else
		gtk_widget_set_sensitive(slcontrols->button_log_load, FALSE);


	if (sldata->has_fit_load &&  sldata->has_fit_unload){
		 if ( data->args->slopesunc.Ec !=NULL || data->args->slopesunc.Hc != NULL || data->args->slopesunc.Ac != NULL)
			 slopes_unc_close_window(data);
		gtk_widget_set_sensitive(slcontrols->button_unc, TRUE);
	}
	else
		gtk_widget_set_sensitive(slcontrols->button_unc, FALSE);

}

void
slopes_fit_and_update_unload(Data *data)
{
	gchar str[300];
	gint  i;
	gint istart, iend, ndata;
	gint fitinfo;
	gboolean fit=TRUE;
	GwyDataLine *x, *y;

	Slopesdata *sldata;
	SlopesControls *slcontrols;
	FDdata *fddata;
	Instdata *instdata;
	GdkColor color;
	GwyGraphCurveModel *cmodel;


	gdk_color_parse ("red", &color);


	// TEMPORARY update GtkEntries for ranges
	/*
	   if (data->args->slopesdata.Finputunload){
	   range_from_pct_Fmax_changed(data);
	   range_to_pct_Fmax_changed(data);
	   }
	   else{
	   range_from_changed(data);
	   range_to_changed(data);
	   }
	   */


	//create pointers for easier reading
	sldata = &(data->args->slopesdata);
	fddata = &(data->args->fddata);
	instdata = &(data->args->instdata);
	slcontrols = data->controls->slopescontrols;

	//if there was a previous fit, free the old data and create new data 
	if (sldata->has_fit_unload){
		g_object_unref(sldata->xunloadfit);
		g_object_unref(sldata->yunloadfit);
	}

	printf("from %g to %g \n", sldata->unloadfrom, sldata->unloadto);

	// check range
	if (sldata->unloadfrom == sldata->unloadto){
		fit = FALSE;
	}

	//find appropriate data, which should be fitted
	//default regime is governed by depth, can be switched to load regime by Finput (if the pct_Fmax ranges were used)
	if (sldata->Finputunload)
	{
		range_to_indices(sldata->unloadfrom_pct_Fmax*0.01*fddata->Fmax, sldata->unloadto_pct_Fmax*0.01*fddata->Fmax,
				fddata->Funload, TRUE, &istart, &iend, &ndata);
	}
	else
		range_to_indices(sldata->unloadfrom, sldata->unloadto, fddata->hunload, TRUE, &istart, &iend, &ndata);

	// if there are not enough data in the selected range,no fitting can be performed 
	ndata = MIN(ndata, fddata->hunload->res - istart);
	if (ndata < 2){
		fit = FALSE;
	}

	sldata->nfitunloaddata = ndata;

	if (fit){
		//get data to be fitted
		x = gwy_data_line_part_extract(fddata->hunload, istart, ndata);
		y = gwy_data_line_part_extract(fddata->Funload, istart, ndata);

		//fit the data and evaluate
		fitinfo = slopes_fit_unload(x,y, sldata, fddata, instdata, NULL);

		if (fitinfo >= 1e6){
			sldata->infologunload = fitinfo/1000000;
			fitinfo = fitinfo % 1000000;
		}
				
		if (fitinfo >=5){
			g_snprintf(str, sizeof(str), "%s","Warning Unload fit: Questionable results \n or fatal errors in fitting!");
			gtk_label_set_text(GTK_LABEL(slcontrols->fitinfounload), str);
			gtk_widget_modify_fg (slcontrols->fitinfounload, GTK_STATE_NORMAL, &color);
			gtk_label_set_line_wrap(GTK_LABEL(slcontrols->fitinfounload), TRUE);
		}
		else if (fitinfo == 4 ){
			g_snprintf(str, sizeof(str), "%s","Warning Unload fit: Iteration limit reached!");
			gtk_label_set_text(GTK_LABEL(slcontrols->fitinfounload), str);
			gtk_widget_modify_fg (slcontrols->fitinfounload, GTK_STATE_NORMAL, &color);
			gtk_label_set_line_wrap(GTK_LABEL(slcontrols->fitinfounload), TRUE);
		}
		else if (sldata->m > 2 ){
			g_snprintf(str, sizeof(str), "%s","Warning Unload fit: Unphysical fit results!");
			gtk_label_set_text(GTK_LABEL(slcontrols->fitinfounload), str);
			gtk_widget_modify_fg (slcontrols->fitinfounload, GTK_STATE_NORMAL, &color);
			gtk_label_set_line_wrap(GTK_LABEL(slcontrols->fitinfounload), TRUE);
		}
		else {
			gtk_label_set_text(GTK_LABEL(slcontrols->fitinfounload), NULL);
		}

		//update range labels
		g_snprintf(str, sizeof(str), DEPTH, sldata->unloadfrom);
		gtk_entry_set_text(GTK_ENTRY(slcontrols->from), str); 
		g_snprintf(str, sizeof(str), DEPTH, sldata->unloadto);
		gtk_entry_set_text(GTK_ENTRY(slcontrols->to), str); 

		g_snprintf(str, sizeof(str), DEPTH, sldata->unloadfrom_pct_Fmax);
		gtk_entry_set_text(GTK_ENTRY(slcontrols->from_pct_Fmax), str);
		g_snprintf(str, sizeof(str), DEPTH, sldata->unloadto_pct_Fmax);
		gtk_entry_set_text(GTK_ENTRY(slcontrols->to_pct_Fmax), str);

		//update labels
		g_snprintf(str, sizeof(str), DEPTH, sldata->hp);
		gtk_label_set_text(GTK_LABEL(slcontrols->hp), str); 
		g_snprintf(str, sizeof(str), NUMBER, sldata->m);
		gtk_label_set_text(GTK_LABEL(slcontrols->m), str); 
		g_snprintf(str,sizeof(str), NUMBER, sldata->eps);
		gtk_label_set_text(GTK_LABEL(slcontrols->eps), str);
		g_snprintf(str, sizeof(str), SLOPE, sldata->Sunload);
		gtk_label_set_text(GTK_LABEL(slcontrols->Sunload), str); 
		g_snprintf(str, sizeof(str), AREA, sldata->Aphc);
		gtk_label_set_text(GTK_LABEL(slcontrols->Aphc), str); 

		g_snprintf(str, sizeof(str), "[%.3f, %.3f]", sldata->unloadfrom, sldata->unloadto);
		gtk_label_set_text(GTK_LABEL(slcontrols->unloadrange), str);
		g_snprintf(str, sizeof(str), "[%.3f, %.3f]", sldata->unloadfrom_pct_Fmax, sldata->unloadto_pct_Fmax);
		gtk_label_set_text(GTK_LABEL(slcontrols->unloadrange_pct_Fmax), str);

		g_snprintf(str, sizeof(str), HARD, sldata->Hit);
		gtk_label_set_text(GTK_LABEL(slcontrols->Hit), str);
		g_snprintf(str, sizeof(str), MODUL, sldata->Eit);
		gtk_label_set_text(GTK_LABEL(slcontrols->Eit), str);
		g_snprintf(str,sizeof(str), MODUL, sldata->Er);
		gtk_label_set_text(GTK_LABEL(slcontrols->Er), str); 

		//calculate curve of fit
		sldata->xunloadfit = gwy_data_line_duplicate(x);
		sldata->yunloadfit = gwy_data_line_new_alike(y,FALSE);

		for (i=0; i< sldata->xunloadfit->res; i++){
			sldata->yunloadfit->data[i] =  sldata->alpha*pow(sldata->xunloadfit->data[i]-sldata->hp,sldata->m);
		}

		// clean up
		g_object_unref(x);
		g_object_unref(y);

		//update fitted curve, if necessary create it
		cmodel = gwy_graph_model_get_curve_by_description (slcontrols->graph_model, "Unloading fit");
		if (cmodel){
			gwy_graph_curve_model_set_data(slcontrols->cmodelunloadfit, sldata->xunloadfit->data, sldata->yunloadfit->data, sldata->xunloadfit->res);
			gwy_graph_curve_model_enforce_order (slcontrols->cmodelunloadfit);
		}
		else {
			gwy_graph_curve_model_set_data(slcontrols->cmodelunloadfit, sldata->xunloadfit->data, sldata->yunloadfit->data, sldata->xunloadfit->res);
			gwy_graph_curve_model_enforce_order (slcontrols->cmodelunloadfit);
			gwy_graph_model_add_curve(slcontrols->graph_model, slcontrols->cmodelunloadfit);
			g_object_set(slcontrols->cmodelunloadfit,
					"mode", GWY_GRAPH_CURVE_LINE,
					"description", "Unloading fit",
					"color", CURVE_COLOR_FIT2,
					"line-style", GDK_LINE_SOLID,
					"line-width", 3,
					NULL);
		}
	}
	else{
		slopes_remove_all_fit_labels_unload(slcontrols);			
		slopes_remove_all_fit_results_unload(sldata);			
		sldata->has_fit_unload = FALSE;
		gwy_graph_model_remove_curve_by_description(slcontrols->graph_model, "Unloading fit");
	}

	// log of fitting procedure is available
	if (sldata->has_fit_unload)
		gtk_widget_set_sensitive(slcontrols->button_log_unload, TRUE);
	else
		gtk_widget_set_sensitive(slcontrols->button_log_unload, FALSE);


	if (sldata->has_fit_load &&  sldata->has_fit_unload){
		 if ( data->args->slopesunc.Ec !=NULL || data->args->slopesunc.Hc != NULL || data->args->slopesunc.Ac != NULL)
			 slopes_unc_close_window(data);
		gtk_widget_set_sensitive(slcontrols->button_unc, TRUE);
	}
	else
		gtk_widget_set_sensitive(slcontrols->button_unc, FALSE);
}

static void slopes_show_log_load(Data *data)
{
	GtkWidget *scrolled_window;
	GtkWidget *textview;
	GtkTextBuffer *buffer;
	gchar *log;
	gint len;
	FILE *logfile;
	gchar *logfnmerr;
	gchar *logfnmrpt;
	SlopesControls *ctrls;
	Slopesdata *sldata;

	ctrls = data->controls->slopescontrols;
	sldata = &(data->args->slopesdata);

	if (ctrls->load_log_error_window  != NULL && GTK_IS_WIDGET(ctrls->load_log_error_window))
		gtk_widget_destroy(ctrls->load_log_error_window);


	/* check if there was a log */
	if (sldata->infologload == 2 || sldata->infologload == 3 ) {
	    show_warning("Fit error log could not be created.");
	}
	else {
		/*open log */
		logfnmerr = g_malloc(sizeof(gchar)*(strlen(sldata->logfnmload)+5));
		strcpy(logfnmerr, sldata->logfnmload);
		strcat(logfnmerr, ".err");
		logfile = fopen(logfnmerr, "r");
		if (!logfile) { /* should not get here */
		    show_error("No error log.");
		    g_free(logfnmerr);
		    return;
		}
		else {
			/*find length and read file */
			fseek(logfile, 0, SEEK_END);
			len = ftell(logfile);
			log = (gchar *) g_malloc((len+1)*sizeof(gchar));
			rewind(logfile);
			fread(log, len,1,logfile);
			fclose(logfile);
			log[len] = '\0';
			g_free(logfnmerr);

			/*create buffer, fill it and create textview */
			buffer = gtk_text_buffer_new (NULL);
			gtk_text_buffer_set_text (buffer, log, -1);
			textview = gtk_text_view_new_with_buffer (buffer);
#ifndef G_OS_WIN32
			gtk_widget_modify_font (textview, pango_font_description_from_string ("monospace"));
#endif
			gtk_text_view_set_editable (GTK_TEXT_VIEW (textview), FALSE);
			g_free(log);

			scrolled_window = gtk_scrolled_window_new(NULL, NULL);
			gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
			gtk_container_add(GTK_CONTAINER(scrolled_window), textview);

			ctrls->load_log_error_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
			gtk_window_set_title(GTK_WINDOW(ctrls->load_log_error_window), "Fit error log loading");
			gtk_widget_set_size_request(GTK_WIDGET(ctrls->load_log_error_window), 450, 400);
			g_signal_connect (ctrls->load_log_error_window, "delete_event", G_CALLBACK (gtk_widget_destroy),NULL);
			gtk_container_add(GTK_CONTAINER(ctrls->load_log_error_window), scrolled_window);
			gtk_widget_show_all(ctrls->load_log_error_window);
		}
	}


	/* check whether there was a log */
	if (sldata->infologload == 1 || sldata->infologload == 3 ) {
	    show_warning("Fit report could not be created.");
	}
	else {
		/*open log */
	    if (ctrls->load_log_report_window != NULL && GTK_IS_WIDGET(ctrls->load_log_report_window))
			gtk_widget_destroy(ctrls->load_log_report_window);

		logfnmrpt = g_malloc(sizeof(gchar)*(strlen(sldata->logfnmload)+5));
		strcpy(logfnmrpt, sldata->logfnmload);
		strcat(logfnmrpt, ".rpt");
		logfile = fopen(logfnmrpt, "r");
		if (!logfile) { /* should not get here */
		    show_error("No report log.");
		    g_free(logfnmrpt);
		    return;
		}
		else{
			/*find length and read file */
			fseek(logfile, 0, SEEK_END);
			len = ftell(logfile);
			log = (gchar *) g_malloc((len+1)*sizeof(gchar));
			rewind(logfile);
			fread(log, len,1,logfile);
			fclose(logfile);
			log[len] = '\0';
			g_free(logfnmrpt);

			/*create buffer, fill it and create textview */
			buffer = gtk_text_buffer_new (NULL);
			gtk_text_buffer_set_text (buffer, log, -1);
			textview = gtk_text_view_new_with_buffer (buffer);
#ifndef G_OS_WIN32
			gtk_widget_modify_font (textview, pango_font_description_from_string ("monospace"));
#endif
			gtk_text_view_set_editable (GTK_TEXT_VIEW (textview), FALSE);
			g_free(log);

			scrolled_window = gtk_scrolled_window_new(NULL, NULL);
			gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
			gtk_container_add(GTK_CONTAINER(scrolled_window), textview);

			ctrls->load_log_report_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
			gtk_window_set_title(GTK_WINDOW(ctrls->load_log_report_window), "Fit report log loading");
			gtk_widget_set_size_request(GTK_WIDGET(ctrls->load_log_report_window), 450, 400);
			g_signal_connect (ctrls->load_log_report_window, "delete_event", G_CALLBACK (gtk_widget_destroy),NULL);
			gtk_container_add(GTK_CONTAINER(ctrls->load_log_report_window), scrolled_window);
			gtk_widget_show_all(ctrls->load_log_report_window);
		}
	}
}

static void slopes_show_log_unload(Data *data)
{

	GtkWidget *scrolled_window;
	GtkWidget *textview;
	GtkTextBuffer *buffer;
	gchar *log;
	gint len;
	FILE *logfile;
	gchar *logfnmerr;
	gchar *logfnmrpt;
	SlopesControls *ctrls;
	Slopesdata *sldata;

	ctrls = data->controls->slopescontrols;
	sldata = &(data->args->slopesdata);

	if (ctrls->unload_log_error_window  != NULL && GTK_IS_WIDGET(ctrls->unload_log_error_window))
		gtk_widget_destroy(ctrls->unload_log_error_window);


	/* check if there was a log */
	if (sldata->infologunload == 2 || sldata->infologunload == 3 ) {
	    show_warning("Fit error log could not be created.");
	}
	else {
		/*open log */
		logfnmerr = g_malloc(sizeof(gchar)*(strlen(sldata->logfnmunload)+5));
		strcpy(logfnmerr, sldata->logfnmunload);
		strcat(logfnmerr, ".err");
		logfile = fopen(logfnmerr, "r");
		if (!logfile) { /* should not get here */
		    show_error("No error log.");
		    g_free(logfnmerr);
		    return;
		}
		else {
			/*find length and read file */
			fseek(logfile, 0, SEEK_END);
			len = ftell(logfile);
			log = (gchar *) g_malloc((len+1)*sizeof(gchar));
			rewind(logfile);
			fread(log, len,1,logfile);
			fclose(logfile);
			log[len] = '\0';
			g_free(logfnmerr);

			/*create buffer, fill it and create textview */
			buffer = gtk_text_buffer_new (NULL);
			gtk_text_buffer_set_text (buffer, log, -1);
			textview = gtk_text_view_new_with_buffer (buffer);
#ifndef G_OS_WIN32
			gtk_widget_modify_font (textview, pango_font_description_from_string ("monospace"));
#endif
			gtk_text_view_set_editable (GTK_TEXT_VIEW (textview), FALSE);
			g_free(log);

			scrolled_window = gtk_scrolled_window_new(NULL, NULL);
			gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
			gtk_container_add(GTK_CONTAINER(scrolled_window), textview);

			ctrls->unload_log_error_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
			gtk_window_set_title(GTK_WINDOW(ctrls->unload_log_error_window), "Fit error log unloading");
			gtk_widget_set_size_request(GTK_WIDGET(ctrls->unload_log_error_window), 450, 400);
			g_signal_connect (ctrls->unload_log_error_window, "delete_event", G_CALLBACK (gtk_widget_destroy),NULL);
			gtk_container_add(GTK_CONTAINER(ctrls->unload_log_error_window), scrolled_window);
			gtk_widget_show_all(ctrls->unload_log_error_window);
		}
	}


	/* check whether there was a log */
	if (sldata->infologunload == 1 || sldata->infologunload == 3 ) {
	    show_warning("Fit report could not be created.");
	}
	else {
		/*open log */
		if (ctrls->unload_log_report_window != NULL && GTK_IS_WIDGET(ctrls->unload_log_report_window))
			gtk_widget_destroy(ctrls->unload_log_report_window);


		logfnmrpt = g_malloc(sizeof(gchar)*(strlen(sldata->logfnmunload)+5));
		strcpy(logfnmrpt, sldata->logfnmunload);
		strcat(logfnmrpt, ".rpt");
		logfile = fopen(logfnmrpt, "r");
		if (!logfile) { /* should not get here */
		    show_error("No report log.");
		    g_free(logfnmrpt);
		    return;
		}
		else {
		 	/*find length and read file */
			fseek(logfile, 0, SEEK_END);
			len = ftell(logfile);
			log = (gchar *) g_malloc((len+1)*sizeof(gchar));
			rewind(logfile);
			fread(log, len,1,logfile);
			fclose(logfile);
			log[len] = '\0';
			g_free(logfnmrpt);

			/*create buffer, fill it and create textview */
			buffer = gtk_text_buffer_new (NULL);
			gtk_text_buffer_set_text (buffer, log, -1);
			textview = gtk_text_view_new_with_buffer (buffer);
#ifndef G_OS_WIN32
			gtk_widget_modify_font (textview, pango_font_description_from_string ("monospace"));
#endif
			gtk_text_view_set_editable (GTK_TEXT_VIEW (textview), FALSE);
			g_free(log);

			scrolled_window = gtk_scrolled_window_new(NULL, NULL);
			gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
			gtk_container_add(GTK_CONTAINER(scrolled_window), textview);

			ctrls->unload_log_report_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
			gtk_window_set_title(GTK_WINDOW(ctrls->unload_log_report_window), "Fit report log unloading");
			gtk_widget_set_size_request(GTK_WIDGET(ctrls->unload_log_report_window), 450, 400);
			g_signal_connect (ctrls->unload_log_report_window, "delete_event", G_CALLBACK (gtk_widget_destroy),NULL);
			gtk_container_add(GTK_CONTAINER(ctrls->unload_log_report_window), scrolled_window);
			gtk_widget_show_all(ctrls->unload_log_report_window);
		}
	}
}


GtkWidget *
tool_slopes_create (Data *data)
{
    	gchar str[300];
	GwyGraphArea *area;
	Args *args;
	SlopesControls *slcontrols;
	gint i;

	args=data->args;
	slcontrols=data->controls->slopescontrols;

	// GraphArea, three curves can be displayed: loading F-d curve, the linear (hp) fit and the powerlaw (S) fit

	slcontrols->graph_model = gwy_graph_model_new();
	g_object_set(slcontrols->graph_model, "title", "Unload", NULL);
	gwy_graph_model_set_axis_label (slcontrols->graph_model, GTK_POS_BOTTOM, "h / nm");
	gwy_graph_model_set_axis_label (slcontrols->graph_model, GTK_POS_LEFT, "F / mN");

	slcontrols->zoomstack = NULL;
	slcontrols->zoomstack = g_slist_prepend (slcontrols->zoomstack, slcontrols->graph_model);

	slcontrols->hbox_graph = gtk_hbox_new (FALSE, CTRLS_SPACING);
	
	slcontrols->graph = gwy_graph_new(slcontrols->graph_model);
	gtk_widget_set_size_request(GTK_WIDGET(slcontrols->graph), 600, 400);

	gwy_graph_enable_user_input(GWY_GRAPH(slcontrols->graph), FALSE);
	gwy_graph_set_status(GWY_GRAPH(slcontrols->graph), GWY_GRAPH_STATUS_XSEL);


	area = GWY_GRAPH_AREA(gwy_graph_get_area(GWY_GRAPH(slcontrols->graph)));
	slcontrols->selection = gwy_graph_area_get_selection(area, GWY_GRAPH_STATUS_XSEL);
	gwy_selection_set_max_objects(slcontrols->selection, 1);
	g_signal_connect(slcontrols->selection, "changed", G_CALLBACK(slopes_graph_selected), data);

	slcontrols->cmodelload = gwy_graph_curve_model_new();
	slcontrols->cmodelhold = gwy_graph_curve_model_new();
	slcontrols->cmodelunload = gwy_graph_curve_model_new();
	slcontrols->cmodelloadfit = gwy_graph_curve_model_new();
	slcontrols->cmodelunloadfit = gwy_graph_curve_model_new();

	g_object_set(slcontrols->cmodelload,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "Loading",
			"point-size", 0,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
		        "color", CURVE_COLOR_LOAD,
			"line-style", GDK_LINE_SOLID,
		        "line-width", 2,
			NULL);

	g_object_set(slcontrols->cmodelhold,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "Hold",
			"point-size", 0,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
		        "color", CURVE_COLOR_HOLD,
			"line-style", GDK_LINE_SOLID,
		        "line-width", 2,
			NULL);

	g_object_set(slcontrols->cmodelunload,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "Unloading",
			"point-size", 0,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
		        "color", CURVE_COLOR_UNLOAD,
			"line-style", GDK_LINE_SOLID,
		        "line-width", 2,
			NULL);

	gwy_graph_model_add_curve(slcontrols->graph_model, slcontrols->cmodelload);
	gwy_graph_model_add_curve(slcontrols->graph_model, slcontrols->cmodelhold);
	gwy_graph_model_add_curve(slcontrols->graph_model, slcontrols->cmodelunload);


	// Table of input data and results

	slcontrols->ctrls = gtk_vbox_new (FALSE, CTRLS_SPACING);

	/* Info section */
	
	slcontrols->frame_info = gtk_frame_new (NULL);
	gtk_frame_set_label_widget (GTK_FRAME (slcontrols->frame_info), GTK_WIDGET (gwy_label_new_header("Info")));
	slcontrols->table_info = gtk_table_new (2, 3, FALSE);
	gtk_container_set_border_width (GTK_CONTAINER (slcontrols->table_info), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (slcontrols->table_info), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (slcontrols->table_info), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add ( GTK_CONTAINER (slcontrols->frame_info), slcontrols->table_info);

	i = 0;

	/* h_max */

	gtk_table_attach (GTK_TABLE (slcontrols->table_info), label_new_with_markup_left ("h<sub>max</sub>"), 0, 1, i,i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	slcontrols->hmax = gtk_label_new (NULL); 
	g_snprintf(str,sizeof(str), DEPTH, args->fddata.hmax);
	gtk_label_set_text (GTK_LABEL (slcontrols->hmax), str);
	gtk_misc_set_alignment (GTK_MISC (slcontrols->hmax), 0.0, 0.5);  
	gtk_table_attach (GTK_TABLE (slcontrols->table_info), slcontrols->hmax, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	gtk_table_attach (GTK_TABLE (slcontrols->table_info), label_new_with_markup_left("nm"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	i++;

	/* F_max */

	gtk_table_attach(GTK_TABLE(slcontrols->table_info), label_new_with_markup_left("F<sub>max</sub>"), 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	slcontrols->Fmax = gtk_label_new(NULL); 
	g_snprintf(str,sizeof(str), FORCE, args->fddata.Fmax); 
	gtk_label_set_text(GTK_LABEL(slcontrols->Fmax), str);
	gtk_misc_set_alignment(GTK_MISC(slcontrols->Fmax), 0.0, 0.5);  
	gtk_table_attach(GTK_TABLE(slcontrols->table_info), slcontrols->Fmax, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	gtk_table_attach(GTK_TABLE(slcontrols->table_info), label_new_with_markup_left("mN"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);  

	/* Parameters section */

	slcontrols->frame_parameters = gtk_frame_new (NULL);
	gtk_frame_set_label_widget (GTK_FRAME (slcontrols->frame_parameters), GTK_WIDGET (gwy_label_new_header("Parameters")));
	slcontrols->table_parameters = gtk_table_new (5, 4, FALSE);
	gtk_container_set_border_width (GTK_CONTAINER (slcontrols->table_parameters), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (slcontrols->table_parameters), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (slcontrols->table_parameters), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add ( GTK_CONTAINER (slcontrols->frame_parameters), slcontrols->table_parameters);
	i = 0;

	gtk_table_attach(GTK_TABLE(slcontrols->table_parameters), label_new_with_markup_left("Fit range"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	slcontrols->from = gtk_entry_new(); 
	g_object_set_data(G_OBJECT(slcontrols->from), "id", (gpointer)"from"); 
	gtk_entry_set_width_chars(GTK_ENTRY(slcontrols->from), 8);
	gtk_table_attach(GTK_TABLE(slcontrols->table_parameters), slcontrols->from, 1, 2, i, i+1, GTK_FILL, 0, 0, 0); 
	gwy_widget_set_activate_on_unfocus(slcontrols->from, TRUE);
	g_signal_connect_swapped(slcontrols->from, "activate", G_CALLBACK(slopes_range_from_changed),data);
/*	g_signal_connect_swapped(slcontrols->from, "focus-out-event", G_CALLBACK(range_from_changed),data); */

	slcontrols->to = gtk_entry_new(); 
	g_object_set_data(G_OBJECT(slcontrols->to), "id", (gpointer)"to"); 
	gtk_entry_set_width_chars(GTK_ENTRY(slcontrols->to), 8); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_parameters), slcontrols->to, 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	gwy_widget_set_activate_on_unfocus(slcontrols->to, TRUE);
	g_signal_connect_swapped(slcontrols->to, "activate", G_CALLBACK(slopes_range_to_changed),data);
/*	g_signal_connect_swapped(slcontrols->to, "focus-out-event", G_CALLBACK(range_to_changed),data); */

	gtk_table_attach(GTK_TABLE(slcontrols->table_parameters), label_new_with_markup_left("nm"), 3, 4, i, i+1, GTK_FILL, 0, 0, 0);
	i++;

	slcontrols->from_pct_Fmax = gtk_entry_new(); 
	g_object_set_data(G_OBJECT(slcontrols->from_pct_Fmax), "id", (gpointer)"from_pct_Fmax"); 
	gtk_entry_set_width_chars(GTK_ENTRY(slcontrols->from_pct_Fmax), 8);
	gtk_table_attach(GTK_TABLE(slcontrols->table_parameters), slcontrols->from_pct_Fmax, 1, 2, i, i+1, GTK_FILL, 0, 0, 0); 
	gwy_widget_set_activate_on_unfocus(slcontrols->from_pct_Fmax, TRUE);
	g_signal_connect_swapped(slcontrols->from_pct_Fmax, "activate", G_CALLBACK(slopes_range_from_pct_Fmax_changed),data);
/*	g_signal_connect_swapped(slcontrols->from_pct_Fmax, "focus-out-event", G_CALLBACK(range_from_pct_Fmax_changed),data);  */


	slcontrols->to_pct_Fmax = gtk_entry_new(); 
	g_object_set_data(G_OBJECT(slcontrols->to_pct_Fmax), "id", (gpointer)"to_pct_Fmax"); 
	gtk_entry_set_width_chars(GTK_ENTRY(slcontrols->to_pct_Fmax), 8); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_parameters), slcontrols->to_pct_Fmax, 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	gwy_widget_set_activate_on_unfocus(slcontrols->to_pct_Fmax, TRUE);
	g_signal_connect_swapped(slcontrols->to_pct_Fmax, "activate", G_CALLBACK(slopes_range_to_pct_Fmax_changed),data);
	/*g_signal_connect_swapped(slcontrols->to_pct_Fmax, "focus-out-event", G_CALLBACK(range_to_pct_Fmax_changed),data);  */

	gtk_table_attach(GTK_TABLE(slcontrols->table_parameters), label_new_with_markup_left("% F<sub>max</sub>"), 3, 4, i, i+1, GTK_FILL, 0, 0, 0);

	/* Fit buttons */

	slcontrols->hbox_fitbuttons = gtk_hbox_new (TRUE, 0);

	slcontrols->button_fit_load = gtk_button_new_with_mnemonic(gwy_sgettext("_Fit loading"));
	gtk_button_set_image (GTK_BUTTON (slcontrols->button_fit_load), gtk_image_new_from_stock (GTK_STOCK_EXECUTE, GTK_ICON_SIZE_BUTTON));
	gtk_box_pack_start (GTK_BOX (slcontrols->hbox_fitbuttons), slcontrols->button_fit_load, TRUE, TRUE, 0);
	g_signal_connect_swapped (slcontrols->button_fit_load, "clicked", G_CALLBACK (slopes_fit_and_update_load), data);

	slcontrols->button_fit_unload = gtk_button_new_with_mnemonic(gwy_sgettext("verb|F_it unloading"));
	gtk_button_set_image (GTK_BUTTON (slcontrols->button_fit_unload), gtk_image_new_from_stock (GTK_STOCK_EXECUTE, GTK_ICON_SIZE_BUTTON));
	gtk_box_pack_start (GTK_BOX (slcontrols->hbox_fitbuttons), slcontrols->button_fit_unload, TRUE, TRUE, 0);
	g_signal_connect_swapped (slcontrols->button_fit_unload, "clicked", G_CALLBACK (slopes_fit_and_update_unload), data);
	

	/* Results section */
	
	slcontrols->frame_results = gtk_frame_new (NULL);
	gtk_frame_set_label_widget (GTK_FRAME (slcontrols->frame_results), GTK_WIDGET (gwy_label_new_header("Results")));
	slcontrols->table_results = gtk_table_new (20, 3, FALSE);
	gtk_container_set_border_width (GTK_CONTAINER (slcontrols->table_results), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (slcontrols->table_results), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (slcontrols->table_results), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add ( GTK_CONTAINER (slcontrols->frame_results), slcontrols->table_results);
	i = 0;

	slcontrols->fitinfoload = gtk_label_new(NULL);
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), slcontrols->fitinfoload, 0, 3, i, i+2, GTK_FILL, GTK_FILL, 0, 0); 
	i+=2;


	gtk_table_attach(GTK_TABLE(slcontrols->table_results), label_new_with_markup_left("h<sub>0</sub>"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	slcontrols->h0 = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(slcontrols->h0), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), slcontrols->h0, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), label_new_with_markup_left("nm"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	i++;

	gtk_table_attach(GTK_TABLE(slcontrols->table_results), label_new_with_markup_left("n"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	slcontrols->n = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(slcontrols->n), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), slcontrols->n, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	i++;

	slcontrols->fitinfounload = gtk_label_new(NULL);
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), slcontrols->fitinfounload, 0, 3, i, i+2, GTK_FILL, GTK_FILL, 0, 0); 
	i+=2;

	gtk_table_attach(GTK_TABLE(slcontrols->table_results), label_new_with_markup_left("h<sub>p</sub>"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	slcontrols->hp = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(slcontrols->hp), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), slcontrols->hp, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), label_new_with_markup_left("nm"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	i++;

	gtk_table_attach(GTK_TABLE(slcontrols->table_results), label_new_with_markup_left("m"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	slcontrols->m = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(slcontrols->m), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), slcontrols->m, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	i++;

	gtk_table_attach(GTK_TABLE(slcontrols->table_results), label_new_with_markup_left("ε"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	slcontrols->eps = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(slcontrols->eps), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), slcontrols->eps, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	i++;

	gtk_table_attach(GTK_TABLE(slcontrols->table_results), label_new_with_markup_left("S<sub>load</sub>"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	slcontrols->Sload = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(slcontrols->Sload), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), slcontrols->Sload, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), label_new_with_markup_left("mN/nm"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	i++;

	gtk_table_attach(GTK_TABLE(slcontrols->table_results), label_new_with_markup_left("S<sub>unload</sub>"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	slcontrols->Sunload = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(slcontrols->Sunload), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), slcontrols->Sunload, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), label_new_with_markup_left("mN/nm"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	i++;

	gtk_table_attach(GTK_TABLE(slcontrols->table_results), label_new_with_markup_left("A<sub>p</sub>(h<sub>c</sub>)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	slcontrols->Aphc = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(slcontrols->Aphc), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), slcontrols->Aphc, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), label_new_with_markup_left("nm<sup>2</sup>"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	i++;


	gtk_table_attach(GTK_TABLE(slcontrols->table_results), label_new_with_markup_left("H<sub>IT</sub>"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	slcontrols->Hit = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(slcontrols->Hit), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), slcontrols->Hit, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), label_new_with_markup_left("MPa"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	i++;

	gtk_table_attach(GTK_TABLE(slcontrols->table_results), label_new_with_markup_left("E<sub>r</sub>"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	slcontrols->Er = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(slcontrols->Er), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), slcontrols->Er, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), label_new_with_markup_left("GPa"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	i++;

	gtk_table_attach(GTK_TABLE(slcontrols->table_results), label_new_with_markup_left("E<sub>IT</sub>"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	slcontrols->Eit = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(slcontrols->Eit), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), slcontrols->Eit, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), label_new_with_markup_left("GPa"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	i++;

	gtk_table_attach(GTK_TABLE(slcontrols->table_results), label_new_with_markup_left("Fitted loading range"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	slcontrols->loadrange = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(slcontrols->loadrange), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), slcontrols->loadrange, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), label_new_with_markup_left("nm"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	i++;
	
	slcontrols->loadrange_pct_Fmax = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(slcontrols->loadrange_pct_Fmax), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), slcontrols->loadrange_pct_Fmax, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), label_new_with_markup_left("% F<sub>max</sub>"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	i++;
	
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), label_new_with_markup_left("Fitted unloading range"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0); 
	slcontrols->unloadrange = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(slcontrols->unloadrange), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), slcontrols->unloadrange, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), label_new_with_markup_left("nm"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	i++;

	slcontrols->unloadrange_pct_Fmax = gtk_label_new(NULL); 
	gtk_misc_set_alignment(GTK_MISC(slcontrols->unloadrange_pct_Fmax), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), slcontrols->unloadrange_pct_Fmax, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	gtk_table_attach(GTK_TABLE(slcontrols->table_results), label_new_with_markup_left("% F<sub>max</sub>"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0); 
	i++;


	
	/* Log buttons */
	slcontrols->hbox_logbuttons = gtk_hbox_new (TRUE, 0);

	slcontrols->button_log_load = gtk_button_new_with_mnemonic("Show _log load");
	gtk_button_set_image (GTK_BUTTON (slcontrols->button_log_load), gtk_image_new_from_stock (GTK_STOCK_DIALOG_QUESTION, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (slcontrols->button_log_load, "clicked", G_CALLBACK (slopes_show_log_load), data);
	gtk_widget_set_sensitive(slcontrols->button_log_load, FALSE);
	slcontrols->load_log_error_window = NULL;
	slcontrols->load_log_report_window = NULL;
	gtk_box_pack_start (GTK_BOX (slcontrols->hbox_logbuttons), slcontrols->button_log_load, TRUE, TRUE, 0);

	slcontrols->button_log_unload = gtk_button_new_with_mnemonic("Show lo_g unload");
	gtk_button_set_image (GTK_BUTTON (slcontrols->button_log_unload), gtk_image_new_from_stock (GTK_STOCK_DIALOG_QUESTION, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (slcontrols->button_log_unload, "clicked", G_CALLBACK (slopes_show_log_unload), data);
	gtk_widget_set_sensitive(slcontrols->button_log_unload, FALSE);
	slcontrols->unload_log_error_window = NULL;
	slcontrols->unload_log_report_window = NULL;
	gtk_box_pack_start (GTK_BOX (slcontrols->hbox_logbuttons), slcontrols->button_log_unload, TRUE, TRUE, 0);


	// Uncertainties Button
	slcontrols->button_unc = gtk_button_new_with_mnemonic("_Uncertainties");
	gtk_button_set_image (GTK_BUTTON (slcontrols->button_unc), gtk_image_new_from_stock (GTK_STOCK_DIALOG_QUESTION, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (slcontrols->button_unc, "clicked", G_CALLBACK (slopes_uncertainty), data);
	gtk_widget_set_sensitive(slcontrols->button_unc, FALSE);


	// Save Button

	slcontrols->button_save = gtk_button_new_from_stock (GTK_STOCK_SAVE);
	g_signal_connect_swapped (slcontrols->button_save, "clicked", G_CALLBACK (slopes_save_data), args);

	gtk_box_pack_start (GTK_BOX (slcontrols->ctrls), slcontrols->frame_info, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (slcontrols->ctrls), slcontrols->frame_parameters, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (slcontrols->ctrls), slcontrols->hbox_fitbuttons, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (slcontrols->ctrls), slcontrols->frame_results, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (slcontrols->ctrls), slcontrols->hbox_logbuttons, FALSE, FALSE, 0);
	/* gtk_box_pack_start (GTK_BOX (slcontrols->ctrls), slcontrols->button_unc, FALSE, FALSE, 0);  */
	gtk_box_pack_start (GTK_BOX (slcontrols->ctrls), slcontrols->button_save, FALSE, FALSE, 0);

	/* Zooming */

	slcontrols->vbox_zoom_buttons = gtk_vbox_new (TRUE, 0);
	slcontrols->vbox_zoom_outer = gtk_vbox_new (FALSE, 0);

	slcontrols->button_zoom_in = gtk_button_new ();
	gtk_button_set_image (GTK_BUTTON (slcontrols->button_zoom_in), gtk_image_new_from_stock (GTK_STOCK_ZOOM_IN, GTK_ICON_SIZE_BUTTON));
	gtk_box_pack_start (GTK_BOX (slcontrols->vbox_zoom_buttons), slcontrols->button_zoom_in, FALSE, FALSE, 0);
	g_signal_connect_swapped(slcontrols->button_zoom_in, "clicked", G_CALLBACK (slopes_zoom_in), data);
	
	slcontrols->button_zoom_out = gtk_button_new ();
	gtk_button_set_image (GTK_BUTTON (slcontrols->button_zoom_out), gtk_image_new_from_stock (GTK_STOCK_ZOOM_OUT, GTK_ICON_SIZE_BUTTON));
	gtk_box_pack_start (GTK_BOX (slcontrols->vbox_zoom_buttons), slcontrols->button_zoom_out, FALSE, FALSE, 0);
	g_signal_connect_swapped(slcontrols->button_zoom_out, "clicked", G_CALLBACK (slopes_zoom_out), data);

	slcontrols->button_zoom_restore = gtk_button_new ();
	gtk_button_set_image (GTK_BUTTON (slcontrols->button_zoom_restore), gtk_image_new_from_stock (GTK_STOCK_ZOOM_FIT, GTK_ICON_SIZE_BUTTON));
	gtk_box_pack_start (GTK_BOX (slcontrols->vbox_zoom_buttons), slcontrols->button_zoom_restore, FALSE, FALSE, 0);
	g_signal_connect_swapped(slcontrols->button_zoom_restore, "clicked", G_CALLBACK (slopes_zoom_restore), data);

	gtk_box_pack_start (GTK_BOX (slcontrols->vbox_zoom_outer), slcontrols->vbox_zoom_buttons, TRUE, FALSE, 0);
  
	gtk_box_pack_start (GTK_BOX (slcontrols->hbox_graph), slcontrols->graph, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (slcontrols->hbox_graph), slcontrols->vbox_zoom_outer, FALSE, FALSE, 0);

	//final 

	slcontrols->slopes_gui = gtk_hbox_new(FALSE,10);
	gtk_box_pack_start (GTK_BOX (slcontrols->slopes_gui), slcontrols->ctrls, FALSE, FALSE, 0); /* expand, fill, padding */
	gtk_box_pack_start (GTK_BOX (slcontrols->slopes_gui), slcontrols->hbox_graph, TRUE, TRUE, 0);

	gtk_widget_show(slcontrols->slopes_gui);

	return slcontrols->slopes_gui;
}

void
slopes_recalculate(Data *data)
{
    recalculate_Eit(data);
}

static void
update_models_slopes(SlopesControls *slcontrols)
{
    GwyGraphCurveModel *cmodel;
    
    slcontrols->graph_model = gwy_graph_get_model (GWY_GRAPH(slcontrols->graph));

    cmodel = gwy_graph_model_get_curve_by_description (slcontrols->graph_model, "Loading");
    if (cmodel)
	slcontrols->cmodelload = cmodel;

    cmodel = gwy_graph_model_get_curve_by_description (slcontrols->graph_model, "Hold");
    if (cmodel)
	slcontrols->cmodelhold = cmodel;

    cmodel = gwy_graph_model_get_curve_by_description (slcontrols->graph_model, "Unloading");
    if (cmodel)
	slcontrols->cmodelunload = cmodel;

    cmodel = gwy_graph_model_get_curve_by_description (slcontrols->graph_model, "Unloading fit");
    if (cmodel)
	slcontrols->cmodelloadfit = cmodel;
    cmodel = gwy_graph_model_get_curve_by_description (slcontrols->graph_model, "Loading fit");
    if (cmodel)
	slcontrols->cmodelunloadfit = cmodel;
}

static void
slopes_zoom_in(Data *data)
{
    SlopesControls *slcontrols;

    slcontrols = data->controls->slopescontrols;
    
    rs_zoom_in (GWY_GRAPH (slcontrols->graph), &(slcontrols->zoomstack), FALSE); /* do not rescale y */
    update_models_slopes(slcontrols);
}

static void
slopes_zoom_out(Data *data)
{
    SlopesControls *slcontrols;
    
    slcontrols = data->controls->slopescontrols;
    
    rs_zoom_out (GWY_GRAPH (slcontrols->graph), &(slcontrols->zoomstack));
    update_models_slopes(slcontrols);

}

void
slopes_zoom_restore(Data *data)
{
    SlopesControls *slcontrols;

    slcontrols = data->controls->slopescontrols;
    
    rs_zoom_restore (GWY_GRAPH (slcontrols->graph), &(slcontrols->zoomstack));
    update_models_slopes(slcontrols);
}

#endif
