#ifndef NOFORTRAN

#include <gtk/gtk.h>
#include <math.h>
#include <app/file.h>
#include "indent-toolbox-common.h"

#include "datatypes.h"
#include "controls.h"

#include "gui-utils.h"
#include "mc-utils.h"

#include "tool_twoslopes_mc.h"
#include "tool_twoslopes_unc.h"
#include "tool_twoslopes.h"

#define NPARAM 8  //  Sload, Sunload,Hit, Er, Eit, Aphc, m, n

#define MAXBAD 5 //maximum percentage of nonsense results before warning is triggered

#define BETA 1

void
init_SlopesMCdata (SlopesMCdata *slopesmc, gint Nmc)
{
	gint i;

	//number of iterations
	slopesmc->Nmc = Nmc;
	printf("initialize for %d iterations \n", slopesmc->Nmc);

	//how much had to be skipped 
	slopesmc->skipmc = 0;

	//dynamically allocate
	slopesmc->slopesmc = (Slopesdata *)g_malloc(slopesmc->Nmc*sizeof(Slopesdata));
	slopesmc->mcavg = (gdouble *)g_malloc(NPARAM*sizeof(gdouble));
	slopesmc->mcstd = (gdouble *)g_malloc(NPARAM*sizeof(gdouble));
	slopesmc->mcdd = (gdouble **) g_malloc(NPARAM*sizeof(gdouble *));
	slopesmc->mcxhist = (gdouble **) g_malloc(NPARAM*sizeof(gdouble *));
	slopesmc->mcyhist = (gdouble **) g_malloc(NPARAM*sizeof(gdouble *));

	slopesmc->mcnstat = (gint)floor(3.49*cbrt(slopesmc->Nmc)+0.5);
	for (i = 0; i< NPARAM; i++){
		slopesmc->mcdd[i] = (gdouble *) g_malloc(slopesmc->Nmc*sizeof(gdouble));
		slopesmc->mcxhist[i] = (gdouble *) g_malloc(slopesmc->mcnstat*sizeof(gdouble));
		slopesmc->mcyhist[i] = (gdouble *) g_malloc(slopesmc->mcnstat*sizeof(gdouble));
	}
}

void
destroy_SlopesMCdata (SlopesMCdata *slopesmc)
{
	gint i;


	//free everything
	g_free(slopesmc->slopesmc);
	g_free(slopesmc->mcavg);
	g_free(slopesmc->mcstd);

	for (i = 0; i< NPARAM; i++){
		g_free(slopesmc->mcdd[i]);
		g_free(slopesmc->mcxhist[i]);
		g_free(slopesmc->mcyhist[i]);
	}

	g_free(slopesmc->mcdd);
	g_free(slopesmc->mcxhist);
	g_free(slopesmc->mcyhist);

	slopesmc->mcnstat = 0;
	slopesmc->skipmc = 0;
}

static	gboolean 
nparam_to_string(gint i, gchar *str, gint nstr)
{

	switch (i){
		case 0:
			g_snprintf(str,nstr,"S_load/(mN/nm)");
			break;
		case 1:
			g_snprintf(str,nstr,"S_unload/(mN/nm)");
			break;
		case 2:
			g_snprintf(str,nstr,"H_IT/MPa");
			break;
		case 3:
			g_snprintf(str,nstr,"E_r/GPa");
			break;
		case 4:
			g_snprintf(str,nstr,"E_IT/GPa");
			break;
		case 5:
			g_snprintf(str,nstr,"A_p(h_c)/nm^2");
			break;
		case 6:
			g_snprintf(str,nstr,"m");
			break;
		case 7:
			g_snprintf(str,nstr,"n");
			break;
		default:
			printf("Unhandled number of results.\n");
			return FALSE;
	}
	return TRUE;
}


void slopes_mc_write_file(gchar *filename, Args *args) 
{
	FILE *file;
	Slopesdata *sldata;
	gint i,j;
	gchar str[200];
	gboolean is;

	if (args->fddata.hall->res == 0 || args->fddata.Fall->res==0) {
	    show_error("No data.");
	    return;
	}

	file = fopen(filename, "w");
	if (!file) {
	    show_error("Could not open file.");
	    return;
	}

	fprintf(file, "# %s \n", args->fddata.filename);
	fprintf(file, "#Monte Carlo simulation of two slopes analysis (ODR fit)\n");
	fprintf(file, "#Input uncertainties \n");
	fprintf(file, "#u(h) = %g nm \n", args->slopesunc.uh);
	fprintf(file, "#u(F) = %g mN \n", args->slopesunc.uF);
	fprintf(file, "#Number of iterations  = %d  \n", args->slopesmc.Nmc);
	fprintf(file, "\n");

	fprintf(file, "#Parameters of two slopes analysis (ODR fit) \n");
	fprintf(file, "#nu: %g                 \n", args->instdata.nu);
	fprintf(file, "#nu_indenter: %g        \n", args->instdata.nui);
	fprintf(file, "#E_indenter: %g      GPa\n", args->instdata.Ei * 1e-9); //TODO units
	fprintf(file, "\n");
	fprintf(file, "#Fitted loadrange: ");
	if (args->slopesdata.Finputload)
		fprintf(file, "	[%g , %g ] %% Fmax \n", args->slopesdata.loadrange_pct_Fmax[0], args->slopesdata.loadrange_pct_Fmax[1]);
	else
		fprintf(file, "[%g , %g ] nm \n",args->slopesdata.loadrange[0], args->slopesdata.loadrange[1]);	
	fprintf(file, "# corresponds : ");
	if (args->slopesdata.Finputload)
		fprintf(file, "[%g , %g ] nm \n",args->slopesdata.loadrange[0], args->slopesdata.loadrange[1]);	
	else
		fprintf(file, "	[%g , %g ] %% Fmax \n", args->slopesdata.loadrange_pct_Fmax[0], args->slopesdata.loadrange_pct_Fmax[1]);
	fprintf(file, "# %d datapoints used.\n", args->slopesdata.nfitloaddata);
	fprintf(file, "\n");
fprintf(file, "#Fitted unloadrange: ");
	if (args->slopesdata.Finputunload)
		fprintf(file, "	[%g , %g ] %% Fmax \n", args->slopesdata.unloadrange_pct_Fmax[0], args->slopesdata.unloadrange_pct_Fmax[1]);
	else
		fprintf(file, "[%g , %g ] nm \n",args->slopesdata.unloadrange[0], args->slopesdata.unloadrange[1]);	
	fprintf(file, "# corresponds : ");
	if (args->slopesdata.Finputunload)
		fprintf(file, "[%g , %g ] nm \n",args->slopesdata.unloadrange[0], args->slopesdata.unloadrange[1]);	
	else
		fprintf(file, "	[%g , %g ] %% Fmax \n", args->slopesdata.unloadrange_pct_Fmax[0], args->slopesdata.unloadrange_pct_Fmax[1]);
	fprintf(file, "# %d datapoints used.\n", args->slopesdata.nfitunloaddata);
	fprintf(file, "\n");


	fprintf(file, "\n\n");


	fprintf(file, "#Results \n");
	for (i = 0; i < NPARAM; i++){
		is = nparam_to_string(i, str, sizeof(str));
		fprintf(file, "#%s = (%g ± %g ) \n", str, args->slopesmc.mcavg[i], args->slopesmc.mcstd[i]);
	}

	fprintf(file, "\n\n\n");


	// Histograms

	if (args->slopesmc.mcnstat >0){
		fprintf(file,"#Histogram\n");
		for (i=0; i < NPARAM; i++){
			fprintf(file,"#Histogram\n");
			is = nparam_to_string(i, str, sizeof(str));
			if (is){
				fprintf(file,"#%s    count/a.u.  \n", str);
				for (j = 0;j < args->slopesmc.mcnstat; j++)
					fprintf(file, "%g %g \n", args->slopesmc.mcxhist[i][j], args->slopesmc.mcyhist[i][j]);
				fprintf(file, "\n \n");
			}
		}
	}


	fprintf(file, "\n\n\n");

	//Full data
	fprintf(file, "#Full data \n");
	fprintf(file, "#");
	for ( j = 0; j< NPARAM; j++){
		is = nparam_to_string(j, str, sizeof(str));
		fprintf(file, "%s ", str);
	}
	fprintf(file,"\n");

	sldata = args->slopesmc.slopesmc;
	for (i = 0; i< args->slopesmc.Nmc; i++){
		fprintf(file, SLOPE, sldata->Sload);
		fprintf(file,"  ");
		fprintf(file, SLOPE, sldata->Sunload);
		fprintf(file,"  ");
		fprintf(file, HARD, sldata->Hit);
		fprintf(file,"  ");
		fprintf(file, MODUL, sldata->Er);
		fprintf(file,"  ");
		fprintf(file, MODUL, sldata->Eit);
		fprintf(file,"  ");
		fprintf(file, AREA, sldata->Aphc);
		fprintf(file,"  ");
		fprintf(file, NUMBER, sldata->m);
		fprintf(file,"  ");
		fprintf(file, NUMBER, sldata->n);
		fprintf(file,"  \n");
		sldata++;
	}


	fclose(file); 
}

static void slopes_mc_hist_Aphc( Data *data)
{
	SlopesMCControls *ctrls;
	GtkWidget *graph;
	GwyGraphModel *graph_model;
	GwyGraphCurveModel *cmodel;
	gint j=5;

	ctrls = data->controls->slopesmccontrols;

	printf("show Aphc histogram \n");
	if (GTK_IS_WIDGET(ctrls->hist_window[j]))
	{
		gtk_window_present(GTK_WINDOW ( ctrls->hist_window[j]));
		return;
	}

	ctrls->hist_window[j] = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->hist_window[j]), "Probability distribution of A_p(h_c)");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->hist_window[j]), 300, 200); 
	g_signal_connect (ctrls->hist_window[j], "delete_event", G_CALLBACK (gtk_widget_destroy),NULL); // CHECK

	graph_model = gwy_graph_model_new();
	g_object_set(graph_model, "title", "PDF h_c", NULL);
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_BOTTOM, "A_p(h_c) / nm^2");
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_LEFT, "count / a.u.");	

	graph = gwy_graph_new(graph_model);
	gtk_widget_set_size_request(GTK_WIDGET(graph), 200, 150);

	gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
	gwy_graph_set_status(GWY_GRAPH(graph), GWY_GRAPH_STATUS_XSEL);

	cmodel = gwy_graph_curve_model_new();

	g_object_set(cmodel,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "PDF(A_p(h_c))",
			"point-size", 3,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
			"color", CURVE_COLOR_UNLOAD,
			"line-style", GDK_LINE_SOLID,
			NULL);
	gwy_graph_model_add_curve(graph_model, cmodel);

	gwy_graph_curve_model_set_data(cmodel,data->args->slopesmc.mcxhist[j], data->args->slopesmc.mcyhist[j], data->args->slopesmc.mcnstat);

	gtk_container_add(GTK_CONTAINER(ctrls->hist_window[j]), graph);
	gtk_widget_show_all(ctrls->hist_window[j]);
}

static void slopes_mc_hist_Hit( Data *data)
{
	SlopesMCControls *ctrls;
	GtkWidget *graph;
	GwyGraphModel *graph_model;
	GwyGraphCurveModel *cmodel;
	gint j=2;

	ctrls = data->controls->slopesmccontrols;

	printf("show Hit histogram \n");
	if (GTK_IS_WIDGET(ctrls->hist_window[j]))
	{
		gtk_window_present(GTK_WINDOW ( ctrls->hist_window[j]));
		return;
	}

	ctrls->hist_window[j] = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->hist_window[j]), "Probability distribution of H_IT");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->hist_window[j]), 300, 200); 
	g_signal_connect (ctrls->hist_window[j], "delete_event", G_CALLBACK (gtk_widget_destroy),NULL); // CHECK

	graph_model = gwy_graph_model_new();
	g_object_set(graph_model, "title", "PDF H_IT", NULL);
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_BOTTOM, "H_IT / MPa");
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_LEFT, "count / a.u.");	

	graph = gwy_graph_new(graph_model);
	gtk_widget_set_size_request(GTK_WIDGET(graph), 200, 150);

	gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
	gwy_graph_set_status(GWY_GRAPH(graph), GWY_GRAPH_STATUS_XSEL);



	cmodel = gwy_graph_curve_model_new();

	g_object_set(cmodel,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "PDF(H_IT)",
			"point-size", 3,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
			"color", CURVE_COLOR_UNLOAD,
			"line-style", GDK_LINE_SOLID,
			NULL);
	gwy_graph_model_add_curve(graph_model, cmodel);

	gwy_graph_curve_model_set_data(cmodel,data->args->slopesmc.mcxhist[j], data->args->slopesmc.mcyhist[j], data->args->slopesmc.mcnstat);

	gtk_container_add(GTK_CONTAINER(ctrls->hist_window[j]), graph);
	gtk_widget_show_all(ctrls->hist_window[j]);
}

static void slopes_mc_hist_Er( Data *data)
{
	SlopesMCControls *ctrls;
	GtkWidget *graph;
	GwyGraphModel *graph_model;
	GwyGraphCurveModel *cmodel;
	gint j=3;

	ctrls = data->controls->slopesmccontrols;

	printf("show Er histogram \n");
	if (GTK_IS_WIDGET(ctrls->hist_window[j]))
	{
		gtk_window_present(GTK_WINDOW ( ctrls->hist_window[j]));
		return;
	}

	ctrls->hist_window[j] = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->hist_window[j]), "Probability distribution of E_r");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->hist_window[j]), 300, 200); 
	g_signal_connect (ctrls->hist_window[j], "delete_event", G_CALLBACK (gtk_widget_destroy),NULL); // CHECK

	graph_model = gwy_graph_model_new();
	g_object_set(graph_model, "title", "PDF E_r", NULL);
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_BOTTOM, "E_r / GPa");
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_LEFT, "count / a.u.");	

	graph = gwy_graph_new(graph_model);
	gtk_widget_set_size_request(GTK_WIDGET(graph), 200, 150);

	gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
	gwy_graph_set_status(GWY_GRAPH(graph), GWY_GRAPH_STATUS_XSEL);



	cmodel = gwy_graph_curve_model_new();

	g_object_set(cmodel,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "PDF(E_r)",
			"point-size", 3,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
			"color", CURVE_COLOR_UNLOAD,
			"line-style", GDK_LINE_SOLID,
			NULL);
	gwy_graph_model_add_curve(graph_model, cmodel);

	gwy_graph_curve_model_set_data(cmodel,data->args->slopesmc.mcxhist[j], data->args->slopesmc.mcyhist[j], data->args->slopesmc.mcnstat);

	gtk_container_add(GTK_CONTAINER(ctrls->hist_window[j]), graph);
	gtk_widget_show_all(ctrls->hist_window[j]);
}

static void slopes_mc_hist_Eit( Data *data)
{
	SlopesMCControls *ctrls;
	GtkWidget *graph;
	GwyGraphModel *graph_model;
	GwyGraphCurveModel *cmodel;
	gint j=4;

	ctrls = data->controls->slopesmccontrols;

	printf("show hc histogram \n");
	if (GTK_IS_WIDGET(ctrls->hist_window[j]))
	{
		gtk_window_present(GTK_WINDOW ( ctrls->hist_window[j]));
		return;
	}

	ctrls->hist_window[j] = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->hist_window[j]), "Probability distribution of E_IT");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->hist_window[j]), 300, 200); 
	g_signal_connect (ctrls->hist_window[j], "delete_event", G_CALLBACK (gtk_widget_destroy),NULL); // CHECK

	graph_model = gwy_graph_model_new();
	g_object_set(graph_model, "title", "PDF E_IT", NULL);
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_BOTTOM, "E_IT / GPa");
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_LEFT, "count / a.u.");	

	graph = gwy_graph_new(graph_model);
	gtk_widget_set_size_request(GTK_WIDGET(graph), 200, 150);

	gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
	gwy_graph_set_status(GWY_GRAPH(graph), GWY_GRAPH_STATUS_XSEL);



	cmodel = gwy_graph_curve_model_new();

	g_object_set(cmodel,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "PDF(E_IT)",
			"point-size", 3,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
			"color", CURVE_COLOR_UNLOAD,
			"line-style", GDK_LINE_SOLID,
			NULL);
	gwy_graph_model_add_curve(graph_model, cmodel);

	gwy_graph_curve_model_set_data(cmodel,data->args->slopesmc.mcxhist[j], data->args->slopesmc.mcyhist[j], data->args->slopesmc.mcnstat);

	gtk_container_add(GTK_CONTAINER(ctrls->hist_window[j]), graph);
	gtk_widget_show_all(ctrls->hist_window[j]);

}

static void slopes_mc_hist_Sl( Data *data)
{
	SlopesMCControls *ctrls;
	GtkWidget *graph;
	GwyGraphModel *graph_model;
	GwyGraphCurveModel *cmodel;
	gint j=0;

	ctrls = data->controls->slopesmccontrols;

	printf("show Sl histogram \n");
	if (GTK_IS_WIDGET(ctrls->hist_window[j]))
	{
		gtk_window_present(GTK_WINDOW ( ctrls->hist_window[j]));
		return;
	}

	ctrls->hist_window[j] = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->hist_window[j]), "Probability distribution of Sl");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->hist_window[j]), 300, 200); 
	g_signal_connect (ctrls->hist_window[j], "delete_event", G_CALLBACK (gtk_widget_destroy),NULL); // CHECK

	graph_model = gwy_graph_model_new();
	g_object_set(graph_model, "title", "PDF Sl", NULL);
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_BOTTOM, "Sl / (mN/nm)");
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_LEFT, "count / a.u.");	

	graph = gwy_graph_new(graph_model);
	gtk_widget_set_size_request(GTK_WIDGET(graph), 200, 150);

	gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
	gwy_graph_set_status(GWY_GRAPH(graph), GWY_GRAPH_STATUS_XSEL);

	cmodel = gwy_graph_curve_model_new();

	g_object_set(cmodel,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "PDF(Sl)",
			"point-size", 3,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
			"color", CURVE_COLOR_UNLOAD,
			"line-style", GDK_LINE_SOLID,
			NULL);
	gwy_graph_model_add_curve(graph_model, cmodel);

	gwy_graph_curve_model_set_data(cmodel,data->args->slopesmc.mcxhist[j], data->args->slopesmc.mcyhist[j], data->args->slopesmc.mcnstat);

	gtk_container_add(GTK_CONTAINER(ctrls->hist_window[j]), graph);
	gtk_widget_show_all(ctrls->hist_window[j]);

}

static void slopes_mc_hist_Su( Data *data)
{
	SlopesMCControls *ctrls;
	GtkWidget *graph;
	GwyGraphModel *graph_model;
	GwyGraphCurveModel *cmodel;
	gint j=1;

	ctrls = data->controls->slopesmccontrols;

	printf("show Su histogram \n");
	if (GTK_IS_WIDGET(ctrls->hist_window[j]))
	{
		gtk_window_present(GTK_WINDOW ( ctrls->hist_window[j]));
		return;
	}

	ctrls->hist_window[j] = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->hist_window[j]), "Probability distribution of Su");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->hist_window[j]), 300, 200); 
	g_signal_connect (ctrls->hist_window[j], "delete_event", G_CALLBACK (gtk_widget_destroy),NULL); // CHECK

	graph_model = gwy_graph_model_new();
	g_object_set(graph_model, "title", "PDF Su", NULL);
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_BOTTOM, "Su / (mN/nm)");
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_LEFT, "count / a.u.");	

	graph = gwy_graph_new(graph_model);
	gtk_widget_set_size_request(GTK_WIDGET(graph), 200, 150);

	gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
	gwy_graph_set_status(GWY_GRAPH(graph), GWY_GRAPH_STATUS_XSEL);

	cmodel = gwy_graph_curve_model_new();

	g_object_set(cmodel,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "PDF(Su)",
			"point-size", 3,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
			"color", CURVE_COLOR_UNLOAD,
			"line-style", GDK_LINE_SOLID,
			NULL);
	gwy_graph_model_add_curve(graph_model, cmodel);

	gwy_graph_curve_model_set_data(cmodel,data->args->slopesmc.mcxhist[j], data->args->slopesmc.mcyhist[j], data->args->slopesmc.mcnstat);

	gtk_container_add(GTK_CONTAINER(ctrls->hist_window[j]), graph);
	gtk_widget_show_all(ctrls->hist_window[j]);

}

static void slopes_mc_hist_m( Data *data)
{
	SlopesMCControls *ctrls;
	GtkWidget *graph;
	GwyGraphModel *graph_model;
	GwyGraphCurveModel *cmodel;
	gint j=6;

	ctrls = data->controls->slopesmccontrols;

	printf("show hc histogram \n");
	if (GTK_IS_WIDGET(ctrls->hist_window[j]))
	{
		gtk_window_present(GTK_WINDOW ( ctrls->hist_window[j]));
		return;
	}

	ctrls->hist_window[j] = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->hist_window[j]), "Probability distribution of m");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->hist_window[j]), 300, 200); 
	g_signal_connect (ctrls->hist_window[j], "delete_event", G_CALLBACK (gtk_widget_destroy),NULL); // CHECK

	graph_model = gwy_graph_model_new();
	g_object_set(graph_model, "title", "PDF m", NULL);
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_BOTTOM, "m");
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_LEFT, "count / a.u.");	

	graph = gwy_graph_new(graph_model);
	gtk_widget_set_size_request(GTK_WIDGET(graph), 200, 150);

	gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
	gwy_graph_set_status(GWY_GRAPH(graph), GWY_GRAPH_STATUS_XSEL);

	cmodel = gwy_graph_curve_model_new();

	g_object_set(cmodel,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "PDF(m)",
			"point-size", 3,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
			"color", CURVE_COLOR_UNLOAD,
			"line-style", GDK_LINE_SOLID,
			NULL);
	gwy_graph_model_add_curve(graph_model, cmodel);

	gwy_graph_curve_model_set_data(cmodel,data->args->slopesmc.mcxhist[j], data->args->slopesmc.mcyhist[j], data->args->slopesmc.mcnstat);

	gtk_container_add(GTK_CONTAINER(ctrls->hist_window[j]), graph);
	gtk_widget_show_all(ctrls->hist_window[j]);

}

static void slopes_mc_hist_n( Data *data)
{
	SlopesMCControls *ctrls;
	GtkWidget *graph;
	GwyGraphModel *graph_model;
	GwyGraphCurveModel *cmodel;
	gint j=7;

	ctrls = data->controls->slopesmccontrols;

	printf("show n histogram \n");
	if (GTK_IS_WIDGET(ctrls->hist_window[j]))
	{
		gtk_window_present(GTK_WINDOW ( ctrls->hist_window[j]));
		return;
	}

	ctrls->hist_window[j] = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->hist_window[j]), "Probability distribution of n");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->hist_window[j]), 300, 200); 
	g_signal_connect (ctrls->hist_window[j], "delete_event", G_CALLBACK (gtk_widget_destroy),NULL); // CHECK

	graph_model = gwy_graph_model_new();
	g_object_set(graph_model, "title", "PDF n", NULL);
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_BOTTOM, "n");
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_LEFT, "count / a.u.");	

	graph = gwy_graph_new(graph_model);
	gtk_widget_set_size_request(GTK_WIDGET(graph), 200, 150);

	gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
	gwy_graph_set_status(GWY_GRAPH(graph), GWY_GRAPH_STATUS_XSEL);

	cmodel = gwy_graph_curve_model_new();

	g_object_set(cmodel,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "PDF(n)",
			"point-size", 3,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
			"color", CURVE_COLOR_UNLOAD,
			"line-style", GDK_LINE_SOLID,
			NULL);
	gwy_graph_model_add_curve(graph_model, cmodel);

	gwy_graph_curve_model_set_data(cmodel,data->args->slopesmc.mcxhist[j], data->args->slopesmc.mcyhist[j], data->args->slopesmc.mcnstat);

	gtk_container_add(GTK_CONTAINER(ctrls->hist_window[j]), graph);
	gtk_widget_show_all(ctrls->hist_window[j]);

}

static void slopes_mc_save_data( Args *args)
{
	GtkWidget *dialog;
	gchar *filename;

	dialog = gtk_file_chooser_dialog_new("Save File",
			NULL,
			GTK_FILE_CHOOSER_ACTION_SAVE,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_SAVE, GTK_RESPONSE_OK,
			NULL);

	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), gwy_app_get_current_directory());

	if ( gtk_dialog_run (GTK_DIALOG ( dialog)) == GTK_RESPONSE_OK)
	{
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		if (gwy_app_file_confirm_overwrite(dialog))
			slopes_mc_write_file(filename, args);
		g_free(filename);
	}
	gtk_widget_destroy ( dialog);
}

void 
slopes_mc_close_window(Data *data)
{
	gint j;

	if (data->controls->slopesmccontrols->has_window){
		gtk_widget_destroy(data->controls->slopesmccontrols->window);

		destroy_SlopesMCdata(&(data->args->slopesmc));
		data->controls->slopesmccontrols->has_window = FALSE;
		for (j=0; j< NPARAM; j++){
			if (GTK_IS_WIDGET(data->controls->slopesmccontrols->hist_window[j])){
				gtk_widget_destroy(data->controls->slopesmccontrols->hist_window[j]);
			}
		}
		g_free(data->controls->slopesmccontrols->hist_window);
		g_free(data->controls->slopesmccontrols->button_hist);
	}
}

void slopes_uncertainty_montecarlo_run_calc( Args *args)
{
	gint istart, iend, ndata;
	gint i, Nmc;
	Slopesdata *slopes;
	SlopesMCdata *mc;
	GwyDataLine *hl, *Fl;
	GwyDataLine *hlnew, *Flnew;
	GwyDataLine *hlnoise, *Flnoise;
	GwyDataLine *hu, *Fu;
	GwyDataLine *hunew, *Funew;
	GwyDataLine *hunoise, *Funoise;
	gdouble hmax_old, Fmax_old;
	GRand *rng;
	gint nn;
	gdouble **dd;
	gdouble *xhist, *yhist;
	gboolean hist;
	gint *infol, *infou;
	gdouble estimateu[3], estimatel[3];

	//not necessary to check range, couldn't get here in the first place

	//find appropriate loading data, which should be fitted, use same regime as for the main fit
	if (args->slopesdata.Finputload)
		range_to_indices(args->slopesdata.loadfrom_pct_Fmax*0.01*args->fddata.Fmax, 
				args->slopesdata.loadto_pct_Fmax*0.01*args->fddata.Fmax,
			       	args->fddata.Fload, FALSE, &istart, &iend, &ndata);
	else
		range_to_indices(args->slopesdata.loadfrom, args->slopesdata.loadto, args->fddata.hload, FALSE, &istart, &iend, &ndata);

	// if there are not enough data in the selected range, not fitting can be performed
	ndata = MIN(ndata, args->fddata.hload->res - istart);

	if (ndata < 2){
		return;
	}

	//get data to be fitted
	hl = gwy_data_line_part_extract(args->fddata.hload, istart, ndata);
	Fl = gwy_data_line_part_extract(args->fddata.Fload, istart, ndata);

	//find appropriate unloadingdata, which should be fitted, use same regime as for the main fit
	if (args->slopesdata.Finputunload)
		range_to_indices(args->slopesdata.unloadfrom_pct_Fmax*0.01*args->fddata.Fmax, 
				args->slopesdata.unloadto_pct_Fmax*0.01*args->fddata.Fmax,
			       	args->fddata.Funload, TRUE, &istart, &iend, &ndata);
	else
		range_to_indices(args->slopesdata.unloadfrom, args->slopesdata.unloadto, args->fddata.hunload, TRUE, &istart, &iend, &ndata);

	// if there are not enough data in the selected range, not fitting can be performed
	ndata = MIN(ndata, args->fddata.hunload->res - istart);

	if (ndata < 2){
		return;
	}

	//get data to be fitted
	hu = gwy_data_line_part_extract(args->fddata.hunload, istart, ndata);
	Fu = gwy_data_line_part_extract(args->fddata.Funload, istart, ndata);


	//back up hmax, Fmax
	hmax_old = args->fddata.hmax;
	Fmax_old = args->fddata.Fmax;

	//use results of previous fit as input estimates
	estimatel[0]= args->slopesdata.gamma;
	estimatel[1]= args->slopesdata.h0;
	estimatel[2]= args->slopesdata.n;
	estimateu[0]= args->slopesdata.alpha;
	estimateu[1]= args->slopesdata.hp;
	estimateu[2]= args->slopesdata.m;


	Nmc = args->slopesmc.Nmc;
	slopes = args->slopesmc.slopesmc;
	//prepare data
	for (i=0; i< Nmc ;i++){
		init_Slopesdata(slopes);
		slopes++;
	}

	slopes = args->slopesmc.slopesmc;
	//copy input parameters
	for (i=0; i< Nmc ;i++){
		/* opodr->nu = args->opodrdata.nu; */
		/* opodr->nui = args->opodrdata.nui; */
		/* opodr->Ei = args->opodrdata.Ei; */
		//		opodr->from = args->opodrdata.from;
		//		opodr->to = args->opodrdata.to;
		slopes++;
	}

	/*
	   for (i=0; i< Nmc ;i++){
	   printf ("nu %g \n", args->opodrmc.opodrmc[i].nu);
	   }
	   */

	// initialize random generator and datafield
	rng = g_rand_new();

	//prepare datafield for noise
	hlnoise=gwy_data_line_new_alike(hl,FALSE);
	Flnoise=gwy_data_line_new_alike(hl,FALSE);
	hunoise=gwy_data_line_new_alike(hu,FALSE);
	Funoise=gwy_data_line_new_alike(hu,FALSE);

	//prepare datafield for varied fields
	hlnew=gwy_data_line_new_alike(hl,FALSE);
	Flnew=gwy_data_line_new_alike(Fl,FALSE);
	hunew=gwy_data_line_new_alike(hu,FALSE);
	Funew=gwy_data_line_new_alike(Fu,FALSE);

	//prepare auxiliary field
	infol = (gint *)g_malloc0(Nmc*sizeof(gint));
	infou = (gint *)g_malloc0(Nmc*sizeof(gint));

	slopes = args->slopesmc.slopesmc;
	for (i=0; i< Nmc; i++){

		//create noise for h-F data
		generate_uncorrelated_noise(hlnoise, args->slopesunc.uh, rng);
		generate_uncorrelated_noise(Flnoise, args->slopesunc.uF, rng);
		generate_uncorrelated_noise(hunoise, args->slopesunc.uh, rng);
		generate_uncorrelated_noise(Funoise, args->slopesunc.uF, rng);

		//add noise to original h-F data
		sum_data_lines(hlnew,hl, hlnoise);
		sum_data_lines(Flnew,Fl, Flnoise);
		sum_data_lines(hunew,hu, hunoise);
		sum_data_lines(Funew,Fu, Funoise);

		//add random noise to hmax, Fmax
		args->fddata.hmax = hmax_old + gaussian_random_number(rng)*args->slopesunc.uh;
		args->fddata.Fmax = Fmax_old + gaussian_random_number(rng)*args->slopesunc.uF;

		//run fit
		infol[i] = slopes_fit_load(hlnew,Flnew,slopes, &(args->fddata), &(args->instdata),  estimatel);
		slopes->has_fit_load = TRUE;
		infou[i] = slopes_fit_unload(hunew,Funew,slopes, &(args->fddata), &(args->instdata),  estimateu);

		slopes++;
	}

	//reset hmax, Fmax to original value
	args->fddata.hmax = hmax_old;
	args->fddata.Fmax = Fmax_old;

//	printf("run Monte Carlo \n");	
	g_object_unref(hl);
	g_object_unref(Fl);
	g_object_unref(hlnew);
	g_object_unref(Flnew);
	g_object_unref(hlnoise);
	g_object_unref(Flnoise);
	g_object_unref(hu);
	g_object_unref(Fu);
	g_object_unref(hunew);
	g_object_unref(Funew);
	g_object_unref(hunoise);
	g_object_unref(Funoise);
	g_rand_free(rng);


		for (i=0; i< Nmc; i++){
//					printf(" %d Er %g Eit %g Hit %g \n", i, args->slopesmc.slopesmc[i].Er, args->slopesmc.slopesmc[i].Eit, args->slopesmc.slopesmc[i].Hit);
		}

		//calculate averages and stdevs
		//hp, Aphc, Hit, Er, Eit, S, m

		mc = &(args->slopesmc);
		for (i = 0; i< NPARAM; i++){
			mc->mcavg[i] =0;
			mc->mcstd[i] =0;
		}

		slopes = args->slopesmc.slopesmc;
		nn = 0;
		for (i = 0; i< Nmc; i++){
			//check sanity
//			if (infol[i] >= 4 && infou[i] >= 4)
			if (slopes->m >1 && slopes->n >1)
			{
				mc->mcavg[0]+=slopes->Sload;
				mc->mcavg[1]+=slopes->Sunload;
				mc->mcavg[2]+=slopes->Hit;
				mc->mcavg[3]+=slopes->Er;
				mc->mcavg[4]+=slopes->Eit;
				mc->mcavg[5]+=slopes->Aphc;
				mc->mcavg[6]+=slopes->m;
				mc->mcavg[7]+=slopes->n;
				mc->mcstd[0]+=slopes->Sload*slopes->Sload;
				mc->mcstd[1]+=slopes->Sunload*slopes->Sunload;
				mc->mcstd[2]+=slopes->Hit*slopes->Hit;
				mc->mcstd[3]+=slopes->Er*slopes->Er;
				mc->mcstd[4]+=slopes->Eit*slopes->Eit;
				mc->mcstd[5]+=slopes->Aphc*slopes->Aphc;
				mc->mcstd[6]+=slopes->m*slopes->m;
				mc->mcstd[7]+=slopes->n*slopes->n;
				nn++;
			}
			slopes++;
		}

		g_free(infol);
		g_free(infou);

		//use only reasonable data for average and standard deviation
		for (i = 0; i< NPARAM; i++){
			mc->mcavg[i] /= nn;
			mc->mcstd[i] /= nn;
			mc->mcstd[i]-= mc->mcavg[i]*mc->mcavg[i];
			mc->mcstd[i] = (mc->mcstd[i] < 0) ? 0 : mc->mcstd[i];
			mc->mcstd[i] = sqrt(mc->mcstd[i]);
		}

		// how many datasets were skipped
		args->slopesmc.skipmc = Nmc-nn;

		/*
		   printf("mean  stdev \n");
		   for (i = 0; i< NPARAM; i++){
		   printf(" %g   %g \n", avg[i], std[i]);
		   }
		   */

		// Histograms
		//create data structure

		dd = args->slopesmc.mcdd;
		slopes = args->slopesmc.slopesmc;
		for (i = 0; i< Nmc; i++){
			dd[0][i] = slopes->Sload;
			dd[1][i] = slopes->Sunload;
			dd[2][i] = slopes->Hit;
			dd[3][i] = slopes->Er;
			dd[4][i] = slopes->Eit;
			dd[5][i] = slopes->Aphc;
			dd[6][i] = slopes->m;
			dd[7][i] = slopes->n;
			slopes++;
		}
		//prepare histogram structure


		for (i=0; i < NPARAM; i++){
			xhist = mc->mcxhist[i];
			yhist = mc->mcyhist[i];
			hist = create_histogram(dd[i], Nmc, xhist, yhist, mc->mcnstat );
			if (!hist){
				printf("Warning: No histogram could be created.\n");
				mc->mcnstat = -1;
				break;
			}
		}
}

void slopes_uncertainty_montecarlo( Data *data)
{

	gint i, j;
	GtkWidget *label;
	gchar str[200];
	GdkColor color;

	SlopesMCControls *ctrls;
	SlopesMCdata *slopesmc;

	gdk_color_parse ("red", &color);


	//if there was a previous window, close it and any histograms, free data
	slopes_mc_close_window(data);

	slopesmc = &(data->args->slopesmc);

	// initialize for number of iterations which was chosen in the Uncertainties dialog
	init_SlopesMCdata(slopesmc, data->args->slopesunc.Nmc); 

	// run calculation
	slopes_uncertainty_montecarlo_run_calc(data->args);

	//create window and display results
	ctrls = data ->controls->slopesmccontrols;

	ctrls->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->window), "Monte Carlo calculation of uncertainties in Oliver Pharr analysis (ODR fit)");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->window), 400, 300); 
	g_signal_connect_swapped (ctrls->window, "delete_event", G_CALLBACK (slopes_mc_close_window), data);
	gtk_window_set_transient_for (GTK_WINDOW(ctrls->window), GTK_WINDOW(data->controls->maincontrols->window_main));

	ctrls->frame_input = gtk_frame_new(NULL);
	gtk_frame_set_label_widget (GTK_FRAME (ctrls->frame_input),
			GTK_WIDGET (gwy_label_new_header("Input for Monte Carlo simulation")));

	ctrls->table_input = gtk_table_new(3, 4, FALSE);

	gtk_container_set_border_width (GTK_CONTAINER (ctrls->table_input), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (ctrls->table_input), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (ctrls->table_input), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add ( GTK_CONTAINER (ctrls->frame_input), ctrls->table_input);

	i = 0;

	/*          u(h)             */

	gtk_table_attach(GTK_TABLE(ctrls->table_input), gwy_label_new_header("u(h)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);

	label = gtk_label_new(NULL);
	g_snprintf(str,sizeof(str), "%g", data->args->slopesunc.uh);
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_input), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	gtk_table_attach(GTK_TABLE(ctrls->table_input), label_new_with_markup_left("nm"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);

	gtk_table_attach(GTK_TABLE(ctrls->table_input), label_new_with_markup_left("normal distribution"), 3, 4, i, i+1, GTK_FILL, 0, 0, 0);

	i++;

	/*          u(F)             */

	gtk_table_attach(GTK_TABLE(ctrls->table_input), gwy_label_new_header("u(F)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);

	label = gtk_label_new(NULL);
	g_snprintf(str,sizeof(str), "%g", data->args->slopesunc.uF);
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_input), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	gtk_table_attach(GTK_TABLE(ctrls->table_input), label_new_with_markup_left("mN"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);

	gtk_table_attach(GTK_TABLE(ctrls->table_input), label_new_with_markup_left("normal distribution"), 3, 4, i, i+1, GTK_FILL, 0, 0, 0);

	i++;

	/*          Number of iteration */

	gtk_table_attach(GTK_TABLE(ctrls->table_input), gwy_label_new_header("Number of iterations "), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);

	label = gtk_label_new(NULL);
	g_snprintf(str,sizeof(str), "%d", slopesmc->Nmc);
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_input), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	// Results
	ctrls->frame_results = gtk_frame_new(NULL);
	gtk_frame_set_label_widget (GTK_FRAME (ctrls->frame_results),
			GTK_WIDGET (gwy_label_new_header("Results of Monte Carlo simulation")));

	ctrls->table_results = gtk_table_new(NPARAM+1, 4, FALSE);

	gtk_container_set_border_width (GTK_CONTAINER (ctrls->table_results), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (ctrls->table_results), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (ctrls->table_results), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add ( GTK_CONTAINER (ctrls->frame_results), ctrls->table_results);

	i = 0;

	gtk_table_attach(GTK_TABLE(ctrls->table_results), gwy_label_new_header("mean"), 1, 2, i, i+1, GTK_FILL, 0, 0, 0);
	gtk_table_attach(GTK_TABLE(ctrls->table_results), gwy_label_new_header("stdev"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);

	i++;

	ctrls->button_hist = (GtkWidget **)g_malloc(NPARAM*sizeof(GtkWidget *));
	ctrls->hist_window = (GtkWidget **)g_malloc(NPARAM*sizeof(GtkWidget *));

	for (j = 0; j< NPARAM; j++)
		ctrls->hist_window[j] = NULL;

	/*                       Sload          */

	gtk_table_attach(GTK_TABLE(ctrls->table_results),
			gwy_label_new_header("Sl/(mN/nm)"), 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units

	label = gtk_label_new(NULL);

	g_snprintf(str, sizeof(str), SLOPE, data->args->slopesmc.mcavg[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	label = gtk_label_new(NULL);
	g_snprintf(str, sizeof(str), SLOPE, data->args->slopesmc.mcstd[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 2, 3, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	ctrls->button_hist[i-1] = gtk_button_new();
	gtk_button_set_image (GTK_BUTTON (ctrls->button_hist[i-1]), gtk_image_new_from_stock (GWY_STOCK_SPECTRUM, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (ctrls->button_hist[i-1], "clicked", G_CALLBACK (slopes_mc_hist_Sl), data);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), ctrls->button_hist[i-1], 3, 4, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 
	i++;

/*                       Sunload          */

	gtk_table_attach(GTK_TABLE(ctrls->table_results),
			gwy_label_new_header("Su/(mN/nm)"), 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units

	label = gtk_label_new(NULL);

	g_snprintf(str, sizeof(str), SLOPE, data->args->slopesmc.mcavg[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	label = gtk_label_new(NULL);
	g_snprintf(str, sizeof(str), SLOPE, data->args->slopesmc.mcstd[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 2, 3, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	ctrls->button_hist[i-1] = gtk_button_new();
	gtk_button_set_image (GTK_BUTTON (ctrls->button_hist[i-1]), gtk_image_new_from_stock (GWY_STOCK_SPECTRUM, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (ctrls->button_hist[i-1], "clicked", G_CALLBACK (slopes_mc_hist_Su), data);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), ctrls->button_hist[i-1], 3, 4, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 
	i++;


	/*                       H_IT           */

	gtk_table_attach(GTK_TABLE(ctrls->table_results),
			gwy_label_new_header("H<sub>it</sub>/MPa"), 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units

	label = gtk_label_new(NULL);
	g_snprintf(str, sizeof(str), HARD, data->args->slopesmc.mcavg[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	label = gtk_label_new(NULL);
	g_snprintf(str, sizeof(str), HARD, data->args->slopesmc.mcstd[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 2, 3, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	ctrls->button_hist[i-1] = gtk_button_new();
	gtk_button_set_image (GTK_BUTTON (ctrls->button_hist[i-1]), gtk_image_new_from_stock (GWY_STOCK_SPECTRUM, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (ctrls->button_hist[i-1], "clicked", G_CALLBACK (slopes_mc_hist_Hit), data);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), ctrls->button_hist[i-1], 3, 4, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 
	i++;

	/*                       E_r           */

	gtk_table_attach(GTK_TABLE(ctrls->table_results),
			gwy_label_new_header("E<sub>r</sub>/GPa"), 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units

	label = gtk_label_new(NULL);
	g_snprintf(str, sizeof(str), MODUL, data->args->slopesmc.mcavg[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	label = gtk_label_new(NULL);
	g_snprintf(str, sizeof(str), MODUL, data->args->slopesmc.mcstd[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 2, 3, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	ctrls->button_hist[i-1] = gtk_button_new();
	gtk_button_set_image (GTK_BUTTON (ctrls->button_hist[i-1]), gtk_image_new_from_stock (GWY_STOCK_SPECTRUM, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (ctrls->button_hist[i-1], "clicked", G_CALLBACK (slopes_mc_hist_Er), data);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), ctrls->button_hist[i-1], 3, 4, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 
	i++;

	/*                       E_IT          */

	gtk_table_attach(GTK_TABLE(ctrls->table_results),
			gwy_label_new_header("E<sub>it</sub>/GPa"), 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units

	label = gtk_label_new(NULL);

	g_snprintf(str, sizeof(str), MODUL, data->args->slopesmc.mcavg[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	label = gtk_label_new(NULL);
	g_snprintf(str, sizeof(str), MODUL, data->args->slopesmc.mcstd[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 2, 3, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	ctrls->button_hist[i-1] = gtk_button_new();
	gtk_button_set_image (GTK_BUTTON (ctrls->button_hist[i-1]), gtk_image_new_from_stock (GWY_STOCK_SPECTRUM, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (ctrls->button_hist[i-1], "clicked", G_CALLBACK (slopes_mc_hist_Eit), data);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), ctrls->button_hist[i-1], 3, 4, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 
	i++;

	/*                       A_p(h_c)           */

	gtk_table_attach(GTK_TABLE(ctrls->table_results),
			gwy_label_new_header("A<sub>p</sub>/nm<sup>2</sup>"), 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units

	label = gtk_label_new(NULL);
	g_snprintf(str, sizeof(str), AREA, data->args->slopesmc.mcavg[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	label = gtk_label_new(NULL);
	g_snprintf(str, sizeof(str), AREA, data->args->slopesmc.mcstd[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 2, 3, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	ctrls->button_hist[i-1] = gtk_button_new();
	gtk_button_set_image (GTK_BUTTON (ctrls->button_hist[i-1]), gtk_image_new_from_stock (GWY_STOCK_SPECTRUM, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (ctrls->button_hist[i-1], "clicked", G_CALLBACK (slopes_mc_hist_Aphc), data);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), ctrls->button_hist[i-1], 3, 4, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 
	i++;



	/*                       m          */

	gtk_table_attach(GTK_TABLE(ctrls->table_results),
			gwy_label_new_header("m"), 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units

	label = gtk_label_new(NULL);

	g_snprintf(str, sizeof(str), NUMBER, data->args->slopesmc.mcavg[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	label = gtk_label_new(NULL);
	g_snprintf(str, sizeof(str), NUMBER, data->args->slopesmc.mcstd[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 2, 3, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	ctrls->button_hist[i-1] = gtk_button_new();
	gtk_button_set_image (GTK_BUTTON (ctrls->button_hist[i-1]), gtk_image_new_from_stock (GWY_STOCK_SPECTRUM, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (ctrls->button_hist[i-1], "clicked", G_CALLBACK (slopes_mc_hist_m), data);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), ctrls->button_hist[i-1], 3, 4, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 
	i++;

	/*                       n          */

	gtk_table_attach(GTK_TABLE(ctrls->table_results),
			gwy_label_new_header("n"), 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units

	label = gtk_label_new(NULL);

	g_snprintf(str, sizeof(str), NUMBER, data->args->slopesmc.mcavg[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	label = gtk_label_new(NULL);
	g_snprintf(str, sizeof(str), NUMBER, data->args->slopesmc.mcstd[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 2, 3, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	ctrls->button_hist[i-1] = gtk_button_new();
	gtk_button_set_image (GTK_BUTTON (ctrls->button_hist[i-1]), gtk_image_new_from_stock (GWY_STOCK_SPECTRUM, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (ctrls->button_hist[i-1], "clicked", G_CALLBACK (slopes_mc_hist_n), data);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), ctrls->button_hist[i-1], 3, 4, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 
	i++;




	if (100.*data->args->slopesmc.skipmc/slopesmc->Nmc > MAXBAD){
		label = gtk_label_new(NULL);
		g_snprintf(str, sizeof(str), "Warning: Approx %.1f %% of the results had to be removed!", 100.*data->args->slopesmc.skipmc/slopesmc->Nmc);
		gtk_label_set_text(GTK_LABEL(label),str);
		gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &color);
		gtk_table_attach(GTK_TABLE(ctrls->table_results), label, 0,1, i, i+1, GTK_FILL, GTK_FILL,0,0);
		i++;
	}


	// save data
	ctrls->button_save = gtk_button_new_from_stock (GTK_STOCK_SAVE);
	g_signal_connect_swapped (ctrls->button_save, "clicked", G_CALLBACK (slopes_mc_save_data), data->args);

	ctrls->vbox = gtk_vbox_new(FALSE, CTRLS_SPACING);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->frame_input, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->frame_results, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->button_save, FALSE, FALSE, 0);

	gtk_container_add(GTK_CONTAINER(ctrls->window), ctrls->vbox);
	gtk_widget_show_all(ctrls->window);
	ctrls->has_window = TRUE;

}
#endif
