#ifndef NOFORTRAN

#include <math.h>

#include <gtk/gtk.h>

#include <app/file.h>

#include "indent-toolbox-common.h"
#include "datatypes.h"
#include "controls.h"
#include "fit-utils.h"
#include "gui-utils.h"
#include "tool_twoslopes_unc.h"
#include "tool_twoslopes_mc.h"
#include "tool_twoslopes.h"


#define CONTACTMAX 2
#define MC_WHEN_WARN 5e2
#define DEBUG TRUE
#define BETA 1

static void slopes_fit_shift_contact(Slopesdata *sldata, FDdata *fddata, Instdata *instdata, Area *area, gdouble *Ec, gdouble *Hc, gdouble *Ac, gint j);

void 
slopes_propagate_uncertainties (Data *data)
{
	gchar str[30];
	SlopesUncControls *ctrls;
	SlopesUncdata *unc;
	Slopesdata *sldata;
	FDdata *fd;
	Instdata *inst;
	gdouble uh, uF;
	gdouble umh, umF, uhph, uhpF, um, uhp;
	gdouble unh, unF, uh0h, uh0F, un, uh0;
	gdouble uSuh, uSuF, uSu;
	gdouble uSlh, uSlF ,uSl;
	gdouble z, depsdm;
	gdouble uepsh, uepsF;
	gdouble ueps;
	gdouble beta[3], std_beta[3], covariance[3];
	gint istart, iend,ndata, info;
	GwyDataLine *x, *y;
	gdouble ux,uy;
	gdouble n,Sl, h0, m, Su, hp, eps, hmax,Fmax, Aphc,Hit,Er,Eit, nu,nui,Ei;
	gdouble covnh0h, covnh0F, covnh0;
	gdouble covmhph, covmhpF, covmhp;
	gdouble K, uKuh, uKuF, uK;
	gdouble covKSuh, covKSuF, covKSu;
	gchar fnm[500];

	unc = &(data->args->slopesunc);
	sldata = &(data->args->slopesdata);
	fd = &(data->args->fddata);
	inst = &(data->args->instdata);
	ctrls =	data->controls->slopesunccontrols;

	if ( !sldata->has_fit_load || !sldata->has_fit_unload )
		return;

	if ( unc->Ec ==NULL || unc->Hc == NULL || unc->Ac == NULL)
		return;

	//make copies to simplify code
	uh = unc->uh;
	uF = unc->uF;
	n = sldata->n;
	h0 = sldata->h0;
	Sl = sldata->Sload;
	m = sldata->m;
	hp = sldata->hp;
	Su = sldata->Sunload;
	eps = sldata->eps;
	hmax = fd->hmax;
	Fmax = fd->Fmax;
	Aphc = sldata->Aphc;
	Hit = sldata->Hit;
	Eit = sldata->Eit;
	Er = sldata->Er;
	Ei = inst->Ei;
	nui = inst->nui;
	nu = inst->nu;

	if (DEBUG)
		printf(" \n\nPropagate uncertainties\n");


	/*   u(n)    */

	/* prepare odr fit */

	beta[0] = sldata->gamma;
	beta[1] = sldata->h0;
	beta[2] = sldata->n;

	//find appropriate data, which should be fitted, use same regime as for the main fit
	if (sldata->Finputload)
		range_to_indices(sldata->loadrange_pct_Fmax[0]*0.01*fd->Fmax, sldata->loadrange_pct_Fmax[1]*0.01*fd->Fmax, fd->Fload, FALSE, &istart, &iend, &ndata);
	else
		range_to_indices(sldata->loadrange[0], sldata->loadrange[1], fd->hload, FALSE, &istart, &iend, &ndata);


	x = gwy_data_line_part_extract(fd->hload, istart, ndata);
	y = gwy_data_line_part_extract(fd->Fload, istart, ndata);

	/* fit with only uh */
	ux = uh;
	uy = 0;
	beta[0] = sldata->gamma;
	beta[1] = sldata->h0;
	beta[2] = sldata->n;
	info = odr_run_fit(x, y, beta, fnm, ux, uy, std_beta, covariance, FALSE); /* do not rescale */
	unh = std_beta[2];
	uh0h = std_beta[1];
	covnh0h = covariance[0]; 

	if (DEBUG)
		printf(" fit with only uh:\n  gamma %g n %g h0 %g \n  u(n) %g u(h0) %g corr(n,h0) %g \n",
			       beta[0], beta[1], beta[2], unh, uh0h, covnh0h/unh/uh0h);	       

	/* fit with only uF */
	ux = 0;
	uy = uF;
	info = odr_run_fit(x, y, beta, fnm, ux, uy, std_beta, covariance, FALSE); /* do not rescale */
	unF = std_beta[2];
	uh0F = std_beta[1];
	covnh0F = covariance[0]; 

	if (DEBUG)
		printf(" fit with only uF:\n  gamma %g n %g h0 %g \n  u(n) %g u(h0) %g corr(n,h0) %g \n",
			       beta[0], beta[1], beta[2], unF, uh0F, covnh0F/unF/uh0F);	       

	unc->unuh = unh;
	unc->unuF = unF;



	/*   u(m)    */

	/* prepare odr fit */

	beta[0] = sldata->alpha;
	beta[1] = sldata->hp;
	beta[2] = sldata->m;

	//find appropriate data, which should be fitted, use same regime as for the main fit
	if (sldata->Finputunload)
		range_to_indices(sldata->unloadrange_pct_Fmax[0]*0.01*fd->Fmax, sldata->unloadrange_pct_Fmax[1]*0.01*fd->Fmax, fd->Funload, TRUE, &istart, &iend, &ndata);
	else
		range_to_indices(sldata->unloadrange[0], sldata->unloadrange[1], fd->hunload, TRUE, &istart, &iend, &ndata);


	x = gwy_data_line_part_extract(fd->hunload, istart, ndata);
	y = gwy_data_line_part_extract(fd->Funload, istart, ndata);

	/* fit with only uh */
	ux = uh;
	uy = 0;
	beta[0] = sldata->alpha;
	beta[1] = sldata->hp;
	beta[2] = sldata->m;
	info = odr_run_fit(x, y, beta, fnm, ux, uy, std_beta, covariance, FALSE); /* do not rescale */
	umh = std_beta[2];
	uhph = std_beta[1];
	covmhph = covariance[0]; 

	if (DEBUG)
		printf(" fit with only uh:\n  alpha %g m %g hp %g \n  u(m) %g u(hp) %g corr(m,hp) %g \n",
			       beta[0], beta[1], beta[2], umh, uhph, covmhph/umh/uhph);	       

	/* fit with only uF */
	ux = 0;
	uy = uF;
	info = odr_run_fit(x, y, beta, fnm, ux, uy, std_beta, covariance, FALSE); /* do not rescale */
	umF = std_beta[2];
	uhpF = std_beta[1];
	covmhpF = covariance[0]; 

	if (DEBUG)
		printf(" fit with only uF:\n  alpha %g m %g hp %g \n  u(m) %g u(hp) %g corr(m,hp) %g \n",
			       beta[0], beta[1], beta[2], umF, uhpF, covmhpF/umF/uhpF);	       

	unc->umuh = umh;
	unc->umuF = umF;

	/*   u(eps)    */
	z = m/(m-1)/2;
	depsdm = eps/m - m/(m -1) * (eps/m -1) -m/2/(m-1)/(m-1)*(eps/m -1)* (psi(z)-psi(z-1./2));
	uepsh = depsdm *umh;
	uepsF = depsdm *umF;
	if (DEBUG){
		printf("depsdm %g  z %g  \n", depsdm,z);
		printf(" u(eps;h) %g u(eps;F) %g  urel(eps;h) %g urel(eps;F) %g \n", uepsh, uepsF, uepsh/eps, uepsF/eps);
	}

	/*    u(Sl)   */
	uSlh = sqrt(unh*unh/n/n +
			uh *uh/(hmax-h0)/(hmax-h0) +
			uh0h*uh0h/(hmax-h0)/(hmax-h0) +
			2*covnh0h/n/(hmax-h0)
		  ) * Sl; 

	uSlF = sqrt(unF*unF/n/n +    
		       	uF*uF/Fmax/Fmax  +    
			uh0F*uh0F/(hmax-h0)/(hmax-h0)+
			2*covnh0F/n/(hmax-h0)
		  ) * Sl; 
	unc->uSluh = uSlh;
	unc->uSluF = uSlF;
	/* u(Sl;h) = Sl *sqrt [ (unh/n)^2 + (uh/(hmax-h0))^2 + (uh0h/(hmax-h0))^2  + 2 cov(n,h0;h)/(n*(hmax-h0))]
	 * u(Sl;F) = Sl *sqrt [ (unF/n)^2 + (uFmax/Fmax)^2   + (uh0F/(hmax-h0))^2  + 2 cov(n,h0;F)/(n*(hmax-h0))]
	 */

	if (DEBUG){
		printf(" u(Sl;h) %g u(Sl;F) %g  urel(Sl;h) %g urel(Sl;F) %g \n", uSlh, uSlF, uSlh/Sl, uSlF/Sl);
	}

	/*    u(Su)   */
	uSuh = sqrt(umh*umh/m/m +
			uh *uh/(hmax-hp)/(hmax-hp) +
			uhph*uhph/(hmax-hp)/(hmax-hp) +
			2*covmhph/m/(hmax-hp)
		  ) * Su; 

	uSuF = sqrt(umF*umF/m/m +    
		       	uF*uF/Fmax/Fmax  +    
			uhpF*uhpF/(hmax-hp)/(hmax-hp)+
			2*covmhpF/m/(hmax-hp)
		  ) * Su; 
	unc->uSuuh = uSuh;
	unc->uSuuF = uSuF;
	/* u(S;h) = S *sqrt [ (umh/m)^2 + (uh/(hmax-hp))^2 + (uhph/(hmax-hp))^2  + 2 cov(m,hp;h)/(m*(hmax-hp))]
	 * u(S;F) = S *sqrt [ (umF/m)^2 + (uFmax/Fmax)^2   + (uhpF/(hmax-hp))^2  + 2 cov(m,hp;F)/(m*(hmax-hp))]
	 */

	if (DEBUG){
		printf(" u(Su;h) %g u(Su;F) %g  urel(Su;h) %g urel(Su;F) %g \n", uSuh, uSuF, uSuh/Su, uSuF/Su);
	}

	/*    u(K)   */
	K= (2*Su - Sl*eps*BETA)/Su/Sl;

	uKuh = sqrt( 4*uSlh*uSlh/Sl/Sl/Sl/Sl + eps *BETA*eps*BETA/Su/Su/Su/Su*uSuh*uSuh);
	uKuF = sqrt( 4*uSlF*uSlF/Sl/Sl/Sl/Sl + eps *BETA*eps*BETA/Su/Su/Su/Su*uSuF*uSuF);

	covKSuh = -2/Su/Su*uSuh*uSuh;
	covKSuF = -2/Su/Su*uSuF*uSuF;
	/*  K = 2 /Sl - eps*BETA/Su
	 *  u(K;h) = sqrt [ (-2/S1^2 u(S1;h))^2 + ( eps*BETA/Su^2 u(Su;h)^2 ]
	 *  u(K;F) = sqrt [ (-2/S1^2 u(S1;F))^2 + ( eps*BETA/Su^2 u(Su;F)^2 ] 
	 *  cov(K,Sl; h) = -2/Sl^2 u(Sl;h)^2 
	 *  cov(K,Sl; F) = -2/Sl^2 u(Sl;F)^2 
	 *  cov(K,Su; h) = eps*BETA/Sl^2 u(Sl;h)^2 
	 *  cov(K,Su; F) = eps*BETASl^2 u(Sl;F)^2  */



	/*    u(A)   */
	unc->uAuh = 2 * Aphc * uKuh/K;
	unc->uAuF = 2 * Aphc * sqrt( uKuF*uKuF/K/K + uF*uF/Fmax/Fmax);
	/* u(A; h) =  2A *(uK;h)/K ]
	   u(A; F) =  2A * sqrt [ (uFmax/Fmax)^2 + ((uK;F)/K)^2 ] */
	if (DEBUG)
		printf(" u(A;h) %g u(A;F) %g  urel(A;h) %g urel(A;F) %g \n", unc->uAuh, unc->uAuF, unc->uAuh/Aphc, unc->uAuF/Aphc);

	/*     u(H_IT)   */
	unc->uHituh = 2 * Hit * uKuh;
	unc->uHituF = Hit *sqrt ( uF*uF/Fmax/Fmax + 4*uKuF*uKuF/K/K );
	/* u(H; h ) = 2 H_IT u(K;h)/ K
	 * u(H; F)  = H_IT sqrt [ (uFmax/Fmax)^2 + (2 u(K;F)/K)^2 ] */
	if (DEBUG)
		printf(" u(Hit;h) %g u(Hit;F) %g  urel(Hit;h) %g urel(Hit;F) %g \n", unc->uHituh, unc->uHituF, unc->uHituh/Hit, unc->uHituF/Hit);


	/*     u(E_r)   */

	unc ->uEruh = Er * sqrt( uKuh*uKuh/K/K + uSuh*uSuh/Su/Su - covKSuh/K/Su);
	unc ->uEruF = Er * sqrt( uF*uF/Fmax/Fmax+ uKuF*uKuF/K/K + uSuF*uSuF/Su/Su - covKSuF/K/Su);
	/* u(Er;h) = Er sqrt [  [u(K;h)/ K]^2 + [u(Su;h)/Su]^2 - cov(K,Su;h)/K/Su ] 
	 * u(Er;F) = Er sqrt [ (uFmax/Fmax)^2 +  [u(K;h)/ K]^2 + [u(Su;h)/Su]^2 - cov(K,Su;F)/K/Su] */


	if (DEBUG)
		printf(" u(Er;h) %g u(Er;F) %g  urel(Er;h) %g urel(Er;F) %g \n", unc->uEruh, unc->uEruF, unc->uEruh/Er, unc->uEruF/Er);


	/*     u(E_IT)   */
	unc->uEituh = Eit*Eit/Er/Er/(1-nu*nu)*unc->uEruh; // Eit^2/Er^2 /(1-nu^2) u(Er;h)
	unc->uEituF = Eit*Eit/Er/Er/(1-nu*nu)*unc->uEruF; // Eit^2/Er^2 /(1-nu^2) u(Er;F)


	unc->uEitunu =  fabs(2*nu/(1-nu*nu)*Eit*unc->unu); // -2nu/(1-nu^2) Eit u(nu)
	unc->uEitunui = fabs(2*nui/(1-nu*nu)* Eit*Eit /Ei *unc->unui)*1e9; // -2nui/(1-nu^2) Eit^2/Ei u(nui)
	unc->uEituEi =  fabs((1-nui*nui)/(1-nu*nu) *Eit*Eit/ Ei*Ei *unc->uEi)*1e-18; //-(1-nui^2)/(1-nu)^2 Eit^2/Ei^2 u(Ei)

	if (DEBUG){
		printf("u(Eit;h) %g  u(Eit;F) %g  urel(Eit;h) %g  urel(Eit;F) %g\n", unc->uEituh, unc->uEituF, unc->uEituh/Eit, unc->uEituF/Eit);
		printf("u(Eit;nu) %g  u(Eit;nui) %g  u(Eit;Ei) %g  urel(Eit;nu) %g  urel(Eit;nui) %g  urel(Eit;Ei) %g\n", unc->uEitunu, unc->uEitunui, unc->uEituEi, unc->uEitunu/Eit, unc->uEitunui/Eit, unc->uEituEi/Eit);
	}



	/* show */
	g_snprintf(str, sizeof(str), NUMBER, unc->unuh);
	gtk_label_set_text(GTK_LABEL(ctrls->unuh), str);
	g_snprintf(str, sizeof(str), NUMBER, unc->unuF);
	gtk_label_set_text(GTK_LABEL(ctrls->unuF), str);

	g_snprintf(str, sizeof(str), NUMBER, unc->umuh);
	gtk_label_set_text(GTK_LABEL(ctrls->umuh), str);
	g_snprintf(str, sizeof(str), NUMBER, unc->umuF);
	gtk_label_set_text(GTK_LABEL(ctrls->umuF), str);

	g_snprintf(str, sizeof(str), SLOPE, unc->uSluh);
	gtk_label_set_text(GTK_LABEL(ctrls->uSluh), str);
	g_snprintf(str, sizeof(str), SLOPE, unc->uSluF);
	gtk_label_set_text(GTK_LABEL(ctrls->uSluF), str);

	g_snprintf(str, sizeof(str), SLOPE, unc->uSuuh);
	gtk_label_set_text(GTK_LABEL(ctrls->uSuuh), str);
	g_snprintf(str, sizeof(str), SLOPE, unc->uSuuF);
	gtk_label_set_text(GTK_LABEL(ctrls->uSuuF), str);

	g_snprintf(str, sizeof(str), AREA, unc->uAuh);
	gtk_label_set_text(GTK_LABEL(ctrls->uAuh), str);
	g_snprintf(str, sizeof(str), AREA, unc->uAuF);
	gtk_label_set_text(GTK_LABEL(ctrls->uAuF), str);

	g_snprintf(str, sizeof(str), HARD, unc->uHituh);
	gtk_label_set_text(GTK_LABEL(ctrls->uHituh), str);
	g_snprintf(str, sizeof(str), HARD, unc->uHituF);
	gtk_label_set_text(GTK_LABEL(ctrls->uHituF), str);

	g_snprintf(str, sizeof(str), MODUL, unc->uEruh);
	gtk_label_set_text(GTK_LABEL(ctrls->uEruh), str);
	g_snprintf(str, sizeof(str), MODUL, unc->uEruF);
	gtk_label_set_text(GTK_LABEL(ctrls->uEruF), str);
	g_snprintf(str, sizeof(str), MODUL, unc->uEituh);
	gtk_label_set_text(GTK_LABEL(ctrls->uEituh), str);
	g_snprintf(str, sizeof(str), MODUL, unc->uEituF);
	gtk_label_set_text(GTK_LABEL(ctrls->uEituF), str);


	g_snprintf(str, sizeof(str), MODUL, unc->uEitunu);
	gtk_label_set_text(GTK_LABEL(ctrls->uEitunu), str);
	g_snprintf(str, sizeof(str), MODUL, unc->uEitunui);
	gtk_label_set_text(GTK_LABEL(ctrls->uEitunui), str);
	g_snprintf(str, sizeof(str), "%.4g", unc->uEituEi);
	gtk_label_set_text(GTK_LABEL(ctrls->uEituEi), str);


	// total uncertainties
	// should take um as given by odr_wrap, otherwise doublecounting
	/* fit with both uh and uF */
	ux = uh;
	uy = uF;
	beta[0] = sldata->gamma;
	beta[1] = sldata->h0;
	beta[2] = sldata->n;

	//find appropriate data, which should be fitted, use same regime as for the main fit
	if (sldata->Finputload)
		range_to_indices(sldata->loadrange_pct_Fmax[0]*0.01*fd->Fmax, sldata->loadrange_pct_Fmax[1]*0.01*fd->Fmax, fd->Fload, FALSE, &istart, &iend, &ndata);
	else
		range_to_indices(sldata->loadrange[0], sldata->loadrange[1], fd->hload, FALSE, &istart, &iend, &ndata);


	x = gwy_data_line_part_extract(fd->hload, istart, ndata);
	y = gwy_data_line_part_extract(fd->Fload, istart, ndata);


	info = odr_run_fit(x, y, beta, fnm, ux, uy, std_beta, covariance, FALSE); /* do not rescale */
	un = std_beta[2];
	uh0 = std_beta[1];
	covnh0 = covariance[0]; 

	unc->un = un;
	if (DEBUG)
		printf(" fit with both uh and uF:\n  gamma %g n %g h0 %g \n  u(n) %g u(h0) %g corr(n,h0) %g \n",
			       beta[0], beta[1], beta[2], un, uh0, covnh0/un/uh0);	       

	//find appropriate data, which should be fitted, use same regime as for the main fit
	if (sldata->Finputunload)
		range_to_indices(sldata->unloadrange_pct_Fmax[0]*0.01*fd->Fmax, sldata->unloadrange_pct_Fmax[1]*0.01*fd->Fmax, fd->Funload, TRUE, &istart, &iend, &ndata);
	else
		range_to_indices(sldata->unloadrange[0], sldata->unloadrange[1], fd->hunload, TRUE, &istart, &iend, &ndata);


	x = gwy_data_line_part_extract(fd->hunload, istart, ndata);
	y = gwy_data_line_part_extract(fd->Funload, istart, ndata);

	beta[0] = sldata->alpha;
	beta[1] = sldata->hp;
	beta[2] = sldata->m;
	info = odr_run_fit(x, y, beta, fnm, ux, uy, std_beta, covariance, FALSE); /* do not rescale */
	um = std_beta[2];
	uhp = std_beta[1];
	covmhp = covariance[0]; 


	/*   u(eps)    */
	z = m/(m-1)/2;
	depsdm = eps/m - m/(m -1) * (eps/m -1) -m/2/(m-1)/(m-1)*(eps/m -1)* (psi(z)-psi(z-1./2));
	ueps = depsdm *um;
	if (DEBUG){
		printf("depsdm %g  z %g  \n", depsdm,z);
		printf(" u(eps) %g    urel(eps) %g  \n", ueps, ueps/eps);
	}


	/*    u(Sl)   */
	uSl = sqrt(um*um/m/m  + uh *uh/(hmax-hp)/(hmax-hp) +  uF*uF/Fmax/Fmax +
			uhp*uhp/(hmax-hp)/(hmax-hp) + 2*covmhp/m/(hmax-hp)) * Sl; 
	unc->uSl = uSl;
	/* u(Sl) = Sl *sqrt [ (umh/m)^2 +  (uhmax/(hmax-hp))^2 + (uFmax/Fmax)^2 +  (uhp/(hmax-hp))^2 + 2 cov(m,hp)/(m*(hmax-hp)) ]
	 */

	if (DEBUG){
		printf(" u(Sl) %g    urel(Sl) %g \n", uSl , uSl/Sl);
	}



	/*    u(Su)   */
	uSu = sqrt(um*um/m/m  + uh *uh/(hmax-hp)/(hmax-hp) +  uF*uF/Fmax/Fmax +
			uhp*uhp/(hmax-hp)/(hmax-hp) + 2*covmhp/m/(hmax-hp)) * Su; 
	unc->uSu = uSu;
	/* u(Su) = Su *sqrt [ (umh/m)^2 +  (uhmax/(hmax-hp))^2 + (uFmax/Fmax)^2 +  (uhp/(hmax-hp))^2 + 2 cov(m,hp)/(m*(hmax-hp)) ]
	 */

	if (DEBUG){
		printf(" u(Su) %g    urel(Su) %g \n", uSu , uSu/Su);
	}
	
	/*    u(K)   */
	K= (2*Su - Sl*eps*BETA)/Su/Sl;

	uK = sqrt( 4*uSl*uSl/Sl/Sl/Sl/Sl + eps *BETA*eps*BETA/Su/Su/Su/Su*uSu*uSu);

	covKSu = -2/Su/Su*uSu*uSu;
	/*  K = 2 /Sl - eps*BETA/Su
	 *  u(K) = sqrt [ (-2/S1^2 u(S1))^2 + ( eps*BETA/Su^2 u(Su)^2 ]
	 *  cov(K,Su) = eps*BETASl^2 u(Sl)^2  */


	/*    u(A)   */
	unc->uA = 2 * Aphc * sqrt( uK*uK/K/K + uF*uF/Fmax/Fmax);
	if (DEBUG)
		printf(" u(A) %g    urel(A) %g  \n", unc->uA, unc->uA/Aphc);

	/*     u(H_IT)   */
	unc->uHit = Hit *sqrt ( uF*uF/Fmax/Fmax + 4*uK*uK/K/K );
	if (DEBUG)
		printf(" u(Hit)  %g   urel(Hit) %g \n", unc->uHit, unc->uHit/Hit);

	/*     u(E_r)   */

	unc ->uEr = Er * sqrt( uF*uF/Fmax/Fmax+ uK*uK/K/K + uSu*uSu/uSu/uSu - covKSu/K/Su);
	if (DEBUG)
		printf(" u(Er) %g   urel(Er) %g  \n", unc->uEr, unc->uEr/Er);


	/*     u(E_IT)   */
	//contribution from h,F
	unc->uEit = Eit*Eit/Er/Er/(1-nu*nu)*unc->uEr; // Eit^2/Er^2 /(1-nu^2) uEr

	//contribution from nu, nui, Ei
	unc->uEitunu =  fabs(2*nu/(1-nu*nu)*Eit*unc->unu); // -2nu/(1-nu^2) Eit u(nu)
	unc->uEitunui = fabs(2*nui/(1-nu*nu)* Eit*Eit /Ei *unc->unui)*1e9; // -2nui/(1-nu^2) Eit^2/Ei u(nui)
	unc->uEituEi =  fabs((1-nui*nui)/(1-nu*nu) *Eit*Eit/Ei*Ei *unc->uEi)*1e-18; //-(1-nui^2)/(1-nu)^2 Eit^2/Ei^2 u(Ei)

	if (DEBUG){
		printf("u(Eit;h,F) %g    urel(Eit;hF) %g  \n", unc->uEit, unc->uEit/Eit);
		printf("u(Eit;nu) %g  u(Eit;nui) %g  u(Eit;Ei) %g  urel(Eit;nu) %g  urel(Eit;nui) %g  urel(Eit;Ei) %g\n", unc->uEitunu, unc->uEitunui, unc->uEituEi, unc->uEitunu/Eit, unc->uEitunui/Eit, unc->uEituEi/Eit);
	}

	unc->uEit = sqrt( unc->uEit*unc->uEit  +unc->uEitunu*unc->uEitunu +unc->uEitunui*unc->uEitunui +unc->uEituEi*unc->uEituEi);
	if (DEBUG)
		printf("u(Eit;hFnuetc) %g    urel(Eit; hF_nu_etc) %g \n", unc->uEit, unc->uEit/Eit);

	if (DEBUG)
		printf(" \nEnd propagation of uncertainties\n\n");


	g_snprintf(str, sizeof(str), NUMBER,  unc->un);
	gtk_label_set_text(GTK_LABEL(ctrls->un), str);

	g_snprintf(str, sizeof(str), NUMBER,  unc->um);
	gtk_label_set_text(GTK_LABEL(ctrls->um), str);

	g_snprintf(str, sizeof(str), SLOPE,  unc->uSl);
	gtk_label_set_text(GTK_LABEL(ctrls->uSl), str);

	g_snprintf(str, sizeof(str), SLOPE,  unc->uSu);
	gtk_label_set_text(GTK_LABEL(ctrls->uSu), str);

	g_snprintf(str, sizeof(str), AREA, unc->uA);
	gtk_label_set_text(GTK_LABEL(ctrls->uA), str);

	g_snprintf(str, sizeof(str), HARD,  unc->uHit);
	gtk_label_set_text(GTK_LABEL(ctrls->uHit), str);

	g_snprintf(str, sizeof(str), MODUL, unc->uEr);
	gtk_label_set_text(GTK_LABEL(ctrls->uEr), str);

	g_snprintf(str, sizeof(str), MODUL, unc->uEit);
	gtk_label_set_text(GTK_LABEL(ctrls->uEit), str);


	g_object_unref(x);
	g_object_unref(y);

	//table_contact doesn't change with any of the adjustables nu, nui or Ei, uh, uF, unu, uniu, uEi
	//no need to update
}

static gboolean 
mc_iterations_changed(Data *data)
{
	gchar str[500];

	data->args->slopesunc.Nmc = (gint)gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->slopesunccontrols->Nmc));
	if (data->args->slopesunc.Nmc > MC_WHEN_WARN){
		g_snprintf(str, sizeof(str), "Warning: Monte Carlo calculation for %d iterations may take very long!", data->args->slopesunc.Nmc);
		gtk_label_set_text(GTK_LABEL(data->controls->slopesunccontrols->mc_warn), str);
	}
	else{
		gtk_label_set_text(GTK_LABEL(data->controls->slopesunccontrols->mc_warn), NULL);
	}

	return FALSE;
}
static void 
unc_depth_changed (Data *data)
{
	data->args->slopesunc.uh = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->slopesunccontrols->uh));
	slopes_propagate_uncertainties (data);
}

static void 
unc_load_changed (Data *data)
{
	data->args->slopesunc.uF = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->slopesunccontrols->uF));
	slopes_propagate_uncertainties (data);
}

static void 
unc_nu_changed (Data *data)
{
	data->args->slopesunc.unu = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->slopesunccontrols->unu));
	slopes_propagate_uncertainties (data);
}

static void 
unc_nui_changed (Data *data)
{
	data->args->slopesunc.unui = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->slopesunccontrols->unui));
	slopes_propagate_uncertainties (data);
}

static void 
unc_Ei_changed (Data *data)
{
	data->args->slopesunc.uEi = gtk_adjustment_get_value(GTK_ADJUSTMENT(data->controls->slopesunccontrols->uEi))*1e9;
	slopes_propagate_uncertainties (data);
}



void slopes_uncertainty_write_file(gchar *filename, Args *args) 
{
	FILE *file;
	Slopesdata *sldata;
	SlopesUncdata *slunc;
	Instdata *instdata;
	gint j;

	sldata = &(args->slopesdata);
	slunc = &(args->slopesunc);
	instdata = &(args->instdata);

	if (args->fddata.hall->res == 0 || args->fddata.Fall->res==0) {
	    show_error("No data.");
	    return;
	}

	file = fopen(filename, "w");
	if (!file) {
	    show_error("Could not open file.");
	    return;
	}

	fprintf(file, "# %s \n", args->fddata.filename);
	fprintf(file, "#Parameters of two slopes analysis (ODR fit) \n");
	fprintf(file, "# u(depth): %g nm\n", slunc->uh);
	fprintf(file, "# u(load): %g nm\n", slunc->uF);
	fprintf(file, "#nu: %g                 u(nu):  %g  \n", instdata->nu, slunc->unu);
	fprintf(file, "#nu_indenter: %g       u(nu_indenter):  %g   \n", instdata->nui, slunc->unui);
	fprintf(file, "#E_indenter: %g        u(E_indenter):   %g GPa\n", instdata->Ei * 1e-9, slunc->uEi*1e-9); //TODO units
	fprintf(file, "#Fitted loading range: ");
	if (sldata->Finputload)
		fprintf(file, "	[%g , %g ] %% Fmax \n", sldata->loadrange_pct_Fmax[0], sldata->loadrange_pct_Fmax[1]);
	else
		fprintf(file, "[%g , %g ] nm \n",sldata->loadrange[0], sldata->loadrange[1]);	
	fprintf(file, "# corresponds : ");
	if (sldata->Finputload)
		fprintf(file, "[%g , %g ] nm \n",sldata->loadrange[0], sldata->loadrange[1]);	
	else
		fprintf(file, "	[%g , %g ] %% Fmax \n", sldata->loadrange_pct_Fmax[0], sldata->loadrange_pct_Fmax[1]);
	fprintf(file, "# %d datapoints used.\n", sldata->nfitloaddata);
	fprintf(file, "#Fitted unloading range: ");
	if (sldata->Finputunload)
		fprintf(file, "	[%g , %g ] %% Fmax \n", sldata->unloadrange_pct_Fmax[0], sldata->unloadrange_pct_Fmax[1]);
	else
		fprintf(file, "[%g , %g ] nm \n",sldata->unloadrange[0], sldata->unloadrange[1]);	
	fprintf(file, "# corresponds : ");
	if (sldata->Finputunload)
		fprintf(file, "[%g , %g ] nm \n",sldata->unloadrange[0], sldata->unloadrange[1]);	
	else
		fprintf(file, "	[%g , %g ] %% Fmax \n", sldata->unloadrange_pct_Fmax[0], sldata->unloadrange_pct_Fmax[1]);
	fprintf(file, "# %d datapoints used.\n", sldata->nfitunloaddata);

	fprintf(file, "#\n");

	fprintf(file, "#Results of two slopes analysis (ODR fit) \n");
	fprintf(file, "#Ap(hc): %g nm^2\n", sldata->Aphc);
	fprintf(file, "#Er: %g GPa\n", sldata->Er);
	fprintf(file, "#Hit: %g GPa\n", sldata->Hit);
	fprintf(file, "#Eit: %g GPa\n", sldata->Eit);
	fprintf(file, "#\n");
	fprintf(file, "#\n");

	fprintf(file, "#Results of the Gaussian propagation of uncertainties \n");
	fprintf(file, "#u(n) =  %g  \n", slunc->un);
	fprintf(file, "#u(Sload) =  %g mN/nm \n", slunc->uSl);
	fprintf(file, "#u(m) =  %g  \n", slunc->um);
	fprintf(file, "#u(Sunload) =  %g mN/nm \n", slunc->uSu);
	fprintf(file, "#u(Ap(hc)) =  %g nm^2 \n", slunc->uA);
	fprintf(file, "#u(Hit) =  %g MPa \n", slunc->uHit);
	fprintf(file, "#u(Er) =  %g GPa \n", slunc->uEr);
	fprintf(file, "#u(Eit) =  %g GPa \n", slunc->uEit);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of n\n");
	fprintf(file, "#u(n;uh) = %g \n", slunc->unuh);
	fprintf(file, "#u(n;uF) = %g \n", slunc->unuF);

	fprintf(file, "#Contributions to uncertainty of Sload\n");
	fprintf(file, "#u(Sl;uh) = %g mN/nm \n", slunc->uSluh);
	fprintf(file, "#u(Sl;uF) = %g mN/nm \n", slunc->uSluF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of m\n");
	fprintf(file, "#u(m;uh) = %g \n", slunc->umuh);
	fprintf(file, "#u(m;uF) = %g \n", slunc->umuF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Sunload\n");
	fprintf(file, "#u(Su;uh) = %g mN/nm \n", slunc->uSuuh);
	fprintf(file, "#u(Su;uF) = %g mN/nm \n", slunc->uSuuF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Ap(hc)\n");
	fprintf(file, "#u(Ap(hc);uh) = %g nm^2 \n", slunc->uAuh);
	fprintf(file, "#u(Ap(hc);uF) = %g nm^2 \n", slunc->uAuF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Hit\n");
	fprintf(file, "#u(Hit;uh) = %g MPa \n", slunc->uHituh);
	fprintf(file, "#u(Hit;uF) = %g MPa \n", slunc->uHituF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Er\n");
	fprintf(file, "#u(Er;uh) = %g GPa \n", slunc->uEruh);
	fprintf(file, "#u(Er;uF) = %g GPa \n", slunc->uEruF);
	fprintf(file, "#\n");

	fprintf(file, "#Contributions to uncertainty of Eit\n");
	fprintf(file, "#u(Eit;uh) = %g GPa \n", slunc->uEituh);
	fprintf(file, "#u(Eit;uF) = %g GPa \n", slunc->uEituF);
	fprintf(file, "#u(Eit;unu) = %g GPa \n", slunc->uEitunu);
	fprintf(file, "#u(Eit;unui) = %g GPa \n", slunc->uEitunui);
	fprintf(file, "#u(Eit;uEi) = %g GPa \n", slunc->uEituEi);
	fprintf(file, "#\n");
	fprintf(file, "#\n");
	fprintf(file, "#\n");


	fprintf(file, "#Changes of the Er corresponding to changes in the contact point \n");
	fprintf(file, "# shift of contact point with respect to the globally chosen contact point (negative - to the left, positive - to the right)   \n");
	fprintf(file, "#Er/GPa \n");
	for (j= -CONTACTMAX; j< CONTACTMAX+1; j++){
		if ( slunc->Ec[j+CONTACTMAX] < 0 )
			fprintf(file, "# %d   not defined \n", j);
		else
			fprintf(file, "# %d   %g  \n", j, slunc->Ec[j+CONTACTMAX]);
	}
	fprintf(file, "#\n");

	fprintf(file, "#Changes of the H corresponding to changes in the contact point \n");
	fprintf(file, "# shift of contact point with respect to the globally chosen contact point (negative - to the left, positive - to the right)   \n");
	fprintf(file, "#H/MPa \n");
	for (j= -CONTACTMAX; j< CONTACTMAX+1; j++){
		if ( slunc->Hc[j+CONTACTMAX] < 0 )
			fprintf(file, "# %d   not defined \n", j);
		else
			fprintf(file, "# %d   %g  \n", j, slunc->Hc[j+CONTACTMAX]);
	}
	fprintf(file, "#\n");

	fprintf(file, "#Changes of the Ap corresponding to changes in the contact point \n");
	fprintf(file, "# shift of contact point with respect to the globally chosen contact point (negative - to the left, positive - to the right)   \n");
	fprintf(file, "#A/nm^2 \n");
	for (j= -CONTACTMAX; j< CONTACTMAX+1; j++){
		if ( slunc->Ac[j+CONTACTMAX] < 0 )
			fprintf(file, "# %d   not defined \n", j);
		else
			fprintf(file, "# %d   %g  \n", j, slunc->Ac[j+CONTACTMAX]);
	}

	fclose(file); 
}

static void slopes_uncertainty_save_data( Args *args)
{
	GtkWidget *dialog;
	gchar *filename;

	dialog = gtk_file_chooser_dialog_new("Save File",
			NULL,
			GTK_FILE_CHOOSER_ACTION_SAVE,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_SAVE, GTK_RESPONSE_OK,
			NULL);

	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), gwy_app_get_current_directory());

	if ( gtk_dialog_run (GTK_DIALOG ( dialog)) == GTK_RESPONSE_OK)
	{
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		if (gwy_app_file_confirm_overwrite(dialog))
			slopes_uncertainty_write_file(filename, args);
		g_free(filename);
	}
	gtk_widget_destroy ( dialog);
}


static void
slopes_fit_shift_contact(Slopesdata *sldata, FDdata *fddata, Instdata *instdata, Area *area,gdouble *Ec,gdouble *Hc, gdouble *Ac,gint j)
{
	Slopesdata slopes;
	FDdata fdc;
	gdouble dh,dF;
	gint istart,iend, ndata;
	GwyDataLine *x, *y;

	Ec[j+CONTACTMAX] = -1;
	Hc[j+CONTACTMAX] = -1;
	Ac[j+CONTACTMAX] = -1;

	// create shifted FDdata

	if (fddata->i_contact_load+j < 0)
		return;
	fdc.i_contact_load = fddata->i_contact_load+j;

	fdc.hall_orig = gwy_data_line_duplicate(fddata->hall_orig);
	fdc.Fall_orig = gwy_data_line_duplicate(fddata->Fall_orig);

	fdc.hunload = gwy_data_line_duplicate(fddata->hunload);
	fdc.Funload = gwy_data_line_duplicate(fddata->Funload);
	fdc.hload = gwy_data_line_duplicate(fddata->hload);
	fdc.Fload = gwy_data_line_duplicate(fddata->Fload);

	//difference in shift between different contactpoints
	dh = fdc.hall_orig->data[fdc.i_contact_load]-fddata->hall_orig->data[fddata->i_contact_load];
	dF = fdc.Fall_orig->data[fdc.i_contact_load]-fddata->Fall_orig->data[fddata->i_contact_load];

	gwy_data_line_add(fdc.hunload, -dh);
	gwy_data_line_add(fdc.Funload, -dF);
	gwy_data_line_add(fdc.hload, -dh);
	gwy_data_line_add(fdc.Fload, -dF);


	fdc.hmax = fddata->hmax -dh;
	fdc.Fmax = fddata->Fmax -dF;


	// create working copy of Slopesdata 
	// shift fitting range
	slopes.eps = sldata->eps;
	slopes.loadrange[0]= sldata->loadrange[0]-dh;
	slopes.loadrange[1]= sldata->loadrange[1]-dh;
	slopes.loadfrom = slopes.loadrange[0];
	slopes.loadto = slopes.loadrange[1];
	slopes.loadfrom_pct_Fmax = sldata->loadfrom_pct_Fmax;
	slopes.loadto_pct_Fmax = sldata->loadto_pct_Fmax;
	slopes.has_fit_load = FALSE;
	slopes.xloadfit = NULL;
	slopes.yloadfit = NULL;
	slopes.unloadrange[0]= sldata->unloadrange[0]-dh;
	slopes.unloadrange[1]= sldata->unloadrange[1]-dh;
	slopes.unloadfrom = slopes.unloadrange[0];
	slopes.unloadto = slopes.unloadrange[1];
	slopes.unloadfrom_pct_Fmax = sldata->unloadfrom_pct_Fmax;
	slopes.unloadto_pct_Fmax = sldata->unloadto_pct_Fmax;
	slopes.has_fit_unload = FALSE;
	slopes.xunloadfit = NULL;
	slopes.yunloadfit = NULL;



	//check range
	if (slopes.loadfrom ==  slopes.loadto){
		return;
	}

	//find appropriate data, which should be fitted, use same regime as for the main fit
	if (sldata->Finputload)
		range_to_indices(slopes.loadfrom_pct_Fmax*0.01*fdc.Fmax, slopes.loadto_pct_Fmax*0.01*fdc.Fmax, fdc.Fload, FALSE, &istart, &iend, &ndata);
	else
		range_to_indices(slopes.loadfrom, slopes.loadto, fdc.hload, FALSE, &istart, &iend, &ndata);
	// if there are not enough data in the selected range, not fitting can be performed
	ndata = MIN(ndata, fdc.hload->res - istart);

	if (ndata < 2){
		return;
	}

	//get data to be fitted
	x = gwy_data_line_part_extract(fdc.hload, istart, ndata);
	y = gwy_data_line_part_extract(fdc.Fload, istart, ndata);


	// perform fit and get results
	slopes_fit_load(x,y,&slopes, &fdc, instdata, NULL);

	// clean up
	g_object_unref(x);
	g_object_unref(y);



	//check range
	if (slopes.unloadfrom ==  slopes.unloadto){
		return;
	}

	//find appropriate data, which should be fitted, use same regime as for the main fit
	if (sldata->Finputunload)
		range_to_indices(slopes.unloadfrom_pct_Fmax*0.01*fdc.Fmax, slopes.unloadto_pct_Fmax*0.01*fdc.Fmax, fdc.Funload, TRUE, &istart, &iend, &ndata);
	else
		range_to_indices(slopes.unloadfrom, slopes.unloadto, fdc.hunload, TRUE, &istart, &iend, &ndata);
	// if there are not enough data in the selected range, not fitting can be performed
	ndata = MIN(ndata, fdc.hunload->res - istart);

	if (ndata < 2){
		return;
	}

	//get data to be fitted
	x = gwy_data_line_part_extract(fdc.hunload, istart, ndata);
	y = gwy_data_line_part_extract(fdc.Funload, istart, ndata);


	// perform fit and get results
	slopes_fit_unload(x,y,&slopes, &fdc, instdata, NULL);

	// clean up
	g_object_unref(x);
	g_object_unref(y);
	g_object_unref(fdc.hall_orig);
	g_object_unref(fdc.Fall_orig);
	g_object_unref(fdc.hload);
	g_object_unref(fdc.Fload);
	g_object_unref(fdc.hunload);
	g_object_unref(fdc.Funload);

	slopes_combine_fit_results(&slopes, fddata, instdata);
	Ec[j+CONTACTMAX] = slopes.Er;
	Hc[j+CONTACTMAX] = slopes.Hit;
	Ac[j+CONTACTMAX] = slopes.Aphc;

}


void 
slopes_unc_close_window(Data *data)
{
	g_free(data->args->slopesunc.Ec);
	g_free(data->args->slopesunc.Hc);
	g_free(data->args->slopesunc.Ac);
	data->args->slopesunc.Ec = NULL;
	data->args->slopesunc.Hc = NULL;
	data->args->slopesunc.Ac = NULL;
	slopes_mc_close_window(data);
	gtk_widget_destroy(data->controls->slopesunccontrols->window);
}
void
slopes_uncertainty (Data *data)
{
	GtkWidget *label, *spin;
	gint i, j;
	SlopesUncControls *ctrls;
	SlopesUncdata *unc;
	gchar str[500];

	if ((data->args->slopesunc.Ec != NULL) & (data->args->slopesunc.Hc != NULL ) & (data->args->slopesunc.Ac != NULL )) {
		gtk_window_present (GTK_WINDOW (data->controls->slopesunccontrols->window));
		return;
	}

	unc = &(data->args->slopesunc);
        ctrls =	data->controls->slopesunccontrols;


	ctrls->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->window), "Uncertainty in Oliver Pharr analysis (ODR fit)");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->window), 400, 300); 
	g_signal_connect_swapped (ctrls->window, "delete_event", G_CALLBACK (slopes_unc_close_window), data);
	gtk_window_set_transient_for (GTK_WINDOW(ctrls->window), GTK_WINDOW(data->controls->maincontrols->window_main));

	/* Uncertainties due to uncertainties in input values */
	
	ctrls->frame_unc_input = gtk_frame_new(NULL);
//	gtk_frame_set_label_widget (GTK_FRAME (ctrls->frame_unc_input),
//				    GTK_WIDGET (gwy_label_new_header("Uncertainties due to uncertainties in input values")));
	gtk_frame_set_label_widget (GTK_FRAME (ctrls->frame_unc_input),
				    GTK_WIDGET (gwy_label_new_header("Uncertainties in input values")));

	ctrls->table_unc_input = gtk_table_new(6, 13, FALSE);

	gtk_container_set_border_width (GTK_CONTAINER (ctrls->table_unc_input), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (ctrls->table_unc_input), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (ctrls->table_unc_input), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add ( GTK_CONTAINER (ctrls->frame_unc_input), ctrls->table_unc_input);

	i = 0;
	
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input),
			 gwy_label_new_header("u(m)"), 3, 4, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input),
			 gwy_label_new_header("u(n)"), 4, 5, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input),
			 gwy_label_new_header("u(S<sub>load</sub>)/(mN/nm)"), 5, 6, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input),
			 gwy_label_new_header("u(S<sub>unload</sub>)/(mN/nm)"), 6, 7, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input),
			 gwy_label_new_header("u(A<sub>p</sub>)/nm<sup>2</sup>"), 7, 8, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input),
			 gwy_label_new_header("u(H<sub>it</sub>)/MPa"), 8, 9, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input),
			 gwy_label_new_header("u(E<sub>r</sub>)/GPa"), 9, 10, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input),
			 gwy_label_new_header("u(E<sub>it</sub>)/GPa"), 10, 11, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

	i++;

	/* u(h) */

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(h)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->uh = gtk_adjustment_new(unc->uh, 0, 1e6, 0.01, 1,0); 
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->uh), 0.001, 3);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	g_signal_connect_swapped(ctrls->uh, "value-changed", G_CALLBACK(unc_depth_changed), data);
	
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), label_new_with_markup_left("nm"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);

	ctrls->umuh = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->umuh), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->umuh, 3, 4, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->unuh = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->unuh), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->unuh, 4, 5, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->uSluh = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uSluh), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uSluh, 5, 6, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->uSuuh = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uSuuh), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uSuuh, 6, 7, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->uAuh = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uAuh), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uAuh, 7, 8, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->uHituh = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uHituh), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uHituh, 8, 9, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->uEruh = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEruh), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEruh, 9, 10, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->uEituh = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEituh), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEituh, 10, 11, i, i+1, GTK_FILL, 0, 0, 0);
	
	i++;

	/* u(F) */

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(F)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->uF = gtk_adjustment_new(unc->uF, 0, 1e6, 0.0001, 0.01,0.);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->uF), 0.0001, 4);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	g_signal_connect_swapped(ctrls->uF, "value-changed", G_CALLBACK(unc_load_changed), data);
	
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), label_new_with_markup_left("mN"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);

	ctrls->umuF = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->umuF), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->umuF, 3, 4, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

	ctrls->unuF = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->unuF), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->unuF, 4, 5, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

	ctrls->uSluF = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uSluF), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uSluF, 5, 6, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

	ctrls->uSuuF = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uSuuF), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uSuuF, 6, 7, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

	ctrls->uAuF = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uAuF), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uAuF, 7, 8, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	ctrls->uHituF = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uHituF), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uHituF, 8, 9, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	ctrls->uEruF = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEruF), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEruF, 9, 10, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	ctrls->uEituF = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEituF), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEituF, 10, 11, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	i++;

	/* u(nu) */

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(ν)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->unu = gtk_adjustment_new(unc->unu, 0, 0.5, 0.01, 0.1, 0.);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->unu), 0.01, 2);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	g_signal_connect_swapped(ctrls->unu, "value-changed", G_CALLBACK(unc_nu_changed), data);
	
	ctrls->uEitunu = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEitunu), 0.0, 0.5); 
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEitunu, 10, 11, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 

	i++;

	/* u(nu_i) */

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(ν<sub>i</sub>)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->unui = gtk_adjustment_new(unc->unui, 0, 0.5, 0.01, 0.1, 0.);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->unui), 0.01, 2);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	g_signal_connect_swapped(ctrls->unui, "value-changed", G_CALLBACK(unc_nui_changed), data);
	
	ctrls->uEitunui = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEitunui), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEitunui, 10, 11, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	i++;

	/* u(E_i) */

	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("u(E<sub>i</sub>)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->uEi = gtk_adjustment_new(unc->uEi*1e-9, 0, 1e5, 1, 10, 0.);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->uEi), 1, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), spin, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	g_signal_connect_swapped(ctrls->uEi, "value-changed", G_CALLBACK(unc_Ei_changed), data);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), label_new_with_markup_left("GPa"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->uEituEi = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEituEi), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEituEi, 10, 11, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

	i++;

	/* total */
	
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), gwy_label_new_header("Total"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->um = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->um), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->um, 3, 4, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	ctrls->un = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->un), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->un, 4, 5, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	ctrls->uSl = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uSl), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uSl, 5, 6, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

	ctrls->uSu = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uSu), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uSu, 6, 7, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

	ctrls->uA = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uA), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uA, 7, 8, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	ctrls->uHit = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uHit), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uHit, 8, 9, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	ctrls->uEr = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEr), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEr, 9, 10, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	
	ctrls->uEit = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(ctrls->uEit), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_input), ctrls->uEit, 10, 11, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);


	/* Uncertainties due to contact point */

	ctrls->frame_unc_contact = gtk_frame_new(NULL);
	gtk_frame_set_label_widget (GTK_FRAME (ctrls->frame_unc_contact),
				    GTK_WIDGET (gwy_label_new_header("Uncertainties due to choice of contact point")));
	
	ctrls->table_unc_contact = gtk_table_new(6, 5, FALSE);
	gtk_container_set_border_width (GTK_CONTAINER (ctrls->table_unc_contact), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (ctrls->table_unc_contact), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (ctrls->table_unc_contact), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add (GTK_CONTAINER (ctrls->frame_unc_contact), ctrls->table_unc_contact);

	/* table headers */

	i=0;
	
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact),
			 gwy_label_new_header("Contact point shift"), 0, 1, i, i+1,  GTK_EXPAND | GTK_FILL, 0, 0, 0);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact),
			 gwy_label_new_header("E<sub>r</sub>/GPa"), 1, 2, i, i+1,  GTK_EXPAND | GTK_FILL, 0, 0, 0);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact),
			 gwy_label_new_header("H<sub>it</sub>/GPa"), 2, 3, i, i+1,  GTK_EXPAND | GTK_FILL, 0, 0, 0);
	gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact),
			 gwy_label_new_header("A<sub>p</sub>/nm^2"), 3, 4, i, i+1,  GTK_EXPAND | GTK_FILL, 0, 0, 0);
	// gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact),
	// gwy_label_new_header("weight"), 2, 3, i,i+1,  GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);

	i++;

	// Ec is Er, Hc is Hit
	unc->Ec = (gdouble *)g_malloc((2*CONTACTMAX+1)*sizeof(gdouble));
	unc->Hc = (gdouble *)g_malloc((2*CONTACTMAX+1)*sizeof(gdouble));
	unc->Ac = (gdouble *)g_malloc((2*CONTACTMAX+1)*sizeof(gdouble));

	/* values */

	for (j = -CONTACTMAX; j <= CONTACTMAX; j++)	{

	    /* vertical alignments: j: 1.0 (bottom), others: 0.0 (top); looks quite nice, still probably incorrect */
	    
		label = gtk_label_new(NULL);
		g_snprintf(str, sizeof(str),"%d\n", j); 
		gtk_label_set_text (GTK_LABEL(label), str);
		gtk_misc_set_alignment (GTK_MISC(label), 0.0, 1.0);
		gtk_table_attach (GTK_TABLE (ctrls->table_unc_contact), label, 0, 1, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

		slopes_fit_shift_contact(&(data->args->slopesdata), &(data->args->fddata), &(data->args->instdata), &(data->args->area), unc->Ec, unc->Hc, unc->Ac, j);

		//fill in Er
		label = gtk_label_new(NULL);
		if (unc->Ec[j+CONTACTMAX] > 0)
			g_snprintf(str, sizeof(str), MODUL, unc->Ec[j+CONTACTMAX]); 
		else
			g_snprintf(str, sizeof(str), "not defined");
		gtk_label_set_text (GTK_LABEL(label), str);
		gtk_misc_set_alignment (GTK_MISC(label), 0.0, 0.0); 
		gtk_table_attach (GTK_TABLE(ctrls->table_unc_contact), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

		/*
		label = gtk_label_new(NULL);
		g_snprintf(str, sizeof(str),"1/%d", 2*CONTACTMAX+1); 
		gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0); 
		gtk_table_attach(GTK_TABLE(ctrls->table_unc_contact), label, 2, 3, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 
		*/

		//fill in Hit
		label = gtk_label_new(NULL);
		if (unc->Hc[j+CONTACTMAX] > 0)
			g_snprintf(str, sizeof(str), HARD, unc->Hc[j+CONTACTMAX]); 
		else
			g_snprintf(str, sizeof(str), "not defined");
		gtk_label_set_text (GTK_LABEL(label), str);
		gtk_misc_set_alignment (GTK_MISC(label), 0.0, 0.0); 
		gtk_table_attach (GTK_TABLE(ctrls->table_unc_contact), label, 2, 3, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

		//fill in Ap
		label = gtk_label_new(NULL);
		if (unc->Ac[j+CONTACTMAX] > 0)
			g_snprintf(str, sizeof(str), AREA, unc->Ac[j+CONTACTMAX]); 
		else
			g_snprintf(str, sizeof(str), "not defined");
		gtk_label_set_text (GTK_LABEL(label), str);
		gtk_misc_set_alignment (GTK_MISC(label), 0.0, 0.0); 
		gtk_table_attach (GTK_TABLE(ctrls->table_unc_contact), label, 3, 4, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 


		i++;
	}

	slopes_propagate_uncertainties (data);

	ctrls->button_save = gtk_button_new_from_stock (GTK_STOCK_SAVE);
	g_signal_connect_swapped (ctrls->button_save, "clicked", G_CALLBACK (slopes_uncertainty_save_data), data->args);

	/* Monte Carlo calculation */
	
	ctrls->frame_mc = gtk_frame_new(NULL);
	gtk_frame_set_label_widget (GTK_FRAME (ctrls->frame_mc),
				    GTK_WIDGET (gwy_label_new_header("Monte Carlo calculation of uncertainties")));

	ctrls->table_mc = gtk_table_new(6, 7, FALSE);

	gtk_container_set_border_width (GTK_CONTAINER (ctrls->table_mc), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (ctrls->table_mc), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (ctrls->table_mc), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add ( GTK_CONTAINER (ctrls->frame_mc), ctrls->table_mc);

	i = 0;
	gtk_table_attach(GTK_TABLE(ctrls->table_mc), gwy_label_new_header("Number of iterations"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);
	
	ctrls->Nmc = gtk_adjustment_new(unc->Nmc, 1, 1e9, 10, 100,0); 
	
	ctrls->spin_Nmc = gtk_spin_button_new(GTK_ADJUSTMENT(ctrls->Nmc), 1, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(ctrls->spin_Nmc), TRUE);
	gtk_table_attach(GTK_TABLE(ctrls->table_mc), ctrls->spin_Nmc, 1, 2, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	g_signal_connect_swapped(ctrls->Nmc, "value-changed", G_CALLBACK(mc_iterations_changed), data);
	i++;
	

	ctrls->mc_warn = gtk_label_new(NULL);
	gtk_table_attach(GTK_TABLE(ctrls->table_mc), ctrls->mc_warn, 0, 3, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); 
	if (unc->Nmc > MC_WHEN_WARN){
		g_snprintf(str, sizeof(str), "Warning: Monte Carlo calculation for %d iterations may take very long!", unc->Nmc);
		gtk_label_set_text(GTK_LABEL(ctrls->mc_warn), str);
	}

	// no Monte Carlo window without the uncertainty dialog
	data->controls->slopesmccontrols->has_window = FALSE;
	ctrls->button_mc = gtk_button_new_with_mnemonic ("_Monte Carlo");
	gtk_button_set_image (GTK_BUTTON (ctrls->button_mc), gtk_image_new_from_stock (GWY_STOCK_POLYNOM, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (ctrls->button_mc, "clicked", G_CALLBACK (slopes_uncertainty_montecarlo), data);

	ctrls->vbox = gtk_vbox_new(FALSE, CTRLS_SPACING);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->frame_unc_input, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->frame_unc_contact, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->button_save, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->frame_mc, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->button_mc, FALSE, FALSE, 0);
	
	gtk_container_add(GTK_CONTAINER(ctrls->window), ctrls->vbox);
	gtk_widget_show_all(ctrls->window);
}

#endif
