#include "xmlsettings.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

#define ROOT_NODE_NAME "niget-settings"

static xmlNodePtr op_assemble_xml_settings();
static void op_parse_xml_settings(xmlDocPtr doc, xmlNodePtr cur);

void
save_xml_settings(const gchar *filename, Args *args)
{
    xmlDocPtr doc;
    xmlNodePtr root, newnode;
    int i;
    char str[16];

    /* root node atribut version="NIGETVERSION", muze se hodit */

    /* gboolean: "true","TRUE" / "false", "FALSE" */
    /* gdouble: asi %g, udela automaticky; (jak to pak nacist?) */

    doc = xmlNewDoc ((xmlChar*)"1.0");

    root = xmlNewNode (NULL, (xmlChar*)ROOT_NODE_NAME);
    xmlSetProp (root, (xmlChar*)"version", (xmlChar*)"0.3");
    xmlDocSetRootElement (doc, root);

    xmlAddChild(root, op_assemble_xml_settings(args));
    //xmlAddChild(root, op_odr_assemble_xml_settings(args));

    newnode = xmlNewNode(NULL, (xmlChar*)"integers");
    xmlSetProp (newnode, (xmlChar*)"array", NULL);
    for (i = 0; i < 10; i++) {
	sprintf(str, "%d", i);
	xmlNewTextChild(newnode, NULL, (xmlChar*)"i", (xmlChar*)str);
    }
    xmlAddChild(root, newnode);

    newnode = xmlNewNode(NULL, (xmlChar*)"doubles");
    xmlSetProp (newnode, (xmlChar*)"array", NULL);
    for (i = 0; i < 10; i++) {
	sprintf(str, "%g", 1.0/i);
	xmlNewTextChild(newnode, NULL, (xmlChar*)"d", (xmlChar*)str);
    }
    xmlAddChild(root, newnode);

    /* xmlSetCompressMode(1); */ /* do not compress settings */
    xmlSaveFormatFile(filename, doc, 1); /* 1: indent output */
    xmlFreeDoc(doc); /* pokud bude lokalne vytvoren */
}

static xmlNodePtr
op_assemble_xml_settings(Args *args)
{
    xmlNodePtr cur;

    cur = xmlNewNode(NULL, (xmlChar*)"OP");
    xmlNewTextChild(cur, NULL, (xmlChar*)"from", (xmlChar*)"10");
    xmlNewTextChild(cur, NULL, (xmlChar*)"to", (xmlChar*)"20");
    return cur;
}


void
load_xml_settings(const gchar *filename, Args *args)
{
    xmlDocPtr doc;
    xmlNodePtr cur;

    doc = xmlParseFile(filename);
    if (doc == NULL) {
	printf("Error: XML settings not parsed successfully\n");
	return;
    }

    cur = xmlDocGetRootElement(doc);
    if (cur == NULL) {
	printf("Error: empty XML document\n");
	xmlFreeDoc(doc);
	return;
    }
	
    if (!xmlStrEqual(cur->name, (const xmlChar*)ROOT_NODE_NAME)) {
	printf("Error: XML document not of settings type\n");
	xmlFreeDoc(doc);
	return;
    }
    else {
	/* zobrazit verzi */
    }

    cur = cur->xmlChildrenNode;
    while (cur != NULL) {
	if ((xmlStrEqual(cur->name, (const xmlChar*)"OP")))
	    op_parse_xml_settings (doc, cur);
	/* if ((xmlStrEqual(cur->name, (const xmlChar*)"OPODR"))) */
	/*     op_odr_parse_xml_settings (doc, cur); */
	cur = cur->next;
    }
}

static void
op_parse_xml_settings(xmlDocPtr doc, xmlNodePtr cur)
{
    xmlChar *key;

    cur = cur->xmlChildrenNode;
    while (cur != NULL) {
	if ((xmlStrEqual(cur->name, (const xmlChar *)"OP"))) {
	    key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
	    /* neco */
	    xmlFree(key);
	}
	cur = cur->next;
    }
    return;
}
